@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -33,9 +33,8 @@
 
 namespace JSC {
 
-#if ENABLE(MASM_PROBE)
-
-extern "C" void ctiMasmProbeTrampoline();
+JSC_DECLARE_JIT_OPERATION(ctiMasmProbeTrampoline, void, ());
+JSC_ANNOTATE_JIT_OPERATION_PROBE(ctiMasmProbeTrampoline);
 
 using namespace ARMRegisters;
 
@@ -229,12 +228,12 @@ asm (
     // Incoming register values:
     //     r0: probe function
     //     r1: probe arg
-    //     r2: Probe::executeProbe
+    //     r2: Probe::executeJSCJITProbe
     //     ip: scratch, was ctiMasmProbeTrampoline
     //     lr: return address
 
     "mov       ip, sp" "\n"
-    "str       r2, [ip, #-" STRINGIZE_VALUE_OF(PTR_SIZE) "]" "\n" // Stash Probe::executeProbe.
+    "str       r2, [ip, #-" STRINGIZE_VALUE_OF(PTR_SIZE) "]" "\n" // Stash Probe::executeJSCJITProbe.
 
     "mov       r2, sp" "\n"
     "sub       r2, r2, #" STRINGIZE_VALUE_OF(PROBE_SIZE + OUT_SIZE) "\n"
@@ -242,7 +241,7 @@ asm (
     // The ARM EABI specifies that the stack needs to be 16 byte aligned.
     "bic       r2, r2, #0xf" "\n"
     "mov       sp, r2" "\n" // Set the sp to protect the Probe::State from interrupts before we initialize it.
-    "ldr       r2, [ip, #-" STRINGIZE_VALUE_OF(PTR_SIZE) "]" "\n" // Reload Probe::executeProbe.
+    "ldr       r2, [ip, #-" STRINGIZE_VALUE_OF(PTR_SIZE) "]" "\n" // Reload Probe::executeJSCJITProbe.
 
     "str       r0, [sp, #" STRINGIZE_VALUE_OF(PROBE_PROBE_FUNCTION_OFFSET) "]" "\n"
     "str       r1, [sp, #" STRINGIZE_VALUE_OF(PROBE_ARG_OFFSET) "]" "\n"
@@ -281,7 +280,7 @@ asm (
     "mov       r5, sp" "\n"
 
     "mov       r0, sp" "\n" // the Probe::State* arg.
-    "blx       r2" "\n" // Call Probe::executeProbe.
+    "blx       r2" "\n" // Call Probe::executeJSCJITProbe.
 
     // Make sure the Probe::State is entirely below the result stack pointer so
     // that register values are still preserved when we call the initializeStack
@@ -362,6 +361,7 @@ asm (
     // Restore the sp and pc.
     "ldr       sp, [sp, #" STRINGIZE_VALUE_OF(PROBE_CPU_SP_OFFSET) "]" "\n"
     "pop       { pc }" "\n"
+    ".previous" "\n"
 );
 #endif // COMPILER(GCC_COMPATIBLE)
 
@@ -381,11 +381,10 @@ void MacroAssembler::probe(Probe::Function function, void* arg)
     // This means we must first preserve the apsr flags above first.
     move(TrustedImmPtr(reinterpret_cast<void*>(function)), r0);
     move(TrustedImmPtr(arg), r1);
-    move(TrustedImmPtr(reinterpret_cast<void*>(Probe::executeProbe)), r2);
+    move(TrustedImmPtr(reinterpret_cast<void*>(Probe::executeJSCJITProbe)), r2);
     move(TrustedImmPtr(reinterpret_cast<void*>(ctiMasmProbeTrampoline)), ip);
     m_assembler.blx(ip);
 }
-#endif // ENABLE(MASM_PROBE)
 
 } // namespace JSC
 
