@@ -86,6 +86,8 @@ struct CallLinkRecord {
 // call to be linked).
 class JITCompiler : public CCallHelpers {
 public:
+    friend class SpeculativeJIT;
+
     JITCompiler(Graph& dfg);
     ~JITCompiler();
     
@@ -145,6 +147,18 @@ public:
         m_calls.append(CallLinkRecord(functionCall, function.retagged<OperationPtrTag>()));
         return functionCall;
     }
+
+    Call appendOperationCall(const FunctionPtr<OperationPtrTag> function)
+    {
+        Call functionCall = call(OperationPtrTag);
+        m_calls.append(CallLinkRecord(functionCall, function));
+        return functionCall;
+    }
+
+    void appendCall(CCallHelpers::Address address)
+    {
+        call(address, OperationPtrTag);
+    }
     
     void exceptionCheck();
 
@@ -185,6 +199,11 @@ public:
         m_putByIds.append(InlineCacheWrapper<JITPutByIdGenerator>(gen, slowPath));
     }
 
+    void addPutByVal(const JITPutByValGenerator& gen, SlowPathGenerator* slowPath)
+    {
+        m_putByVals.append(InlineCacheWrapper<JITPutByValGenerator>(gen, slowPath));
+    }
+
     void addDelById(const JITDelByIdGenerator& gen, SlowPathGenerator* slowPath)
     {
         m_delByIds.append(InlineCacheWrapper<JITDelByIdGenerator>(gen, slowPath));
@@ -205,19 +224,24 @@ public:
         m_inByIds.append(InlineCacheWrapper<JITInByIdGenerator>(gen, slowPath));
     }
 
-    void addJSCall(Call fastCall, Call slowCall, DataLabelPtr targetToCheck, CallLinkInfo* info)
+    void addInByVal(const JITInByValGenerator& gen, SlowPathGenerator* slowPath)
     {
-        m_jsCalls.append(JSCallRecord(fastCall, slowCall, targetToCheck, info));
+        m_inByVals.append(InlineCacheWrapper<JITInByValGenerator>(gen, slowPath));
+    }
+
+    void addPrivateBrandAccess(const JITPrivateBrandAccessGenerator& gen, SlowPathGenerator* slowPath)
+    {
+        m_privateBrandAccesses.append(InlineCacheWrapper<JITPrivateBrandAccessGenerator>(gen, slowPath));
+    }
+
+    void addJSCall(Label slowPathStart, Label doneLocation, OptimizingCallLinkInfo* info)
+    {
+        m_jsCalls.append(JSCallRecord(slowPathStart, doneLocation, info));
     }
     
-    void addJSDirectCall(Call call, Label slowPath, CallLinkInfo* info)
+    void addJSDirectCall(Label slowPath, OptimizingCallLinkInfo* info)
     {
-        m_jsDirectCalls.append(JSDirectCallRecord(call, slowPath, info));
-    }
-    
-    void addJSDirectTailCall(PatchableJump patchableJump, Call call, Label slowPath, CallLinkInfo* info)
-    {
-        m_jsDirectTailCalls.append(JSDirectTailCallRecord(patchableJump, call, slowPath, info));
+        m_jsDirectCalls.append(JSDirectCallRecord(slowPath, info));
     }
     
     void addWeakReference(JSCell* target)
@@ -244,7 +268,7 @@ public:
     {
         Structure* structure = weakStructure.get();
 #if USE(JSVALUE64)
-        Jump result = branch32(cond, left, TrustedImm32(structure->id()));
+        Jump result = branch32(cond, left, TrustedImm32(structure->id().bits()));
         return result;
 #else
         return branchPtr(cond, left, TrustedImmPtr(structure));
@@ -253,7 +277,21 @@ public:
 
     void noticeOSREntry(BasicBlock&, JITCompiler::Label blockHead, LinkBuffer&);
     void noticeCatchEntrypoint(BasicBlock&, JITCompiler::Label blockHead, LinkBuffer&, Vector<FlushFormat>&& argumentFormats);
-    
+
+    unsigned appendOSRExit(OSRExit&& exit)
+    {
+        unsigned result = m_osrExit.size();
+        m_osrExit.append(WTFMove(exit));
+        return result;
+    }
+
+    unsigned appendSpeculationRecovery(const SpeculationRecovery& recovery)
+    {
+        unsigned result = m_speculationRecovery.size();
+        m_speculationRecovery.append(recovery);
+        return result;
+    }
+
     RefPtr<JITCode> jitCode() { return m_jitCode; }
     
     Vector<Label>& blockHeads() { return m_blockHeads; }
@@ -264,6 +302,16 @@ public:
 
     VM& vm() { return m_graph.m_vm; }
 
+    void emitRestoreCalleeSaves()
+    {
+        emitRestoreCalleeSavesFor(&RegisterAtOffsetList::dfgCalleeSaveRegisters());
+    }
+
+    void emitSaveCalleeSaves()
+    {
+        emitSaveCalleeSavesFor(&RegisterAtOffsetList::dfgCalleeSaveRegisters());
+    }
+
 private:
     friend class OSRExitJumpPlaceholder;
     
@@ -295,71 +343,52 @@ private:
     Vector<CallLinkRecord> m_calls;
     JumpList m_exceptionChecks;
     JumpList m_exceptionChecksWithCallFrameRollback;
-    
+
     Vector<Label> m_blockHeads;
 
 
     struct JSCallRecord {
-        JSCallRecord(Call fastCall, Call slowCall, DataLabelPtr targetToCheck, CallLinkInfo* info)
-            : fastCall(fastCall)
-            , slowCall(slowCall)
-            , targetToCheck(targetToCheck)
+        JSCallRecord(Label slowPathStart, Label doneLocation, OptimizingCallLinkInfo* info)
+            : slowPathStart(slowPathStart)
+            , doneLocation(doneLocation)
             , info(info)
         {
-            ASSERT(fastCall.isFlagSet(Call::Near));
-            ASSERT(slowCall.isFlagSet(Call::Near));
         }
         
-        Call fastCall;
-        Call slowCall;
-        DataLabelPtr targetToCheck;
-        CallLinkInfo* info;
+        Label slowPathStart;
+        Label doneLocation;
+        OptimizingCallLinkInfo* info;
     };
     
     struct JSDirectCallRecord {
-        JSDirectCallRecord(Call call, Label slowPath, CallLinkInfo* info)
-            : call(call)
-            , slowPath(slowPath)
+        JSDirectCallRecord(Label slowPath, OptimizingCallLinkInfo* info)
+            : slowPath(slowPath)
             , info(info)
         {
-            ASSERT(call.isFlagSet(Call::Near));
         }
         
-        Call call;
         Label slowPath;
-        CallLinkInfo* info;
+        OptimizingCallLinkInfo* info;
     };
     
-    struct JSDirectTailCallRecord {
-        JSDirectTailCallRecord(PatchableJump patchableJump, Call call, Label slowPath, CallLinkInfo* info)
-            : patchableJump(patchableJump)
-            , call(call)
-            , slowPath(slowPath)
-            , info(info)
-        {
-            ASSERT(call.isFlagSet(Call::Near) && call.isFlagSet(Call::Tail));
-        }
-        
-        PatchableJump patchableJump;
-        Call call;
-        Label slowPath;
-        CallLinkInfo* info;
-    };
-
-    
     Vector<InlineCacheWrapper<JITGetByIdGenerator>, 4> m_getByIds;
     Vector<InlineCacheWrapper<JITGetByIdWithThisGenerator>, 4> m_getByIdsWithThis;
     Vector<InlineCacheWrapper<JITGetByValGenerator>, 4> m_getByVals;
     Vector<InlineCacheWrapper<JITPutByIdGenerator>, 4> m_putByIds;
+    Vector<InlineCacheWrapper<JITPutByValGenerator>, 4> m_putByVals;
     Vector<InlineCacheWrapper<JITDelByIdGenerator>, 4> m_delByIds;
     Vector<InlineCacheWrapper<JITDelByValGenerator>, 4> m_delByVals;
     Vector<InlineCacheWrapper<JITInByIdGenerator>, 4> m_inByIds;
+    Vector<InlineCacheWrapper<JITInByValGenerator>, 4> m_inByVals;
     Vector<InlineCacheWrapper<JITInstanceOfGenerator>, 4> m_instanceOfs;
+    Vector<InlineCacheWrapper<JITPrivateBrandAccessGenerator>, 4> m_privateBrandAccesses;
     Vector<JSCallRecord, 4> m_jsCalls;
     Vector<JSDirectCallRecord, 4> m_jsDirectCalls;
-    Vector<JSDirectTailCallRecord, 4> m_jsDirectTailCalls;
     SegmentedVector<OSRExitCompilationInfo, 4> m_exitCompilationInfo;
     Vector<Vector<Label>> m_exitSiteLabels;
+    Vector<DFG::OSREntryData> m_osrEntry;
+    Vector<DFG::OSRExit> m_osrExit;
+    Vector<DFG::SpeculationRecovery> m_speculationRecovery;
     
     struct ExceptionHandlingOSRExitInfo {
         OSRExitCompilationInfo& exitInfo;
