@@ -1,4 +1,4 @@
-# Copyright (C) 2018 Apple Inc. All rights reserved.
+# Copyright (C) 2018-2021 Apple Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -138,14 +138,17 @@ module DSL
 #include "GetByValHistory.h"
 #include "Instruction.h"
 #include "IterationModeMetadata.h"
+#include "JSPropertyNameEnumerator.h"
 #include "Opcode.h"
 #include "PrivateFieldPutKind.h"
-#include "PutByIdStatus.h"
+#include "PutByStatus.h"
 #include "PutByIdFlags.h"
 #include "ToThisStatus.h"
 
 namespace JSC {
 
+class BasicBlockLocation;
+
 void dumpBytecode(BytecodeDumperBase* dumper, InstructionStream::Offset, const Instruction*);
 
 #if ENABLE(WEBASSEMBLY)
