@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,8 +26,6 @@
 #include "config.h"
 #include "SlotVisitor.h"
 
-#include "BlockDirectoryInlines.h"
-#include "CPU.h"
 #include "ConservativeRoots.h"
 #include "GCSegmentedArrayInlines.h"
 #include "HeapAnalyzer.h"
@@ -35,15 +33,11 @@
 #include "HeapProfiler.h"
 #include "IntegrityInlines.h"
 #include "JSArray.h"
-#include "JSDestructibleObject.h"
-#include "JSObject.h"
+#include "JSCellInlines.h"
 #include "JSString.h"
-#include "JSCInlines.h"
-#include "MarkedBlockInlines.h"
 #include "MarkingConstraintSolver.h"
 #include "SlotVisitorInlines.h"
 #include "StopIfNecessaryTimer.h"
-#include "SuperSampler.h"
 #include "VM.h"
 #include <wtf/ListDump.h>
 #include <wtf/Lock.h>
@@ -83,15 +77,10 @@ static void validate(JSCell* cell)
 #endif
 
 SlotVisitor::SlotVisitor(Heap& heap, CString codeName)
-    : m_bytesVisited(0)
-    , m_visitCount(0)
-    , m_isInParallelMode(false)
+    : Base(heap, codeName, heap.m_opaqueRoots)
     , m_markingVersion(MarkedSpace::initialVersion)
-    , m_heap(heap)
-    , m_codeName(codeName)
 #if ASSERT_ENABLED
     , m_isCheckingForDefaultMarkViolation(false)
-    , m_isDraining(false)
 #endif
 {
 }
@@ -123,8 +112,8 @@ void SlotVisitor::didStartMarking()
 
 void SlotVisitor::reset()
 {
+    AbstractSlotVisitor::reset();
     m_bytesVisited = 0;
-    m_visitCount = 0;
     m_heapAnalyzer = nullptr;
     RELEASE_ASSERT(!m_currentCell);
 }
@@ -163,10 +152,9 @@ void SlotVisitor::appendJSCellOrAuxiliary(HeapCell* heapCell)
                     out.print("GC type: ", heap()->collectionScope(), "\n");
                     out.print("Object at: ", RawPointer(jsCell), "\n");
 #if USE(JSVALUE64)
-                    out.print("Structure ID: ", structureID, " (0x", format("%x", structureID), ")\n");
-                    out.print("Structure ID table size: ", heap()->structureIDTable().size(), "\n");
+                    out.print("Structure ID: ", structureID.bits(), " (", RawPointer(structureID.decode()), ")\n");
 #else
-                    out.print("Structure: ", RawPointer(structureID), "\n");
+                    out.print("Structure: ", RawPointer(structureID.decode()), "\n");
 #endif
                     out.print("Object contents:");
                     for (unsigned i = 0; i < 2; ++i)
@@ -197,16 +185,13 @@ void SlotVisitor::appendJSCellOrAuxiliary(HeapCell* heapCell)
             die("GC scan found corrupt object: structureID is zero!\n");
         
         // It's not OK for the structure to be nuked at any GC scan point.
-        if (isNuked(structureID))
+        if (structureID.isNuked())
             die("GC scan found object in bad state: structureID is nuked!\n");
-        
-#if USE(JSVALUE64)
+
         // This detects the worst of the badness.
-        if (!heap()->structureIDTable().isValid(structureID))
-            die("GC scan found corrupt object: structureID is invalid!\n");
-#endif
+        Integrity::auditStructureID(structureID);
     };
-    
+
     // In debug mode, we validate before marking since this makes it clearer what the problem
     // was. It's also slower, so we don't do it normally.
     if (ASSERT_ENABLED && isJSCellKind(heapCell->cellKind()))
@@ -217,7 +202,7 @@ void SlotVisitor::appendJSCellOrAuxiliary(HeapCell* heapCell)
     
     switch (heapCell->cellKind()) {
     case HeapCell::JSCell:
-    case HeapCell::JSCellWithInteriorPointers: {
+    case HeapCell::JSCellWithIndexingHeader: {
         // We have ample budget to perform validation here.
     
         JSCell* jsCell = static_cast<JSCell*>(heapCell);
@@ -239,7 +224,7 @@ void SlotVisitor::appendJSCellOrAuxiliary(HeapCell* heapCell)
 void SlotVisitor::appendSlow(JSCell* cell, Dependency dependency)
 {
     if (UNLIKELY(m_heapAnalyzer))
-        m_heapAnalyzer->analyzeEdge(m_currentCell, cell, m_rootMarkReason);
+        m_heapAnalyzer->analyzeEdge(m_currentCell, cell, rootMarkReason());
 
     appendHiddenSlowImpl(cell, dependency);
 }
@@ -294,7 +279,7 @@ ALWAYS_INLINE void SlotVisitor::appendToMarkStack(ContainerType& container, JSCe
 #if CPU(X86_64)
     if (UNLIKELY(Options::dumpZappedCellCrashData())) {
         if (UNLIKELY(cell->isZapped()))
-            reportZappedCellAndCrash(cell);
+            reportZappedCellAndCrash(m_heap, cell);
     }
 #endif
     ASSERT(!cell->isZapped());
@@ -404,7 +389,7 @@ ALWAYS_INLINE void SlotVisitor::visitChildren(const JSCell* cell)
                 methodTable->visitChildren(const_cast<JSCell*>(cell), *this);
                 break;
             }
-            reportZappedCellAndCrash(const_cast<JSCell*>(cell));
+            reportZappedCellAndCrash(m_heap, const_cast<JSCell*>(cell));
         }
 #endif
         cell->methodTable(vm())->visitChildren(const_cast<JSCell*>(cell), *this);
@@ -429,7 +414,7 @@ inline void SlotVisitor::propagateExternalMemoryVisitedIfNecessary()
         if (m_extraMemorySize.hasOverflowed())
             heap()->reportExtraMemoryVisited(std::numeric_limits<size_t>::max());
         else if (m_extraMemorySize)
-            heap()->reportExtraMemoryVisited(m_extraMemorySize.unsafeGet());
+            heap()->reportExtraMemoryVisited(m_extraMemorySize);
         m_extraMemorySize = 0;
     }
 }
@@ -450,9 +435,9 @@ void SlotVisitor::donateKnownParallel(MarkStackArray& from, MarkStackArray& to)
 
     // If we're contending on the lock, be conservative and assume that another
     // thread is already donating.
-    std::unique_lock<Lock> lock(m_heap.m_markingMutex, std::try_to_lock);
-    if (!lock.owns_lock())
+    if (!m_heap.m_markingMutex.tryLock())
         return;
+    Locker locker { AdoptLock, m_heap.m_markingMutex };
 
     // Otherwise, assume that a thread will go idle soon, and donate.
     from.donateSomeCellsTo(to);
@@ -478,7 +463,7 @@ void SlotVisitor::updateMutatorIsStopped()
 {
     if (mutatorIsStoppedIsUpToDate())
         return;
-    updateMutatorIsStopped(holdLock(m_rightToRun));
+    updateMutatorIsStopped(Locker { m_rightToRun });
 }
 
 bool SlotVisitor::hasAcknowledgedThatTheMutatorIsResumed() const
@@ -503,7 +488,7 @@ NEVER_INLINE void SlotVisitor::drain(MonotonicTime timeout)
         RELEASE_ASSERT_NOT_REACHED();
     }
     
-    auto locker = holdLock(m_rightToRun);
+    Locker locker { m_rightToRun };
     
     while (!hasElapsed(timeout)) {
         updateMutatorIsStopped(locker);
@@ -535,7 +520,7 @@ size_t SlotVisitor::performIncrementOfDraining(size_t bytesRequested)
 
     size_t cellsRequested = bytesRequested / MarkedBlock::atomSize;
     {
-        auto locker = holdLock(m_heap.m_markingMutex);
+        Locker locker { m_heap.m_markingMutex };
         forEachMarkStack(
             [&] (MarkStackArray& stack) -> IterationStatus {
                 cellsRequested -= correspondingGlobalStack(stack).transferTo(stack, cellsRequested);
@@ -555,7 +540,7 @@ size_t SlotVisitor::performIncrementOfDraining(size_t bytesRequested)
     };
     
     {
-        auto locker = holdLock(m_rightToRun);
+        Locker locker { m_rightToRun };
         
         while (!isDone()) {
             updateMutatorIsStopped(locker);
@@ -592,7 +577,7 @@ size_t SlotVisitor::performIncrementOfDraining(size_t bytesRequested)
 
 bool SlotVisitor::didReachTermination()
 {
-    LockHolder locker(m_heap.m_markingMutex);
+    Locker locker { m_heap.m_markingMutex };
     return didReachTermination(locker);
 }
 
@@ -620,12 +605,12 @@ NEVER_INLINE SlotVisitor::SharedDrainResult SlotVisitor::drainFromShared(SharedD
         RefPtr<SharedTask<void(SlotVisitor&)>> bonusTask;
         
         {
-            auto locker = holdLock(m_heap.m_markingMutex);
+            Locker locker { m_heap.m_markingMutex };
             if (isActive)
                 m_heap.m_numberOfActiveParallelMarkers--;
             m_heap.m_numberOfWaitingParallelMarkers++;
             
-            if (sharedDrainMode == MasterDrain) {
+            if (sharedDrainMode == MainDrain) {
                 while (true) {
                     if (hasElapsed(timeout))
                         return SharedDrainResult::TimedOut;
@@ -641,7 +626,7 @@ NEVER_INLINE SlotVisitor::SharedDrainResult SlotVisitor::drainFromShared(SharedD
                     m_heap.m_markingConditionVariable.waitUntil(m_heap.m_markingMutex, timeout);
                 }
             } else {
-                ASSERT(sharedDrainMode == SlaveDrain);
+                ASSERT(sharedDrainMode == HelperDrain);
 
                 if (hasElapsed(timeout))
                     return SharedDrainResult::TimedOut;
@@ -699,7 +684,7 @@ NEVER_INLINE SlotVisitor::SharedDrainResult SlotVisitor::drainFromShared(SharedD
             // The main thread could still be running, and may run for a while. Unless we clear the task
             // ourselves, we will keep looping around trying to run the task.
             {
-                auto locker = holdLock(m_heap.m_markingMutex);
+                Locker locker { m_heap.m_markingMutex };
                 if (m_heap.m_bonusVisitorTask == bonusTask)
                     m_heap.m_bonusVisitorTask = nullptr;
                 bonusTask = nullptr;
@@ -717,7 +702,7 @@ NEVER_INLINE SlotVisitor::SharedDrainResult SlotVisitor::drainFromShared(SharedD
 SlotVisitor::SharedDrainResult SlotVisitor::drainInParallel(MonotonicTime timeout)
 {
     donateAndDrain(timeout);
-    return drainFromShared(MasterDrain, timeout);
+    return drainFromShared(MainDrain, timeout);
 }
 
 SlotVisitor::SharedDrainResult SlotVisitor::drainInParallelPassively(MonotonicTime timeout)
@@ -735,13 +720,13 @@ SlotVisitor::SharedDrainResult SlotVisitor::drainInParallelPassively(MonotonicTi
         return drainInParallel(timeout);
     }
 
-    donateAll(holdLock(m_heap.m_markingMutex));
+    donateAll(Locker { m_heap.m_markingMutex });
     return waitForTermination(timeout);
 }
 
 SlotVisitor::SharedDrainResult SlotVisitor::waitForTermination(MonotonicTime timeout)
 {
-    auto locker = holdLock(m_heap.m_markingMutex);
+    Locker locker { m_heap.m_markingMutex };
     for (;;) {
         if (hasElapsed(timeout))
             return SharedDrainResult::TimedOut;
@@ -760,7 +745,7 @@ void SlotVisitor::donateAll()
     if (isEmpty())
         return;
     
-    donateAll(holdLock(m_heap.m_markingMutex));
+    donateAll(Locker { m_heap.m_markingMutex });
 }
 
 void SlotVisitor::donateAll(const AbstractLocker&)
@@ -797,7 +782,7 @@ void SlotVisitor::didRace(const VisitRaceKey& race)
 {
     dataLogLnIf(Options::verboseVisitRace(), toCString("GC visit race: ", race));
     
-    auto locker = holdLock(heap()->m_raceMarkStackLock);
+    Locker locker { heap()->m_raceMarkStackLock };
     JSCell* cell = race.cell();
     cell->setCellState(CellState::PossiblyGrey);
     heap()->m_raceMarkStack->append(cell);
@@ -816,55 +801,18 @@ MarkStackArray& SlotVisitor::correspondingGlobalStack(MarkStackArray& stack)
     return *m_heap.m_sharedMutatorMarkStack;
 }
 
+NO_RETURN_DUE_TO_CRASH void SlotVisitor::addParallelConstraintTask(RefPtr<SharedTask<void(AbstractSlotVisitor&)>>)
+{
+    RELEASE_ASSERT_NOT_REACHED();
+}
+
 void SlotVisitor::addParallelConstraintTask(RefPtr<SharedTask<void(SlotVisitor&)>> task)
 {
     RELEASE_ASSERT(m_currentSolver);
     RELEASE_ASSERT(m_currentConstraint);
     RELEASE_ASSERT(task);
-    
+
     m_currentSolver->addParallelTask(task, *m_currentConstraint);
 }
 
-#if CPU(X86_64)
-NEVER_INLINE NO_RETURN_DUE_TO_CRASH NOT_TAIL_CALLED void SlotVisitor::reportZappedCellAndCrash(JSCell* cell)
-{
-    MarkedBlock::Handle* foundBlockHandle = nullptr;
-    uint64_t* cellWords = reinterpret_cast_ptr<uint64_t*>(cell);
-
-    uintptr_t cellAddress = bitwise_cast<uintptr_t>(cell);
-    uint64_t headerWord = cellWords[0];
-    uint64_t zapReasonAndMore = cellWords[1];
-    unsigned subspaceHash = 0;
-    size_t cellSize = 0;
-
-    m_heap.objectSpace().forEachBlock([&] (MarkedBlock::Handle* blockHandle) {
-        if (blockHandle->contains(cell)) {
-            foundBlockHandle = blockHandle;
-            return IterationStatus::Done;
-        }
-        return IterationStatus::Continue;
-    });
-
-    uint64_t variousState = 0;
-    MarkedBlock* foundBlock = nullptr;
-    if (foundBlockHandle) {
-        foundBlock = &foundBlockHandle->block();
-        subspaceHash = StringHasher::computeHash(foundBlockHandle->subspace()->name());
-        cellSize = foundBlockHandle->cellSize();
-
-        variousState |= static_cast<uint64_t>(foundBlockHandle->isFreeListed()) << 0;
-        variousState |= static_cast<uint64_t>(foundBlockHandle->isAllocated()) << 1;
-        variousState |= static_cast<uint64_t>(foundBlockHandle->isEmpty()) << 2;
-        variousState |= static_cast<uint64_t>(foundBlockHandle->needsDestruction()) << 3;
-        variousState |= static_cast<uint64_t>(foundBlock->isNewlyAllocated(cell)) << 4;
-
-        ptrdiff_t cellOffset = cellAddress - reinterpret_cast<uint64_t>(foundBlockHandle->start());
-        bool cellIsProperlyAligned = !(cellOffset % cellSize);
-        variousState |= static_cast<uint64_t>(cellIsProperlyAligned) << 5;
-    }
-
-    CRASH_WITH_INFO(cellAddress, headerWord, zapReasonAndMore, subspaceHash, cellSize, foundBlock, variousState);
-}
-#endif // PLATFORM(MAC)
-
 } // namespace JSC
