@@ -81,24 +81,18 @@ Stream &operator<<(Stream &out, CommaSeparatedListItemPrefixGenerator &gen)
 
 }  // namespace
 
-TOutputGLSLBase::TOutputGLSLBase(TInfoSinkBase &objSink,
-                                 ShArrayIndexClampingStrategy clampingStrategy,
-                                 ShHashFunction64 hashFunction,
-                                 NameMap &nameMap,
-                                 TSymbolTable *symbolTable,
-                                 sh::GLenum shaderType,
-                                 int shaderVersion,
-                                 ShShaderOutput output,
+TOutputGLSLBase::TOutputGLSLBase(TCompiler *compiler,
+                                 TInfoSinkBase &objSink,
                                  ShCompileOptions compileOptions)
-    : TIntermTraverser(true, true, true, symbolTable),
+    : TIntermTraverser(true, true, true, &compiler->getSymbolTable()),
       mObjSink(objSink),
       mDeclaringVariable(false),
-      mClampingStrategy(clampingStrategy),
-      mHashFunction(hashFunction),
-      mNameMap(nameMap),
-      mShaderType(shaderType),
-      mShaderVersion(shaderVersion),
-      mOutput(output),
+      mHashFunction(compiler->getHashFunction()),
+      mNameMap(compiler->getNameMap()),
+      mShaderType(compiler->getShaderType()),
+      mShaderVersion(compiler->getShaderVersion()),
+      mOutput(compiler->getOutputType()),
+      mHighPrecisionSupported(compiler->isHighPrecisionSupported()),
       mCompileOptions(compileOptions)
 {}
 
@@ -143,21 +137,20 @@ void TOutputGLSLBase::writeTriplet(Visit visit,
         out << postStr;
 }
 
-void TOutputGLSLBase::writeBuiltInFunctionTriplet(Visit visit,
-                                                  TOperator op,
-                                                  bool useEmulatedFunction)
+void TOutputGLSLBase::writeFunctionTriplet(Visit visit,
+                                           const ImmutableString &functionName,
+                                           bool useEmulatedFunction)
 {
     TInfoSinkBase &out = objSink();
     if (visit == PreVisit)
     {
-        const char *opStr(GetOperatorString(op));
         if (useEmulatedFunction)
         {
-            BuiltInFunctionEmulator::WriteEmulatedFunctionName(out, opStr);
+            BuiltInFunctionEmulator::WriteEmulatedFunctionName(out, functionName.data());
         }
         else
         {
-            out << opStr;
+            out << functionName;
         }
         out << "(";
     }
@@ -169,7 +162,7 @@ void TOutputGLSLBase::writeBuiltInFunctionTriplet(Visit visit,
 
 // Outputs what goes inside layout(), except for location and binding qualifiers, as they are
 // handled differently between GL GLSL and Vulkan GLSL.
-std::string TOutputGLSLBase::getCommonLayoutQualifiers(TIntermTyped *variable)
+std::string TOutputGLSLBase::getCommonLayoutQualifiers(TIntermSymbol *variable)
 {
     std::ostringstream out;
     CommaSeparatedListItemPrefixGenerator listItemPrefix;
@@ -247,7 +240,7 @@ std::string TOutputGLSLBase::getMemoryQualifiers(const TType &type)
     return out.str();
 }
 
-void TOutputGLSLBase::writeLayoutQualifier(TIntermTyped *variable)
+void TOutputGLSLBase::writeLayoutQualifier(TIntermSymbol *variable)
 {
     const TType &type = variable->getType();
 
@@ -258,8 +251,7 @@ void TOutputGLSLBase::writeLayoutQualifier(TIntermTyped *variable)
 
     if (type.getBasicType() == EbtInterfaceBlock)
     {
-        const TInterfaceBlock *interfaceBlock = type.getInterfaceBlock();
-        declareInterfaceBlockLayout(interfaceBlock);
+        declareInterfaceBlockLayout(type);
         return;
     }
 
@@ -368,9 +360,36 @@ const char *TOutputGLSLBase::mapQualifierToString(TQualifier qualifier)
                 break;
         }
     }
+
+    // Handle qualifiers that produce different output based on shader type.
+    switch (qualifier)
+    {
+        case EvqClipDistance:
+        case EvqCullDistance:
+            return mShaderType == GL_FRAGMENT_SHADER ? "in" : "out";
+        default:
+            break;
+    }
+
     return sh::getQualifierString(qualifier);
 }
 
+namespace
+{
+
+constexpr char kIndent[]      = "                    ";  // 10x2 spaces
+constexpr int kIndentWidth    = 2;
+constexpr int kMaxIndentLevel = sizeof(kIndent) / kIndentWidth;
+
+}  // namespace
+
+const char *TOutputGLSLBase::getIndentPrefix(int extraIndentation)
+{
+    int indentDepth = std::min(kMaxIndentLevel, getCurrentBlockDepth() + extraIndentation);
+    ASSERT(indentDepth >= 0);
+    return kIndent + (kMaxIndentLevel - indentDepth) * kIndentWidth;
+}
+
 void TOutputGLSLBase::writeVariableType(const TType &type,
                                         const TSymbol *symbol,
                                         bool isFunctionArgument)
@@ -396,8 +415,8 @@ void TOutputGLSLBase::writeVariableType(const TType &type,
         out << getMemoryQualifiers(type);
     }
 
-    // Declare the struct if we have not done so already.
-    if (type.getBasicType() == EbtStruct && !structDeclared(type.getStruct()))
+    // Declare the struct.
+    if (type.isStructSpecifier())
     {
         const TStructure *structure = type.getStruct();
 
@@ -405,8 +424,7 @@ void TOutputGLSLBase::writeVariableType(const TType &type,
     }
     else if (type.getBasicType() == EbtInterfaceBlock)
     {
-        const TInterfaceBlock *interfaceBlock = type.getInterfaceBlock();
-        declareInterfaceBlock(interfaceBlock);
+        declareInterfaceBlock(type);
     }
     else
     {
@@ -427,9 +445,13 @@ void TOutputGLSLBase::writeFunctionParameters(const TFunction *func)
         writeVariableType(type, param, true);
 
         if (param->symbolType() != SymbolType::Empty)
+        {
             out << " " << hashName(param);
+        }
         if (type.isArray())
+        {
             out << ArrayString(type);
+        }
 
         // Put a comma if this is not the last argument.
         if (i != paramCount - 1)
@@ -599,62 +621,8 @@ bool TOutputGLSLBase::visitBinary(Visit visit, TIntermBinary *node)
             break;
 
         case EOpIndexDirect:
-            writeTriplet(visit, nullptr, "[", "]");
-            break;
         case EOpIndexIndirect:
-            if (node->getAddIndexClamp())
-            {
-                if (visit == InVisit)
-                {
-                    if (mClampingStrategy == SH_CLAMP_WITH_CLAMP_INTRINSIC)
-                        out << "[int(clamp(float(";
-                    else
-                        out << "[webgl_int_clamp(";
-                }
-                else if (visit == PostVisit)
-                {
-                    TIntermTyped *left = node->getLeft();
-                    TType leftType     = left->getType();
-
-                    if (mClampingStrategy == SH_CLAMP_WITH_CLAMP_INTRINSIC)
-                        out << "), 0.0, float(";
-                    else
-                        out << ", 0, ";
-
-                    if (leftType.isUnsizedArray())
-                    {
-                        // For runtime-sized arrays in ESSL 3.10 we need to call the length method
-                        // to get the length to clamp against. See ESSL 3.10 section 4.1.9. Note
-                        // that a runtime-sized array expression is guaranteed not to have side
-                        // effects, so it's fine to add the expression to the output twice.
-                        ASSERT(mShaderVersion >= 310);
-                        ASSERT(!left->hasSideEffects());
-                        left->traverse(this);
-                        out << ".length() - 1";
-                    }
-                    else
-                    {
-                        int maxSize;
-                        if (leftType.isArray())
-                        {
-                            maxSize = static_cast<int>(leftType.getOutermostArraySize()) - 1;
-                        }
-                        else
-                        {
-                            maxSize = leftType.getNominalSize() - 1;
-                        }
-                        out << maxSize;
-                    }
-                    if (mClampingStrategy == SH_CLAMP_WITH_CLAMP_INTRINSIC)
-                        out << ")))]";
-                    else
-                        out << ")]";
-                }
-            }
-            else
-            {
-                writeTriplet(visit, nullptr, "[", "]");
-            }
+            writeTriplet(visit, nullptr, "[", "]");
             break;
         case EOpIndexDirectStruct:
             if (visit == InVisit)
@@ -800,69 +768,10 @@ bool TOutputGLSLBase::visitUnary(Visit visit, TIntermUnary *node)
             postString = ").length())";
             break;
 
-        case EOpRadians:
-        case EOpDegrees:
-        case EOpSin:
-        case EOpCos:
-        case EOpTan:
-        case EOpAsin:
-        case EOpAcos:
-        case EOpAtan:
-        case EOpSinh:
-        case EOpCosh:
-        case EOpTanh:
-        case EOpAsinh:
-        case EOpAcosh:
-        case EOpAtanh:
-        case EOpExp:
-        case EOpLog:
-        case EOpExp2:
-        case EOpLog2:
-        case EOpSqrt:
-        case EOpInversesqrt:
-        case EOpAbs:
-        case EOpSign:
-        case EOpFloor:
-        case EOpTrunc:
-        case EOpRound:
-        case EOpRoundEven:
-        case EOpCeil:
-        case EOpFract:
-        case EOpIsnan:
-        case EOpIsinf:
-        case EOpFloatBitsToInt:
-        case EOpFloatBitsToUint:
-        case EOpIntBitsToFloat:
-        case EOpUintBitsToFloat:
-        case EOpPackSnorm2x16:
-        case EOpPackUnorm2x16:
-        case EOpPackHalf2x16:
-        case EOpUnpackSnorm2x16:
-        case EOpUnpackUnorm2x16:
-        case EOpUnpackHalf2x16:
-        case EOpPackUnorm4x8:
-        case EOpPackSnorm4x8:
-        case EOpUnpackUnorm4x8:
-        case EOpUnpackSnorm4x8:
-        case EOpLength:
-        case EOpNormalize:
-        case EOpDFdx:
-        case EOpDFdy:
-        case EOpFwidth:
-        case EOpTranspose:
-        case EOpDeterminant:
-        case EOpInverse:
-        case EOpAny:
-        case EOpAll:
-        case EOpLogicalNotComponentWise:
-        case EOpBitfieldReverse:
-        case EOpBitCount:
-        case EOpFindLSB:
-        case EOpFindMSB:
-            writeBuiltInFunctionTriplet(visit, node->getOp(), node->getUseEmulatedFunction());
-            return true;
         default:
-            UNREACHABLE();
+            writeFunctionTriplet(visit, node->getFunction()->name(),
+                                 node->getUseEmulatedFunction());
+            return true;
     }
 
     writeTriplet(visit, preString, nullptr, postString);
@@ -899,7 +808,7 @@ bool TOutputGLSLBase::visitIfElse(Visit visit, TIntermIfElse *node)
 
     if (node->getFalseBlock())
     {
-        out << "else\n";
+        out << getIndentPrefix() << "else\n";
         visitCodeBlock(node->getFalseBlock());
     }
     return false;
@@ -942,6 +851,9 @@ bool TOutputGLSLBase::visitBlock(Visit visit, TIntermBlock *node)
     {
         TIntermNode *curNode = *iter;
         ASSERT(curNode != nullptr);
+
+        out << getIndentPrefix(curNode->getAsCaseNode() ? -1 : 0);
+
         curNode->traverse(this);
 
         if (isSingleStatement(curNode))
@@ -951,7 +863,7 @@ bool TOutputGLSLBase::visitBlock(Visit visit, TIntermBlock *node)
     // Scope the blocks except when at the global scope.
     if (getCurrentTraversalDepth() > 0)
     {
-        out << "}\n";
+        out << getIndentPrefix(-1) << "}\n";
     }
     return false;
 }
@@ -995,88 +907,30 @@ void TOutputGLSLBase::visitFunctionPrototype(TIntermFunctionPrototype *node)
 bool TOutputGLSLBase::visitAggregate(Visit visit, TIntermAggregate *node)
 {
     bool visitChildren = true;
-    TInfoSinkBase &out = objSink();
-    switch (node->getOp())
+    if (node->getOp() == EOpConstruct)
     {
-        case EOpCallFunctionInAST:
-        case EOpCallInternalRawFunction:
-        case EOpCallBuiltInFunction:
-            // Function call.
-            if (visit == PreVisit)
-            {
-                if (node->getOp() == EOpCallBuiltInFunction)
-                {
-                    out << translateTextureFunction(node->getFunction()->name(), mCompileOptions);
-                }
-                else
-                {
-                    out << hashFunctionNameIfNeeded(node->getFunction());
-                }
-                out << "(";
-            }
-            else if (visit == InVisit)
-                out << ", ";
-            else
-                out << ")";
-            break;
-        case EOpConstruct:
-            writeConstructorTriplet(visit, node->getType());
-            break;
+        writeConstructorTriplet(visit, node->getType());
+    }
+    else
+    {
+        // Function call.
+        ImmutableString functionName = node->getFunction()->name();
+        if (visit == PreVisit)
+        {
+            // No raw function is expected.
+            ASSERT(node->getOp() != EOpCallInternalRawFunction);
 
-        case EOpEqualComponentWise:
-        case EOpNotEqualComponentWise:
-        case EOpLessThanComponentWise:
-        case EOpGreaterThanComponentWise:
-        case EOpLessThanEqualComponentWise:
-        case EOpGreaterThanEqualComponentWise:
-        case EOpMod:
-        case EOpModf:
-        case EOpPow:
-        case EOpAtan:
-        case EOpMin:
-        case EOpMax:
-        case EOpClamp:
-        case EOpMix:
-        case EOpStep:
-        case EOpSmoothstep:
-        case EOpFma:
-        case EOpFrexp:
-        case EOpLdexp:
-        case EOpDistance:
-        case EOpDot:
-        case EOpCross:
-        case EOpFaceforward:
-        case EOpReflect:
-        case EOpRefract:
-        case EOpMulMatrixComponentWise:
-        case EOpOuterProduct:
-        case EOpBitfieldExtract:
-        case EOpBitfieldInsert:
-        case EOpUaddCarry:
-        case EOpUsubBorrow:
-        case EOpUmulExtended:
-        case EOpImulExtended:
-        case EOpBarrier:
-        case EOpMemoryBarrier:
-        case EOpMemoryBarrierAtomicCounter:
-        case EOpMemoryBarrierBuffer:
-        case EOpMemoryBarrierImage:
-        case EOpMemoryBarrierShared:
-        case EOpGroupMemoryBarrier:
-        case EOpAtomicAdd:
-        case EOpAtomicMin:
-        case EOpAtomicMax:
-        case EOpAtomicAnd:
-        case EOpAtomicOr:
-        case EOpAtomicXor:
-        case EOpAtomicExchange:
-        case EOpAtomicCompSwap:
-        case EOpEmitVertex:
-        case EOpEndPrimitive:
-            writeBuiltInFunctionTriplet(visit, node->getOp(), node->getUseEmulatedFunction());
-            break;
-        default:
-            UNREACHABLE();
+            if (node->getOp() == EOpCallFunctionInAST)
+            {
+                functionName = hashFunctionNameIfNeeded(node->getFunction());
+            }
+            else
+            {
+                functionName =
+                    translateTextureFunction(node->getFunction()->name(), mCompileOptions);
+            }
+        }
+        writeFunctionTriplet(visit, functionName, node->getUseEmulatedFunction());
     }
     return visitChildren;
 }
@@ -1089,13 +943,24 @@ bool TOutputGLSLBase::visitDeclaration(Visit visit, TIntermDeclaration *node)
     if (visit == PreVisit)
     {
         const TIntermSequence &sequence = *(node->getSequence());
-        TIntermTyped *variable          = sequence.front()->getAsTyped();
-        writeLayoutQualifier(variable);
-        TIntermSymbol *symbolNode = variable->getAsSymbolNode();
-        writeVariableType(variable->getType(), symbolNode ? &symbolNode->variable() : nullptr,
-                          false);
-        if (variable->getAsSymbolNode() == nullptr ||
-            variable->getAsSymbolNode()->variable().symbolType() != SymbolType::Empty)
+        TIntermTyped *decl              = sequence.front()->getAsTyped();
+        TIntermSymbol *symbolNode       = decl->getAsSymbolNode();
+        if (symbolNode == nullptr)
+        {
+            ASSERT(decl->getAsBinaryNode() && decl->getAsBinaryNode()->getOp() == EOpInitialize);
+            symbolNode = decl->getAsBinaryNode()->getLeft()->getAsSymbolNode();
+        }
+        ASSERT(symbolNode);
+
+        if (symbolNode->getName() != "gl_ClipDistance" &&
+            symbolNode->getName() != "gl_CullDistance")
+        {
+            // gl_Clip/CullDistance re-declaration doesn't need layout.
+            writeLayoutQualifier(symbolNode);
+        }
+
+        writeVariableType(symbolNode->getType(), &symbolNode->variable(), false);
+        if (symbolNode->variable().symbolType() != SymbolType::Empty)
         {
             out << " ";
         }
@@ -1190,6 +1055,7 @@ void TOutputGLSLBase::visitCodeBlock(TIntermBlock *node)
     TInfoSinkBase &out = objSink();
     if (node != nullptr)
     {
+        out << getIndentPrefix();
         node->traverse(this);
         // Single statements not part of a sequence need to be terminated
         // with semi-colon.
@@ -1276,17 +1142,6 @@ ImmutableString TOutputGLSLBase::hashFunctionNameIfNeeded(const TFunction *func)
     }
 }
 
-bool TOutputGLSLBase::structDeclared(const TStructure *structure) const
-{
-    ASSERT(structure);
-    if (structure->symbolType() == SymbolType::Empty)
-    {
-        return false;
-    }
-
-    return (mDeclaredStructs.count(structure->uniqueId().get()) > 0);
-}
-
 void TOutputGLSLBase::declareStruct(const TStructure *structure)
 {
     TInfoSinkBase &out = objSink();
@@ -1301,25 +1156,39 @@ void TOutputGLSLBase::declareStruct(const TStructure *structure)
     const TFieldList &fields = structure->fields();
     for (size_t i = 0; i < fields.size(); ++i)
     {
-        const TField *field = fields[i];
-        if (writeVariablePrecision(field->type()->getPrecision()))
+        out << getIndentPrefix(1);
+        const TField *field    = fields[i];
+        const TType &fieldType = *field->type();
+        if (writeVariablePrecision(fieldType.getPrecision()))
+        {
             out << " ";
-        out << getTypeName(*field->type()) << " " << hashFieldName(field);
-        if (field->type()->isArray())
-            out << ArrayString(*field->type());
+        }
+        if (fieldType.isPrecise())
+        {
+            writePreciseQualifier(fieldType);
+        }
+        out << getTypeName(fieldType) << " " << hashFieldName(field);
+        if (fieldType.isArray())
+        {
+            out << ArrayString(fieldType);
+        }
         out << ";\n";
     }
-    out << "}";
-
-    if (structure->symbolType() != SymbolType::Empty)
-    {
-        mDeclaredStructs.insert(structure->uniqueId().get());
-    }
+    out << getIndentPrefix() << "}";
 }
 
-void TOutputGLSLBase::declareInterfaceBlockLayout(const TInterfaceBlock *interfaceBlock)
+void TOutputGLSLBase::declareInterfaceBlockLayout(const TType &type)
 {
-    TInfoSinkBase &out = objSink();
+    // 4.4.5 Uniform and Shader Storage Block Layout Qualifiers in GLSL 4.5 spec.
+    // Layout qualifiers can be used for uniform and shader storage blocks,
+    // but not for non-block uniform declarations.
+    if (IsShaderIoBlock(type.getQualifier()))
+    {
+        return;
+    }
+
+    const TInterfaceBlock *interfaceBlock = type.getInterfaceBlock();
+    TInfoSinkBase &out                    = objSink();
 
     out << "layout(";
 
@@ -1357,27 +1226,84 @@ void TOutputGLSLBase::declareInterfaceBlockLayout(const TInterfaceBlock *interfa
     out << ") ";
 }
 
-void TOutputGLSLBase::declareInterfaceBlock(const TInterfaceBlock *interfaceBlock)
+const char *getVariableInterpolation(TQualifier qualifier)
 {
-    TInfoSinkBase &out = objSink();
+    switch (qualifier)
+    {
+        case EvqSmoothOut:
+            return "smooth out ";
+        case EvqFlatOut:
+            return "flat out ";
+        case EvqNoPerspectiveOut:
+            return "noperspective out ";
+        case EvqCentroidOut:
+            return "centroid out ";
+        case EvqSmoothIn:
+            return "smooth in ";
+        case EvqFlatIn:
+            return "flat in ";
+        case EvqNoPerspectiveIn:
+            return "noperspective in ";
+        case EvqCentroidIn:
+            return "centroid in ";
+        default:
+            break;
+    }
+    return nullptr;
+}
+
+void TOutputGLSLBase::declareInterfaceBlock(const TType &type)
+{
+    const TInterfaceBlock *interfaceBlock = type.getInterfaceBlock();
+    TInfoSinkBase &out                    = objSink();
 
     out << hashName(interfaceBlock) << "{\n";
     const TFieldList &fields = interfaceBlock->fields();
     for (const TField *field : fields)
     {
-        writeFieldLayoutQualifier(field);
-        out << getMemoryQualifiers(*field->type());
+        out << getIndentPrefix(1);
+        if (!IsShaderIoBlock(type.getQualifier()) && type.getQualifier() != EvqPatchIn &&
+            type.getQualifier() != EvqPatchOut)
+        {
+            writeFieldLayoutQualifier(field);
+        }
 
-        if (writeVariablePrecision(field->type()->getPrecision()))
+        const TType &fieldType = *field->type();
+
+        out << getMemoryQualifiers(fieldType);
+        if (writeVariablePrecision(fieldType.getPrecision()))
             out << " ";
-        out << getTypeName(*field->type()) << " " << hashFieldName(field);
-        if (field->type()->isArray())
-            out << ArrayString(*field->type());
+        if (fieldType.isInvariant())
+        {
+            writeInvariantQualifier(fieldType);
+        }
+        if (fieldType.isPrecise())
+        {
+            writePreciseQualifier(fieldType);
+        }
+
+        const char *qualifier = getVariableInterpolation(fieldType.getQualifier());
+        if (qualifier != nullptr)
+            out << qualifier;
+
+        out << getTypeName(fieldType) << " " << hashFieldName(field);
+
+        if (fieldType.isArray())
+            out << ArrayString(fieldType);
         out << ";\n";
     }
     out << "}";
 }
 
+void WritePragma(TInfoSinkBase &out, ShCompileOptions compileOptions, const TPragma &pragma)
+{
+    if ((compileOptions & SH_FLATTEN_PRAGMA_STDGL_INVARIANT_ALL) == 0)
+    {
+        if (pragma.stdgl.invariantAll)
+            out << "#pragma STDGL invariant(all)\n";
+    }
+}
+
 void WriteGeometryShaderLayoutQualifiers(TInfoSinkBase &out,
                                          sh::TLayoutPrimitiveType inputPrimitive,
                                          int invocations,
@@ -1426,6 +1352,40 @@ void WriteGeometryShaderLayoutQualifiers(TInfoSinkBase &out,
     }
 }
 
+void WriteTessControlShaderLayoutQualifiers(TInfoSinkBase &out, int inputVertices)
+{
+    if (inputVertices != 0)
+    {
+        out << "layout (vertices = " << inputVertices << ") out;\n";
+    }
+}
+
+void WriteTessEvaluationShaderLayoutQualifiers(TInfoSinkBase &out,
+                                               sh::TLayoutTessEvaluationType inputPrimitive,
+                                               sh::TLayoutTessEvaluationType inputVertexSpacing,
+                                               sh::TLayoutTessEvaluationType inputOrdering,
+                                               sh::TLayoutTessEvaluationType inputPoint)
+{
+    if (inputPrimitive != EtetUndefined)
+    {
+        out << "layout (";
+        out << getTessEvaluationShaderTypeString(inputPrimitive);
+        if (inputVertexSpacing != EtetUndefined)
+        {
+            out << ", " << getTessEvaluationShaderTypeString(inputVertexSpacing);
+        }
+        if (inputOrdering != EtetUndefined)
+        {
+            out << ", " << getTessEvaluationShaderTypeString(inputOrdering);
+        }
+        if (inputPoint != EtetUndefined)
+        {
+            out << ", " << getTessEvaluationShaderTypeString(inputPoint);
+        }
+        out << ") in;\n";
+    }
+}
+
 // If SH_SCALARIZE_VEC_AND_MAT_CONSTRUCTOR_ARGS is enabled, layout qualifiers are spilled whenever
 // variables with specified layout qualifiers are copied. Additional checks are needed against the
 // type and storage qualifier of the variable to verify that layout qualifiers have to be outputted.
@@ -1464,4 +1424,64 @@ bool NeedsToWriteLayoutQualifier(const TType &type)
     return false;
 }
 
+void EmitEarlyFragmentTestsGLSL(const TCompiler &compiler, TInfoSinkBase &sink)
+{
+    if (compiler.isEarlyFragmentTestsSpecified() || compiler.isEarlyFragmentTestsOptimized())
+    {
+        sink << "layout (early_fragment_tests) in;\n";
+    }
+}
+
+void EmitWorkGroupSizeGLSL(const TCompiler &compiler, TInfoSinkBase &sink)
+{
+    if (compiler.isComputeShaderLocalSizeDeclared())
+    {
+        const sh::WorkGroupSize &localSize = compiler.getComputeShaderLocalSize();
+        sink << "layout (local_size_x=" << localSize[0] << ", local_size_y=" << localSize[1]
+             << ", local_size_z=" << localSize[2] << ") in;\n";
+    }
+}
+
+void EmitMultiviewGLSL(const TCompiler &compiler,
+                       const ShCompileOptions &compileOptions,
+                       const TExtension extension,
+                       const TBehavior behavior,
+                       TInfoSinkBase &sink)
+{
+    ASSERT(behavior != EBhUndefined);
+    if (behavior == EBhDisable)
+        return;
+
+    const bool isVertexShader = (compiler.getShaderType() == GL_VERTEX_SHADER);
+    if ((compileOptions & SH_INITIALIZE_BUILTINS_FOR_INSTANCED_MULTIVIEW) != 0)
+    {
+        // Emit ARB_shader_viewport_layer_array/NV_viewport_array2 in a vertex shader if the
+        // SH_SELECT_VIEW_IN_NV_GLSL_VERTEX_SHADER option is set and the
+        // OVR_multiview(2) extension is requested.
+        if (isVertexShader && (compileOptions & SH_SELECT_VIEW_IN_NV_GLSL_VERTEX_SHADER) != 0)
+        {
+            sink << "#if defined(GL_ARB_shader_viewport_layer_array)\n"
+                 << "#extension GL_ARB_shader_viewport_layer_array : require\n"
+                 << "#elif defined(GL_NV_viewport_array2)\n"
+                 << "#extension GL_NV_viewport_array2 : require\n"
+                 << "#endif\n";
+        }
+    }
+    else
+    {
+        sink << "#extension GL_OVR_multiview";
+        if (extension == TExtension::OVR_multiview2)
+        {
+            sink << "2";
+        }
+        sink << " : " << GetBehaviorString(behavior) << "\n";
+
+        const auto &numViews = compiler.getNumViews();
+        if (isVertexShader && numViews != -1)
+        {
+            sink << "layout(num_views=" << numViews << ") in;\n";
+        }
+    }
+}
+
 }  // namespace sh
