@@ -8,6 +8,8 @@
 //
 
 #include "compiler/translator/UtilsHLSL.h"
+
+#include "common/utilities.h"
 #include "compiler/translator/IntermNode.h"
 #include "compiler/translator/StructureHLSL.h"
 #include "compiler/translator/SymbolTable.h"
@@ -839,7 +841,7 @@ TString DecoratePrivate(const ImmutableString &privateText)
 
 TString Decorate(const ImmutableString &string)
 {
-    if (!string.beginsWith("gl_"))
+    if (!gl::IsBuiltInName(string.data()))
     {
         return "_" + TString(string.data());
     }
@@ -850,7 +852,7 @@ TString Decorate(const ImmutableString &string)
 TString DecorateVariableIfNeeded(const TVariable &variable)
 {
     if (variable.symbolType() == SymbolType::AngleInternal ||
-        variable.symbolType() == SymbolType::Empty)
+        variable.symbolType() == SymbolType::BuiltIn || variable.symbolType() == SymbolType::Empty)
     {
         // Besides handling internal variables, we generate names for nameless parameters here.
         const ImmutableString &name = variable.name();
@@ -881,7 +883,7 @@ TString DecorateFunctionIfNeeded(const TFunction *func)
         ASSERT(!func->name().beginsWith("_"));
         return TString(func->name().data());
     }
-    ASSERT(!func->name().beginsWith("gl_"));
+    ASSERT(!gl::IsBuiltInName(func->name().data()));
     // Add an additional f prefix to functions so that they're always disambiguated from variables.
     // This is necessary in the corner case where a variable declaration hides a function that it
     // uses in its initializer.
@@ -1009,7 +1011,7 @@ TString StructNameString(const TStructure &structure)
 TString QualifiedStructNameString(const TStructure &structure,
                                   bool useHLSLRowMajorPacking,
                                   bool useStd140Packing,
-                                  bool forcePackingEnd)
+                                  bool forcePadding)
 {
     if (structure.symbolType() == SymbolType::Empty)
     {
@@ -1031,9 +1033,9 @@ TString QualifiedStructNameString(const TStructure &structure,
         prefix += "rm_";
     }
 
-    if (forcePackingEnd)
+    if (forcePadding)
     {
-        prefix += "pe_";
+        prefix += "fp_";
     }
 
     return prefix + StructNameString(structure);
@@ -1063,6 +1065,8 @@ const char *InterpolationString(TQualifier qualifier)
             return "nointerpolation";
         case EvqCentroidOut:
             return "centroid";
+        case EvqSampleIn:
+            return "sample";
         default:
             UNREACHABLE();
     }
@@ -1074,15 +1078,17 @@ const char *QualifierString(TQualifier qualifier)
 {
     switch (qualifier)
     {
-        case EvqIn:
+        case EvqParamIn:
             return "in";
-        case EvqOut:
+        case EvqParamOut:
             return "inout";  // 'out' results in an HLSL error if not all fields are written, for
                              // GLSL it's undefined
-        case EvqInOut:
+        case EvqParamInOut:
             return "inout";
-        case EvqConstReadOnly:
+        case EvqParamConst:
             return "const";
+        case EvqSampleOut:
+            return "sample";
         default:
             UNREACHABLE();
     }
