@@ -18,6 +18,7 @@
 #include "libANGLE/IndexRangeCache.h"
 #include "libANGLE/Observer.h"
 #include "libANGLE/RefCountObject.h"
+#include "libANGLE/angletypes.h"
 
 namespace rx
 {
@@ -63,8 +64,23 @@ class BufferState final : angle::NonCopyable
     int mBindingCount;
     int mTransformFeedbackIndexedBindingCount;
     int mTransformFeedbackGenericBindingCount;
+    GLboolean mImmutable;
+    GLbitfield mStorageExtUsageFlags;
+    GLboolean mExternal;
 };
 
+// Some Vertex Array Objects track buffer data updates.
+struct ContentsObserver
+{
+    VertexArray *vertexArray = nullptr;
+    uint32_t bufferIndex     = 0;
+};
+
+ANGLE_INLINE bool operator==(const ContentsObserver &lhs, const ContentsObserver &rhs)
+{
+    return lhs.vertexArray == rhs.vertexArray && lhs.bufferIndex == rhs.bufferIndex;
+}
+
 class Buffer final : public RefCountObject<BufferID>,
                      public LabeledObject,
                      public angle::ObserverInterface,
@@ -78,6 +94,16 @@ class Buffer final : public RefCountObject<BufferID>,
     void setLabel(const Context *context, const std::string &label) override;
     const std::string &getLabel() const override;
 
+    angle::Result bufferStorageExternal(Context *context,
+                                        BufferBinding target,
+                                        GLsizeiptr size,
+                                        GLeglClientBufferEXT clientBuffer,
+                                        GLbitfield flags);
+    angle::Result bufferStorage(Context *context,
+                                BufferBinding target,
+                                GLsizeiptr size,
+                                const void *data,
+                                GLbitfield flags);
     angle::Result bufferData(Context *context,
                              BufferBinding target,
                              const void *data,
@@ -114,18 +140,32 @@ class Buffer final : public RefCountObject<BufferID>,
     GLbitfield getAccessFlags() const { return mState.mAccessFlags; }
     GLenum getAccess() const { return mState.mAccess; }
     GLboolean isMapped() const { return mState.mMapped; }
+    bool isPersistentlyMapped() const
+    {
+        return (mState.mStorageExtUsageFlags & GL_MAP_PERSISTENT_BIT_EXT) != 0;
+    }
     void *getMapPointer() const { return mState.mMapPointer; }
     GLint64 getMapOffset() const { return mState.mMapOffset; }
     GLint64 getMapLength() const { return mState.mMapLength; }
     GLint64 getSize() const { return mState.mSize; }
     GLint64 getMemorySize() const;
+    GLboolean isImmutable() const { return mState.mImmutable; }
+    GLbitfield getStorageExtUsageFlags() const { return mState.mStorageExtUsageFlags; }
+
+    // Buffers are always initialized immediately when allocated
+    InitState initState() const { return InitState::Initialized; }
 
     rx::BufferImpl *getImplementation() const { return mImpl; }
 
-    ANGLE_INLINE bool isBound() const { return mState.mBindingCount > 0; }
-
-    ANGLE_INLINE bool isBoundForTransformFeedbackAndOtherUse() const
+    // Note: we pass "isWebGL" to this function to clarify it's only valid if WebGL is enabled.
+    // We pass the boolean flag instead of the pointer because this header can't read Context.h.
+    ANGLE_INLINE bool hasWebGLXFBBindingConflict(bool isWebGL) const
     {
+        if (!isWebGL)
+        {
+            return false;
+        }
+
         // The transform feedback generic binding point is not an indexed binding point but it also
         // does not count as a non-transform-feedback use of the buffer, so we subtract it from the
         // binding count when checking if the buffer is bound to a non-transform-feedback location.
@@ -146,11 +186,30 @@ class Buffer final : public RefCountObject<BufferID>,
     // angle::ObserverInterface implementation.
     void onSubjectStateChange(angle::SubjectIndex index, angle::SubjectMessage message) override;
 
+    void addContentsObserver(VertexArray *vertexArray, uint32_t bufferIndex);
+    void removeContentsObserver(VertexArray *vertexArray, uint32_t bufferIndex);
+
   private:
+    angle::Result bufferDataImpl(Context *context,
+                                 BufferBinding target,
+                                 const void *data,
+                                 GLsizeiptr size,
+                                 BufferUsage usage,
+                                 GLbitfield flags);
+    angle::Result bufferExternalDataImpl(Context *context,
+                                         BufferBinding target,
+                                         GLeglClientBufferEXT clientBuffer,
+                                         GLsizeiptr size,
+                                         GLbitfield flags);
+
+    void onContentsChange();
+    size_t getContentsObserverIndex(VertexArray *vertexArray, uint32_t bufferIndex) const;
+
     BufferState mState;
     rx::BufferImpl *mImpl;
     angle::ObserverBinding mImplObserver;
 
+    angle::FastVector<ContentsObserver, angle::kMaxFixedObservers> mContentsObservers;
     mutable IndexRangeCache mIndexRangeCache;
 };
 
