@@ -10,6 +10,7 @@
 #include "angle_gl.h"
 #include "libANGLE/Version.h"
 #include "libANGLE/angletypes.h"
+#include "libANGLE/gles_extensions_autogen.h"
 #include "libANGLE/renderer/Format.h"
 
 #include <array>
@@ -20,13 +21,12 @@
 
 namespace gl
 {
-
-struct Extensions;
-
 struct TextureCaps
 {
     TextureCaps();
     TextureCaps(const TextureCaps &other);
+    TextureCaps &operator=(const TextureCaps &other);
+
     ~TextureCaps();
 
     // Supports for basic texturing: glTexImage, glTexSubImage, etc
@@ -79,7 +79,7 @@ class TextureCapsMap final : angle::NonCopyable
     TextureCaps &get(angle::FormatID formatID);
 
     // Indexed by angle::FormatID
-    std::array<TextureCaps, angle::kNumANGLEFormats> mFormatData;
+    angle::FormatMap<TextureCaps> mFormatData;
 };
 
 void InitMinimumTextureCapsMap(const Version &clientVersion,
@@ -90,552 +90,7 @@ void InitMinimumTextureCapsMap(const Version &clientVersion,
 // present. Does not determine if they are natively supported without decompression.
 bool DetermineCompressedTextureETCSupport(const TextureCapsMap &textureCaps);
 
-struct Extensions
-{
-    Extensions();
-    Extensions(const Extensions &other);
-
-    // Generate a vector of supported extension strings
-    std::vector<std::string> getStrings() const;
-
-    // Set all texture related extension support based on the supported textures.
-    // Determines support for:
-    // GL_OES_packed_depth_stencil
-    // GL_OES_rgb8_rgba8
-    // GL_EXT_texture_format_BGRA8888
-    // GL_EXT_color_buffer_half_float,
-    // GL_OES_texture_half_float, GL_OES_texture_half_float_linear
-    // GL_OES_texture_float, GL_OES_texture_float_linear
-    // GL_EXT_texture_rg
-    // GL_EXT_texture_compression_dxt1, GL_ANGLE_texture_compression_dxt3,
-    // GL_ANGLE_texture_compression_dxt5
-    // GL_KHR_texture_compression_astc_ldr, GL_OES_texture_compression_astc.
-    //     NOTE: GL_KHR_texture_compression_astc_hdr must be enabled separately. Support for the
-    //           HDR profile cannot be determined from the format enums alone.
-    // GL_OES_compressed_ETC1_RGB8_texture
-    // GL_EXT_sRGB
-    // GL_ANGLE_depth_texture, GL_OES_depth32
-    // GL_EXT_color_buffer_float
-    // GL_EXT_texture_norm16
-    // GL_EXT_texture_compression_bptc
-    // GL_EXT_texture_compression_rgtc
-    void setTextureExtensionSupport(const TextureCapsMap &textureCaps);
-
-    // indicate if any depth texture extension is available
-    bool depthTextureAny() const { return (depthTextureANGLE || depthTextureOES); }
-
-    // ES2 Extension support
-
-    // GL_OES_element_index_uint
-    bool elementIndexUintOES = false;
-
-    // GL_OES_packed_depth_stencil
-    bool packedDepthStencilOES = false;
-
-    // GL_NV_read_depth
-    bool readDepthNV = false;
-
-    // GL_NV_read_stencil
-    bool readStencilNV = false;
-
-    // GL_NV_depth_buffer_float2
-    bool depthBufferFloat2NV = false;
-
-    // GL_OES_get_program_binary
-    bool getProgramBinaryOES = false;
-
-    // GL_OES_rgb8_rgba8
-    // Implies that TextureCaps for GL_RGB8 and GL_RGBA8 exist
-    bool rgb8rgba8OES = false;
-
-    // GL_EXT_texture_format_BGRA8888
-    // Implies that TextureCaps for GL_BGRA8 exist
-    bool textureFormatBGRA8888 = false;
-
-    // GL_EXT_read_format_bgra
-    bool readFormatBGRA = false;
-
-    // GL_NV_pixel_buffer_object
-    bool pixelBufferObjectNV = false;
-
-    // GL_ARB_sync
-    bool glSyncARB = false;
-
-    // GL_OES_mapbuffer and GL_EXT_map_buffer_range
-    bool mapBufferOES   = false;
-    bool mapBufferRange = false;
-
-    // GL_EXT_color_buffer_half_float
-    // Together with GL_OES_texture_half_float in a GLES 2.0 context, implies that half-float
-    // textures are renderable.
-    bool colorBufferHalfFloat = false;
-
-    // GL_OES_texture_half_float and GL_OES_texture_half_float_linear
-    // Implies that TextureCaps for GL_RGB16F, GL_RGBA16F, GL_ALPHA32F_EXT, GL_LUMINANCE32F_EXT and
-    // GL_LUMINANCE_ALPHA32F_EXT exist
-    bool textureHalfFloat       = false;
-    bool textureHalfFloatLinear = false;
-
-    // GL_EXT_texture_type_2_10_10_10_REV
-    bool textureFormat2101010REV = false;
-
-    // GL_OES_texture_float and GL_OES_texture_float_linear
-    // Implies that TextureCaps for GL_RGB32F, GL_RGBA32F, GL_ALPHA16F_EXT, GL_LUMINANCE16F_EXT and
-    // GL_LUMINANCE_ALPHA16F_EXT exist
-    bool textureFloatOES       = false;
-    bool textureFloatLinearOES = false;
-
-    // GL_EXT_texture_rg
-    // Implies that TextureCaps for GL_R8, GL_RG8 (and floating point R/RG texture formats if
-    // floating point extensions are also present) exist
-    bool textureRG = false;
-
-    // GL_EXT_texture_compression_dxt1, GL_ANGLE_texture_compression_dxt3 and
-    // GL_ANGLE_texture_compression_dxt5 Implies that TextureCaps exist for
-    // GL_COMPRESSED_RGB_S3TC_DXT1_EXT, GL_COMPRESSED_RGBA_S3TC_DXT1_EXT
-    // GL_COMPRESSED_RGBA_S3TC_DXT3_ANGLE and GL_COMPRESSED_RGBA_S3TC_DXT5_ANGLE
-    bool textureCompressionDXT1 = false;
-    bool textureCompressionDXT3 = false;
-    bool textureCompressionDXT5 = false;
-
-    // GL_EXT_texture_compression_s3tc_srgb
-    // Implies that TextureCaps exist for GL_COMPRESSED_SRGB_S3TC_DXT1_EXT,
-    // GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT, GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT, and
-    // GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT
-    bool textureCompressionS3TCsRGB = false;
-
-    // GL_KHR_texture_compression_astc_ldr
-    bool textureCompressionASTCLDRKHR = false;
-
-    // GL_KHR_texture_compression_astc_hdr
-    bool textureCompressionASTCHDRKHR = false;
-
-    // GL_OES_texture_compression_astc
-    bool textureCompressionASTCOES = false;
-
-    // GL_KHR_texture_compression_astc_sliced_3d
-    bool textureCompressionSliced3dASTCKHR = false;
-
-    // GL_EXT_texture_compression_bptc
-    bool textureCompressionBPTC = false;
-
-    // GL_EXT_texture_compression_rgtc
-    bool textureCompressionRGTC = false;
-
-    // GL_OES_compressed_ETC1_RGB8_texture
-    // Implies that TextureCaps for GL_ETC1_RGB8_OES exist
-    bool compressedETC1RGB8TextureOES = false;
-
-    // GL_EXT_compressed_ETC1_RGB8_sub_texture
-    bool compressedETC1RGB8SubTexture = false;
-
-    // OES_compressed_ETC2_RGB8_texture
-    bool compressedETC2RGB8TextureOES = false;
-
-    // OES_compressed_ETC2_sRGB8_texture
-    bool compressedETC2sRGB8TextureOES = false;
-
-    // OES_compressed_ETC2_punchthroughA_RGBA8_texture
-    bool compressedETC2PunchthroughARGB8TextureOES = false;
-
-    // OES_compressed_ETC2_punchthroughA_sRGB8_alpha_texture
-    bool compressedETC2PunchthroughAsRGB8AlphaTextureOES = false;
-
-    // OES_compressed_ETC2_RGBA8_texture
-    bool compressedETC2RGBA8TextureOES = false;
-
-    // OES_compressed_ETC2_sRGB8_alpha8_texture
-    bool compressedETC2sRGB8Alpha8TextureOES = false;
-
-    // OES_compressed_EAC_R11_unsigned_texture
-    bool compressedEACR11UnsignedTextureOES = false;
-
-    // OES_compressed_EAC_R11_signed_texture
-    bool compressedEACR11SignedTextureOES = false;
-
-    // OES_compressed_EAC_RG11_unsigned_texture
-    bool compressedEACRG11UnsignedTextureOES = false;
-
-    // OES_compressed_EAC_RG11_signed_texture
-    bool compressedEACRG11SignedTextureOES = false;
-
-    // ANGLE_compressed_texture_etc
-    // ONLY exposed if ETC texture formats are natively supported without decompression
-    // Backends should enable this extension explicitly. It is not enabled with
-    // setTextureExtensionSupport, use DetermineCompressedTextureETCSupport to check if all of the
-    // individual formats required to support this extension are available.
-    bool compressedTextureETC = false;
-
-    // GL_IMG_texture_compression_pvrtc
-    bool compressedTexturePVRTC = false;
-
-    // GL_EXT_pvrtc_sRGB
-    bool compressedTexturePVRTCsRGB = false;
-
-    // GL_EXT_sRGB
-    // Implies that TextureCaps for GL_SRGB8_ALPHA8 and GL_SRGB8 exist
-    // TODO: Don't advertise this extension in ES3
-    bool sRGB = false;
-
-    // GL_EXT_texture_sRGB_R8
-    bool sRGBR8EXT = false;
-
-    // GL_ANGLE_depth_texture
-    bool depthTextureANGLE = false;
-
-    // OES_depth_texture
-    bool depthTextureOES = false;
-
-    // GL_OES_depth_texture_cube_map
-    bool depthTextureCubeMapOES = false;
-
-    // GL_OES_depth24
-    // Allows DEPTH_COMPONENT24_OES as a valid Renderbuffer format.
-    bool depth24OES = false;
-
-    // GL_OES_depth32
-    // Allows DEPTH_COMPONENT32_OES as a valid Renderbuffer format.
-    bool depth32OES = false;
-
-    // GL_OES_texture_3D
-    bool texture3DOES = false;
-
-    // GL_EXT_texture_storage
-    bool textureStorage = false;
-
-    // GL_OES_texture_npot
-    bool textureNPOTOES = false;
-
-    // GL_EXT_draw_buffers
-    bool drawBuffers = false;
-
-    // GL_EXT_draw_buffers_indexed
-    bool drawBuffersIndexedEXT = false;
-
-    // GL_OES_draw_buffers_indexed
-    bool drawBuffersIndexedOES = false;
-
-    // Any version of the draw_buffers_indexed
-    bool drawBuffersIndexedAny() const { return (drawBuffersIndexedEXT || drawBuffersIndexedOES); }
-
-    // GL_EXT_texture_filter_anisotropic
-    bool textureFilterAnisotropic = false;
-    GLfloat maxTextureAnisotropy  = 0.0f;
-
-    // GL_EXT_occlusion_query_boolean
-    bool occlusionQueryBoolean = false;
-
-    // GL_NV_fence
-    bool fenceNV = false;
-
-    // GL_EXT_disjoint_timer_query
-    bool disjointTimerQuery            = false;
-    GLuint queryCounterBitsTimeElapsed = 0;
-    GLuint queryCounterBitsTimestamp   = 0;
-
-    // GL_EXT_robustness
-    bool robustness = false;
-
-    // GL_KHR_robust_buffer_access_behavior
-    bool robustBufferAccessBehavior = false;
-
-    // GL_EXT_blend_minmax
-    bool blendMinMax = false;
-
-    // GL_ANGLE_framebuffer_blit
-    bool framebufferBlit = false;
-
-    // GL_ANGLE_framebuffer_multisample
-    bool framebufferMultisample = false;
-
-    // GL_EXT_multisampled_render_to_texture
-    bool multisampledRenderToTexture = false;
-
-    // GL_ANGLE_instanced_arrays
-    bool instancedArraysANGLE = false;
-    // GL_EXT_instanced_arrays
-    bool instancedArraysEXT = false;
-    // Any version of the instanced arrays extension
-    bool instancedArraysAny() const { return (instancedArraysANGLE || instancedArraysEXT); }
-
-    // GL_ANGLE_pack_reverse_row_order
-    bool packReverseRowOrder = false;
-
-    // GL_OES_standard_derivatives
-    bool standardDerivativesOES = false;
-
-    // GL_EXT_shader_texture_lod
-    bool shaderTextureLOD = false;
-
-    // GL_EXT_frag_depth
-    bool fragDepth = false;
-
-    // OVR_multiview
-    bool multiview  = false;
-    GLuint maxViews = 1;
-
-    // OVR_multiview2
-    bool multiview2 = false;
-
-    // GL_ANGLE_texture_usage
-    bool textureUsage = false;
-
-    // GL_ANGLE_translated_shader_source
-    bool translatedShaderSource = false;
-
-    // GL_OES_fbo_render_mipmap
-    bool fboRenderMipmapOES = false;
-
-    // GL_EXT_discard_framebuffer
-    bool discardFramebuffer = false;
-
-    // EXT_debug_marker
-    bool debugMarker = false;
-
-    // GL_OES_EGL_image
-    bool eglImageOES = false;
-
-    // GL_OES_EGL_image_external
-    bool eglImageExternalOES = false;
-
-    // GL_OES_EGL_image_external_essl3
-    bool eglImageExternalEssl3OES = false;
-
-    // GL_EXT_EGL_image_external_wrap_modes
-    bool eglImageExternalWrapModesEXT = false;
-
-    // GL_OES_EGL_sync
-    bool eglSyncOES = false;
-
-    // GL_EXT_memory_object
-    bool memoryObject = false;
-
-    // GL_EXT_memory_object_fd
-    bool memoryObjectFd = false;
-
-    // GL_ANGLE_memory_object_fuchsia
-    bool memoryObjectFuchsiaANGLE = false;
-
-    // GL_EXT_semaphore
-    bool semaphore = false;
-
-    // GL_EXT_semaphore_fd
-    bool semaphoreFd = false;
-
-    // GL_ANGLE_semaphore_fuchsia
-    bool semaphoreFuchsiaANGLE = false;
-
-    // NV_EGL_stream_consumer_external
-    bool eglStreamConsumerExternalNV = false;
-
-    // EXT_unpack_subimage
-    bool unpackSubimage = false;
-
-    // NV_pack_subimage
-    bool packSubimage = false;
-
-    // GL_NV_shader_noperspective_interpolation
-    bool noperspectiveInterpolationNV = false;
-
-    // GL_OES_vertex_half_float
-    bool vertexHalfFloatOES = false;
-
-    // GL_OES_vertex_array_object
-    bool vertexArrayObjectOES = false;
-
-    // GL_OES_vertex_type_10_10_10_2
-    bool vertexAttribType1010102OES = false;
-
-    // GL_KHR_debug
-    bool debug                     = false;
-    GLuint maxDebugMessageLength   = 0;
-    GLuint maxDebugLoggedMessages  = 0;
-    GLuint maxDebugGroupStackDepth = 0;
-    GLuint maxLabelLength          = 0;
-
-    // KHR_no_error
-    bool noError = false;
-
-    // GL_ANGLE_lossy_etc_decode
-    bool lossyETCDecode = false;
-
-    // GL_CHROMIUM_bind_uniform_location
-    bool bindUniformLocation = false;
-
-    // GL_CHROMIUM_sync_query
-    bool syncQuery = false;
-
-    // GL_CHROMIUM_copy_texture
-    bool copyTexture = false;
-
-    // GL_CHROMIUM_copy_compressed_texture
-    bool copyCompressedTexture = false;
-
-    // GL_ANGLE_copy_texture_3d
-    bool copyTexture3d = false;
-
-    // GL_ANGLE_webgl_compatibility
-    bool webglCompatibility = false;
-
-    // GL_ANGLE_request_extension
-    bool requestExtension = false;
-
-    // GL_CHROMIUM_bind_generates_resource
-    bool bindGeneratesResource = false;
-
-    // GL_ANGLE_robust_client_memory
-    bool robustClientMemory = false;
-
-    // GL_OES_texture_border_clamp
-    bool textureBorderClampOES = false;
-
-    // GL_EXT_texture_sRGB_decode
-    bool textureSRGBDecode = false;
-
-    // GL_EXT_texture_sRGB_override
-    bool textureSRGBOverride = false;
-
-    // GL_EXT_sRGB_write_control
-    bool sRGBWriteControl = false;
-
-    // GL_CHROMIUM_color_buffer_float_rgb
-    bool colorBufferFloatRGB = false;
-
-    // GL_CHROMIUM_color_buffer_float_rgba
-    bool colorBufferFloatRGBA = false;
-
-    // GL_EXT_EGL_image_array
-    bool eglImageArray = false;
-
-    // ES3 Extension support
-
-    // GL_EXT_color_buffer_float
-    bool colorBufferFloat = false;
-
-    // GL_EXT_multisample_compatibility.
-    // written against ES 3.1 but can apply to earlier versions.
-    bool multisampleCompatibility = false;
-
-    // GL_CHROMIUM_framebuffer_mixed_samples
-    bool framebufferMixedSamples = false;
-
-    // GL_EXT_texture_norm16
-    // written against ES 3.1 but can apply to ES 3.0 as well.
-    bool textureNorm16 = false;
-
-    // GL_OES_surfaceless_context
-    bool surfacelessContextOES = false;
-
-    // GL_ANGLE_client_arrays
-    bool clientArrays = false;
-
-    // GL_ANGLE_robust_resource_initialization
-    bool robustResourceInitialization = false;
-
-    // GL_ANGLE_program_cache_control
-    bool programCacheControl = false;
-
-    // GL_ANGLE_texture_rectangle
-    bool textureRectangle = false;
-
-    // GL_EXT_geometry_shader
-    bool geometryShader = false;
-
-    // GLES1 emulation: GLES1 extensions
-    // GL_OES_point_size_array
-    bool pointSizeArrayOES = false;
-
-    // GL_OES_texture_cube_map
-    bool textureCubeMapOES = false;
-
-    // GL_OES_point_sprite
-    bool pointSpriteOES = false;
-
-    // GL_OES_draw_texture
-    bool drawTextureOES = false;
-
-    // EGL_ANGLE_explicit_context GL subextensions
-    // GL_ANGLE_explicit_context_gles1
-    bool explicitContextGles1 = false;
-    // GL_ANGLE_explicit_context
-    bool explicitContext = false;
-
-    // GL_KHR_parallel_shader_compile
-    bool parallelShaderCompile = false;
-
-    // GL_OES_texture_storage_multisample_2d_array
-    bool textureStorageMultisample2DArrayOES = false;
-
-    // GL_ANGLE_multiview_multisample
-    bool multiviewMultisample = false;
-
-    // GL_EXT_blend_func_extended
-    bool blendFuncExtended          = false;
-    GLuint maxDualSourceDrawBuffers = 0;
-
-    // GL_EXT_float_blend
-    bool floatBlend = false;
-
-    // GL_ANGLE_memory_size
-    bool memorySize = false;
-
-    // GL_ANGLE_texture_multisample
-    bool textureMultisample = false;
-
-    // GL_ANGLE_multi_draw
-    bool multiDraw = false;
-
-    // GL_ANGLE_provoking_vertex
-    bool provokingVertex = false;
-
-    // GL_CHROMIUM_texture_filtering_hint
-    bool textureFilteringCHROMIUM = false;
-
-    // GL_CHROMIUM_lose_context
-    bool loseContextCHROMIUM = false;
-
-    // GL_ANGLE_texture_external_update
-    bool textureExternalUpdateANGLE = false;
-
-    // GL_ANGLE_base_vertex_base_instance
-    bool baseVertexBaseInstance = false;
-
-    // GL_ANGLE_get_image
-    bool getImageANGLE = false;
-
-    // GL_OES_draw_elements_base_vertex
-    bool drawElementsBaseVertexOES = false;
-    // GL_EXT_draw_elements_base_vertex
-    bool drawElementsBaseVertexEXT = false;
-    // Any version of the base vertex extension
-    bool drawElementsBaseVertexAny() const
-    {
-        return (drawElementsBaseVertexOES || drawElementsBaseVertexEXT);
-    }
-
-    // GL_EXT_shader_non_constant_global_initializers
-    bool shaderNonConstGlobalInitializersEXT = false;
-
-    // GL_EXT_gpu_shader5
-    bool gpuShader5EXT = false;
-    // WEBGL_video_texture
-    bool webglVideoTexture = false;
-
-    // GL_APPLE_clip_distance
-    bool clipDistanceAPPLE = false;
-
-    // GL_OES_texture_cube_map_array
-    bool textureCubeMapArrayOES = false;
-    // GL_EXT_texture_cube_map_array
-    bool textureCubeMapArrayEXT = false;
-    // Any version of the texture cube map array extension
-    bool textureCubeMapArrayAny() const
-    {
-        return (textureCubeMapArrayOES || textureCubeMapArrayEXT);
-    }
-};
-
-// Pointer to a boolean memeber of the Extensions struct
+// Pointer to a boolean member of the Extensions struct
 using ExtensionBool = bool Extensions::*;
 
 struct ExtensionInfo
@@ -655,6 +110,9 @@ const ExtensionInfoMap &GetExtensionInfoMap();
 struct Limitations
 {
     Limitations();
+    Limitations(const Limitations &other);
+
+    Limitations &operator=(const Limitations &other);
 
     // Renderer doesn't support gl_FrontFacing in fragment shaders
     bool noFrontFacingSupport = false;
@@ -683,6 +141,19 @@ struct Limitations
 
     // D3D does not support vertex attribute aliasing
     bool noVertexAttributeAliasing = false;
+
+    // Renderer doesn't support GL_TEXTURE_COMPARE_MODE=GL_NONE on a shadow sampler.
+    // TODO(http://anglebug.com/5231): add validation code to front-end.
+    bool noShadowSamplerCompareModeNone = false;
+
+    // PVRTC1 textures must be squares.
+    bool squarePvrtc1 = false;
+
+    // ETC1 texture support is emulated.
+    bool emulatedEtc1 = false;
+
+    // No compressed TEXTURE_3D support.
+    bool noCompressedTexture3D = false;
 };
 
 struct TypePrecision
@@ -690,6 +161,8 @@ struct TypePrecision
     TypePrecision();
     TypePrecision(const TypePrecision &other);
 
+    TypePrecision &operator=(const TypePrecision &other);
+
     void setIEEEFloat();
     void setTwosComplementInt(unsigned int bits);
     void setSimulatedFloat(unsigned int range, unsigned int precision);
@@ -705,12 +178,18 @@ struct Caps
 {
     Caps();
     Caps(const Caps &other);
+    Caps &operator=(const Caps &other);
+
     ~Caps();
 
     // If the values could be got by using GetIntegeri_v, they should
     // be GLint instead of GLuint and call LimitToInt() to ensure
     // they will not overflow.
 
+    GLfloat minInterpolationOffset        = 0;
+    GLfloat maxInterpolationOffset        = 0;
+    GLint subPixelInterpolationOffsetBits = 0;
+
     // ES 3.1 (April 29, 2015) 20.39: implementation dependent values
     GLint64 maxElementIndex       = 0;
     GLint max3DTextureSize        = 0;
@@ -845,10 +324,43 @@ struct Caps
     GLint maxGeometryTotalOutputComponents = 0;
     GLint maxGeometryShaderInvocations     = 0;
 
+    // GL_EXT_tessellation_shader
+    GLint maxTessControlInputComponents       = 0;
+    GLint maxTessControlOutputComponents      = 0;
+    GLint maxTessControlTotalOutputComponents = 0;
+
+    GLint maxTessPatchComponents = 0;
+    GLint maxPatchVertices       = 0;
+    GLint maxTessGenLevel        = 0;
+
+    GLint maxTessEvaluationInputComponents  = 0;
+    GLint maxTessEvaluationOutputComponents = 0;
+
     GLuint subPixelBits = 4;
 
+    // GL_EXT_blend_func_extended
+    GLuint maxDualSourceDrawBuffers = 0;
+
+    // GL_EXT_texture_filter_anisotropic
+    GLfloat maxTextureAnisotropy = 0.0f;
+
+    // GL_EXT_disjoint_timer_query
+    GLuint queryCounterBitsTimeElapsed = 0;
+    GLuint queryCounterBitsTimestamp   = 0;
+
+    // OVR_multiview
+    GLuint maxViews = 1;
+
+    // GL_KHR_debug
+    GLuint maxDebugMessageLength   = 0;
+    GLuint maxDebugLoggedMessages  = 0;
+    GLuint maxDebugGroupStackDepth = 0;
+    GLuint maxLabelLength          = 0;
+
     // GL_APPLE_clip_distance/GL_EXT_clip_cull_distance
-    GLuint maxClipDistances = 0;
+    GLuint maxClipDistances                = 0;
+    GLuint maxCullDistances                = 0;
+    GLuint maxCombinedClipAndCullDistances = 0;
 
     // GLES1 emulation: Caps for ES 1.1. Taken from Table 6.20 / 6.22 in the OpenGL ES 1.1 spec.
     GLuint maxMultitextureUnits                 = 0;
@@ -862,6 +374,15 @@ struct Caps
     GLfloat maxSmoothPointSize                  = 0.0f;
     GLfloat minSmoothLineWidth                  = 0.0f;
     GLfloat maxSmoothLineWidth                  = 0.0f;
+
+    // ES 3.2 Table 20.41: Implementation Dependent Values (cont.)
+    GLint maxTextureBufferSize         = 0;
+    GLint textureBufferOffsetAlignment = 0;
+
+    // Direct-to-metal constants:
+    GLuint driverUniformsBindingIndex    = 0;
+    GLuint defaultUniformsBindingIndex   = 0;
+    GLuint UBOArgumentBufferBindingIndex = 0;
 };
 
 Caps GenerateMinimumCaps(const Version &clientVersion, const Extensions &extensions);
@@ -876,6 +397,9 @@ struct Caps
 
     // Support for NPOT surfaces
     bool textureNPOT;
+
+    // Support for Stencil8 configs
+    bool stencil8;
 };
 
 struct DisplayExtensions
@@ -915,9 +439,6 @@ struct DisplayExtensions
     // EGL_KHR_create_context
     bool createContext = false;
 
-    // EGL_EXT_device_query
-    bool deviceQuery = false;
-
     // EGL_KHR_image
     bool image = false;
 
@@ -942,9 +463,6 @@ struct DisplayExtensions
     // EGL_KHR_get_all_proc_addresses
     bool getAllProcAddresses = false;
 
-    // EGL_ANGLE_flexible_surface_compatibility
-    bool flexibleSurfaceCompatibility = false;
-
     // EGL_ANGLE_direct_composition
     bool directComposition = false;
 
@@ -996,18 +514,24 @@ struct DisplayExtensions
     // EGL_ANGLE_display_texture_share_group
     bool displayTextureShareGroup = false;
 
+    // EGL_ANGLE_display_semaphore_share_group
+    bool displaySemaphoreShareGroup = false;
+
     // EGL_ANGLE_create_context_client_arrays
     bool createContextClientArrays = false;
 
     // EGL_ANGLE_program_cache_control
-    bool programCacheControl = false;
+    bool programCacheControlANGLE = false;
 
     // EGL_ANGLE_robust_resource_initialization
-    bool robustResourceInitialization = false;
+    bool robustResourceInitializationANGLE = false;
 
     // EGL_ANGLE_iosurface_client_buffer
     bool iosurfaceClientBuffer = false;
 
+    // EGL_ANGLE_metal_texture_client_buffer
+    bool mtlTextureClientBuffer = false;
+
     // EGL_ANGLE_create_context_extensions_enabled
     bool createContextExtensionsEnabled = false;
 
@@ -1035,6 +559,9 @@ struct DisplayExtensions
     // EGL_ANDROID_get_native_client_buffer
     bool getNativeClientBufferANDROID = false;
 
+    // EGL_ANDROID_create_native_client_buffer
+    bool createNativeClientBufferANDROID = false;
+
     // EGL_ANDROID_native_fence_sync
     bool nativeFenceSyncANDROID = false;
 
@@ -1085,6 +612,39 @@ struct DisplayExtensions
 
     // EGL_NOK_texture_from_pixmap
     bool textureFromPixmapNOK = false;
+
+    // EGL_NV_robustness_video_memory_purge
+    bool robustnessVideoMemoryPurgeNV = false;
+
+    // EGL_KHR_reusable_sync
+    bool reusableSyncKHR = false;
+
+    // EGL_ANGLE_external_context_and_surface
+    bool externalContextAndSurface = false;
+
+    // EGL_EXT_buffer_age
+    bool bufferAgeEXT = false;
+
+    // EGL_KHR_mutable_render_buffer
+    bool mutableRenderBufferKHR = false;
+
+    // EGL_EXT_protected_content
+    bool protectedContentEXT = false;
+
+    // EGL_ANGLE_create_surface_swap_interval
+    bool createSurfaceSwapIntervalANGLE = false;
+
+    // EGL_ANGLE_context_virtualization
+    bool contextVirtualizationANGLE = false;
+
+    // EGL_KHR_lock_surface3
+    bool lockSurface3KHR = false;
+
+    // EGL_ANGLE_vulkan_image
+    bool vulkanImageANGLE = false;
+
+    // EGL_ANGLE_metal_create_context_ownership_identity
+    bool metalCreateContextOwnershipIdentityANGLE = false;
 };
 
 struct DeviceExtensions
@@ -1102,6 +662,12 @@ struct DeviceExtensions
 
     // EGL_ANGLE_device_eagl
     bool deviceEAGL = false;
+
+    // EGL_ANGLE_device_metal
+    bool deviceMetal = false;
+
+    // EGL_ANGLE_device_vulkan
+    bool deviceVulkan = false;
 };
 
 struct ClientExtensions
@@ -1142,8 +708,11 @@ struct ClientExtensions
     // EGL_ANGLE_platform_angle_metal
     bool platformANGLEMetal = false;
 
-    // EGL_ANGLE_platform_angle_context_virtualization
-    bool platformANGLEContextVirtualization = false;
+    // EGL_ANGLE_platform_angle_device_context_volatile_eagl
+    bool platformANGLEDeviceContextVolatileEagl = false;
+
+    // EGL_ANGLE_platform_angle_device_context_volatile_cgl
+    bool platformANGLEDeviceContextVolatileCgl = false;
 
     // EGL_ANGLE_device_creation
     bool deviceCreation = false;
@@ -1163,9 +732,6 @@ struct ClientExtensions
     // EGL_KHR_debug
     bool debug = false;
 
-    // EGL_ANGLE_explicit_context
-    bool explicitContext = false;
-
     // EGL_ANGLE_feature_control
     bool featureControlANGLE = false;
 
@@ -1174,6 +740,12 @@ struct ClientExtensions
 
     // EGL_ANGLE_platform_angle_device_type_egl_angle
     bool platformANGLEDeviceTypeEGLANGLE = false;
+
+    // EGL_EXT_device_query
+    bool deviceQueryEXT = false;
+
+    // EGL_ANGLE_display_power_preference
+    bool displayPowerPreferenceANGLE = false;
 };
 
 }  // namespace egl
