@@ -11,6 +11,7 @@
 #define LIBANGLE_OVERLAYWIDGETS_H_
 
 #include "common/angleutils.h"
+#include "libANGLE/Overlay_autogen.h"
 
 namespace gl
 {
@@ -20,7 +21,7 @@ class OverlayState;
 namespace overlay_impl
 {
 class AppendWidgetDataHelper;
-}
+}  // namespace overlay_impl
 
 enum class WidgetType
 {
@@ -44,28 +45,6 @@ enum class WidgetType
     EnumCount = InvalidEnum,
 };
 
-enum class WidgetId
-{
-    // Front-end widgets:
-
-    // Frames per second (PerSecond).
-    FPS,
-
-    // Vulkan backend:
-
-    // Last validation error (Text).
-    VulkanLastValidationMessage,
-    // Number of validation errors and warnings (Count).
-    VulkanValidationMessageCount,
-    // Number of RenderPasses in a frame (RunningGraph).
-    VulkanRenderPassCount,
-    // Secondary Command Buffer pool memory waste (RunningHistogram).
-    VulkanSecondaryCommandBufferPoolWaste,
-
-    InvalidEnum,
-    EnumCount = InvalidEnum,
-};
-
 namespace overlay
 {
 class Widget
@@ -184,9 +163,9 @@ class RunningHistogram : public RunningGraph
     }
 };
 
-// If overlay is disabled, all the above classes would be replaced with Dummy, turning them into
+// If overlay is disabled, all the above classes would be replaced with Mock, turning them into
 // noop.
-class Dummy
+class Mock
 {
   public:
     void reset() const {}
