@@ -45,6 +45,8 @@ struct FunctionsEGL::EGLDispatchTable
           destroyContextPtr(nullptr),
           destroySurfacePtr(nullptr),
           getConfigAttribPtr(nullptr),
+          getConfigsPtr(nullptr),
+          getCurrentSurfacePtr(nullptr),
           getDisplayPtr(nullptr),
           getErrorPtr(nullptr),
           initializePtr(nullptr),
@@ -59,6 +61,8 @@ struct FunctionsEGL::EGLDispatchTable
           surfaceAttribPtr(nullptr),
           swapIntervalPtr(nullptr),
 
+          getCurrentContextPtr(nullptr),
+
           createImageKHRPtr(nullptr),
           destroyImageKHRPtr(nullptr),
 
@@ -93,6 +97,8 @@ struct FunctionsEGL::EGLDispatchTable
     PFNEGLDESTROYCONTEXTPROC destroyContextPtr;
     PFNEGLDESTROYSURFACEPROC destroySurfacePtr;
     PFNEGLGETCONFIGATTRIBPROC getConfigAttribPtr;
+    PFNEGLGETCONFIGSPROC getConfigsPtr;
+    PFNEGLGETCURRENTSURFACEPROC getCurrentSurfacePtr;
     PFNEGLGETDISPLAYPROC getDisplayPtr;
     PFNEGLGETERRORPROC getErrorPtr;
     PFNEGLINITIALIZEPROC initializePtr;
@@ -108,6 +114,9 @@ struct FunctionsEGL::EGLDispatchTable
     PFNEGLSURFACEATTRIBPROC surfaceAttribPtr;
     PFNEGLSWAPINTERVALPROC swapIntervalPtr;
 
+    // 1.4
+    PFNEGLGETCURRENTCONTEXTPROC getCurrentContextPtr;
+
     // EGL_KHR_image
     PFNEGLCREATEIMAGEKHRPROC createImageKHRPtr;
     PFNEGLDESTROYIMAGEKHRPROC destroyImageKHRPtr;
@@ -169,6 +178,8 @@ egl::Error FunctionsEGL::initialize(EGLNativeDisplayType nativeDisplay)
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->destroyContextPtr, eglDestroyContext);
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->destroySurfacePtr, eglDestroySurface);
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->getConfigAttribPtr, eglGetConfigAttrib);
+    ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->getConfigsPtr, eglGetConfigs);
+    ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->getCurrentSurfacePtr, eglGetCurrentSurface);
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->getDisplayPtr, eglGetDisplay);
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->getErrorPtr, eglGetError);
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->initializePtr, eglInitialize);
@@ -201,6 +212,8 @@ egl::Error FunctionsEGL::initialize(EGLNativeDisplayType nativeDisplay)
         return egl::Error(mFnPtrs->getErrorPtr(), "Failed to bind API in system egl");
     }
 
+    ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->getCurrentContextPtr, eglGetCurrentContext);
+
     const char *extensions = queryString(EGL_EXTENSIONS);
     if (!extensions)
     {
@@ -328,11 +341,23 @@ EGLBoolean FunctionsEGL::chooseConfig(EGLint const *attribList,
     return mFnPtrs->chooseConfigPtr(mEGLDisplay, attribList, configs, configSize, numConfig);
 }
 
+EGLBoolean FunctionsEGL::getConfigs(EGLConfig *configs,
+                                    EGLint config_size,
+                                    EGLint *num_config) const
+{
+    return mFnPtrs->getConfigsPtr(mEGLDisplay, configs, config_size, num_config);
+}
+
 EGLBoolean FunctionsEGL::getConfigAttrib(EGLConfig config, EGLint attribute, EGLint *value) const
 {
     return mFnPtrs->getConfigAttribPtr(mEGLDisplay, config, attribute, value);
 }
 
+EGLSurface FunctionsEGL::getCurrentSurface(EGLint readdraw) const
+{
+    return mFnPtrs->getCurrentSurfacePtr(readdraw);
+}
+
 EGLContext FunctionsEGL::createContext(EGLConfig config,
                                        EGLContext share_context,
                                        EGLint const *attrib_list) const
@@ -402,6 +427,11 @@ EGLBoolean FunctionsEGL::swapInterval(EGLint interval) const
     return mFnPtrs->swapIntervalPtr(mEGLDisplay, interval);
 }
 
+EGLContext FunctionsEGL::getCurrentContext() const
+{
+    return mFnPtrs->getCurrentContextPtr();
+}
+
 EGLImageKHR FunctionsEGL::createImageKHR(EGLContext context,
                                          EGLenum target,
                                          EGLClientBuffer buffer,
@@ -441,7 +471,7 @@ EGLint FunctionsEGL::waitSyncKHR(EGLSyncKHR sync, EGLint flags) const
 }
 
 EGLBoolean FunctionsEGL::swapBuffersWithDamageKHR(EGLSurface surface,
-                                                  EGLint *rects,
+                                                  const EGLint *rects,
                                                   EGLint n_rects) const
 {
     return mFnPtrs->swapBuffersWithDamageKHRPtr(mEGLDisplay, surface, rects, n_rects);
