@@ -1,3 +1,4 @@
+
 //
 // Copyright 2019 The ANGLE Project Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
@@ -17,9 +18,12 @@
 #include "common/Optional.h"
 #include "common/utilities.h"
 #include "libANGLE/renderer/ProgramImpl.h"
+#include "libANGLE/renderer/ShaderInterfaceVariableInfoMap.h"
 #include "libANGLE/renderer/glslang_wrapper_utils.h"
 #include "libANGLE/renderer/metal/mtl_buffer_pool.h"
 #include "libANGLE/renderer/metal/mtl_command_buffer.h"
+#include "libANGLE/renderer/metal/mtl_common.h"
+#include "libANGLE/renderer/metal/mtl_context_device.h"
 #include "libANGLE/renderer/metal/mtl_glslang_mtl_utils.h"
 #include "libANGLE/renderer/metal/mtl_resources.h"
 #include "libANGLE/renderer/metal/mtl_state_cache.h"
@@ -128,17 +132,19 @@ class ProgramMtl : public ProgramImpl, public mtl::RenderPipelineCacheSpecialize
     void getUniformuiv(const gl::Context *context, GLint location, GLuint *params) const override;
 
     // Override mtl::RenderPipelineCacheSpecializeShaderFactory
-    angle::Result getSpecializedShader(mtl::Context *context,
+    angle::Result getSpecializedShader(ContextMtl *context,
                                        gl::ShaderType shaderType,
                                        const mtl::RenderPipelineDesc &renderPipelineDesc,
                                        id<MTLFunction> *shaderOut) override;
     bool hasSpecializedShader(gl::ShaderType shaderType,
                               const mtl::RenderPipelineDesc &renderPipelineDesc) override;
 
-    angle::Result createMslShaderLib(mtl::Context *context,
-                                     gl::ShaderType shaderType,
-                                     gl::InfoLog &infoLog,
-                                     mtl::TranslatedShaderInfo *translatedMslInfo);
+    angle::Result createMslShaderLib(
+        ContextMtl *context,
+        gl::ShaderType shaderType,
+        gl::InfoLog &infoLog,
+        mtl::TranslatedShaderInfo *translatedMslInfo,
+        NSDictionary<NSString *, NSObject *> *subtitutionDictionary = @{});
     // Calls this before drawing, changedPipelineDesc is passed when vertex attributes desc and/or
     // shader program changed.
     angle::Result setupDraw(const gl::Context *glContext,
@@ -146,12 +152,19 @@ class ProgramMtl : public ProgramImpl, public mtl::RenderPipelineCacheSpecialize
                             const mtl::RenderPipelineDesc &pipelineDesc,
                             bool pipelineDescChanged,
                             bool forceTexturesSetting,
-                            bool uniformBuffersDirty,
-                            bool transformFeedbackDraw);
+                            bool uniformBuffersDirty);
 
-    std::string getXfbMslSource() const { return mXfbMslSource; }
+    std::string getTranslatedShaderSource(const gl::ShaderType shaderType) const
+    {
+        return mMslShaderTranslateInfo[shaderType].metalShaderSource;
+    }
 
-    mtl::RenderPipelineCache *mMetalXfbRenderPipelineCache;
+    mtl::TranslatedShaderInfo getTranslatedShaderInfo(const gl::ShaderType shaderType) const
+    {
+        return mMslShaderTranslateInfo[shaderType];
+    }
+
+    bool hasFlatAttribute() const { return mProgramHasFlatAttributes; }
 
   private:
     template <int cols, int rows>
@@ -180,6 +193,9 @@ class ProgramMtl : public ProgramImpl, public mtl::RenderPipelineCacheSpecialize
     angle::Result updateUniformBuffers(ContextMtl *context,
                                        mtl::RenderCommandEncoder *cmdEncoder,
                                        const mtl::RenderPipelineDesc &pipelineDesc);
+    angle::Result updateXfbBuffers(ContextMtl *context,
+                                   mtl::RenderCommandEncoder *cmdEncoder,
+                                   const mtl::RenderPipelineDesc &pipelineDesc);
     angle::Result legalizeUniformBufferOffsets(ContextMtl *context,
                                                const std::vector<gl::InterfaceBlock> &blocks);
     angle::Result bindUniformBuffersToDiscreteSlots(ContextMtl *context,
@@ -200,6 +216,8 @@ class ProgramMtl : public ProgramImpl, public mtl::RenderPipelineCacheSpecialize
     void saveShaderInternalInfo(gl::BinaryOutputStream *stream);
     void loadShaderInternalInfo(gl::BinaryInputStream *stream);
 
+    void linkUpdateHasFlatAttributes();
+
 #if ANGLE_ENABLE_METAL_SPIRV
 
     angle::Result linkImplSpirv(const gl::Context *glContext,
@@ -220,6 +238,8 @@ class ProgramMtl : public ProgramImpl, public mtl::RenderPipelineCacheSpecialize
                                         gl::BinaryInputStream *stream,
                                         gl::InfoLog &infoLog);
 
+    mtl::BufferPool *getBufferPool(ContextMtl *context);
+
     // State for the default uniform blocks.
     struct DefaultUniformBlock final : private angle::NonCopyable
     {
@@ -234,6 +254,7 @@ class ProgramMtl : public ProgramImpl, public mtl::RenderPipelineCacheSpecialize
         std::vector<sh::BlockMemberInfo> uniformLayout;
     };
 
+    bool mProgramHasFlatAttributes;
     gl::ShaderBitSet mDefaultUniformBlocksDirty;
     gl::ShaderBitSet mSamplerBindingsDirty;
     gl::ShaderMap<DefaultUniformBlock> mDefaultUniformBlocks;
@@ -257,7 +278,7 @@ class ProgramMtl : public ProgramImpl, public mtl::RenderPipelineCacheSpecialize
     // Cached references of current shader variants.
     gl::ShaderMap<ProgramShaderObjVariantMtl *> mCurrentShaderVariants;
 
-    ShaderMapInterfaceVariableInfoMap mVariableInfoMap;
+    ShaderInterfaceVariableInfoMap mVariableInfoMap;
     // Scratch data:
     // Legalized buffers and their offsets. For example, uniform buffer's offset=1 is not a valid
     // offset, it will be converted to legal offset and the result is stored in this array.
@@ -266,10 +287,10 @@ class ProgramMtl : public ProgramImpl, public mtl::RenderPipelineCacheSpecialize
     // into an argument buffer.
     std::vector<uint32_t> mArgumentBufferRenderStageUsages;
 
-    uint32_t mShadowCompareModes[mtl::kMaxShaderSamplers] = {0};
+    uint32_t mShadowCompareModes[mtl::kMaxShaderSamplers];
 
     mtl::RenderPipelineCache mMetalRenderPipelineCache;
-    std::string mXfbMslSource;
+    mtl::BufferPool *mAuxBufferPool;
 };
 
 }  // namespace rx
