@@ -18,7 +18,7 @@ namespace rx
 class ShaderVk : public ShaderImpl
 {
   public:
-    ShaderVk(const gl::ShaderState &data);
+    ShaderVk(const gl::ShaderState &state);
     ~ShaderVk() override;
 
     std::shared_ptr<WaitableCompileEvent> compile(const gl::Context *context,
