@@ -25,7 +25,7 @@ class DisplayVkGGP : public DisplayVk
                                        EGLNativeWindowType window) override;
 
     egl::ConfigSet generateConfigs() override;
-    bool checkConfigSupport(egl::Config *config) override;
+    void checkConfigSupport(egl::Config *config) override;
 
     const char *getWSIExtension() const override;
 };
