@@ -76,8 +76,9 @@ TEST_P(EGLX11VisualHintTest, InvalidVisualID)
     static const int gInvalidVisualId = -1;
     auto attributes                   = getDisplayAttributes(gInvalidVisualId);
 
-    EGLDisplay display =
-        eglGetPlatformDisplayEXT(EGL_PLATFORM_ANGLE_ANGLE, EGL_DEFAULT_DISPLAY, attributes.data());
+    EGLDisplay display = eglGetPlatformDisplayEXT(
+        EGL_PLATFORM_ANGLE_ANGLE, reinterpret_cast<_XDisplay *>(EGL_DEFAULT_DISPLAY),
+        attributes.data());
     ASSERT_TRUE(display != EGL_NO_DISPLAY);
 
     ASSERT_TRUE(EGL_FALSE == eglInitialize(display, nullptr, nullptr));
@@ -100,9 +101,10 @@ TEST_P(EGLX11VisualHintTest, ValidVisualIDAndClear)
     ASSERT_NE(0, XGetWindowAttributes(mDisplay, xWindow, &windowAttributes));
     int visualId = windowAttributes.visual->visualid;
 
-    auto attributes = getDisplayAttributes(visualId);
-    EGLDisplay display =
-        eglGetPlatformDisplayEXT(EGL_PLATFORM_ANGLE_ANGLE, EGL_DEFAULT_DISPLAY, attributes.data());
+    auto attributes    = getDisplayAttributes(visualId);
+    EGLDisplay display = eglGetPlatformDisplayEXT(
+        EGL_PLATFORM_ANGLE_ANGLE, reinterpret_cast<_XDisplay *>(EGL_DEFAULT_DISPLAY),
+        attributes.data());
     ASSERT_NE(EGL_NO_DISPLAY, display);
 
     ASSERT_TRUE(EGL_TRUE == eglInitialize(display, nullptr, nullptr));
@@ -145,6 +147,9 @@ TEST_P(EGLX11VisualHintTest, ValidVisualIDAndClear)
 
         eglDestroyContext(display, context);
         ASSERT_EGL_SUCCESS();
+
+        eglMakeCurrent(display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
+        ASSERT_EGL_SUCCESS();
     }
 
     OSWindow::Delete(&osWindow);
@@ -153,7 +158,7 @@ TEST_P(EGLX11VisualHintTest, ValidVisualIDAndClear)
     eglTerminate(display);
 }
 
-// Test that EGL_BAD_MATCH is generated when trying to create an EGL window from
+// Test that a child window is created when trying to create an EGL window from
 // an X11 window whose visual ID doesn't match the visual ID passed at display creation.
 TEST_P(EGLX11VisualHintTest, InvalidWindowVisualID)
 {
@@ -174,9 +179,10 @@ TEST_P(EGLX11VisualHintTest, InvalidWindowVisualID)
         OSWindow::Delete(&osWindow);
     }
 
-    auto attributes = getDisplayAttributes(visualId);
-    EGLDisplay display =
-        eglGetPlatformDisplayEXT(EGL_PLATFORM_ANGLE_ANGLE, EGL_DEFAULT_DISPLAY, attributes.data());
+    auto attributes    = getDisplayAttributes(visualId);
+    EGLDisplay display = eglGetPlatformDisplayEXT(
+        EGL_PLATFORM_ANGLE_ANGLE, reinterpret_cast<_XDisplay *>(EGL_DEFAULT_DISPLAY),
+        attributes.data());
     ASSERT_NE(EGL_NO_DISPLAY, display);
 
     ASSERT_TRUE(EGL_TRUE == eglInitialize(display, nullptr, nullptr));
@@ -191,15 +197,25 @@ TEST_P(EGLX11VisualHintTest, InvalidWindowVisualID)
 
     Window xWindow = osWindow->getNativeWindow();
 
-    // Creating the EGL window should fail with EGL_BAD_MATCH
+    // Creating the EGL window should succeed
     int nReturnedConfigs = 0;
     EGLConfig config;
     ASSERT_TRUE(EGL_TRUE == eglGetConfigs(display, &config, 1, &nReturnedConfigs));
     ASSERT_EQ(1, nReturnedConfigs);
 
     EGLSurface window = eglCreateWindowSurface(display, config, xWindow, nullptr);
-    ASSERT_EQ(EGL_NO_SURFACE, window);
-    ASSERT_EGL_ERROR(EGL_BAD_MATCH);
+    ASSERT_TRUE(window);
+    ASSERT_EGL_SUCCESS();
+
+    // When trying to create a window with a visual other than the one specified
+    // with EGL_X11_VISUAL_ID_ANGLE, ANGLE should fallback to using a child window.
+    Window root;
+    Window parent;
+    Window *children;
+    unsigned int nchildren;
+    XQueryTree(mDisplay, xWindow, &root, &parent, &children, &nchildren);
+    EXPECT_EQ(nchildren, 1U);
+    XFree(children);
 
     OSWindow::Delete(&osWindow);
 }
