@@ -55,13 +55,28 @@ class MultisampleTest : public ANGLETestWithParam<MultisampleTestParams>
 
         // Nexus 5X and 6P fail to eglMakeCurrent with a config they advertise they support.
         // http://anglebug.com/3464
-        ANGLE_SKIP_TEST_IF(IsNexus5X() || IsNexus6P());
+        ANGLE_SKIP_TEST_IF(IsNexus5X());
 
         // Find a config that uses RGBA8 and allows 4x multisampling.
-        const EGLint configAttributes[] = {
-            EGL_RED_SIZE,       8, EGL_GREEN_SIZE, 8,  EGL_BLUE_SIZE,    8,
-            EGL_ALPHA_SIZE,     8, EGL_DEPTH_SIZE, 24, EGL_STENCIL_SIZE, 8,
-            EGL_SAMPLE_BUFFERS, 1, EGL_SAMPLES,    4,  EGL_NONE};
+        const EGLint configAttributes[] = {EGL_SURFACE_TYPE,
+                                           EGL_WINDOW_BIT,
+                                           EGL_RED_SIZE,
+                                           8,
+                                           EGL_GREEN_SIZE,
+                                           8,
+                                           EGL_BLUE_SIZE,
+                                           8,
+                                           EGL_ALPHA_SIZE,
+                                           8,
+                                           EGL_DEPTH_SIZE,
+                                           24,
+                                           EGL_STENCIL_SIZE,
+                                           8,
+                                           EGL_SAMPLE_BUFFERS,
+                                           1,
+                                           EGL_SAMPLES,
+                                           4,
+                                           EGL_NONE};
 
         EGLint configCount;
         EGLConfig multisampledConfig;
@@ -76,7 +91,7 @@ class MultisampleTest : public ANGLETestWithParam<MultisampleTestParams>
 
         // Create a window, context and surface if multisampling is possible.
         mOSWindow = OSWindow::New();
-        mOSWindow->initialize("MultisampleTest", kWindowSize, kWindowSize);
+        mOSWindow->initialize("MultisampleTest", kWindowWidth, kWindowHeight);
         setWindowVisible(mOSWindow, true);
 
         EGLint contextAttributes[] = {
@@ -140,7 +155,8 @@ class MultisampleTest : public ANGLETestWithParam<MultisampleTestParams>
     }
 
   protected:
-    static constexpr int kWindowSize = 8;
+    static constexpr int kWindowWidth  = 16;
+    static constexpr int kWindowHeight = 8;
 
     OSWindow *mOSWindow            = nullptr;
     EGLDisplay mDisplay            = EGL_NO_DISPLAY;
@@ -158,6 +174,8 @@ TEST_P(MultisampleTest, Point)
     ANGLE_SKIP_TEST_IF(!mMultisampledConfigExists);
     // http://anglebug.com/3470
     ANGLE_SKIP_TEST_IF(IsAndroid() && IsNVIDIAShield() && IsOpenGLES());
+    // http://anglebug.com/5727
+    ANGLE_SKIP_TEST_IF(IsOzone());
 
     constexpr char kPointsVS[] = R"(precision highp float;
 attribute vec4 a_position;
@@ -182,29 +200,33 @@ void main()
     ASSERT_GL_NO_ERROR();
 
     // The center pixels should be all red.
-    EXPECT_PIXEL_COLOR_EQ(kWindowSize / 2, kWindowSize / 2, GLColor::red);
-    EXPECT_PIXEL_COLOR_EQ(kWindowSize / 2 - 1, kWindowSize / 2, GLColor::red);
-    EXPECT_PIXEL_COLOR_EQ(kWindowSize / 2, kWindowSize / 2 - 1, GLColor::red);
-    EXPECT_PIXEL_COLOR_EQ(kWindowSize / 2 - 1, kWindowSize / 2 - 1, GLColor::red);
+    EXPECT_PIXEL_COLOR_EQ(kWindowWidth / 2, kWindowHeight / 2, GLColor::red);
+    EXPECT_PIXEL_COLOR_EQ(kWindowWidth / 2 - 1, kWindowHeight / 2, GLColor::red);
+    EXPECT_PIXEL_COLOR_EQ(kWindowWidth / 2, kWindowHeight / 2 - 1, GLColor::red);
+    EXPECT_PIXEL_COLOR_EQ(kWindowWidth / 2 - 1, kWindowHeight / 2 - 1, GLColor::red);
 
     // Border pixels should be between red and black, and not exactly either; corners are darker and
     // sides are brighter.
     const GLColor kSideColor   = {128, 0, 0, 128};
     const GLColor kCornerColor = {64, 0, 0, 64};
     constexpr int kErrorMargin = 16;
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 - 2, kWindowSize / 2 - 2, kCornerColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 - 2, kWindowSize / 2 + 1, kCornerColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 + 1, kWindowSize / 2 - 2, kCornerColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 + 1, kWindowSize / 2 + 1, kCornerColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 - 2, kWindowHeight / 2 - 2, kCornerColor,
+                            kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 - 2, kWindowHeight / 2 + 1, kCornerColor,
+                            kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 + 1, kWindowHeight / 2 - 2, kCornerColor,
+                            kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 + 1, kWindowHeight / 2 + 1, kCornerColor,
+                            kErrorMargin);
 
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 - 2, kWindowSize / 2 - 1, kSideColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 - 2, kWindowSize / 2, kSideColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 - 1, kWindowSize / 2 - 2, kSideColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 - 1, kWindowSize / 2 + 1, kSideColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2, kWindowSize / 2 - 2, kSideColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2, kWindowSize / 2 + 1, kSideColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 + 1, kWindowSize / 2 - 1, kSideColor, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2 + 1, kWindowSize / 2, kSideColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 - 2, kWindowHeight / 2 - 1, kSideColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 - 2, kWindowHeight / 2, kSideColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 - 1, kWindowHeight / 2 - 2, kSideColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 - 1, kWindowHeight / 2 + 1, kSideColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2, kWindowHeight / 2 - 2, kSideColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2, kWindowHeight / 2 + 1, kSideColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 + 1, kWindowHeight / 2 - 1, kSideColor, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2 + 1, kWindowHeight / 2, kSideColor, kErrorMargin);
 }
 
 // Test line rendering on a multisampled surface.  GLES2 section 3.4.4.
@@ -212,6 +234,8 @@ TEST_P(MultisampleTest, Line)
 {
     ANGLE_SKIP_TEST_IF(!mMultisampledConfigExists);
     ANGLE_SKIP_TEST_IF(IsARM64() && IsWindows() && IsD3D());
+    // http://anglebug.com/5727
+    ANGLE_SKIP_TEST_IF(IsOzone());
 
     ANGLE_GL_PROGRAM(program, essl1_shaders::vs::Simple(), essl1_shaders::fs::Red());
     glUseProgram(program);
@@ -242,15 +266,15 @@ TEST_P(MultisampleTest, Line)
     constexpr int kErrorMargin = 16;
     constexpr int kLargeMargin = 80;
 
-    static_assert(kWindowSize == 8, "Verification code written for 8x8 window");
+    static_assert(kWindowWidth == 16, "Verification code written for 16x8 window");
     EXPECT_PIXEL_COLOR_NEAR(0, 2, kDarkRed, kLargeMargin);
-    EXPECT_PIXEL_COLOR_NEAR(1, 3, GLColor::red, kLargeMargin);
-    EXPECT_PIXEL_COLOR_NEAR(2, 3, GLColor::red, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(3, 3, kMidRed, kLargeMargin);
-    EXPECT_PIXEL_COLOR_NEAR(4, 4, kMidRed, kLargeMargin);
-    EXPECT_PIXEL_COLOR_NEAR(5, 4, GLColor::red, kErrorMargin);
-    EXPECT_PIXEL_COLOR_NEAR(6, 4, GLColor::red, kLargeMargin);
-    EXPECT_PIXEL_COLOR_NEAR(7, 5, kDarkRed, kLargeMargin);
+    EXPECT_PIXEL_COLOR_NEAR(3, 3, GLColor::red, kLargeMargin);
+    EXPECT_PIXEL_COLOR_NEAR(4, 3, GLColor::red, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(6, 3, kMidRed, kLargeMargin);
+    EXPECT_PIXEL_COLOR_NEAR(8, 4, kMidRed, kLargeMargin);
+    EXPECT_PIXEL_COLOR_NEAR(11, 4, GLColor::red, kErrorMargin);
+    EXPECT_PIXEL_COLOR_NEAR(12, 4, GLColor::red, kLargeMargin);
+    EXPECT_PIXEL_COLOR_NEAR(15, 5, kDarkRed, kLargeMargin);
 }
 
 // Test polygon rendering on a multisampled surface.  GLES2 section 3.5.3.
@@ -259,6 +283,8 @@ TEST_P(MultisampleTest, Triangle)
     ANGLE_SKIP_TEST_IF(!mMultisampledConfigExists);
     // http://anglebug.com/3470
     ANGLE_SKIP_TEST_IF(IsAndroid() && IsNVIDIAShield() && IsOpenGLES());
+    // http://anglebug.com/5727
+    ANGLE_SKIP_TEST_IF(IsOzone());
 
     ANGLE_GL_PROGRAM(program, essl1_shaders::vs::Simple(), essl1_shaders::fs::Red());
     glUseProgram(program);
@@ -275,16 +301,17 @@ TEST_P(MultisampleTest, Triangle)
 
     // Top-left pixels should be all red.
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
-    EXPECT_PIXEL_COLOR_EQ(kWindowSize / 4, kWindowSize / 4, GLColor::red);
+    EXPECT_PIXEL_COLOR_EQ(kWindowWidth / 4, kWindowHeight / 4, GLColor::red);
 
     // Diagonal pixels from bottom-left to top-right are between red and black.  Pixels above the
     // diagonal are red and pixels below it are black.
-    const GLColor kMidRed      = {128, 0, 0, 128};
-    constexpr int kErrorMargin = 16;
+    const GLColor kMidRed = {128, 0, 0, 128};
+    // D3D11 is off by 63 for red (191 instead of 128), where other back-ends get 128
+    constexpr int kErrorMargin = 64;
 
-    for (int i = 1; i + 1 < kWindowSize; ++i)
+    for (int i = 2; i + 2 < kWindowWidth; i += 2)
     {
-        int j = kWindowSize - 1 - i;
+        int j = kWindowHeight - 1 - (i / 2);
         EXPECT_PIXEL_COLOR_NEAR(i, j, kMidRed, kErrorMargin);
         EXPECT_PIXEL_COLOR_EQ(i, j - 1, GLColor::red);
         EXPECT_PIXEL_COLOR_EQ(i, j + 1, GLColor::transparentBlack);
@@ -300,9 +327,10 @@ TEST_P(MultisampleTest, ContentPresevedAfterInterruption)
     ANGLE_SKIP_TEST_IF(!IsGLExtensionEnabled("GL_OES_rgb8_rgba8"));
     // http://anglebug.com/3470
     ANGLE_SKIP_TEST_IF(IsAndroid() && IsNVIDIAShield() && IsOpenGLES());
-
     // http://anglebug.com/4609
     ANGLE_SKIP_TEST_IF(IsD3D11());
+    // http://anglebug.com/5727
+    ANGLE_SKIP_TEST_IF(IsOzone());
 
     ANGLE_GL_PROGRAM(program, essl1_shaders::vs::Simple(), essl1_shaders::fs::Red());
     glUseProgram(program);
@@ -338,7 +366,7 @@ TEST_P(MultisampleTest, ContentPresevedAfterInterruption)
 
     // Top-left pixels should be all red.
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
-    EXPECT_PIXEL_COLOR_EQ(kWindowSize / 4, kWindowSize / 4, GLColor::red);
+    EXPECT_PIXEL_COLOR_EQ(kWindowWidth / 4, kWindowHeight / 4, GLColor::red);
 
     // Triangle edge:
     // Diagonal pixels from bottom-left to top-right are between red and black.  Pixels above the
@@ -347,14 +375,14 @@ TEST_P(MultisampleTest, ContentPresevedAfterInterruption)
         const GLColor kMidRed      = {128, 0, 0, 128};
         constexpr int kErrorMargin = 16;
 
-        for (int i = 1; i + 1 < kWindowSize; ++i)
+        for (int i = 2; i + 2 < kWindowWidth; i += 2)
         {
             // Exclude the middle pixel where the triangle and line cross each other.
-            if (abs(kWindowSize / 2 - i) <= 1)
+            if (abs(kWindowHeight / 2 - (i / 2)) <= 1)
             {
                 continue;
             }
-            int j = kWindowSize - 1 - i;
+            int j = kWindowHeight - 1 - (i / 2);
             EXPECT_PIXEL_COLOR_NEAR(i, j, kMidRed, kErrorMargin);
             EXPECT_PIXEL_COLOR_EQ(i, j - 1, GLColor::red);
             EXPECT_PIXEL_COLOR_EQ(i, j + 1, GLColor::transparentBlack);
@@ -367,14 +395,59 @@ TEST_P(MultisampleTest, ContentPresevedAfterInterruption)
         constexpr int kErrorMargin = 16;
         constexpr int kLargeMargin = 80;
 
-        static_assert(kWindowSize == 8, "Verification code written for 8x8 window");
+        static_assert(kWindowWidth == 16, "Verification code written for 16x8 window");
         // Exclude the triangle region.
-        EXPECT_PIXEL_COLOR_NEAR(5, 4, GLColor::red, kErrorMargin);
-        EXPECT_PIXEL_COLOR_NEAR(6, 4, GLColor::red, kLargeMargin);
-        EXPECT_PIXEL_COLOR_NEAR(7, 5, kDarkRed, kLargeMargin);
+        EXPECT_PIXEL_COLOR_NEAR(11, 4, GLColor::red, kErrorMargin);
+        EXPECT_PIXEL_COLOR_NEAR(12, 4, GLColor::red, kLargeMargin);
+        EXPECT_PIXEL_COLOR_NEAR(15, 5, kDarkRed, kLargeMargin);
     }
 }
 
+// Test that alpha to coverage is enabled works properly along with early fragment test.
+TEST_P(MultisampleTest, AlphaToSampleCoverage)
+{
+    ANGLE_SKIP_TEST_IF(!mMultisampledConfigExists);
+    // http://anglebug.com/5727
+    ANGLE_SKIP_TEST_IF(IsOzone());
+
+    constexpr char kFS[] =
+        "precision highp float;\n"
+        "void main()\n"
+        "{\n"
+        "    gl_FragColor = vec4(1.0, 0.0, 0.0, 0.0);\n"
+        "}\n";
+    ANGLE_GL_PROGRAM(transparentRedProgram, essl1_shaders::vs::Simple(), kFS);
+    glUseProgram(transparentRedProgram);
+    glEnable(GL_DEPTH_TEST);
+    glDepthFunc(GL_LESS);
+    glClearDepthf(1.0f);
+    glClearColor(0.0f, 1.0f, 0.0f, 1.0f);  // clear to green
+    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+    // This should pass depth test, but because of the alpha to coverage enabled, and alpha is 0,
+    // the fragment should be discarded. If early fragment test is disabled, no depth will be
+    // written. depth buffer should be 1.0.
+    glEnable(GL_SAMPLE_ALPHA_TO_COVERAGE);
+    // There was a bug in ANGLE that we are checking sampler coverage enabled or not instead of
+    // alpha to sample coverage enabled or not. This is specically try to trick ANGLE so that it
+    // will enable early fragment test. When early fragment test is accidentally enabled, then the
+    // depth test will occur before fragment shader, and depth buffer maybe written with value
+    // (0.0+1.0)/2.0=0.5.
+    glEnable(GL_SAMPLE_COVERAGE);
+    drawQuad(transparentRedProgram, essl1_shaders::PositionAttrib(), 0.0f);
+
+    // Now draw with blue color but to test against 0.0f. This should fail depth test
+    glDisable(GL_SAMPLE_ALPHA_TO_COVERAGE);
+    glDisable(GL_SAMPLE_COVERAGE);
+    glDepthFunc(GL_GREATER);
+    ANGLE_GL_PROGRAM(blueProgram, essl1_shaders::vs::Simple(), essl1_shaders::fs::Blue());
+    // Zd = 0.5f means (0.5+1.0)/2.0=0.75. Depends on early fragment on or off this will pass or
+    // fail depth test.
+    drawQuad(blueProgram, essl1_shaders::PositionAttrib(), 0.5f);
+    EXPECT_PIXEL_COLOR_EQ(1, 1, GLColor::green);
+
+    ASSERT_GL_NO_ERROR();
+}
+
 // Test that resolve from multisample default framebuffer works.
 TEST_P(MultisampleTestES3, ResolveToFBO)
 {
@@ -382,8 +455,8 @@ TEST_P(MultisampleTestES3, ResolveToFBO)
 
     GLTexture resolveTexture;
     glBindTexture(GL_TEXTURE_2D, resolveTexture);
-    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, kWindowSize, kWindowSize, 0, GL_RGBA, GL_UNSIGNED_BYTE,
-                 nullptr);
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, kWindowWidth, kWindowHeight, 0, GL_RGBA,
+                 GL_UNSIGNED_BYTE, nullptr);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 
@@ -400,17 +473,17 @@ TEST_P(MultisampleTestES3, ResolveToFBO)
     glBindFramebuffer(GL_DRAW_FRAMEBUFFER, resolveFBO);
     glClearColor(1, 0, 0, 1);
     glClear(GL_COLOR_BUFFER_BIT);
-    glBlitFramebuffer(0, 0, kWindowSize, kWindowSize, 0, 0, kWindowSize, kWindowSize,
+    glBlitFramebuffer(0, 0, kWindowWidth, kWindowHeight, 0, 0, kWindowWidth, kWindowHeight,
                       GL_COLOR_BUFFER_BIT, GL_NEAREST);
     ASSERT_GL_NO_ERROR();
 
     const GLColor kResult = GLColor(63, 127, 191, 63);
     glBindFramebuffer(GL_READ_FRAMEBUFFER, resolveFBO);
     EXPECT_PIXEL_COLOR_NEAR(0, 0, kResult, 1);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize - 1, 0, kResult, 1);
-    EXPECT_PIXEL_COLOR_NEAR(0, kWindowSize - 1, kResult, 1);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize - 1, kWindowSize - 1, kResult, 1);
-    EXPECT_PIXEL_COLOR_NEAR(kWindowSize / 2, kWindowSize / 2, kResult, 1);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth - 1, 0, kResult, 1);
+    EXPECT_PIXEL_COLOR_NEAR(0, kWindowHeight - 1, kResult, 1);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth - 1, kWindowHeight - 1, kResult, 1);
+    EXPECT_PIXEL_COLOR_NEAR(kWindowWidth / 2, kWindowHeight / 2, kResult, 1);
 }
 
 ANGLE_INSTANTIATE_TEST_COMBINE_1(MultisampleTest,
@@ -439,6 +512,7 @@ ANGLE_INSTANTIATE_TEST_COMBINE_1(MultisampleTest,
                                  WithNoFixture(ES2_METAL()));
 }  // namespace store_and_resolve_feature_off
 
+GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(MultisampleTestES3);
 ANGLE_INSTANTIATE_TEST_COMBINE_1(MultisampleTestES3,
                                  PrintToStringParamName,
                                  testing::Values(false),
@@ -449,5 +523,6 @@ ANGLE_INSTANTIATE_TEST_COMBINE_1(MultisampleTestES3,
                                  WithNoFixture(ES3_OPENGLES()),
                                  WithNoFixture(ES31_OPENGLES()),
                                  WithNoFixture(ES3_VULKAN()),
-                                 WithNoFixture(ES31_VULKAN()));
+                                 WithNoFixture(ES31_VULKAN()),
+                                 WithNoFixture(ES3_METAL()));
 }  // anonymous namespace
