@@ -15,7 +15,28 @@ using namespace angle;
 namespace
 {
 
-class ProgramInterfaceTestES31 : public ANGLETest
+// Variations:
+//
+// - bool: whether the program must be created and recreated, so that it's reloaded from cache.
+using ProgramInterfaceTestParams = std::tuple<angle::PlatformParameters, bool>;
+
+std::string ProgramInterfaceTestPrint(
+    const ::testing::TestParamInfo<ProgramInterfaceTestParams> &paramsInfo)
+{
+    const ProgramInterfaceTestParams &params = paramsInfo.param;
+    std::ostringstream out;
+
+    out << std::get<0>(params);
+
+    if (std::get<1>(params))
+    {
+        out << "__cached";
+    }
+
+    return out.str();
+}
+
+class ProgramInterfaceTestES31 : public ANGLETestWithParam<ProgramInterfaceTestParams>
 {
   protected:
     ProgramInterfaceTestES31()
@@ -27,8 +48,45 @@ class ProgramInterfaceTestES31 : public ANGLETest
         setConfigBlueBits(8);
         setConfigAlphaBits(8);
     }
+
+    void createGraphicsProgram(GLProgram &program,
+                               const char *vs,
+                               const char *fs,
+                               bool cacheAndReload);
+    void createComputeProgram(GLProgram &program, const char *cs, bool cacheAndReload);
 };
 
+void ProgramInterfaceTestES31::createGraphicsProgram(GLProgram &program,
+                                                     const char *vs,
+                                                     const char *fs,
+                                                     bool cacheAndReload)
+{
+    program.makeRaster(vs, fs);
+    ASSERT_TRUE(program.valid());
+
+    if (cacheAndReload)
+    {
+        program.reset();
+        program.makeRaster(vs, fs);
+        ASSERT_TRUE(program.valid());
+    }
+}
+
+void ProgramInterfaceTestES31::createComputeProgram(GLProgram &program,
+                                                    const char *cs,
+                                                    bool cacheAndReload)
+{
+    program.makeCompute(cs);
+    ASSERT_TRUE(program.valid());
+
+    if (cacheAndReload)
+    {
+        program.reset();
+        program.makeCompute(cs);
+        ASSERT_TRUE(program.valid());
+    }
+}
+
 // Tests glGetProgramResourceIndex.
 TEST_P(ProgramInterfaceTestES31, GetResourceIndex)
 {
@@ -42,7 +100,8 @@ TEST_P(ProgramInterfaceTestES31, GetResourceIndex)
         "    oColor = color;\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, essl31_shaders::vs::Simple(), kFS);
+    GLProgram program;
+    createGraphicsProgram(program, essl31_shaders::vs::Simple(), kFS, std::get<1>(GetParam()));
 
     GLuint index =
         glGetProgramResourceIndex(program, GL_PROGRAM_INPUT, essl31_shaders::PositionAttrib());
@@ -78,7 +137,8 @@ TEST_P(ProgramInterfaceTestES31, GetResourceName)
         "    oColor[0] = color;\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, essl31_shaders::vs::Simple(), kFS);
+    GLProgram program;
+    createGraphicsProgram(program, essl31_shaders::vs::Simple(), kFS, std::get<1>(GetParam()));
 
     GLuint index =
         glGetProgramResourceIndex(program, GL_PROGRAM_INPUT, essl31_shaders::PositionAttrib());
@@ -144,7 +204,8 @@ TEST_P(ProgramInterfaceTestES31, GetResourceLocation)
         "    oColor[0] = color;\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, kVS, kFS);
+    GLProgram program;
+    createGraphicsProgram(program, kVS, kFS, std::get<1>(GetParam()));
 
     GLenum invalidInterfaces[] = {GL_UNIFORM_BLOCK, GL_TRANSFORM_FEEDBACK_VARYING,
                                   GL_BUFFER_VARIABLE, GL_SHADER_STORAGE_BLOCK,
@@ -204,7 +265,8 @@ TEST_P(ProgramInterfaceTestES31, GetResource)
         "    oColor[0] = color;\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, kVS, kFS);
+    GLProgram program;
+    createGraphicsProgram(program, kVS, kFS, std::get<1>(GetParam()));
 
     GLuint index = glGetProgramResourceIndex(program, GL_PROGRAM_INPUT, "position");
     EXPECT_GL_NO_ERROR();
@@ -283,7 +345,8 @@ TEST_P(ProgramInterfaceTestES31, GetProgramInterface)
         "    blockInstance2[0].target = vec3(1, 1, 1);\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, essl31_shaders::vs::Simple(), kFS);
+    GLProgram program;
+    createGraphicsProgram(program, essl31_shaders::vs::Simple(), kFS, std::get<1>(GetParam()));
 
     GLint num;
     glGetProgramInterfaceiv(program, GL_PROGRAM_INPUT, GL_ACTIVE_RESOURCES, &num);
@@ -347,9 +410,11 @@ TEST_P(ProgramInterfaceTestES31, GetProgramInterface)
 // Tests the resource property query for uniform can be done correctly.
 TEST_P(ProgramInterfaceTestES31, GetUniformProperties)
 {
-    // TODO(jiajia.qin@intel.com): Don't skip this test once atomic counter is supported on d3d
-    // backend. http://anglebug.com/1729
-    ANGLE_SKIP_TEST_IF(IsD3D11());
+    // Check atomic support.
+    GLint numSupported;
+    glGetIntegerv(GL_MAX_VERTEX_ATOMIC_COUNTERS, &numSupported);
+    EXPECT_GL_NO_ERROR();
+    ANGLE_SKIP_TEST_IF(numSupported < 1);
 
     constexpr char kVS[] =
         "#version 310 es\n"
@@ -371,7 +436,8 @@ TEST_P(ProgramInterfaceTestES31, GetUniformProperties)
         "    oColor = color;\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, kVS, kFS);
+    GLProgram program;
+    createGraphicsProgram(program, kVS, kFS, std::get<1>(GetParam()));
 
     GLuint index = glGetProgramResourceIndex(program, GL_UNIFORM, "color");
     EXPECT_GL_NO_ERROR();
@@ -478,7 +544,8 @@ TEST_P(ProgramInterfaceTestES31, GetUniformBlockProperties)
         "  color = vec4(v, 0, 1);\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, kVS, kFS);
+    GLProgram program;
+    createGraphicsProgram(program, kVS, kFS, std::get<1>(GetParam()));
 
     GLuint index = glGetProgramResourceIndex(program, GL_UNIFORM_BLOCK, "blockName");
     EXPECT_GL_NO_ERROR();
@@ -540,9 +607,13 @@ TEST_P(ProgramInterfaceTestES31, GetUniformBlockProperties)
 // Tests atomic counter buffer qeury works correctly.
 TEST_P(ProgramInterfaceTestES31, QueryAtomicCounteBuffer)
 {
-    // TODO(jiajia.qin@intel.com): Don't skip this test once atomic counter is supported on d3d
-    // backend. http://anglebug.com/1729
-    ANGLE_SKIP_TEST_IF(IsD3D11());
+    // Check atomic support.
+    GLint numSupportedInVertex;
+    GLint numSupportedInFragment;
+    glGetIntegerv(GL_MAX_VERTEX_ATOMIC_COUNTERS, &numSupportedInVertex);
+    glGetIntegerv(GL_MAX_FRAGMENT_ATOMIC_COUNTERS, &numSupportedInFragment);
+    EXPECT_GL_NO_ERROR();
+    ANGLE_SKIP_TEST_IF(numSupportedInVertex < 1 || numSupportedInFragment < 1);
 
     constexpr char kVS[] =
         "#version 310 es\n"
@@ -566,7 +637,9 @@ TEST_P(ProgramInterfaceTestES31, QueryAtomicCounteBuffer)
         "    my_color = vec4(0.0);\n"
         "}\n";
 
-    ANGLE_GL_PROGRAM(program, kVS, kFS);
+    GLProgram program;
+    createGraphicsProgram(program, kVS, kFS, std::get<1>(GetParam()));
+
     GLint num;
     glGetProgramInterfaceiv(program, GL_ATOMIC_COUNTER_BUFFER, GL_ACTIVE_RESOURCES, &num);
     EXPECT_GL_NO_ERROR();
@@ -640,7 +713,8 @@ TEST_P(ProgramInterfaceTestES31, GetBufferVariableProperties)
         "    oColor = vec4(instanceName1[0].f1, 0, 0, 1);\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, kVS, kFS);
+    GLProgram program;
+    createGraphicsProgram(program, kVS, kFS, std::get<1>(GetParam()));
 
     GLuint index = glGetProgramResourceIndex(program, GL_BUFFER_VARIABLE, "blockName1.f1");
     EXPECT_GL_NO_ERROR();
@@ -755,7 +829,8 @@ void main()
 }
 )";
 
-    ANGLE_GL_COMPUTE_PROGRAM(program, kComputeShaderSource);
+    GLProgram program;
+    createComputeProgram(program, kComputeShaderSource, std::get<1>(GetParam()));
 
     GLuint index = glGetProgramResourceIndex(program, GL_BUFFER_VARIABLE, "blockIn.a");
     EXPECT_GL_NO_ERROR();
@@ -868,7 +943,8 @@ void main()
 }
 )";
 
-    ANGLE_GL_COMPUTE_PROGRAM(program, kComputeShaderSource);
+    GLProgram program;
+    createComputeProgram(program, kComputeShaderSource, std::get<1>(GetParam()));
 
     GLuint index = glGetProgramResourceIndex(program, GL_BUFFER_VARIABLE, "blockOut.s[0][0].m");
     EXPECT_GL_NO_ERROR();
@@ -996,7 +1072,8 @@ TEST_P(ProgramInterfaceTestES31, GetShaderStorageBlockProperties)
         "    oColor = color;\n"
         "}";
 
-    ANGLE_GL_PROGRAM(program, kVS, kFS);
+    GLProgram program;
+    createGraphicsProgram(program, kVS, kFS, std::get<1>(GetParam()));
 
     GLuint index = glGetProgramResourceIndex(program, GL_SHADER_STORAGE_BLOCK, "blockName0");
     EXPECT_GL_NO_ERROR();
@@ -1061,12 +1138,13 @@ layout(binding = 0) uniform atomic_uint ac2;
 
 void main()
 {
-    atomicCounter(acbase);
-    atomicCounter(ac[0]);
-    atomicCounter(ac2);
+    atomicCounterIncrement(acbase);
+    atomicCounterIncrement(ac[0]);
+    atomicCounterIncrement(ac2);
 })";
 
-    ANGLE_GL_COMPUTE_PROGRAM(program, kCSSource);
+    GLProgram program;
+    createComputeProgram(program, kCSSource, std::get<1>(GetParam()));
 
     GLuint index = glGetProgramResourceIndex(program, GL_UNIFORM, "ac");
     EXPECT_GL_NO_ERROR();
@@ -1211,6 +1289,64 @@ void main() {
     glDeleteProgram(program);
 }
 
-ANGLE_INSTANTIATE_TEST_ES31(ProgramInterfaceTestES31);
+// Regression test for crash report in http://anglebug.com/6073.
+TEST_P(ProgramInterfaceTestES31, ReloadFromCacheShouldNotCrash)
+{
+    // TODO(jiajia.qin@intel.com): Don't skip this test once non-simple SSBO sentences are supported
+    // on d3d backend. http://anglebug.com/1951
+    ANGLE_SKIP_TEST_IF(IsD3D11());
+
+    constexpr char kVS[] = R"(#version 310 es
+#extension GL_ANGLE_multi_draw : require
+precision highp int;
+precision highp float;
+layout(std140) buffer;
+struct TransformInfo
+{
+    mat4 mvp;
+};
+layout(binding = 0) buffer pe_transforms
+{
+    TransformInfo transforms[];
+};
+out vec2 texCoord;
+uniform int pe_base_draw_id;
+layout(location = 0) in vec3 pe_vertex;
+layout(location = 1) in vec3 pe_normal;
+layout(location = 2) in vec2 pe_tex_coord;
+void main()
+{
+    vec4 v = vec4(pe_vertex, 1.0);
+    texCoord = pe_tex_coord;
+    gl_Position = transforms[(gl_DrawID + pe_base_draw_id)].mvp * v;
+})";
+
+    constexpr char kFS[] = R"(#version 310 es
+#extension GL_ANGLE_multi_draw : require
+precision highp int;
+precision highp float;
+layout(std140) buffer;
+in vec2 texCoord;
+layout(binding = 0) uniform sampler2D pe_tex_main;
+out vec4 pe_frag_color;
+void main()
+{
+    vec4 u = texture(pe_tex_main, texCoord);
+    if(u.a < 0.05)
+        discard;
+    pe_frag_color = u;
+}
+)";
+
+    GLProgram program;
+    createGraphicsProgram(program, kVS, kFS, std::get<1>(GetParam()));
+    EXPECT_GL_NO_ERROR();
+}
+
+GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(ProgramInterfaceTestES31);
+ANGLE_INSTANTIATE_TEST_COMBINE_1(ProgramInterfaceTestES31,
+                                 ProgramInterfaceTestPrint,
+                                 testing::Bool(),
+                                 ANGLE_ALL_TEST_PLATFORMS_ES31);
 
 }  // anonymous namespace
