@@ -9,12 +9,16 @@
 
 #include "test_utils/angle_test_instantiate.h"
 
+#include <algorithm>
+#include <array>
 #include <iostream>
 #include <map>
 
 #include "angle_gl.h"
+#include "common/debug.h"
 #include "common/platform.h"
 #include "common/system_utils.h"
+#include "common/third_party/base/anglebase/no_destructor.h"
 #include "gpu_info_util/SystemInfo.h"
 #include "test_utils/angle_test_configs.h"
 #include "util/EGLWindow.h"
@@ -34,25 +38,26 @@ namespace angle
 {
 namespace
 {
-bool IsANGLEConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
+bool IsAngleEGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
 {
     std::unique_ptr<angle::Library> eglLibrary;
 
 #if defined(ANGLE_USE_UTIL_LOADER)
     eglLibrary.reset(
-        angle::OpenSharedLibrary(ANGLE_EGL_LIBRARY_NAME, angle::SearchType::ApplicationDir));
+        angle::OpenSharedLibrary(ANGLE_EGL_LIBRARY_NAME, angle::SearchType::ModuleDir));
 #endif
 
     EGLWindow *eglWindow = EGLWindow::New(param.majorVersion, param.minorVersion);
     ConfigParameters configParams;
     bool result =
-        eglWindow->initializeGL(osWindow, eglLibrary.get(), param.eglParameters, configParams);
+        eglWindow->initializeGL(osWindow, eglLibrary.get(), angle::GLESDriverType::AngleEGL,
+                                param.eglParameters, configParams);
     eglWindow->destroyGL();
     EGLWindow::Delete(&eglWindow);
     return result;
 }
 
-bool IsWGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
+bool IsSystemWGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
 {
 #if defined(ANGLE_PLATFORM_WINDOWS) && defined(ANGLE_USE_UTIL_LOADER)
     std::unique_ptr<angle::Library> openglLibrary(
@@ -61,7 +66,8 @@ bool IsWGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
     WGLWindow *wglWindow = WGLWindow::New(param.majorVersion, param.minorVersion);
     ConfigParameters configParams;
     bool result =
-        wglWindow->initializeGL(osWindow, openglLibrary.get(), param.eglParameters, configParams);
+        wglWindow->initializeGL(osWindow, openglLibrary.get(), angle::GLESDriverType::SystemWGL,
+                                param.eglParameters, configParams);
     wglWindow->destroyGL();
     WGLWindow::Delete(&wglWindow);
     return result;
@@ -70,13 +76,26 @@ bool IsWGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
 #endif  // defined(ANGLE_PLATFORM_WINDOWS) && defined(ANGLE_USE_UTIL_LOADER)
 }
 
-bool IsNativeConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
+bool IsSystemEGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
 {
-    // Not yet implemented.
-    return false;
-}
+#if defined(ANGLE_USE_UTIL_LOADER)
+    std::unique_ptr<angle::Library> eglLibrary;
 
-std::map<PlatformParameters, bool> gParamAvailabilityCache;
+    eglLibrary.reset(OpenSharedLibraryWithExtension(GetNativeEGLLibraryNameWithExtension(),
+                                                    SearchType::SystemDir));
+
+    EGLWindow *eglWindow = EGLWindow::New(param.majorVersion, param.minorVersion);
+    ConfigParameters configParams;
+    bool result =
+        eglWindow->initializeGL(osWindow, eglLibrary.get(), angle::GLESDriverType::SystemEGL,
+                                param.eglParameters, configParams);
+    eglWindow->destroyGL();
+    EGLWindow::Delete(&eglWindow);
+    return result;
+#else
+    return false;
+#endif
+}
 
 bool IsAndroidDevice(const std::string &deviceName)
 {
@@ -92,20 +111,77 @@ bool IsAndroidDevice(const std::string &deviceName)
     return false;
 }
 
-bool HasSystemVendorID(VendorID vendorID)
+bool IsAndroid9OrNewer()
+{
+    if (!IsAndroid())
+    {
+        return false;
+    }
+    SystemInfo *systemInfo = GetTestSystemInfo();
+    if (systemInfo->androidSdkLevel >= 28)
+    {
+        return true;
+    }
+    return false;
+}
+
+GPUDeviceInfo *GetActiveGPUDeviceInfo()
 {
     SystemInfo *systemInfo = GetTestSystemInfo();
     // Unfortunately sometimes GPU info collection can fail.
     if (systemInfo->gpus.empty())
     {
-        return false;
+        return nullptr;
     }
-    return systemInfo->gpus[systemInfo->activeGPUIndex].vendorId == vendorID;
+    return &systemInfo->gpus[systemInfo->activeGPUIndex];
 }
+
+bool HasSystemVendorID(VendorID vendorID)
+{
+    GPUDeviceInfo *gpuInfo = GetActiveGPUDeviceInfo();
+
+    return gpuInfo && gpuInfo->vendorId == vendorID;
+}
+
+bool HasSystemDeviceID(VendorID vendorID, DeviceID deviceID)
+{
+    GPUDeviceInfo *gpuInfo = GetActiveGPUDeviceInfo();
+
+    return gpuInfo && gpuInfo->vendorId == vendorID && gpuInfo->deviceId == deviceID;
+}
+
+using ParamAvailabilityCache = std::map<PlatformParameters, bool>;
+
+ParamAvailabilityCache &GetAvailabilityCache()
+{
+    static angle::base::NoDestructor<std::unique_ptr<ParamAvailabilityCache>>
+        sParamAvailabilityCache(new ParamAvailabilityCache());
+    return **sParamAvailabilityCache;
+}
+
+constexpr size_t kMaxConfigNameLen = 100;
+std::array<char, kMaxConfigNameLen> gSelectedConfig;
 }  // namespace
 
-std::string gSelectedConfig;
-bool gSeparateProcessPerConfig = false;
+bool gEnableANGLEPerTestCaptureLabel = false;
+
+bool IsConfigSelected()
+{
+    return gSelectedConfig[0] != 0;
+}
+
+#if !defined(ANGLE_PLATFORM_APPLE)
+// For Apple platform, see angle_test_instantiate_apple.mm
+bool IsMetalTextureSwizzleAvailable()
+{
+    return false;
+}
+
+bool IsMetalCompressedTexture3DAvailable()
+{
+    return false;
+}
+#endif
 
 SystemInfo *GetTestSystemInfo()
 {
@@ -130,7 +206,7 @@ SystemInfo *GetTestSystemInfo()
         // Print complete system info when available.
         // Seems to trip up Android test expectation parsing.
         // Also don't print info when a config is selected to prevent test spam.
-        if (!IsAndroid() && gSelectedConfig.empty())
+        if (!IsAndroid() && !IsConfigSelected())
         {
             PrintSystemInfo(*sSystemInfo);
         }
@@ -158,7 +234,26 @@ bool IsLinux()
 
 bool IsOSX()
 {
-#if defined(ANGLE_PLATFORM_APPLE)
+#if defined(ANGLE_PLATFORM_MACOS)
+    return true;
+#else
+    return false;
+#endif
+}
+
+bool IsIOS()
+{
+#if defined(ANGLE_PLATFORM_IOS)
+    return true;
+#else
+    return false;
+#endif
+}
+
+bool IsARM64()
+{
+// _M_ARM64 is Windows-specific, while __aarch64__ is for other platforms.
+#if defined(_M_ARM64) || defined(__aarch64__)
     return true;
 #else
     return false;
@@ -167,7 +262,23 @@ bool IsOSX()
 
 bool IsOzone()
 {
-#if defined(USE_OZONE)
+#if defined(USE_OZONE) && (defined(USE_X11) || defined(ANGLE_USE_VULKAN_DISPLAY))
+    // We do not have a proper support for Ozone/Linux yet. Still, we need to figure out how to
+    // properly initialize tests and differentiate between X11 and Wayland. Probably, passing a
+    // command line argument could be sufficient. At the moment, run tests only for X11 backend
+    // as we don't have Wayland support in Angle. Yes, this is a bit weird to return false, but
+    // it makes it possible to continue angle tests with X11 regardless of the Chromium config
+    // for linux, which is use_x11 && use_ozone.  Also, IsOzone is a bit vague now. It was only
+    // expected that angle could run with ozone/drm backend for ChromeOS. And returning true
+    // for desktop Linux when USE_OZONE && USE_X11 are both defined results in incorrect tests'
+    // expectations. We should also rework them and make IsOzone less vague.
+    //
+    // TODO(crbug.com/angleproject/4977): make it possible to switch between X11 and Wayland on
+    // Ozone/Linux builds. Probably, it's possible to identify the WAYLAND backend by checking
+    // the WAYLAND_DISPLAY or XDG_SESSION_TYPE env vars. And also make the IsOzone method less
+    // vague (read the comment above).
+    return false;
+#elif defined(USE_OZONE)
     return true;
 #else
     return false;
@@ -206,11 +317,6 @@ bool IsNexus5X()
     return IsAndroidDevice("Nexus 5X");
 }
 
-bool IsNexus6P()
-{
-    return IsAndroidDevice("Nexus 6P");
-}
-
 bool IsNexus9()
 {
     return IsAndroidDevice("Nexus 9");
@@ -231,6 +337,16 @@ bool IsPixel2XL()
     return IsAndroidDevice("Pixel 2 XL");
 }
 
+bool IsPixel4()
+{
+    return IsAndroidDevice("Pixel 4");
+}
+
+bool IsPixel4XL()
+{
+    return IsAndroidDevice("Pixel 4 XL");
+}
+
 bool IsNVIDIAShield()
 {
     return IsAndroidDevice("SHIELD Android TV");
@@ -241,16 +357,31 @@ bool IsIntel()
     return HasSystemVendorID(kVendorID_Intel);
 }
 
+bool IsIntelUHD630Mobile()
+{
+    return HasSystemDeviceID(kVendorID_Intel, kDeviceID_UHD630Mobile);
+}
+
 bool IsAMD()
 {
     return HasSystemVendorID(kVendorID_AMD);
 }
 
+bool IsApple()
+{
+    return HasSystemVendorID(kVendorID_Apple);
+}
+
 bool IsARM()
 {
     return HasSystemVendorID(kVendorID_ARM);
 }
 
+bool IsSwiftshaderDevice()
+{
+    return HasSystemDeviceID(kVendorID_GOOGLE, kDeviceID_Swiftshader);
+}
+
 bool IsNVIDIA()
 {
 #if defined(ANGLE_PLATFORM_ANDROID)
@@ -263,16 +394,22 @@ bool IsNVIDIA()
     return HasSystemVendorID(kVendorID_NVIDIA);
 }
 
-bool IsARM64()
+bool IsQualcomm()
 {
-#if defined(_M_ARM64)
+    return IsNexus5X() || IsNexus9() || IsPixelXL() || IsPixel2() || IsPixel2XL() || IsPixel4() ||
+           IsPixel4XL();
+}
+
+bool Is64Bit()
+{
+#if defined(ANGLE_IS_64_BIT_CPU)
     return true;
 #else
     return false;
-#endif
+#endif  // defined(ANGLE_IS_64_BIT_CPU)
 }
 
-bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters &param)
+bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters &param)
 {
     VendorID vendorID =
         systemInfo.gpus.empty() ? 0 : systemInfo.gpus[systemInfo.activeGPUIndex].vendorId;
@@ -286,6 +423,19 @@ bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters
             return true;
     }
 
+    // TODO: http://crbug.com/swiftshader/145
+    // Swiftshader does not currently have all the robustness features
+    // we need for ANGLE. In particular, it is unable to detect and recover
+    // from infinitely looping shaders. That bug is the tracker for fixing
+    // that and when resolved we can remove the following code.
+    // This test will disable tests marked with the config WithRobustness
+    // when run with the swiftshader Vulkan driver and on Android.
+    if ((param.isSwiftshader() || IsSwiftshaderDevice()) &&
+        param.eglParameters.robustness == EGL_TRUE)
+    {
+        return false;
+    }
+
     if (IsWindows())
     {
         switch (param.driver)
@@ -295,8 +445,11 @@ bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters
                 {
                     case EGL_PLATFORM_ANGLE_TYPE_D3D9_ANGLE:
                     case EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE:
-                    case EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE:
                         return true;
+                    case EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE:
+                        // Note we disable AMD OpenGL testing on Windows due to using a very old and
+                        // outdated card with many driver bugs. See http://anglebug.com/5123
+                        return !IsAMD();
                     case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
                         if (IsARM64())
                         {
@@ -326,7 +479,7 @@ bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters
     }
 
 #if defined(ANGLE_PLATFORM_APPLE)
-    if (IsOSX())
+    if (IsOSX() || IsIOS())
     {
         // We do not support non-ANGLE bindings on OSX.
         if (param.driver != GLESDriverType::AngleEGL)
@@ -344,10 +497,8 @@ bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters
                 }
                 return true;
             case EGL_PLATFORM_ANGLE_TYPE_METAL_ANGLE:
-                if (!IsMetalRendererAvailable() || IsIntel(vendorID))
+                if (!IsMetalRendererAvailable())
                 {
-                    // TODO(hqle): Intel metal tests seem to have problems. Disable for now.
-                    // http://anglebug.com/4133
                     return false;
                 }
                 return true;
@@ -396,21 +547,32 @@ bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters
         return (param.getRenderer() == EGL_PLATFORM_ANGLE_TYPE_OPENGLES_ANGLE);
     }
 
+    if (IsLinux() || IsAndroid())
+    {
+        // We do not support WGL bindings on Linux/Android. We do support system EGL.
+        switch (param.driver)
+        {
+            case GLESDriverType::SystemEGL:
+                return param.getRenderer() == EGL_PLATFORM_ANGLE_TYPE_DEFAULT_ANGLE;
+            case GLESDriverType::SystemWGL:
+                return false;
+            default:
+                break;
+        }
+    }
+
     if (IsLinux())
     {
-        // We do not support non-ANGLE bindings on Linux.
-        if (param.driver != GLESDriverType::AngleEGL)
-        {
-            return false;
-        }
+        ASSERT(param.driver == GLESDriverType::AngleEGL);
 
         // Currently we support the OpenGL and Vulkan back-ends on Linux.
         switch (param.getRenderer())
         {
             case EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE:
-            case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
-                // Note that system info collection depends on Vulkan support.
                 return true;
+            case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
+                // http://issuetracker.google.com/173004081
+                return !IsIntel() || param.eglParameters.asyncCommandQueueFeatureVulkan != EGL_TRUE;
             default:
                 return false;
         }
@@ -418,16 +580,12 @@ bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters
 
     if (IsAndroid())
     {
-        // We do not support non-ANGLE bindings on Android.
-        if (param.driver != GLESDriverType::AngleEGL)
-        {
-            return false;
-        }
+        ASSERT(param.driver == GLESDriverType::AngleEGL);
 
         // Nexus Android devices don't support backing 3.2 contexts
         if (param.eglParameters.majorVersion == 3 && param.eglParameters.minorVersion == 2)
         {
-            if (IsNexus5X() || IsNexus6P())
+            if (IsNexus5X())
             {
                 return false;
             }
@@ -437,12 +595,21 @@ bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters
         switch (param.getRenderer())
         {
             case EGL_PLATFORM_ANGLE_TYPE_OPENGLES_ANGLE:
+                return true;
             case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
                 // Swiftshader's vulkan frontend doesn't build on Android.
                 if (param.getDeviceType() == EGL_PLATFORM_ANGLE_DEVICE_TYPE_SWIFTSHADER_ANGLE)
                 {
                     return false;
                 }
+                if (!IsAndroid9OrNewer())
+                {
+                    return false;
+                }
+                if (param.eglParameters.supportsVulkanViewportFlip == EGL_FALSE)
+                {
+                    return false;
+                }
                 return true;
             default:
                 return false;
@@ -462,13 +629,13 @@ bool IsConfigSupported(const PlatformParameters &param)
         switch (param.driver)
         {
             case GLESDriverType::AngleEGL:
-                result = IsANGLEConfigSupported(param, osWindow);
+                result = IsAngleEGLConfigSupported(param, osWindow);
                 break;
             case GLESDriverType::SystemEGL:
-                result = IsNativeConfigSupported(param, osWindow);
+                result = IsSystemEGLConfigSupported(param, osWindow);
                 break;
             case GLESDriverType::SystemWGL:
-                result = IsWGLConfigSupported(param, osWindow);
+                result = IsSystemWGLConfigSupported(param, osWindow);
                 break;
         }
 
@@ -481,6 +648,15 @@ bool IsConfigSupported(const PlatformParameters &param)
 
 bool IsPlatformAvailable(const PlatformParameters &param)
 {
+    // Disable "null" device when not on ANGLE or in D3D9.
+    if (param.getDeviceType() == EGL_PLATFORM_ANGLE_DEVICE_TYPE_NULL_ANGLE)
+    {
+        if (param.driver != GLESDriverType::AngleEGL)
+            return false;
+        if (param.getRenderer() == EGL_PLATFORM_ANGLE_TYPE_D3D9_ANGLE)
+            return false;
+    }
+
     switch (param.getRenderer())
     {
         case EGL_PLATFORM_ANGLE_TYPE_DEFAULT_ANGLE:
@@ -511,15 +687,6 @@ bool IsPlatformAvailable(const PlatformParameters &param)
         case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
 #if !defined(ANGLE_ENABLE_VULKAN)
             return false;
-#elif defined(ANGLE_IS_32_BIT_CPU)
-            // http://anglebug.com/4092
-            //  Currently hitting crashes in SwS w/ 32-bit Windows so skipping for now
-            if (IsWindows() && (param.eglParameters.deviceType ==
-                                EGL_PLATFORM_ANGLE_DEVICE_TYPE_SWIFTSHADER_ANGLE))
-            {
-                return false;
-            }
-            break;
 #else
             break;
 #endif
@@ -532,10 +699,11 @@ bool IsPlatformAvailable(const PlatformParameters &param)
 #endif
 
         case EGL_PLATFORM_ANGLE_TYPE_NULL_ANGLE:
-#ifndef ANGLE_ENABLE_NULL
+#if !defined(ANGLE_ENABLE_NULL)
             return false;
-#endif
+#else
             break;
+#endif
 
         default:
             std::cout << "Unknown test platform: " << param << std::endl;
@@ -544,18 +712,18 @@ bool IsPlatformAvailable(const PlatformParameters &param)
 
     bool result = false;
 
-    auto iter = gParamAvailabilityCache.find(param);
-    if (iter != gParamAvailabilityCache.end())
+    auto iter = GetAvailabilityCache().find(param);
+    if (iter != GetAvailabilityCache().end())
     {
         result = iter->second;
     }
     else
     {
-        if (!gSelectedConfig.empty())
+        if (IsConfigSelected())
         {
             std::stringstream strstr;
             strstr << param;
-            if (strstr.str() == gSelectedConfig)
+            if (strstr.str() == std::string(gSelectedConfig.data()))
             {
                 result = true;
             }
@@ -566,7 +734,7 @@ bool IsPlatformAvailable(const PlatformParameters &param)
 
             if (systemInfo)
             {
-                result = IsConfigWhitelisted(*systemInfo, param);
+                result = IsConfigAllowlisted(*systemInfo, param);
             }
             else
             {
@@ -574,10 +742,10 @@ bool IsPlatformAvailable(const PlatformParameters &param)
             }
         }
 
-        gParamAvailabilityCache[param] = result;
+        GetAvailabilityCache()[param] = result;
 
         // Enable this unconditionally to print available platforms.
-        if (!gSelectedConfig.empty())
+        if (IsConfigSelected())
         {
             if (result)
             {
@@ -590,12 +758,6 @@ bool IsPlatformAvailable(const PlatformParameters &param)
                       << " because it is not available.\n";
         }
     }
-
-    // Disable all tests in the parent process when running child processes.
-    if (gSeparateProcessPerConfig)
-    {
-        return false;
-    }
     return result;
 }
 
@@ -603,7 +765,7 @@ std::vector<std::string> GetAvailableTestPlatformNames()
 {
     std::vector<std::string> platformNames;
 
-    for (const auto &iter : gParamAvailabilityCache)
+    for (const auto &iter : GetAvailabilityCache())
     {
         if (iter.second)
         {
@@ -618,4 +780,10 @@ std::vector<std::string> GetAvailableTestPlatformNames()
 
     return platformNames;
 }
+
+void SetSelectedConfig(const char *selectedConfig)
+{
+    gSelectedConfig.fill(0);
+    strncpy(gSelectedConfig.data(), selectedConfig, kMaxConfigNameLen - 1);
+}
 }  // namespace angle
