@@ -23,38 +23,38 @@ struct PlatformParameters;
 bool IsAndroid();
 bool IsLinux();
 bool IsOSX();
+bool IsIOS();
 bool IsOzone();
 bool IsWindows();
 bool IsWindows7();
 bool IsFuchsia();
 
+// CPU architectures
+bool IsARM64();
+
 // Android devices
 bool IsNexus5X();
-bool IsNexus6P();
 bool IsNexus9();
 bool IsPixelXL();
 bool IsPixel2();
 bool IsPixel2XL();
+bool IsPixel4();
+bool IsPixel4XL();
 bool IsNVIDIAShield();
 
 // GPU vendors.
 bool IsIntel();
 bool IsAMD();
-bool IsNVIDIA();
+bool IsApple();
 bool IsARM();
-bool IsARM64();
+bool IsNVIDIA();
+bool IsQualcomm();
 
 // GPU devices.
 bool IsSwiftshaderDevice();
+bool IsIntelUHD630Mobile();
 
-inline bool IsASan()
-{
-#if defined(ANGLE_WITH_ASAN)
-    return true;
-#else
-    return false;
-#endif  // defined(ANGLE_WITH_ASAN)
-}
+bool Is64Bit();
 
 bool IsPlatformAvailable(const PlatformParameters &param);
 
@@ -120,18 +120,28 @@ struct CombinedPrintToStringParamName
     INSTANTIATE_TEST_SUITE_P(, testName, testing::ValuesIn(::angle::FilterTestParams(valuesin)), \
                              testing::PrintToStringParamName())
 
-#define ANGLE_ALL_TEST_PLATFORMS_ES1 \
-    ES1_D3D11(), ES1_OPENGL(), ES1_OPENGLES(), ES1_VULKAN(), ES1_VULKAN_SWIFTSHADER()
+#define ANGLE_ALL_TEST_PLATFORMS_ES1                                                   \
+    ES1_D3D11(), ES1_OPENGL(), ES1_OPENGLES(), ES1_VULKAN(), ES1_VULKAN_SWIFTSHADER(), \
+        WithAsyncCommandQueueFeatureVulkan(ES1_VULKAN()),                              \
+        WithAsyncCommandQueueFeatureVulkan(ES1_VULKAN_SWIFTSHADER())
 
 #define ANGLE_ALL_TEST_PLATFORMS_ES2                                                               \
     ES2_D3D9(), ES2_D3D11(), ES2_OPENGL(), ES2_OPENGLES(), ES2_VULKAN(), ES2_VULKAN_SWIFTSHADER(), \
-        ES2_METAL()
+        ES2_METAL(), WithAsyncCommandQueueFeatureVulkan(ES2_VULKAN()),                             \
+        WithAsyncCommandQueueFeatureVulkan(ES2_VULKAN_SWIFTSHADER())
 
-#define ANGLE_ALL_TEST_PLATFORMS_ES3 \
-    ES3_D3D11(), ES3_OPENGL(), ES3_OPENGLES(), ES3_VULKAN(), ES3_VULKAN_SWIFTSHADER()
+#define ANGLE_ALL_TEST_PLATFORMS_ES3                                                   \
+    ES3_D3D11(), ES3_OPENGL(), ES3_OPENGLES(), ES3_VULKAN(), ES3_VULKAN_SWIFTSHADER(), \
+        ES3_METAL(), WithAsyncCommandQueueFeatureVulkan(ES3_VULKAN()),                 \
+        WithAsyncCommandQueueFeatureVulkan(ES3_VULKAN_SWIFTSHADER())
 
-#define ANGLE_ALL_TEST_PLATFORMS_ES31 \
-    ES31_D3D11(), ES31_OPENGL(), ES31_OPENGLES(), ES31_VULKAN(), ES31_VULKAN_SWIFTSHADER()
+#define ANGLE_ALL_TEST_PLATFORMS_ES31                                                       \
+    ES31_D3D11(), ES31_OPENGL(), ES31_OPENGLES(), ES31_VULKAN(), ES31_VULKAN_SWIFTSHADER(), \
+        WithAsyncCommandQueueFeatureVulkan(ES31_VULKAN()),                                  \
+        WithAsyncCommandQueueFeatureVulkan(ES31_VULKAN_SWIFTSHADER())
+
+#define ANGLE_ALL_TEST_PLATFORMS_ES32 \
+    ES32_VULKAN(), WithAsyncCommandQueueFeatureVulkan(ES32_VULKAN())
 
 #define ANGLE_ALL_TEST_PLATFORMS_NULL ES2_NULL(), ES3_NULL(), ES31_NULL()
 
@@ -147,15 +157,20 @@ struct CombinedPrintToStringParamName
     INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName), \
                              testing::PrintToStringParamName())
 
+#define ANGLE_INSTANTIATE_TEST_ES2_AND(testName, ...)                                          \
+    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES2, __VA_ARGS__}; \
+    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),           \
+                             testing::PrintToStringParamName())
+
 // Instantiate the test once for each GLES3 platform
 #define ANGLE_INSTANTIATE_TEST_ES3(testName)                                         \
     const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES3};    \
     INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName), \
                              testing::PrintToStringParamName())
 
-#define ANGLE_INSTANTIATE_TEST_ES3_AND(testName, extra)                                  \
-    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES3, extra}; \
-    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),     \
+#define ANGLE_INSTANTIATE_TEST_ES3_AND(testName, ...)                                          \
+    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES3, __VA_ARGS__}; \
+    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),           \
                              testing::PrintToStringParamName())
 
 // Instantiate the test once for each GLES31 platform
@@ -164,9 +179,20 @@ struct CombinedPrintToStringParamName
     INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName), \
                              testing::PrintToStringParamName())
 
-#define ANGLE_INSTANTIATE_TEST_ES31_AND(testName, extra)                                  \
-    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES31, extra}; \
-    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),      \
+#define ANGLE_INSTANTIATE_TEST_ES31_AND(testName, ...)                                          \
+    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES31, __VA_ARGS__}; \
+    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),            \
+                             testing::PrintToStringParamName())
+
+// Instantiate the test once for each GLES32 platform
+#define ANGLE_INSTANTIATE_TEST_ES32(testName)                                        \
+    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES32};   \
+    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName), \
+                             testing::PrintToStringParamName())
+
+#define ANGLE_INSTANTIATE_TEST_ES32_AND(testName, ...)                                          \
+    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES32, __VA_ARGS__}; \
+    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),            \
                              testing::PrintToStringParamName())
 
 // Multiple ES Version macros
@@ -176,10 +202,10 @@ struct CombinedPrintToStringParamName
     INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName), \
                              testing::PrintToStringParamName())
 
-#define ANGLE_INSTANTIATE_TEST_ES2_AND_ES3_AND(testName, extra)                          \
-    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES2,         \
-                                                   ANGLE_ALL_TEST_PLATFORMS_ES3, extra}; \
-    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),     \
+#define ANGLE_INSTANTIATE_TEST_ES2_AND_ES3_AND(testName, ...)                                  \
+    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES2,               \
+                                                   ANGLE_ALL_TEST_PLATFORMS_ES3, __VA_ARGS__}; \
+    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),           \
                              testing::PrintToStringParamName())
 
 #define ANGLE_INSTANTIATE_TEST_ES2_AND_ES3_AND_ES31(testName)                        \
@@ -202,6 +228,12 @@ struct CombinedPrintToStringParamName
     INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName), \
                              testing::PrintToStringParamName())
 
+#define ANGLE_INSTANTIATE_TEST_ES3_AND_ES31_AND(testName, ...)                                  \
+    const PlatformParameters testName##params[] = {ANGLE_ALL_TEST_PLATFORMS_ES3,                \
+                                                   ANGLE_ALL_TEST_PLATFORMS_ES31, __VA_ARGS__}; \
+    INSTANTIATE_TEST_SUITE_P(, testName, ANGLE_INSTANTIATE_TEST_PLATFORMS(testName),            \
+                             testing::PrintToStringParamName())
+
 // Instantiate the test for a combination of N parameters and the
 // enumeration of platforms in the extra args, similar to
 // ANGLE_INSTANTIATE_TEST.  The macros are defined only for the Ns
@@ -225,8 +257,8 @@ struct CombinedPrintToStringParamName
                                               combine1, combine2, combine3, combine4, combine5),  \
                              print)
 
-// Checks if a config is expected to be supported by checking a system-based white list.
-bool IsConfigWhitelisted(const SystemInfo &systemInfo, const PlatformParameters &param);
+// Checks if a config is expected to be supported by checking a system-based allow list.
+bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters &param);
 
 // Determines if a config is supported by trying to initialize it. Does
 // not require SystemInfo.
@@ -244,10 +276,13 @@ std::vector<std::string> GetAvailableTestPlatformNames();
 void SetSelectedConfig(const char *selectedConfig);
 bool IsConfigSelected();
 
-// Use a separate isolated process per test config. This works around
-// driver flakiness when using multiple APIs/windows/etc in the same
-// process.
-extern bool gSeparateProcessPerConfig;
+// Check whether texture swizzle is natively supported on Metal device.
+bool IsMetalTextureSwizzleAvailable();
+
+// Check whether TEXTURE_3D target is supported for compressed formats on Metal device.
+bool IsMetalCompressedTexture3DAvailable();
+
+extern bool gEnableANGLEPerTestCaptureLabel;
 
 // For use with ANGLE_INSTANTIATE_TEST_ARRAY
 template <typename ParamsT>
@@ -300,6 +335,20 @@ std::vector<ParamT> CombineWithValues(const std::vector<ParamT> &in,
 {
     return CombineWithValues(in, std::begin(modifiers), std::end(modifiers), combine);
 }
+
+template <typename ParamT, typename FilterFunc>
+std::vector<ParamT> FilterWithFunc(const std::vector<ParamT> &in, FilterFunc filter)
+{
+    std::vector<ParamT> out;
+    for (const ParamT &param : in)
+    {
+        if (filter(param))
+        {
+            out.push_back(param);
+        }
+    }
+    return out;
+}
 }  // namespace angle
 
 #define ANGLE_SKIP_TEST_IF(COND)                                  \
