@@ -13,9 +13,14 @@
 #include <string.h>
 
 #include "common/system_utils.h"
-#include "platform/Platform.h"
+#include "platform/PlatformMethods.h"
 #include "util/OSWindow.h"
 
+namespace
+{
+constexpr EGLint kDefaultSwapInterval = 1;
+}  // anonymous namespace
+
 // ConfigParameters implementation.
 ConfigParameters::ConfigParameters()
     : redBits(-1),
@@ -32,7 +37,9 @@ ConfigParameters::ConfigParameters()
       clientArraysEnabled(true),
       robustAccess(false),
       samples(-1),
-      resetStrategy(EGL_NO_RESET_NOTIFICATION_EXT)
+      resetStrategy(EGL_NO_RESET_NOTIFICATION_EXT),
+      colorSpace(EGL_COLORSPACE_LINEAR),
+      swapInterval(kDefaultSwapInterval)
 {}
 
 ConfigParameters::~ConfigParameters() = default;
@@ -90,22 +97,46 @@ EGLContext EGLWindow::getContext() const
     return mContext;
 }
 
+bool EGLWindow::isContextVersion(EGLint glesMajorVersion, EGLint glesMinorVersion) const
+{
+    return mClientMajorVersion == glesMajorVersion && mClientMinorVersion == glesMinorVersion;
+}
+
+GLWindowResult EGLWindow::initializeGLWithResult(OSWindow *osWindow,
+                                                 angle::Library *glWindowingLibrary,
+                                                 angle::GLESDriverType driverType,
+                                                 const EGLPlatformParameters &platformParams,
+                                                 const ConfigParameters &configParams)
+{
+    if (!initializeDisplay(osWindow, glWindowingLibrary, driverType, platformParams))
+    {
+        return GLWindowResult::Error;
+    }
+    GLWindowResult res = initializeSurface(osWindow, glWindowingLibrary, configParams);
+    if (res != GLWindowResult::NoError)
+    {
+        return res;
+    }
+    if (!initializeContext())
+    {
+        return GLWindowResult::Error;
+    }
+    return GLWindowResult::NoError;
+}
+
 bool EGLWindow::initializeGL(OSWindow *osWindow,
                              angle::Library *glWindowingLibrary,
+                             angle::GLESDriverType driverType,
                              const EGLPlatformParameters &platformParams,
                              const ConfigParameters &configParams)
 {
-    if (!initializeDisplay(osWindow, glWindowingLibrary, platformParams))
-        return false;
-    if (!initializeSurface(osWindow, glWindowingLibrary, configParams))
-        return false;
-    if (!initializeContext())
-        return false;
-    return true;
+    return initializeGLWithResult(osWindow, glWindowingLibrary, driverType, platformParams,
+                                  configParams) == GLWindowResult::NoError;
 }
 
 bool EGLWindow::initializeDisplay(OSWindow *osWindow,
                                   angle::Library *glWindowingLibrary,
+                                  angle::GLESDriverType driverType,
                                   const EGLPlatformParameters &params)
 {
 #if defined(ANGLE_USE_UTIL_LOADER)
@@ -113,6 +144,7 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
     glWindowingLibrary->getAs("eglGetProcAddress", &getProcAddress);
     if (!getProcAddress)
     {
+        fprintf(stderr, "Cannot load eglGetProcAddress\n");
         return false;
     }
 
@@ -156,12 +188,6 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
         displayAttributes.push_back(params.debugLayersEnabled);
     }
 
-    if (params.contextVirtualization != EGL_DONT_CARE)
-    {
-        displayAttributes.push_back(EGL_PLATFORM_ANGLE_CONTEXT_VIRTUALIZATION_ANGLE);
-        displayAttributes.push_back(params.contextVirtualization);
-    }
-
     if (params.platformMethods)
     {
         static_assert(sizeof(EGLAttrib) == sizeof(params.platformMethods),
@@ -170,48 +196,166 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
         displayAttributes.push_back(reinterpret_cast<EGLAttrib>(params.platformMethods));
     }
 
+    if (params.displayPowerPreference != EGL_DONT_CARE)
+    {
+        displayAttributes.push_back(EGL_POWER_PREFERENCE_ANGLE);
+        displayAttributes.push_back(params.displayPowerPreference);
+    }
+
     std::vector<const char *> disabledFeatureOverrides;
     std::vector<const char *> enabledFeatureOverrides;
 
-    if (params.commandGraphFeature == EGL_TRUE)
-    {
-        enabledFeatureOverrides.push_back("command_graph");
-    }
-    else if (params.commandGraphFeature == EGL_FALSE)
-    {
-        disabledFeatureOverrides.push_back("command_graph");
-    }
-
     if (params.transformFeedbackFeature == EGL_FALSE)
     {
-        disabledFeatureOverrides.push_back("supports_transform_feedback_extension");
-        disabledFeatureOverrides.push_back("emulate_transform_feedback");
+        disabledFeatureOverrides.push_back("supportsTransformFeedbackExtension");
+        disabledFeatureOverrides.push_back("supportsGeometryStreamsCapability");
+        disabledFeatureOverrides.push_back("emulateTransformFeedback");
+    }
+
+    if (params.allocateNonZeroMemoryFeature == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("allocateNonZeroMemory");
+    }
+    else if (params.allocateNonZeroMemoryFeature == EGL_FALSE)
+    {
+        disabledFeatureOverrides.push_back("allocateNonZeroMemory");
+    }
+
+    if (params.emulateCopyTexImage2DFromRenderbuffers == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("emulate_copyteximage2d_from_renderbuffers");
+    }
+
+    if (params.shaderStencilOutputFeature == EGL_FALSE)
+    {
+        disabledFeatureOverrides.push_back("has_shader_stencil_output");
+    }
+
+    if (params.genMultipleMipsPerPassFeature == EGL_FALSE)
+    {
+        disabledFeatureOverrides.push_back("gen_multiple_mips_per_pass");
+    }
+
+    if (params.supportsVulkanViewportFlip == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("supportsViewportFlip");
+    }
+    else if (params.supportsVulkanViewportFlip == EGL_FALSE)
+    {
+        disabledFeatureOverrides.push_back("supportsViewportFlip");
+    }
+
+    if (params.supportsVulkanMultiDrawIndirect == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("supportsMultiDrawIndirect");
+    }
+    else if (params.supportsVulkanMultiDrawIndirect == EGL_FALSE)
+    {
+        disabledFeatureOverrides.push_back("supportsMultiDrawIndirect");
+    }
+
+    if (params.WithVulkanPreferCPUForBufferSubData == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("preferCPUForBufferSubData");
+    }
+    else if (params.WithVulkanPreferCPUForBufferSubData == EGL_FALSE)
+    {
+        disabledFeatureOverrides.push_back("preferCPUForBufferSubData");
+    }
+
+    switch (params.emulatedPrerotation)
+    {
+        case 90:
+            enabledFeatureOverrides.push_back("emulatedPrerotation90");
+            break;
+        case 180:
+            enabledFeatureOverrides.push_back("emulatedPrerotation180");
+            break;
+        case 270:
+            enabledFeatureOverrides.push_back("emulatedPrerotation270");
+            break;
+        default:
+            break;
+    }
+
+    if (params.asyncCommandQueueFeatureVulkan == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("asyncCommandQueue");
+    }
+
+    if (params.generateSPIRVThroughGlslang == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("generateSPIRVThroughGlslang");
+    }
+
+    if (params.directMetalGeneration == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("directMetalGeneration");
+    }
+
+    if (params.hasExplicitMemBarrierFeatureMtl == EGL_FALSE)
+    {
+        disabledFeatureOverrides.push_back("has_explicit_mem_barrier_mtl");
+    }
+
+    if (params.hasCheapRenderPassFeatureMtl == EGL_FALSE)
+    {
+        disabledFeatureOverrides.push_back("has_cheap_render_pass_mtl");
+    }
+
+    if (params.forceBufferGPUStorageFeatureMtl == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("force_buffer_gpu_storage_mtl");
+    }
+
+    if (params.emulatedVAOs == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("sync_vertex_arrays_to_default");
+    }
+
+    if (params.captureLimits == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("enable_capture_limits");
+    }
+
+    if (params.forceRobustResourceInit == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("forceRobustResourceInit");
+    }
+
+    if (params.forceInitShaderVariables == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("forceInitShaderVariables");
+    }
+
+    if (params.forceVulkanFallbackFormat == EGL_TRUE)
+    {
+        enabledFeatureOverrides.push_back("forceFallbackFormat");
+    }
+
+    const bool hasFeatureControlANGLE =
+        strstr(extensionString, "EGL_ANGLE_feature_control") != nullptr;
+
+    if (!hasFeatureControlANGLE &&
+        (!enabledFeatureOverrides.empty() || !disabledFeatureOverrides.empty()))
+    {
+        fprintf(stderr, "Missing EGL_ANGLE_feature_control.\n");
+        destroyGL();
+        return false;
     }
 
     if (!disabledFeatureOverrides.empty())
     {
-        if (strstr(extensionString, "EGL_ANGLE_feature_control") == nullptr)
-        {
-            std::cout << "Missing EGL_ANGLE_feature_control.\n";
-            destroyGL();
-            return false;
-        }
-
         disabledFeatureOverrides.push_back(nullptr);
 
         displayAttributes.push_back(EGL_FEATURE_OVERRIDES_DISABLED_ANGLE);
         displayAttributes.push_back(reinterpret_cast<EGLAttrib>(disabledFeatureOverrides.data()));
     }
 
-    if (!enabledFeatureOverrides.empty())
+    if (hasFeatureControlANGLE)
     {
-        if (strstr(extensionString, "EGL_ANGLE_feature_control") == nullptr)
-        {
-            std::cout << "Missing EGL_ANGLE_feature_control.\n";
-            destroyGL();
-            return false;
-        }
-
+        // Always enable exposeNonConformantExtensionsAndVersions in ANGLE tests.
+        enabledFeatureOverrides.push_back("exposeNonConformantExtensionsAndVersions");
         enabledFeatureOverrides.push_back(nullptr);
 
         displayAttributes.push_back(EGL_FEATURE_OVERRIDES_ENABLED_ANGLE);
@@ -220,17 +364,31 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
 
     displayAttributes.push_back(EGL_NONE);
 
-    mDisplay = eglGetPlatformDisplay(EGL_PLATFORM_ANGLE_ANGLE,
-                                     reinterpret_cast<void *>(osWindow->getNativeDisplay()),
-                                     &displayAttributes[0]);
+    if (driverType == angle::GLESDriverType::SystemWGL)
+        return false;
+
+    if (driverType == angle::GLESDriverType::AngleEGL &&
+        strstr(extensionString, "EGL_ANGLE_platform_angle"))
+    {
+        mDisplay = eglGetPlatformDisplay(EGL_PLATFORM_ANGLE_ANGLE,
+                                         reinterpret_cast<void *>(osWindow->getNativeDisplay()),
+                                         &displayAttributes[0]);
+    }
+    else
+    {
+        mDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
+    }
+
     if (mDisplay == EGL_NO_DISPLAY)
     {
+        fprintf(stderr, "Failed to get display: 0x%X\n", eglGetError());
         destroyGL();
         return false;
     }
 
     if (eglInitialize(mDisplay, &mEGLMajorVersion, &mEGLMinorVersion) == EGL_FALSE)
     {
+        fprintf(stderr, "eglInitialize failed: 0x%X\n", eglGetError());
         destroyGL();
         return false;
     }
@@ -239,14 +397,16 @@ bool EGLWindow::initializeDisplay(OSWindow *osWindow,
     return true;
 }
 
-bool EGLWindow::initializeSurface(OSWindow *osWindow,
-                                  angle::Library *glWindowingLibrary,
-                                  const ConfigParameters &params)
+GLWindowResult EGLWindow::initializeSurface(OSWindow *osWindow,
+                                            angle::Library *glWindowingLibrary,
+                                            const ConfigParameters &params)
 {
     mConfigParams                 = params;
     const char *displayExtensions = eglQueryString(mDisplay, EGL_EXTENSIONS);
 
     std::vector<EGLint> configAttributes = {
+        EGL_SURFACE_TYPE,
+        EGL_WINDOW_BIT,
         EGL_RED_SIZE,
         (mConfigParams.redBits >= 0) ? mConfigParams.redBits : EGL_DONT_CARE,
         EGL_GREEN_SIZE,
@@ -269,8 +429,9 @@ bool EGLWindow::initializeSurface(OSWindow *osWindow,
     bool hasPixelFormatFloat = strstr(displayExtensions, "EGL_EXT_pixel_format_float") != nullptr;
     if (!hasPixelFormatFloat && mConfigParams.componentType != EGL_COLOR_COMPONENT_TYPE_FIXED_EXT)
     {
+        fprintf(stderr, "Mising EGL_EXT_pixel_format_float.\n");
         destroyGL();
-        return false;
+        return GLWindowResult::Error;
     }
     if (hasPixelFormatFloat)
     {
@@ -283,9 +444,9 @@ bool EGLWindow::initializeSurface(OSWindow *osWindow,
 
     if (!FindEGLConfig(mDisplay, configAttributes.data(), &mConfig))
     {
-        std::cout << "Could not find a suitable EGL config!" << std::endl;
+        fprintf(stderr, "Could not find a suitable EGL config!\n");
         destroyGL();
-        return false;
+        return GLWindowResult::Error;
     }
 
     eglGetConfigAttrib(mDisplay, mConfig, EGL_RED_SIZE, &mConfigParams.redBits);
@@ -312,6 +473,27 @@ bool EGLWindow::initializeSurface(OSWindow *osWindow,
                                                                              : EGL_FALSE);
     }
 
+    bool hasGLColorSpace = strstr(displayExtensions, "EGL_KHR_gl_colorspace") != nullptr;
+    if (!hasGLColorSpace && mConfigParams.colorSpace != EGL_COLORSPACE_LINEAR)
+    {
+        fprintf(stderr, "Mising EGL_KHR_gl_colorspace.\n");
+        destroyGL();
+        return GLWindowResult::NoColorspaceSupport;
+    }
+    if (hasGLColorSpace)
+    {
+        surfaceAttributes.push_back(EGL_GL_COLORSPACE_KHR);
+        surfaceAttributes.push_back(mConfigParams.colorSpace);
+    }
+
+    bool hasCreateSurfaceSwapInterval =
+        strstr(displayExtensions, "EGL_ANGLE_create_surface_swap_interval") != nullptr;
+    if (hasCreateSurfaceSwapInterval && mConfigParams.swapInterval != kDefaultSwapInterval)
+    {
+        surfaceAttributes.push_back(EGL_SWAP_INTERVAL_ANGLE);
+        surfaceAttributes.push_back(mConfigParams.swapInterval);
+    }
+
     surfaceAttributes.push_back(EGL_NONE);
 
     osWindow->resetNativeWindow();
@@ -320,18 +502,30 @@ bool EGLWindow::initializeSurface(OSWindow *osWindow,
                                       &surfaceAttributes[0]);
     if (eglGetError() != EGL_SUCCESS || (mSurface == EGL_NO_SURFACE))
     {
+        fprintf(stderr, "eglCreateWindowSurface failed: 0x%X\n", eglGetError());
         destroyGL();
-        return false;
+        return GLWindowResult::Error;
     }
 
 #if defined(ANGLE_USE_UTIL_LOADER)
     angle::LoadGLES(eglGetProcAddress);
 #endif  // defined(ANGLE_USE_UTIL_LOADER)
 
-    return true;
+    return GLWindowResult::NoError;
 }
 
-EGLContext EGLWindow::createContext(EGLContext share) const
+GLWindowContext EGLWindow::getCurrentContextGeneric()
+{
+    return reinterpret_cast<GLWindowContext>(mContext);
+}
+
+GLWindowContext EGLWindow::createContextGeneric(GLWindowContext share)
+{
+    EGLContext shareContext = reinterpret_cast<EGLContext>(share);
+    return reinterpret_cast<GLWindowContext>(createContext(shareContext, nullptr));
+}
+
+EGLContext EGLWindow::createContext(EGLContext share, EGLint *extraAttributes)
 {
     const char *displayExtensions = eglQueryString(mDisplay, EGL_EXTENSIONS);
 
@@ -340,7 +534,15 @@ EGLContext EGLWindow::createContext(EGLContext share) const
     if (mClientMajorVersion > 2 && !(mEGLMajorVersion > 1 || mEGLMinorVersion >= 5) &&
         !hasKHRCreateContext)
     {
-        std::cerr << "EGL_KHR_create_context incompatibility.\n";
+        fprintf(stderr, "EGL_KHR_create_context incompatibility.\n");
+        return EGL_NO_CONTEXT;
+    }
+
+    // EGL_CONTEXT_OPENGL_DEBUG is only valid as of EGL 1.5.
+    bool hasDebug = mEGLMinorVersion >= 5;
+    if (mConfigParams.debug && !hasDebug)
+    {
+        fprintf(stderr, "EGL 1.5 is required for EGL_CONTEXT_OPENGL_DEBUG.\n");
         return EGL_NO_CONTEXT;
     }
 
@@ -348,7 +550,7 @@ EGLContext EGLWindow::createContext(EGLContext share) const
         strstr(displayExtensions, "EGL_ANGLE_create_context_webgl_compatibility") != nullptr;
     if (mConfigParams.webGLCompatibility.valid() && !hasWebGLCompatibility)
     {
-        std::cerr << "EGL_ANGLE_create_context_webgl_compatibility missing.\n";
+        fprintf(stderr, "EGL_ANGLE_create_context_webgl_compatibility missing.\n");
         return EGL_NO_CONTEXT;
     }
 
@@ -356,7 +558,7 @@ EGLContext EGLWindow::createContext(EGLContext share) const
         strstr(displayExtensions, "EGL_ANGLE_create_context_extensions_enabled") != nullptr;
     if (mConfigParams.extensionsEnabled.valid() && !hasCreateContextExtensionsEnabled)
     {
-        std::cerr << "EGL_ANGLE_create_context_extensions_enabled missing.\n";
+        fprintf(stderr, "EGL_ANGLE_create_context_extensions_enabled missing.\n");
         return EGL_NO_CONTEXT;
     }
 
@@ -365,7 +567,7 @@ EGLContext EGLWindow::createContext(EGLContext share) const
          mConfigParams.resetStrategy != EGL_NO_RESET_NOTIFICATION_EXT) &&
         !hasRobustness)
     {
-        std::cerr << "EGL_EXT_create_context_robustness missing.\n";
+        fprintf(stderr, "EGL_EXT_create_context_robustness missing.\n");
         return EGL_NO_CONTEXT;
     }
 
@@ -373,7 +575,7 @@ EGLContext EGLWindow::createContext(EGLContext share) const
         strstr(displayExtensions, "EGL_CHROMIUM_create_context_bind_generates_resource") != nullptr;
     if (!mConfigParams.bindGeneratesResource && !hasBindGeneratesResource)
     {
-        std::cerr << "EGL_CHROMIUM_create_context_bind_generates_resource missing.\n";
+        fprintf(stderr, "EGL_CHROMIUM_create_context_bind_generates_resource missing.\n");
         return EGL_NO_CONTEXT;
     }
 
@@ -382,7 +584,7 @@ EGLContext EGLWindow::createContext(EGLContext share) const
     if (!mConfigParams.clientArraysEnabled && !hasClientArraysExtension)
     {
         // Non-default state requested without the extension present
-        std::cerr << "EGL_ANGLE_create_context_client_arrays missing.\n";
+        fprintf(stderr, "EGL_ANGLE_create_context_client_arrays missing.\n");
         return EGL_NO_CONTEXT;
     }
 
@@ -390,26 +592,33 @@ EGLContext EGLWindow::createContext(EGLContext share) const
         strstr(displayExtensions, "EGL_ANGLE_program_cache_control ") != nullptr;
     if (mConfigParams.contextProgramCacheEnabled.valid() && !hasProgramCacheControlExtension)
     {
-        std::cerr << "EGL_ANGLE_program_cache_control missing.\n";
+        fprintf(stderr, "EGL_ANGLE_program_cache_control missing.\n");
         return EGL_NO_CONTEXT;
     }
 
-    bool hasBackwardsCompatibleContextExtension =
-        strstr(displayExtensions, "EGL_ANGLE_create_context_backwards_compatible") != nullptr;
-    if (!hasProgramCacheControlExtension)
+    bool hasKHRCreateContextNoError =
+        strstr(displayExtensions, "EGL_KHR_create_context_no_error") != nullptr;
+    if (mConfigParams.noError && !hasKHRCreateContextNoError)
     {
-        std::cerr << "EGL_ANGLE_create_context_backwards_compatible missing.\n";
+        fprintf(stderr, "EGL_KHR_create_context_no_error missing.\n");
         return EGL_NO_CONTEXT;
     }
 
     eglBindAPI(EGL_OPENGL_ES_API);
     if (eglGetError() != EGL_SUCCESS)
     {
-        std::cerr << "Error on eglBindAPI.\n";
+        fprintf(stderr, "Error on eglBindAPI.\n");
         return EGL_NO_CONTEXT;
     }
 
     std::vector<EGLint> contextAttributes;
+    for (EGLint *extraAttrib = extraAttributes;
+         extraAttrib != nullptr && extraAttrib[0] != EGL_NONE; extraAttrib += 2)
+    {
+        contextAttributes.push_back(extraAttrib[0]);
+        contextAttributes.push_back(extraAttrib[1]);
+    }
+
     if (hasKHRCreateContext)
     {
         contextAttributes.push_back(EGL_CONTEXT_MAJOR_VERSION_KHR);
@@ -418,15 +627,21 @@ EGLContext EGLWindow::createContext(EGLContext share) const
         contextAttributes.push_back(EGL_CONTEXT_MINOR_VERSION_KHR);
         contextAttributes.push_back(mClientMinorVersion);
 
-        contextAttributes.push_back(EGL_CONTEXT_OPENGL_DEBUG);
-        contextAttributes.push_back(mConfigParams.debug ? EGL_TRUE : EGL_FALSE);
+        // Note that the Android loader currently doesn't handle this flag despite reporting 1.5.
+        // Work around this by only using the debug bit when we request a debug context.
+        if (hasDebug && mConfigParams.debug)
+        {
+            contextAttributes.push_back(EGL_CONTEXT_OPENGL_DEBUG);
+            contextAttributes.push_back(mConfigParams.debug ? EGL_TRUE : EGL_FALSE);
+        }
 
-        // TODO(jmadill): Check for the extension string.
-        // bool hasKHRCreateContextNoError = strstr(displayExtensions,
-        // "EGL_KHR_create_context_no_error") != nullptr;
-
-        contextAttributes.push_back(EGL_CONTEXT_OPENGL_NO_ERROR_KHR);
-        contextAttributes.push_back(mConfigParams.noError ? EGL_TRUE : EGL_FALSE);
+        // TODO (http://anglebug.com/5809)
+        // Mesa does not allow EGL_CONTEXT_OPENGL_NO_ERROR_KHR for GLES1.
+        if (hasKHRCreateContextNoError && mConfigParams.noError)
+        {
+            contextAttributes.push_back(EGL_CONTEXT_OPENGL_NO_ERROR_KHR);
+            contextAttributes.push_back(mConfigParams.noError ? EGL_TRUE : EGL_FALSE);
+        }
 
         if (mConfigParams.webGLCompatibility.valid())
         {
@@ -470,6 +685,8 @@ EGLContext EGLWindow::createContext(EGLContext share) const
                 mConfigParams.contextProgramCacheEnabled.value() ? EGL_TRUE : EGL_FALSE);
         }
 
+        bool hasBackwardsCompatibleContextExtension =
+            strstr(displayExtensions, "EGL_ANGLE_create_context_backwards_compatible") != nullptr;
         if (hasBackwardsCompatibleContextExtension)
         {
             // Always request the exact context version that the config wants
@@ -489,9 +706,9 @@ EGLContext EGLWindow::createContext(EGLContext share) const
     contextAttributes.push_back(EGL_NONE);
 
     EGLContext context = eglCreateContext(mDisplay, mConfig, share, &contextAttributes[0]);
-    if (eglGetError() != EGL_SUCCESS)
+    if (context == EGL_NO_CONTEXT)
     {
-        std::cerr << "Error on eglCreateContext.\n";
+        fprintf(stderr, "eglCreateContext failed: 0x%X\n", eglGetError());
         return EGL_NO_CONTEXT;
     }
 
@@ -500,7 +717,7 @@ EGLContext EGLWindow::createContext(EGLContext share) const
 
 bool EGLWindow::initializeContext()
 {
-    mContext = createContext(EGL_NO_CONTEXT);
+    mContext = createContext(EGL_NO_CONTEXT, nullptr);
     if (mContext == EGL_NO_CONTEXT)
     {
         destroyGL();
@@ -557,7 +774,9 @@ bool EGLWindow::isGLInitialized() const
 }
 
 // Find an EGLConfig that is an exact match for the specified attributes. EGL_FALSE is returned if
-// the EGLConfig is found.  This indicates that the EGLConfig is not supported.
+// the EGLConfig is found.  This indicates that the EGLConfig is not supported.  Surface type is
+// special-cased as it's possible for a config to return support for both EGL_WINDOW_BIT and
+// EGL_PBUFFER_BIT even though only one of them is requested.
 EGLBoolean EGLWindow::FindEGLConfig(EGLDisplay dpy, const EGLint *attrib_list, EGLConfig *config)
 {
     EGLint numConfigs = 0;
@@ -577,7 +796,9 @@ EGLBoolean EGLWindow::FindEGLConfig(EGLDisplay dpy, const EGLint *attrib_list, E
 
             EGLint actualValue = EGL_DONT_CARE;
             eglGetConfigAttrib(dpy, allConfigs[i], curAttrib[0], &actualValue);
-            if (curAttrib[1] != actualValue)
+            if ((curAttrib[0] == EGL_SURFACE_TYPE &&
+                 (curAttrib[1] & actualValue) != curAttrib[1]) ||
+                (curAttrib[0] != EGL_SURFACE_TYPE && curAttrib[1] != actualValue))
             {
                 matchFound = false;
                 break;
@@ -594,13 +815,27 @@ EGLBoolean EGLWindow::FindEGLConfig(EGLDisplay dpy, const EGLint *attrib_list, E
     return EGL_FALSE;
 }
 
+bool EGLWindow::makeCurrentGeneric(GLWindowContext context)
+{
+    EGLContext eglContext = reinterpret_cast<EGLContext>(context);
+    return makeCurrent(eglContext);
+}
+
 bool EGLWindow::makeCurrent()
 {
-    if (eglMakeCurrent(mDisplay, mSurface, mSurface, mContext) == EGL_FALSE ||
-        eglGetError() != EGL_SUCCESS)
+    return makeCurrent(mContext);
+}
+
+bool EGLWindow::makeCurrent(EGLContext context)
+{
+    if (isGLInitialized())
     {
-        std::cerr << "Error during eglMakeCurrent.\n";
-        return false;
+        if (eglMakeCurrent(mDisplay, mSurface, mSurface, context) == EGL_FALSE ||
+            eglGetError() != EGL_SUCCESS)
+        {
+            fprintf(stderr, "Error during eglMakeCurrent.\n");
+            return false;
+        }
     }
 
     return true;
@@ -610,7 +845,7 @@ bool EGLWindow::setSwapInterval(EGLint swapInterval)
 {
     if (eglSwapInterval(mDisplay, swapInterval) == EGL_FALSE || eglGetError() != EGL_SUCCESS)
     {
-        std::cerr << "Error during eglSwapInterval.\n";
+        fprintf(stderr, "Error during eglSwapInterval.\n");
         return false;
     }
 
@@ -622,11 +857,9 @@ bool EGLWindow::hasError() const
     return eglGetError() != EGL_SUCCESS;
 }
 
-bool CheckExtensionExists(const char *allExtensions, const std::string &extName)
+angle::GenericProc EGLWindow::getProcAddress(const char *name)
 {
-    const std::string paddedExtensions = std::string(" ") + allExtensions + std::string(" ");
-    return paddedExtensions.find(std::string(" ") + extName + std::string(" ")) !=
-           std::string::npos;
+    return eglGetProcAddress(name);
 }
 
 // static
