@@ -59,30 +59,25 @@ enum TimeType {
 struct LocalTimeOffset {
     WTF_MAKE_STRUCT_FAST_ALLOCATED;
 
-    LocalTimeOffset()
-        : isDST(false)
-        , offset(0)
-    {
-    }
-
-    LocalTimeOffset(bool isDST, int offset)
+    LocalTimeOffset() = default;
+    constexpr LocalTimeOffset(bool isDST, int offset)
         : isDST(isDST)
         , offset(offset)
     {
     }
 
-    bool operator==(const LocalTimeOffset& other)
+    bool operator==(const LocalTimeOffset& other) const
     {
         return isDST == other.isDST && offset == other.offset;
     }
 
-    bool operator!=(const LocalTimeOffset& other)
+    bool operator!=(const LocalTimeOffset& other) const
     {
         return isDST != other.isDST || offset != other.offset;
     }
 
-    bool isDST;
-    int offset;
+    bool isDST { false };
+    int offset { 0 };
 };
 
 void initializeDates();
@@ -167,7 +162,7 @@ inline double daysFrom1970ToYear(int year)
     static constexpr int excludedLeapDaysBefore1971By100Rule = 1970 / 100;
     static constexpr int leapDaysBefore1971By400Rule = 1970 / 400;
 
-    const double yearMinusOne = year - 1;
+    const double yearMinusOne = static_cast<double>(year) - 1;
     const double yearsToAddBy4Rule = floor(yearMinusOne / 4.0) - leapDaysBefore1971By4Rule;
     const double yearsToExcludeBy100Rule = floor(yearMinusOne / 100.0) - excludedLeapDaysBefore1971By100Rule;
     const double yearsToAddBy400Rule = floor(yearMinusOne / 400.0) - leapDaysBefore1971By400Rule;
@@ -223,7 +218,8 @@ inline int dayInYear(int year, int month, int day)
 
 inline int dayInYear(double ms, int year)
 {
-    return static_cast<int>(msToDays(ms) - daysFrom1970ToYear(year));
+    double result = msToDays(ms) - daysFrom1970ToYear(year);
+    return std::isnan(result) ? 0 : static_cast<int>(result);
 }
 
 inline int dayInYear(TimeClippedPositiveMilliseconds ms, int year)
@@ -249,7 +245,10 @@ inline double dateToDaysFrom1970(int year, int month, int day)
 
 inline int msToYear(double ms)
 {
-    int approxYear = static_cast<int>(floor(ms / (msPerDay * 365.2425)) + 1970);
+    double msAsYears = std::floor(ms / (msPerDay * 365.2425));
+    if (std::isnan(msAsYears))
+        msAsYears = 0;
+    int approxYear = static_cast<int>(msAsYears + 1970);
     double msFromApproxYearTo1970 = msPerDay * daysFrom1970ToYear(approxYear);
     if (msFromApproxYearTo1970 > ms)
         return approxYear - 1;
@@ -389,6 +388,11 @@ inline int dayInMonthFromDayInYear(int dayInYear, bool leapYear)
     return d - step;
 }
 
+inline double timeToMS(double hour, double min, double sec, double ms)
+{
+    return (((hour * WTF::minutesPerHour + min) * WTF::secondsPerMinute + sec) * WTF::msPerSecond + ms);
+}
+
 // Returns combined offset in millisecond (UTC + DST).
 WTF_EXPORT_PRIVATE LocalTimeOffset calculateLocalTimeOffset(double utcInMilliseconds, TimeType = UTCTime);
 
@@ -411,6 +415,7 @@ using WTF::msToHours;
 using WTF::secondsPerDay;
 using WTF::secondsPerMinute;
 using WTF::parseDateFromNullTerminatedCharacters;
+using WTF::timeToMS;
 using WTF::makeRFC2822DateString;
 using WTF::LocalTimeOffset;
 using WTF::calculateLocalTimeOffset;
