@@ -26,6 +26,7 @@
 #include "CommonVM.h"
 #include "ContentSecurityPolicy.h"
 #include "DOMWrapperWorld.h"
+#include "DocumentInlines.h"
 #include "DocumentLoader.h"
 #include "Event.h"
 #include "Frame.h"
@@ -42,7 +43,6 @@
 #include "Logging.h"
 #include "ModuleFetchFailureKind.h"
 #include "ModuleFetchParameters.h"
-#include "NP_jsobject.h"
 #include "Page.h"
 #include "PageConsoleClient.h"
 #include "PageGroup.h"
@@ -58,7 +58,6 @@
 #include "UserGestureIndicator.h"
 #include "WebCoreJITOperations.h"
 #include "WebCoreJSClientData.h"
-#include "npruntime_impl.h"
 #include "runtime_root.h"
 #include <JavaScriptCore/Debugger.h>
 #include <JavaScriptCore/Heap.h>
@@ -78,7 +77,7 @@
 #include <wtf/Threading.h>
 #include <wtf/text/TextPosition.h>
 
-#define RELEASE_LOG_ERROR_IF_ALLOWED(channel, fmt, ...) RELEASE_LOG_ERROR_IF(m_frame.isAlwaysOnLoggingAllowed(), channel, "%p - ScriptController::" fmt, this, ##__VA_ARGS__)
+#define SCRIPTCONTROLLER_RELEASE_LOG_ERROR(channel, fmt, ...) RELEASE_LOG_ERROR(channel, "%p - ScriptController::" fmt, this, ##__VA_ARGS__)
 
 namespace WebCore {
 using namespace JSC;
@@ -96,9 +95,6 @@ ScriptController::ScriptController(Frame& frame)
     : m_frame(frame)
     , m_sourceURL(0)
     , m_paused(false)
-#if ENABLE(NETSCAPE_PLUGIN_API)
-    , m_windowScriptNPObject(0)
-#endif
 #if PLATFORM(COCOA)
     , m_windowScriptObject(0)
 #endif
@@ -124,7 +120,13 @@ JSC::JSValue ScriptController::evaluateInWorldIgnoringException(const ScriptSour
 
 ValueOrException ScriptController::evaluateInWorld(const ScriptSourceCode& sourceCode, DOMWrapperWorld& world)
 {
-    JSLockHolder lock(world.vm());
+    auto& vm = world.vm();
+    JSLockHolder lock(vm);
+
+    if (vm.hasPendingTerminationException()) {
+        ExceptionDetails details;
+        return makeUnexpected(details);
+    }
 
     const SourceCode& jsSourceCode = sourceCode.jsSourceCode();
     const URL& sourceURL = jsSourceCode.provider()->sourceOrigin().url();
@@ -149,10 +151,10 @@ ValueOrException ScriptController::evaluateInWorld(const ScriptSourceCode& sourc
 
     InspectorInstrumentation::didEvaluateScript(m_frame);
 
-    Optional<ExceptionDetails> optionalDetails;
+    std::optional<ExceptionDetails> optionalDetails;
     if (evaluationException) {
         ExceptionDetails details;
-        reportException(&globalObject, evaluationException, sourceCode.cachedScript(), &details);
+        reportException(&globalObject, evaluationException, sourceCode.cachedScript(), false, &details);
         optionalDetails = WTFMove(details);
     }
 
@@ -216,7 +218,8 @@ JSC::JSValue ScriptController::linkAndEvaluateModuleScriptInWorld(LoadableModule
     if (evaluationException) {
         // FIXME: Give a chance to dump the stack trace if the "crossorigin" attribute allows.
         // https://bugs.webkit.org/show_bug.cgi?id=164539
-        reportException(&lexicalGlobalObject, evaluationException, nullptr);
+        constexpr bool fromModule = true;
+        reportException(&lexicalGlobalObject, evaluationException, nullptr, fromModule);
         return jsUndefined();
     }
     return returnValue;
@@ -276,7 +279,7 @@ void ScriptController::initScriptForWindowProxy(JSWindowProxy& windowProxy)
     if (Page* page = m_frame.page()) {
         windowProxy.attachDebugger(page->debugger());
         windowProxy.window()->setProfileGroup(page->group().identifier());
-        windowProxy.window()->setConsoleClient(&page->console());
+        windowProxy.window()->setConsoleClient(page->console());
     }
 
     m_frame.loader().dispatchDidClearWindowObjectInWorld(world);
@@ -313,15 +316,31 @@ void ScriptController::setupModuleScriptHandlers(LoadableModuleScript& moduleScr
     auto& rejectHandler = *JSNativeStdFunction::create(lexicalGlobalObject.vm(), proxy.window(), 1, String(), [moduleScript](JSGlobalObject* globalObject, CallFrame* callFrame) {
         VM& vm = globalObject->vm();
         JSValue errorValue = callFrame->argument(0);
+        auto scope = DECLARE_CATCH_SCOPE(vm);
         if (errorValue.isObject()) {
             auto* object = JSC::asObject(errorValue);
             if (JSValue failureKindValue = object->getDirect(vm, static_cast<JSVMClientData&>(*vm.clientData).builtinNames().failureKindPrivateName())) {
                 // This is host propagated error in the module loader pipeline.
                 switch (static_cast<ModuleFetchFailureKind>(failureKindValue.asInt32())) {
-                case ModuleFetchFailureKind::WasErrored:
+                case ModuleFetchFailureKind::WasPropagatedError:
                     moduleScript->notifyLoadFailed(LoadableScript::Error {
                         LoadableScript::ErrorType::CachedScript,
-                        WTF::nullopt
+                        std::nullopt,
+                        std::nullopt
+                    });
+                    break;
+                // For a fetch error that was not propagated from further in the
+                // pipeline, we include the console error message but do not
+                // include an error value as it should not be reported.
+                case ModuleFetchFailureKind::WasFetchError:
+                    moduleScript->notifyLoadFailed(LoadableScript::Error {
+                        LoadableScript::ErrorType::CachedScript,
+                        LoadableScript::ConsoleMessage {
+                            MessageSource::JS,
+                            MessageLevel::Error,
+                            retrieveErrorMessage(*globalObject, vm, errorValue, scope),
+                        },
+                        std::nullopt
                     });
                     break;
                 case ModuleFetchFailureKind::WasCanceled:
@@ -332,14 +351,16 @@ void ScriptController::setupModuleScriptHandlers(LoadableModuleScript& moduleScr
             }
         }
 
-        auto scope = DECLARE_CATCH_SCOPE(vm);
         moduleScript->notifyLoadFailed(LoadableScript::Error {
             LoadableScript::ErrorType::CachedScript,
             LoadableScript::ConsoleMessage {
                 MessageSource::JS,
                 MessageLevel::Error,
                 retrieveErrorMessage(*globalObject, vm, errorValue, scope),
-            }
+            },
+            // The error value is included so that it can be reported to the
+            // appropriate global object.
+            errorValue
         });
         return JSValue::encode(jsUndefined());
     });
@@ -470,29 +491,6 @@ void ScriptController::collectIsolatedContexts(Vector<std::pair<JSC::JSGlobalObj
     }
 }
 
-#if ENABLE(NETSCAPE_PLUGIN_API)
-NPObject* ScriptController::windowScriptNPObject()
-{
-    if (!m_windowScriptNPObject) {
-        JSLockHolder lock(commonVM());
-        if (canExecuteScripts(NotAboutToExecuteScript)) {
-            // JavaScript is enabled, so there is a JavaScript window object.
-            // Return an NPObject bound to the window object.
-            auto* window = jsWindowProxy(pluginWorld()).window();
-            ASSERT(window);
-            Bindings::RootObject* root = bindingRootObject();
-            m_windowScriptNPObject = _NPN_CreateScriptObject(0, window, root);
-        } else {
-            // JavaScript is not enabled, so we cannot bind the NPObject to the JavaScript window object.
-            // Instead, we create an NPObject of a different class, one which is not bound to a JavaScript object.
-            m_windowScriptNPObject = _NPN_CreateNoScriptObject();
-        }
-    }
-
-    return m_windowScriptNPObject;
-}
-#endif
-
 #if !PLATFORM(COCOA)
 RefPtr<JSC::Bindings::Instance> ScriptController::createScriptInstanceForWidget(Widget* widget)
 {
@@ -556,16 +554,6 @@ void ScriptController::clearScriptObjects()
         m_bindingRootObject->invalidate();
         m_bindingRootObject = nullptr;
     }
-
-#if ENABLE(NETSCAPE_PLUGIN_API)
-    if (m_windowScriptNPObject) {
-        // Call _NPN_DeallocateObject() instead of _NPN_ReleaseObject() so that we don't leak if a plugin fails to release the window
-        // script object properly.
-        // This shouldn't cause any problems for plugins since they should have already been stopped and destroyed at this point.
-        _NPN_DeallocateObject(m_windowScriptNPObject);
-        m_windowScriptNPObject = nullptr;
-    }
-#endif
 }
 
 JSC::JSValue ScriptController::executeScriptIgnoringException(const String& script, bool forceUserGesture)
@@ -575,7 +563,7 @@ JSC::JSValue ScriptController::executeScriptIgnoringException(const String& scri
 
 JSC::JSValue ScriptController::executeScriptInWorldIgnoringException(DOMWrapperWorld& world, const String& script, bool forceUserGesture)
 {
-    auto result = executeScriptInWorld(world, { script, URL { }, false, WTF::nullopt, forceUserGesture });
+    auto result = executeScriptInWorld(world, { script, URL { }, false, std::nullopt, forceUserGesture });
     return result ? result.value() : JSC::JSValue { };
 }
 
@@ -585,13 +573,13 @@ ValueOrException ScriptController::executeScriptInWorld(DOMWrapperWorld& world,
     if (m_frame.loader().client().shouldEnableInAppBrowserPrivacyProtections()) {
         if (auto* document = m_frame.document())
             document->addConsoleMessage(MessageSource::Security, MessageLevel::Warning, "Ignoring user script injection for non-app bound domain.");
-        RELEASE_LOG_ERROR_IF_ALLOWED(Loading, "executeScriptInWorld: Ignoring user script injection for non app-bound domain");
+        SCRIPTCONTROLLER_RELEASE_LOG_ERROR(Loading, "executeScriptInWorld: Ignoring user script injection for non app-bound domain");
         return makeUnexpected(ExceptionDetails { "Ignoring user script injection for non-app bound domain"_s });
     }
     m_frame.loader().client().notifyPageOfAppBoundBehavior();
 #endif
 
-    UserGestureIndicator gestureIndicator(parameters.forceUserGesture == ForceUserGesture::Yes ? Optional<ProcessingUserGestureState>(ProcessingUserGesture) : WTF::nullopt, m_frame.document());
+    UserGestureIndicator gestureIndicator(parameters.forceUserGesture == ForceUserGesture::Yes ? std::optional<ProcessingUserGestureState>(ProcessingUserGesture) : std::nullopt, m_frame.document());
 
     if (!canExecuteScripts(AboutToExecuteScript) || isPaused())
         return makeUnexpected(ExceptionDetails { "Cannot execute JavaScript in this document"_s });
@@ -659,7 +647,7 @@ ValueOrException ScriptController::callInWorld(RunJavaScriptParameters&& paramet
     InspectorInstrumentation::willEvaluateScript(m_frame, sourceURL.string(), sourceCode.startLine(), sourceCode.startColumn());
 
     NakedPtr<JSC::Exception> evaluationException;
-    Optional<ExceptionDetails> optionalDetails;
+    std::optional<ExceptionDetails> optionalDetails;
     JSValue returnValue;
     do {
         JSValue functionObject = JSExecState::profiledEvaluate(&globalObject, JSC::ProfilingReason::Other, jsSourceCode, &proxy, evaluationException);
@@ -687,7 +675,7 @@ ValueOrException ScriptController::callInWorld(RunJavaScriptParameters&& paramet
 
     if (evaluationException && !optionalDetails) {
         ExceptionDetails details;
-        reportException(&globalObject, evaluationException, sourceCode.cachedScript(), &details);
+        reportException(&globalObject, evaluationException, sourceCode.cachedScript(), false, &details);
         optionalDetails = WTFMove(details);
     }
 
@@ -703,23 +691,12 @@ JSC::JSValue ScriptController::executeUserAgentScriptInWorldIgnoringException(DO
 }
 ValueOrException ScriptController::executeUserAgentScriptInWorld(DOMWrapperWorld& world, const String& script, bool forceUserGesture)
 {
-    return executeUserAgentScriptInWorldInternal(world, { script, URL { }, false, WTF::nullopt, forceUserGesture });
-}
-
-ValueOrException ScriptController::executeUserAgentScriptInWorldInternal(DOMWrapperWorld& world, RunJavaScriptParameters&& parameters)
-{
-    auto& document = *m_frame.document();
-    auto allowed = shouldAllowUserAgentScripts(document);
-    if (!allowed)
-        return makeUnexpected(allowed.error());
-
-    document.setHasEvaluatedUserAgentScripts();
-    return executeScriptInWorld(world, WTFMove(parameters));
+    return executeScriptInWorld(world, { script, URL { }, false, std::nullopt, forceUserGesture });
 }
 
 void ScriptController::executeAsynchronousUserAgentScriptInWorld(DOMWrapperWorld& world, RunJavaScriptParameters&& parameters, ResolveFunction&& resolveCompletionHandler)
 {
-    auto result = executeUserAgentScriptInWorldInternal(world, WTFMove(parameters));
+    auto result = executeScriptInWorld(world, WTFMove(parameters));
     
     if (parameters.runAsAsyncFunction == RunAsAsyncFunction::No || !result || !result.value().isObject()) {
         resolveCompletionHandler(result);
@@ -779,17 +756,6 @@ void ScriptController::executeAsynchronousUserAgentScriptInWorld(DOMWrapperWorld
     call(&globalObject, thenFunction, callData, result.value(), arguments);
 }
 
-Expected<void, ExceptionDetails> ScriptController::shouldAllowUserAgentScripts(Document& document) const
-{
-#if ENABLE(APPLE_PAY)
-    if (auto page = m_frame.page())
-        return page->paymentCoordinator().shouldAllowUserAgentScripts(document);
-#else
-    UNUSED_PARAM(document);
-#endif
-    return { };
-}
-
 bool ScriptController::canExecuteScripts(ReasonForCallingCanExecuteScripts reason)
 {
     if (reason == AboutToExecuteScript)
@@ -815,7 +781,7 @@ void ScriptController::executeJavaScriptURL(const URL& url, RefPtr<SecurityOrigi
     if (requesterSecurityOrigin && !requesterSecurityOrigin->isSameOriginDomain(m_frame.document()->securityOrigin()))
         return;
 
-    if (!m_frame.page() || !m_frame.document()->contentSecurityPolicy()->allowJavaScriptURLs(m_frame.document()->url().string(), eventHandlerPosition().m_line))
+    if (!m_frame.page() || !m_frame.document()->contentSecurityPolicy()->allowJavaScriptURLs(m_frame.document()->url().string(), eventHandlerPosition().m_line, url.string()))
         return;
 
     // We need to hold onto the Frame here because executing script can
@@ -829,7 +795,7 @@ void ScriptController::executeJavaScriptURL(const URL& url, RefPtr<SecurityOrigi
     VM& vm = globalObject->vm();
     auto throwScope = DECLARE_THROW_SCOPE(vm);
 
-    String decodedURL = decodeURLEscapeSequences(url.string());
+    String decodedURL = PAL::decodeURLEscapeSequences(url.string());
     auto result = executeScriptIgnoringException(decodedURL.substring(javascriptSchemeLength));
     RELEASE_ASSERT(&vm == &jsWindowProxy(mainThreadNormalWorld()).window()->vm());
 
@@ -868,6 +834,18 @@ void ScriptController::executeJavaScriptURL(const URL& url, RefPtr<SecurityOrigi
     }
 }
 
+void ScriptController::reportExceptionFromScriptError(LoadableScript::Error error, bool isModule)
+{
+    auto& world = mainThreadNormalWorld();
+    JSC::VM& vm = world.vm();
+    JSLockHolder lock(vm);
+
+    auto& proxy = jsWindowProxy(world);
+    auto& lexicalGlobalObject = *proxy.window();
+
+    reportException(&lexicalGlobalObject, error.errorValue.value(), nullptr, isModule);
+}
+
 } // namespace WebCore
 
-#undef RELEASE_LOG_ERROR_IF_ALLOWED
+#undef SCRIPTCONTROLLER_RELEASE_LOG_ERROR
