@@ -93,14 +93,14 @@ static ExceptionOr<std::unique_ptr<CryptoAlgorithmParameters>> normalizeCryptoAl
     VM& vm = state.vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
-    if (WTF::holds_alternative<String>(algorithmIdentifier)) {
+    if (std::holds_alternative<String>(algorithmIdentifier)) {
         auto newParams = Strong<JSObject>(vm, constructEmptyObject(&state));
-        newParams->putDirect(vm, Identifier::fromString(vm, "name"), jsString(vm, WTF::get<String>(algorithmIdentifier)));
+        newParams->putDirect(vm, Identifier::fromString(vm, "name"), jsString(vm, std::get<String>(algorithmIdentifier)));
         
         return normalizeCryptoAlgorithmParameters(state, newParams, operation);
     }
 
-    auto& value = WTF::get<JSC::Strong<JSC::JSObject>>(algorithmIdentifier);
+    auto& value = std::get<JSC::Strong<JSC::JSObject>>(algorithmIdentifier);
 
     auto params = convertDictionary<CryptoAlgorithmParameters>(state, value.get());
     RETURN_IF_EXCEPTION(scope, Exception { ExistingExceptionError });
@@ -442,36 +442,34 @@ static void normalizeJsonWebKey(JsonWebKey& webKey)
     webKey.usages = webKey.key_ops ? toCryptoKeyUsageBitmap(webKey.key_ops.value()) : 0;
 }
 
-// FIXME: This returns an Optional<KeyData> and takes a promise, rather than returning an
+// FIXME: This returns an std::optional<KeyData> and takes a promise, rather than returning an
 // ExceptionOr<KeyData> and letting the caller handle the promise, to work around an issue where
 // Variant types (which KeyData is) in ExceptionOr<> cause compile issues on some platforms. This
 // should be resolved by adopting a standards compliant std::variant (see https://webkit.org/b/175583)
-static Optional<KeyData> toKeyData(SubtleCrypto::KeyFormat format, SubtleCrypto::KeyDataVariant&& keyDataVariant, Ref<DeferredPromise>& promise)
+static std::optional<KeyData> toKeyData(SubtleCrypto::KeyFormat format, SubtleCrypto::KeyDataVariant&& keyDataVariant, Ref<DeferredPromise>& promise)
 {
     switch (format) {
     case SubtleCrypto::KeyFormat::Spki:
     case SubtleCrypto::KeyFormat::Pkcs8:
     case SubtleCrypto::KeyFormat::Raw:
         return WTF::switchOn(keyDataVariant,
-            [&promise] (JsonWebKey&) -> Optional<KeyData> {
+            [&promise] (JsonWebKey&) -> std::optional<KeyData> {
                 promise->reject(Exception { TypeError });
-                return WTF::nullopt;
+                return std::nullopt;
             },
-            [] (auto& bufferSource) -> Optional<KeyData> {
-                Vector<uint8_t> result;
-                result.append(static_cast<const uint8_t*>(bufferSource->data()), bufferSource->byteLength());
-                return KeyData { result };
+            [] (auto& bufferSource) -> std::optional<KeyData> {
+                return KeyData { Vector { static_cast<const uint8_t*>(bufferSource->data()), bufferSource->byteLength() } };
             }
         );
     case SubtleCrypto::KeyFormat::Jwk:
         return WTF::switchOn(keyDataVariant,
-            [] (JsonWebKey& webKey) -> Optional<KeyData> {
+            [] (JsonWebKey& webKey) -> std::optional<KeyData> {
                 normalizeJsonWebKey(webKey);
                 return KeyData { webKey };
             },
-            [&promise] (auto&) -> Optional<KeyData> {
+            [&promise] (auto&) -> std::optional<KeyData> {
                 promise->reject(Exception { TypeError });
-                return WTF::nullopt;
+                return std::nullopt;
             }
         );
     }
@@ -481,9 +479,7 @@ static Optional<KeyData> toKeyData(SubtleCrypto::KeyFormat format, SubtleCrypto:
 
 static Vector<uint8_t> copyToVector(BufferSource&& data)
 {
-    Vector<uint8_t> dataVector;
-    dataVector.append(data.data(), data.length());
-    return dataVector;
+    return { data.data(), data.length() };
 }
 
 static bool isSupportedExportKey(CryptoAlgorithmIdentifier identifier)
@@ -507,12 +503,10 @@ static bool isSupportedExportKey(CryptoAlgorithmIdentifier identifier)
     }
 }
 
-RefPtr<DeferredPromise> getPromise(DeferredPromise* index, WeakPtr<SubtleCrypto> subtleCryptoWeakPointer)
+RefPtr<DeferredPromise> getPromise(DeferredPromise* index, WeakPtr<SubtleCrypto> weakThis)
 {
-    if (subtleCryptoWeakPointer) {
-        if (auto promise = subtleCryptoWeakPointer->m_pendingPromises.take(index))
-            return WTFMove(promise.value());
-    }
+    if (weakThis)
+        return weakThis->m_pendingPromises.take(index);
     return nullptr;
 }
 
@@ -536,10 +530,18 @@ static std::unique_ptr<CryptoAlgorithmParameters> crossThreadCopyImportParams(co
     }
 }
 
+void SubtleCrypto::addAuthenticatedEncryptionWarningIfNecessary(CryptoAlgorithmIdentifier algorithmIdentifier)
+{
+    if (algorithmIdentifier == CryptoAlgorithmIdentifier::AES_CBC || algorithmIdentifier == CryptoAlgorithmIdentifier::AES_CTR)
+        scriptExecutionContext()->addConsoleMessage(MessageSource::Security, MessageLevel::Warning, "AES-CBC and AES-CTR do not provide authentication by default, and implementing it manually can result in minor, but serious mistakes. We recommended using authenticated encryption like AES-GCM to protect against chosen-ciphertext attacks.");
+}
+
 // MARK: - Exposed functions.
 
 void SubtleCrypto::encrypt(JSC::JSGlobalObject& state, AlgorithmIdentifier&& algorithmIdentifier, CryptoKey& key, BufferSource&& dataBufferSource, Ref<DeferredPromise>&& promise)
 {
+    addAuthenticatedEncryptionWarningIfNecessary(key.algorithmIdentifier());
+
     auto paramsOrException = normalizeCryptoAlgorithmParameters(state, WTFMove(algorithmIdentifier), Operations::Encrypt);
     if (paramsOrException.hasException()) {
         promise->reject(paramsOrException.releaseException());
@@ -563,13 +565,13 @@ void SubtleCrypto::encrypt(JSC::JSGlobalObject& state, AlgorithmIdentifier&& alg
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](const Vector<uint8_t>& cipherText) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](const Vector<uint8_t>& cipherText) mutable {
+        if (auto promise = getPromise(index, weakThis))
             fulfillPromiseWithArrayBuffer(promise.releaseNonNull(), cipherText.data(), cipherText.size());
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -578,6 +580,8 @@ void SubtleCrypto::encrypt(JSC::JSGlobalObject& state, AlgorithmIdentifier&& alg
 
 void SubtleCrypto::decrypt(JSC::JSGlobalObject& state, AlgorithmIdentifier&& algorithmIdentifier, CryptoKey& key, BufferSource&& dataBufferSource, Ref<DeferredPromise>&& promise)
 {
+    addAuthenticatedEncryptionWarningIfNecessary(key.algorithmIdentifier());
+
     auto paramsOrException = normalizeCryptoAlgorithmParameters(state, WTFMove(algorithmIdentifier), Operations::Decrypt);
     if (paramsOrException.hasException()) {
         promise->reject(paramsOrException.releaseException());
@@ -601,13 +605,13 @@ void SubtleCrypto::decrypt(JSC::JSGlobalObject& state, AlgorithmIdentifier&& alg
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](const Vector<uint8_t>& plainText) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](const Vector<uint8_t>& plainText) mutable {
+        if (auto promise = getPromise(index, weakThis))
             fulfillPromiseWithArrayBuffer(promise.releaseNonNull(), plainText.data(), plainText.size());
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -639,13 +643,13 @@ void SubtleCrypto::sign(JSC::JSGlobalObject& state, AlgorithmIdentifier&& algori
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](const Vector<uint8_t>& signature) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](const Vector<uint8_t>& signature) mutable {
+        if (auto promise = getPromise(index, weakThis))
             fulfillPromiseWithArrayBuffer(promise.releaseNonNull(), signature.data(), signature.size());
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -678,13 +682,13 @@ void SubtleCrypto::verify(JSC::JSGlobalObject& state, AlgorithmIdentifier&& algo
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](bool result) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](bool result) mutable {
+        if (auto promise = getPromise(index, weakThis))
             promise->resolve<IDLBoolean>(result);
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -706,13 +710,13 @@ void SubtleCrypto::digest(JSC::JSGlobalObject& state, AlgorithmIdentifier&& algo
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](const Vector<uint8_t>& digest) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](const Vector<uint8_t>& digest) mutable {
+        if (auto promise = getPromise(index, weakThis))
             fulfillPromiseWithArrayBuffer(promise.releaseNonNull(), digest.data(), digest.size());
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -734,9 +738,9 @@ void SubtleCrypto::generateKey(JSC::JSGlobalObject& state, AlgorithmIdentifier&&
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](KeyOrKeyPair&& keyOrKeyPair) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer)) {
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](KeyOrKeyPair&& keyOrKeyPair) mutable {
+        if (auto promise = getPromise(index, weakThis)) {
             WTF::switchOn(keyOrKeyPair,
                 [&promise] (RefPtr<CryptoKey>& key) {
                     if ((key->type() == CryptoKeyType::Private || key->type() == CryptoKeyType::Secret) && !key->usagesBitmap()) {
@@ -755,8 +759,8 @@ void SubtleCrypto::generateKey(JSC::JSGlobalObject& state, AlgorithmIdentifier&&
             );
         }
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -815,12 +819,12 @@ void SubtleCrypto::deriveKey(JSC::JSGlobalObject& state, AlgorithmIdentifier&& a
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer, importAlgorithm = WTFMove(importAlgorithm), importParams = crossThreadCopyImportParams(*importParams), extractable, keyUsagesBitmap](const Vector<uint8_t>& derivedKey) mutable {
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis, importAlgorithm = WTFMove(importAlgorithm), importParams = crossThreadCopyImportParams(*importParams), extractable, keyUsagesBitmap](const Vector<uint8_t>& derivedKey) mutable {
         // FIXME: https://bugs.webkit.org/show_bug.cgi?id=169395
         KeyData data = derivedKey;
-        auto callback = [index, subtleCryptoWeakPointer](CryptoKey& key) mutable {
-            if (auto promise = getPromise(index, subtleCryptoWeakPointer)) {
+        auto callback = [index, weakThis](CryptoKey& key) mutable {
+            if (auto promise = getPromise(index, weakThis)) {
                 if ((key.type() == CryptoKeyType::Private || key.type() == CryptoKeyType::Secret) && !key.usagesBitmap()) {
                     rejectWithException(promise.releaseNonNull(), SyntaxError);
                     return;
@@ -828,15 +832,15 @@ void SubtleCrypto::deriveKey(JSC::JSGlobalObject& state, AlgorithmIdentifier&& a
                 promise->resolve<IDLInterface<CryptoKey>>(key);
             }
         };
-        auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-            if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+        auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+            if (auto promise = getPromise(index, weakThis))
                 rejectWithException(promise.releaseNonNull(), ec);
         };
 
         importAlgorithm->importKey(SubtleCrypto::KeyFormat::Raw, WTFMove(data), *importParams, extractable, keyUsagesBitmap, WTFMove(callback), WTFMove(exceptionCallback));
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -866,13 +870,13 @@ void SubtleCrypto::deriveBits(JSC::JSGlobalObject& state, AlgorithmIdentifier&&
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](const Vector<uint8_t>& derivedKey) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](const Vector<uint8_t>& derivedKey) mutable {
+        if (auto promise = getPromise(index, weakThis))
             fulfillPromiseWithArrayBuffer(promise.releaseNonNull(), derivedKey.data(), derivedKey.size());
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -901,9 +905,9 @@ void SubtleCrypto::importKey(JSC::JSGlobalObject& state, KeyFormat format, KeyDa
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](CryptoKey& key) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer)) {
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](CryptoKey& key) mutable {
+        if (auto promise = getPromise(index, weakThis)) {
             if ((key.type() == CryptoKeyType::Private || key.type() == CryptoKeyType::Secret) && !key.usagesBitmap()) {
                 rejectWithException(promise.releaseNonNull(), SyntaxError);
                 return;
@@ -911,8 +915,8 @@ void SubtleCrypto::importKey(JSC::JSGlobalObject& state, KeyFormat format, KeyDa
             promise->resolve<IDLInterface<CryptoKey>>(key);
         }
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -938,26 +942,26 @@ void SubtleCrypto::exportKey(KeyFormat format, CryptoKey& key, Ref<DeferredPromi
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer](SubtleCrypto::KeyFormat format, KeyData&& key) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer)) {
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis](SubtleCrypto::KeyFormat format, KeyData&& key) mutable {
+        if (auto promise = getPromise(index, weakThis)) {
             switch (format) {
             case SubtleCrypto::KeyFormat::Spki:
             case SubtleCrypto::KeyFormat::Pkcs8:
             case SubtleCrypto::KeyFormat::Raw: {
-                Vector<uint8_t>& rawKey = WTF::get<Vector<uint8_t>>(key);
+                Vector<uint8_t>& rawKey = std::get<Vector<uint8_t>>(key);
                 fulfillPromiseWithArrayBuffer(promise.releaseNonNull(), rawKey.data(), rawKey.size());
                 return;
             }
             case SubtleCrypto::KeyFormat::Jwk:
-                promise->resolve<IDLDictionary<JsonWebKey>>(WTFMove(WTF::get<JsonWebKey>(key)));
+                promise->resolve<IDLDictionary<JsonWebKey>>(WTFMove(std::get<JsonWebKey>(key)));
                 return;
             }
             ASSERT_NOT_REACHED();
         }
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -1012,32 +1016,32 @@ void SubtleCrypto::wrapKey(JSC::JSGlobalObject& state, KeyFormat format, CryptoK
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer, wrapAlgorithm, wrappingKey = makeRef(wrappingKey), wrapParams = WTFMove(wrapParams), isEncryption, context, workQueue = m_workQueue.copyRef()](SubtleCrypto::KeyFormat format, KeyData&& key) mutable {
-        if (subtleCryptoWeakPointer) {
-            if (auto promise = subtleCryptoWeakPointer->m_pendingPromises.get(index)) {
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis, wrapAlgorithm, wrappingKey = Ref { wrappingKey }, wrapParams = WTFMove(wrapParams), isEncryption, context, workQueue = m_workQueue](SubtleCrypto::KeyFormat format, KeyData&& key) mutable {
+        if (weakThis) {
+            if (auto promise = weakThis->m_pendingPromises.get(index)) {
                 Vector<uint8_t> bytes;
                 switch (format) {
                 case SubtleCrypto::KeyFormat::Spki:
                 case SubtleCrypto::KeyFormat::Pkcs8:
                 case SubtleCrypto::KeyFormat::Raw:
-                    bytes = WTF::get<Vector<uint8_t>>(key);
+                    bytes = std::get<Vector<uint8_t>>(key);
                     break;
                 case SubtleCrypto::KeyFormat::Jwk: {
                     // FIXME: Converting to JS just to JSON-Stringify seems inefficient. We should find a way to go directly from the struct to JSON.
-                    auto jwk = toJS<IDLDictionary<JsonWebKey>>(*(promise->globalObject()), *(promise->globalObject()), WTFMove(WTF::get<JsonWebKey>(key)));
+                    auto jwk = toJS<IDLDictionary<JsonWebKey>>(*(promise->globalObject()), *(promise->globalObject()), WTFMove(std::get<JsonWebKey>(key)));
                     String jwkString = JSONStringify(promise->globalObject(), jwk, 0);
                     CString jwkUtf8String = jwkString.utf8(StrictConversion);
                     bytes.append(jwkUtf8String.data(), jwkUtf8String.length());
                 }
                 }
 
-                auto callback = [index, subtleCryptoWeakPointer](const Vector<uint8_t>& wrappedKey) mutable {
-                    if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+                auto callback = [index, weakThis](const Vector<uint8_t>& wrappedKey) mutable {
+                    if (auto promise = getPromise(index, weakThis))
                         fulfillPromiseWithArrayBuffer(promise.releaseNonNull(), wrappedKey.data(), wrappedKey.size());
                 };
-                auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-                    if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+                auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+                    if (auto promise = getPromise(index, weakThis))
                         rejectWithException(promise.releaseNonNull(), ec);
                 };
 
@@ -1053,8 +1057,8 @@ void SubtleCrypto::wrapKey(JSC::JSGlobalObject& state, KeyFormat format, CryptoK
             }
         }
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
@@ -1070,8 +1074,6 @@ void SubtleCrypto::unwrapKey(JSC::JSGlobalObject& state, KeyFormat format, Buffe
 
     auto unwrapParamsOrException = normalizeCryptoAlgorithmParameters(state, unwrapAlgorithmIdentifier, Operations::UnwrapKey);
     if (unwrapParamsOrException.hasException()) {
-        ASSERT(unwrapParamsOrException.exception().code() != ExistingExceptionError);
-
         unwrapParamsOrException = normalizeCryptoAlgorithmParameters(state, unwrapAlgorithmIdentifier, Operations::Decrypt);
         if (unwrapParamsOrException.hasException()) {
             promise->reject(unwrapParamsOrException.releaseException());
@@ -1115,10 +1117,10 @@ void SubtleCrypto::unwrapKey(JSC::JSGlobalObject& state, KeyFormat format, Buffe
 
     auto index = promise.ptr();
     m_pendingPromises.add(index, WTFMove(promise));
-    auto subtleCryptoWeakPointer = makeWeakPtr(*this);
-    auto callback = [index, subtleCryptoWeakPointer, format, importAlgorithm, unwrappedKeyAlgorithm = crossThreadCopyImportParams(*unwrappedKeyAlgorithm), extractable, keyUsagesBitmap](const Vector<uint8_t>& bytes) mutable {
-        if (subtleCryptoWeakPointer) {
-            if (auto promise = subtleCryptoWeakPointer->m_pendingPromises.get(index)) {
+    WeakPtr weakThis { *this };
+    auto callback = [index, weakThis, format, importAlgorithm, unwrappedKeyAlgorithm = crossThreadCopyImportParams(*unwrappedKeyAlgorithm), extractable, keyUsagesBitmap](const Vector<uint8_t>& bytes) mutable {
+        if (weakThis) {
+            if (auto promise = weakThis->m_pendingPromises.get(index)) {
                 KeyData keyData;
                 switch (format) {
                 case SubtleCrypto::KeyFormat::Spki:
@@ -1131,7 +1133,7 @@ void SubtleCrypto::unwrapKey(JSC::JSGlobalObject& state, KeyFormat format, Buffe
                     auto& vm = state.vm();
                     auto scope = DECLARE_THROW_SCOPE(vm);
 
-                    String jwkString(reinterpret_cast_ptr<const char*>(bytes.data()), bytes.size());
+                    String jwkString(bytes.data(), bytes.size());
                     JSLockHolder locker(vm);
                     auto jwkObject = JSONParse(&state, jwkString);
                     if (!jwkObject) {
@@ -1147,8 +1149,8 @@ void SubtleCrypto::unwrapKey(JSC::JSGlobalObject& state, KeyFormat format, Buffe
                 }
                 }
 
-                auto callback = [index, subtleCryptoWeakPointer](CryptoKey& key) mutable {
-                    if (auto promise = getPromise(index, subtleCryptoWeakPointer)) {
+                auto callback = [index, weakThis](CryptoKey& key) mutable {
+                    if (auto promise = getPromise(index, weakThis)) {
                         if ((key.type() == CryptoKeyType::Private || key.type() == CryptoKeyType::Secret) && !key.usagesBitmap()) {
                             rejectWithException(promise.releaseNonNull(), SyntaxError);
                             return;
@@ -1156,8 +1158,8 @@ void SubtleCrypto::unwrapKey(JSC::JSGlobalObject& state, KeyFormat format, Buffe
                         promise->resolve<IDLInterface<CryptoKey>>(key);
                     }
                 };
-                auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-                    if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+                auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+                    if (auto promise = getPromise(index, weakThis))
                         rejectWithException(promise.releaseNonNull(), ec);
                 };
 
@@ -1166,8 +1168,8 @@ void SubtleCrypto::unwrapKey(JSC::JSGlobalObject& state, KeyFormat format, Buffe
             }
         }
     };
-    auto exceptionCallback = [index, subtleCryptoWeakPointer](ExceptionCode ec) mutable {
-        if (auto promise = getPromise(index, subtleCryptoWeakPointer))
+    auto exceptionCallback = [index, weakThis](ExceptionCode ec) mutable {
+        if (auto promise = getPromise(index, weakThis))
             rejectWithException(promise.releaseNonNull(), ec);
     };
 
