@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2008-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,10 +26,10 @@
 #include "config.h"
 #include "CSSGradientValue.h"
 
-#include "CSSCalculationValue.h"
+#include "CSSCalcValue.h"
 #include "CSSToLengthConversionData.h"
 #include "CSSValueKeywords.h"
-#include "ColorBlending.h"
+#include "ColorInterpolation.h"
 #include "GradientImage.h"
 #include "NodeRenderStyle.h"
 #include "Pair.h"
@@ -68,9 +68,9 @@ RefPtr<Image> CSSGradientValue::image(RenderElement& renderer, const FloatSize&
 
 struct GradientStop {
     Color color;
-    Optional<float> offset;
+    std::optional<float> offset;
 
-    bool isSpecified() const { return offset.hasValue(); }
+    bool isSpecified() const { return offset.has_value(); }
     bool isMidpoint() const { return !color.isValid(); }
 };
 
@@ -106,9 +106,9 @@ bool CSSGradientValue::hasColorDerivedFromElement() const
     return *m_hasColorDerivedFromElement;
 }
 
-Ref<CSSGradientValue> CSSGradientValue::gradientWithStylesResolved(Style::BuilderState& builderState)
+Ref<CSSGradientValue> CSSGradientValue::valueWithStylesResolved(Style::BuilderState& builderState)
 {
-    auto result = hasColorDerivedFromElement() ? clone(*this) : makeRef(*this);
+    auto result = hasColorDerivedFromElement() ? clone(*this) : Ref { *this };
     resolveStopColors(result->m_stops, [&](const CSSPrimitiveValue& colorValue) {
         return builderState.colorFromPrimitiveValueWithResolvedCurrentColor(colorValue);
     });
@@ -137,7 +137,7 @@ public:
     }
     float maxExtent(float, float) const { return 1; }
 
-    void normalizeStopsAndEndpointsOutsideRange(Vector<GradientStop>& stops)
+    void normalizeStopsAndEndpointsOutsideRange(Vector<GradientStop>& stops, ColorInterpolationMethod)
     {
         float firstOffset = *stops.first().offset;
         float lastOffset = *stops.last().offset;
@@ -180,7 +180,7 @@ public:
         return 1;
     }
 
-    void normalizeStopsAndEndpointsOutsideRange(Vector<GradientStop>& stops)
+    void normalizeStopsAndEndpointsOutsideRange(Vector<GradientStop>& stops, ColorInterpolationMethod colorInterpolationMethod)
     {
         auto numStops = stops.size();
 
@@ -204,8 +204,7 @@ public:
                 float nextOffset = *stops[firstZeroOrGreaterIndex].offset;
 
                 float interStopProportion = -prevOffset / (nextOffset - prevOffset);
-                // FIXME: when we interpolate gradients using premultiplied colors, this should do premultiplication.
-                Color blendedColor = blend(stops[firstZeroOrGreaterIndex - 1].color, stops[firstZeroOrGreaterIndex].color, interStopProportion);
+                auto blendedColor = interpolateColors(colorInterpolationMethod, stops[firstZeroOrGreaterIndex - 1].color, 1.0f - interStopProportion, stops[firstZeroOrGreaterIndex].color, interStopProportion);
 
                 // Clamp the positions to 0 and set the color.
                 for (size_t i = 0; i < firstZeroOrGreaterIndex; ++i) {
@@ -235,11 +234,12 @@ public:
     float gradientLength() const { return 1; }
     float maxExtent(float, float) const { return 1; }
 
-    void normalizeStopsAndEndpointsOutsideRange(Vector<GradientStop>& stops)
+    void normalizeStopsAndEndpointsOutsideRange(Vector<GradientStop>& stops, ColorInterpolationMethod colorInterpolationMethod)
     {
-        auto numStops = stops.size();
-        
-        size_t firstZeroOrGreaterIndex = numStops;
+        size_t numStops = stops.size();
+        size_t lastStopIndex = numStops - 1;
+
+        std::optional<size_t> firstZeroOrGreaterIndex;
         for (size_t i = 0; i < numStops; ++i) {
             if (*stops[i].offset >= 0) {
                 firstZeroOrGreaterIndex = i;
@@ -247,65 +247,67 @@ public:
             }
         }
 
-        if (firstZeroOrGreaterIndex > 0) {
-            if (firstZeroOrGreaterIndex < numStops && *stops[firstZeroOrGreaterIndex].offset > 0) {
-                float prevOffset = *stops[firstZeroOrGreaterIndex - 1].offset;
-                float nextOffset = *stops[firstZeroOrGreaterIndex].offset;
-                
-                float interStopProportion = -prevOffset / (nextOffset - prevOffset);
-                // FIXME: when we interpolate gradients using premultiplied colors, this should do premultiplication.
-                Color blendedColor = blend(stops[firstZeroOrGreaterIndex - 1].color, stops[firstZeroOrGreaterIndex].color, interStopProportion);
-                
+        if (firstZeroOrGreaterIndex) {
+            size_t index = *firstZeroOrGreaterIndex;
+            if (index > 0) {
+                float previousOffset = *stops[index - 1].offset;
+                float nextOffset = *stops[index].offset;
+
+                float interStopProportion = -previousOffset / (nextOffset - previousOffset);
+                auto blendedColor = interpolateColors(colorInterpolationMethod, stops[index - 1].color, 1.0f - interStopProportion, stops[index].color, interStopProportion);
+
                 // Clamp the positions to 0 and set the color.
-                for (size_t i = 0; i < firstZeroOrGreaterIndex; ++i) {
+                for (size_t i = 0; i < index; ++i) {
                     stops[i].offset = 0;
                     stops[i].color = blendedColor;
                 }
-            } else {
-                // All stops are below 0; just clamp them.
-                for (size_t i = 0; i < firstZeroOrGreaterIndex; ++i)
-                    stops[i].offset = 0;
             }
+        } else {
+            // All stop offsets below 0, clamp them.
+            for (auto& stop : stops)
+                stop.offset = 0;
         }
 
-        size_t lastOneOrLessIndex = numStops;
-        for (int i = numStops - 1; i >= 0; --i) {
+        std::optional<size_t> lastOneOrLessIndex;
+        for (int i = lastStopIndex; i >= 0; --i) {
             if (*stops[i].offset <= 1) {
                 lastOneOrLessIndex = i;
                 break;
             }
         }
-        
-        if (lastOneOrLessIndex < numStops - 1) {
-            if (lastOneOrLessIndex < numStops && *stops[lastOneOrLessIndex].offset < 1) {
-                float prevOffset = *stops[lastOneOrLessIndex].offset;
-                float nextOffset = *stops[lastOneOrLessIndex + 1].offset;
-                
-                float interStopProportion = (1 - prevOffset) / (nextOffset - prevOffset);
-                // FIXME: when we interpolate gradients using premultiplied colors, this should do premultiplication.
-                Color blendedColor = blend(stops[lastOneOrLessIndex].color, stops[lastOneOrLessIndex + 1].color, interStopProportion);
-                
+
+        if (lastOneOrLessIndex) {
+            size_t index = *lastOneOrLessIndex;
+            if (index < lastStopIndex) {
+                float previousOffset = *stops[index].offset;
+                float nextOffset = *stops[index + 1].offset;
+
+                float interStopProportion = (1 - previousOffset) / (nextOffset - previousOffset);
+                auto blendedColor = interpolateColors(colorInterpolationMethod, stops[index].color, 1.0f - interStopProportion, stops[index + 1].color, interStopProportion);
+
                 // Clamp the positions to 1 and set the color.
-                for (size_t i = lastOneOrLessIndex + 1; i < numStops; ++i) {
+                for (size_t i = index + 1; i < numStops; ++i) {
                     stops[i].offset = 1;
                     stops[i].color = blendedColor;
                 }
-            } else {
-                // All stops are above 1; just clamp them.
-                for (size_t i = lastOneOrLessIndex; i < numStops; ++i)
-                    stops[i].offset = 1;
             }
+        } else {
+            // All stop offsets above 1, clamp them.
+            for (auto& stop : stops)
+                stop.offset = 1;
         }
     }
 };
 
 template<typename GradientAdapter>
-Gradient::ColorStopVector CSSGradientValue::computeStops(GradientAdapter& gradientAdapter, const CSSToLengthConversionData& conversionData, const RenderStyle& style, float maxLengthForRepeat)
+GradientColorStops CSSGradientValue::computeStops(GradientAdapter& gradientAdapter, const CSSToLengthConversionData& conversionData, const RenderStyle& style, float maxLengthForRepeat)
 {
-    if (m_gradientType == CSSDeprecatedLinearGradient || m_gradientType == CSSDeprecatedRadialGradient) {
-        Gradient::ColorStopVector result;
-        result.reserveInitialCapacity(m_stops.size());
+    bool hasColorFilter = style.hasAppleColorFilter();
 
+    if (m_gradientType == CSSDeprecatedLinearGradient || m_gradientType == CSSDeprecatedRadialGradient) {
+        GradientColorStops::StopVector result;
+        result.reserveInitialCapacity(m_stops.size());
+        
         for (auto& stop : m_stops) {
             float offset;
             if (stop.position->isPercentage())
@@ -313,17 +315,14 @@ Gradient::ColorStopVector CSSGradientValue::computeStops(GradientAdapter& gradie
             else
                 offset = stop.position->floatValue(CSSUnitType::CSS_NUMBER);
 
-            Color color = stop.resolvedColor;
-            if (style.hasAppleColorFilter())
-                style.appleColorFilter().transformColor(color);
-            result.uncheckedAppend({ offset, color });
+            result.uncheckedAppend({ offset, hasColorFilter ? style.colorByApplyingColorFilter(stop.resolvedColor) : stop.resolvedColor });
         }
 
-        std::stable_sort(result.begin(), result.end(), [] (const Gradient::ColorStop& a, const Gradient::ColorStop& b) {
+        std::stable_sort(result.begin(), result.end(), [] (const GradientColorStop& a, const GradientColorStop& b) {
             return a.offset < b.offset;
         });
 
-        return result;
+        return GradientColorStops::Sorted { WTFMove(result) };
     }
 
     size_t numStops = m_stops.size();
@@ -334,11 +333,7 @@ Gradient::ColorStopVector CSSGradientValue::computeStops(GradientAdapter& gradie
     for (size_t i = 0; i < numStops; ++i) {
         auto& stop = m_stops[i];
 
-        Color color = stop.resolvedColor;
-        if (style.hasAppleColorFilter())
-            style.appleColorFilter().transformColor(color);
-
-        stops[i].color = color;
+        stops[i].color = hasColorFilter ? style.colorByApplyingColorFilter(stop.resolvedColor) : stop.resolvedColor;
 
         if (stop.position) {
             auto& positionValue = *stop.position;
@@ -481,8 +476,7 @@ Gradient::ColorStopVector CSSGradientValue::computeStops(GradientAdapter& gradie
         for (size_t y = 0; y < 9; ++y) {
             float relativeOffset = (*newStops[y].offset - offset1) / (offset2 - offset1);
             float multiplier = std::pow(relativeOffset, std::log(.5f) / std::log(midpoint));
-            // FIXME: Why not premultiply here?
-            newStops[y].color = blendWithoutPremultiply(color1, color2, multiplier);
+            newStops[y].color = interpolateColors(m_colorInterpolationMethod.method, color1, 1.0f - multiplier, color2, multiplier);
         }
 
         stops.remove(x);
@@ -553,14 +547,14 @@ Gradient::ColorStopVector CSSGradientValue::computeStops(GradientAdapter& gradie
 
     // If the gradient goes outside the 0-1 range, normalize it by moving the endpoints, and adjusting the stops.
     if (stops.size() > 1 && (*stops.first().offset < 0 || *stops.last().offset > 1))
-        gradientAdapter.normalizeStopsAndEndpointsOutsideRange(stops);
+        gradientAdapter.normalizeStopsAndEndpointsOutsideRange(stops, m_colorInterpolationMethod.method);
     
-    Gradient::ColorStopVector result;
+    GradientColorStops::StopVector result;
     result.reserveInitialCapacity(stops.size());
     for (auto& stop : stops)
-        result.uncheckedAppend({ *stop.offset, stop.color });
+        result.uncheckedAppend({ *stop.offset, WTFMove(stop.color) });
 
-    return result;
+    return GradientColorStops::Sorted { WTFMove(result) };
 }
 
 static float positionFromValue(const CSSPrimitiveValue* value, const CSSToLengthConversionData& conversionData, const FloatSize& size, bool isHorizontal)
@@ -653,7 +647,84 @@ bool CSSGradientValue::equals(const CSSGradientValue& other) const
         && compareCSSValuePtr(m_secondY, other.m_secondY)
         && m_stops == other.m_stops
         && m_gradientType == other.m_gradientType
-        && m_repeating == other.m_repeating;
+        && m_repeating == other.m_repeating
+        && m_colorInterpolationMethod == other.m_colorInterpolationMethod;
+}
+
+static void appendHueInterpolationMethod(StringBuilder& builder, HueInterpolationMethod hueInterpolationMethod)
+{
+    switch (hueInterpolationMethod) {
+    case HueInterpolationMethod::Shorter:
+        break;
+    case HueInterpolationMethod::Longer:
+        builder.append(" longer hue");
+        break;
+    case HueInterpolationMethod::Increasing:
+        builder.append(" increasing hue");
+        break;
+    case HueInterpolationMethod::Decreasing:
+        builder.append(" decreasing hue");
+        break;
+    case HueInterpolationMethod::Specified:
+        builder.append(" specified hue");
+        break;
+    }
+}
+
+static bool appendColorInterpolationMethod(StringBuilder& builder, CSSGradientColorInterpolationMethod colorInterpolationMethod, bool needsLeadingSpace)
+{
+    return WTF::switchOn(colorInterpolationMethod.method.colorSpace,
+        [&] (const ColorInterpolationMethod::HSL& hsl) {
+            builder.append(needsLeadingSpace ? " " : "", "in hsl");
+            appendHueInterpolationMethod(builder, hsl.hueInterpolationMethod);
+            return true;
+        },
+        [&] (const ColorInterpolationMethod::HWB& hwb) {
+            builder.append(needsLeadingSpace ? " " : "", "in hwb");
+            appendHueInterpolationMethod(builder, hwb.hueInterpolationMethod);
+            return true;
+        },
+        [&] (const ColorInterpolationMethod::LCH& lch) {
+            builder.append(needsLeadingSpace ? " " : "", "in lch");
+            appendHueInterpolationMethod(builder, lch.hueInterpolationMethod);
+            return true;
+        },
+        [&] (const ColorInterpolationMethod::Lab&) {
+            builder.append(needsLeadingSpace ? " " : "", "in lab");
+            return true;
+        },
+        [&] (const ColorInterpolationMethod::OKLCH& oklch) {
+            builder.append(needsLeadingSpace ? " " : "", "in oklch");
+            appendHueInterpolationMethod(builder, oklch.hueInterpolationMethod);
+            return true;
+        },
+        [&] (const ColorInterpolationMethod::OKLab&) {
+            if (colorInterpolationMethod.defaultMethod != CSSGradientColorInterpolationMethod::Default::OKLab) {
+                builder.append(needsLeadingSpace ? " " : "", "in oklab");
+                return true;
+            }
+            return false;
+        },
+        [&] (const ColorInterpolationMethod::SRGB&) {
+            if (colorInterpolationMethod.defaultMethod != CSSGradientColorInterpolationMethod::Default::SRGB) {
+                builder.append(needsLeadingSpace ? " " : "", "in srgb");
+                return true;
+            }
+            return false;
+        },
+        [&] (const ColorInterpolationMethod::SRGBLinear&) {
+            builder.append(needsLeadingSpace ? " " : "", "in srgb-linear");
+            return true;
+        },
+        [&] (const ColorInterpolationMethod::XYZD50&) {
+            builder.append(needsLeadingSpace ? " " : "", "in xyz-d50");
+            return true;
+        },
+        [&] (const ColorInterpolationMethod::XYZD65&) {
+            builder.append(needsLeadingSpace ? " " : "", "in xyz-d65");
+            return true;
+        }
+    );
 }
 
 static void appendGradientStops(StringBuilder& builder, const Vector<CSSGradientColorStop, 2>& stops)
@@ -692,9 +763,9 @@ String CSSLinearGradientValue::customCSSText() const
         appendGradientStops(result, stops());
     } else if (gradientType() == CSSPrefixedLinearGradient) {
         if (isRepeating())
-            result.appendLiteral("-webkit-repeating-linear-gradient(");
+            result.append("-webkit-repeating-linear-gradient(");
         else
-            result.appendLiteral("-webkit-linear-gradient(");
+            result.append("-webkit-linear-gradient(");
 
         if (m_angle)
             result.append(m_angle->cssText());
@@ -702,29 +773,32 @@ String CSSLinearGradientValue::customCSSText() const
             appendSpaceSeparatedOptionalCSSPtrText(result, firstX(), firstY());
 
         for (auto& stop : stops()) {
-            result.appendLiteral(", ");
+            result.append(", ");
             writeColorStop(result, stop);
         }
     } else {
         if (isRepeating())
-            result.appendLiteral("repeating-linear-gradient(");
+            result.append("repeating-linear-gradient(");
         else
-            result.appendLiteral("linear-gradient(");
+            result.append("linear-gradient(");
 
         bool wroteSomething = false;
 
         if (m_angle && m_angle->computeDegrees() != 180) {
             result.append(m_angle->cssText());
             wroteSomething = true;
-        } else if ((firstX() || firstY()) && !(!firstX() && firstY() && firstY()->valueID() == CSSValueBottom)) {
-            result.appendLiteral("to ");
+        } else if (firstX() || (firstY() && firstY()->valueID() != CSSValueBottom)) {
+            result.append("to ");
             appendSpaceSeparatedOptionalCSSPtrText(result, firstX(), firstY());
             wroteSomething = true;
         }
 
+        if (appendColorInterpolationMethod(result, colorInterpolationMethod(), wroteSomething))
+            wroteSomething = true;
+
         for (auto& stop : stops()) {
             if (wroteSomething)
-                result.appendLiteral(", ");
+                result.append(", ");
             wroteSomething = true;
             writeColorStop(result, stop);
         }
@@ -806,7 +880,11 @@ Ref<Gradient> CSSLinearGradientValue::createGradient(RenderElement& renderer, co
 {
     ASSERT(!size.isEmpty());
 
-    CSSToLengthConversionData conversionData(&renderer.style(), renderer.document().documentElement()->renderStyle(), renderer.parentStyle(), &renderer.view());
+    const RenderStyle* rootStyle = nullptr;
+    if (auto* documentElement = renderer.document().documentElement())
+        rootStyle = documentElement->renderStyle();
+
+    CSSToLengthConversionData conversionData(&renderer.style(), rootStyle, renderer.parentStyle(), &renderer.view());
 
     FloatPoint firstPoint;
     FloatPoint secondPoint;
@@ -863,9 +941,7 @@ Ref<Gradient> CSSLinearGradientValue::createGradient(RenderElement& renderer, co
     LinearGradientAdapter adapter { data };
     auto stops = computeStops(adapter, conversionData, renderer.style(), 1);
 
-    auto gradient = Gradient::create(WTFMove(data));
-    gradient->setSortedColorStops(WTFMove(stops));
-    return gradient;
+    return Gradient::create(WTFMove(data), colorInterpolationMethod().method, GradientSpreadMethod::Pad, WTFMove(stops));
 }
 
 bool CSSLinearGradientValue::equals(const CSSLinearGradientValue& other) const
@@ -883,44 +959,44 @@ String CSSRadialGradientValue::customCSSText() const
         appendGradientStops(result, stops());
     } else if (gradientType() == CSSPrefixedRadialGradient) {
         if (isRepeating())
-            result.appendLiteral("-webkit-repeating-radial-gradient(");
+            result.append("-webkit-repeating-radial-gradient(");
         else
-            result.appendLiteral("-webkit-radial-gradient(");
+            result.append("-webkit-radial-gradient(");
 
         if (firstX() || firstY())
             appendSpaceSeparatedOptionalCSSPtrText(result, firstX(), firstY());
         else
-            result.appendLiteral("center");
+            result.append("center");
 
         if (m_shape || m_sizingBehavior) {
-            result.appendLiteral(", ");
+            result.append(", ");
             if (m_shape)
                 result.append(m_shape->cssText(), ' ');
             else
-                result.appendLiteral("ellipse ");
+                result.append("ellipse ");
             if (m_sizingBehavior)
                 result.append(m_sizingBehavior->cssText());
             else
-                result.appendLiteral("cover");
+                result.append("cover");
         } else if (m_endHorizontalSize && m_endVerticalSize)
             result.append(", ", m_endHorizontalSize->cssText(), ' ', m_endVerticalSize->cssText());
 
         for (auto& stop : stops()) {
-            result.appendLiteral(", ");
+            result.append(", ");
             writeColorStop(result, stop);
         }
     } else {
         if (isRepeating())
-            result.appendLiteral("repeating-radial-gradient(");
+            result.append("repeating-radial-gradient(");
         else
-            result.appendLiteral("radial-gradient(");
+            result.append("radial-gradient(");
 
         bool wroteSomething = false;
 
         // The only ambiguous case that needs an explicit shape to be provided
         // is when a sizing keyword is used (or all sizing is omitted).
         if (m_shape && m_shape->valueID() != CSSValueEllipse && (m_sizingBehavior || (!m_sizingBehavior && !m_endHorizontalSize))) {
-            result.appendLiteral("circle");
+            result.append("circle");
             wroteSomething = true;
         }
 
@@ -938,21 +1014,24 @@ String CSSRadialGradientValue::customCSSText() const
             wroteSomething = true;
         }
 
-        if (firstX() || firstY()) {
+        if ((firstX() && !firstX()->isCenterPosition()) || (firstY() && !firstY()->isCenterPosition())) {
             if (wroteSomething)
                 result.append(' ');
-            result.appendLiteral("at ");
+            result.append("at ");
             appendSpaceSeparatedOptionalCSSPtrText(result, firstX(), firstY());
             wroteSomething = true;
         }
 
+        if (appendColorInterpolationMethod(result, colorInterpolationMethod(), wroteSomething))
+            wroteSomething = true;
+
         if (wroteSomething)
-            result.appendLiteral(", ");
+            result.append(", ");
 
         bool wroteFirstStop = false;
         for (auto& stop : stops()) {
             if (wroteFirstStop)
-                result.appendLiteral(", ");
+                result.append(", ");
             wroteFirstStop = true;
             writeColorStop(result, stop);
         }
@@ -969,6 +1048,10 @@ float CSSRadialGradientValue::resolveRadius(CSSPrimitiveValue& radius, const CSS
         result = radius.floatValue() * conversionData.zoom();
     else if (widthOrHeight && radius.isPercentage())
         result = *widthOrHeight * radius.floatValue() / 100;
+    else if (widthOrHeight && radius.isCalculatedPercentageWithLength()) {
+        auto expression = radius.cssCalcValue()->createCalculationValue(conversionData);
+        result = expression->evaluate(*widthOrHeight);
+    }
     else
         result = radius.computeLength<float>(conversionData);
     return result;
@@ -1055,7 +1138,11 @@ Ref<Gradient> CSSRadialGradientValue::createGradient(RenderElement& renderer, co
 {
     ASSERT(!size.isEmpty());
 
-    CSSToLengthConversionData conversionData(&renderer.style(), renderer.document().documentElement()->renderStyle(), renderer.parentStyle(), &renderer.view());
+    const RenderStyle* rootStyle = nullptr;
+    if (auto* documentElement = renderer.document().documentElement())
+        rootStyle = documentElement->renderStyle();
+
+    CSSToLengthConversionData conversionData(&renderer.style(), rootStyle, renderer.parentStyle(), &renderer.view());
 
     FloatPoint firstPoint = computeEndPoint(firstX(), firstY(), conversionData, size);
     if (!firstX())
@@ -1189,9 +1276,7 @@ Ref<Gradient> CSSRadialGradientValue::createGradient(RenderElement& renderer, co
     RadialGradientAdapter adapter { data };
     auto stops = computeStops(adapter, conversionData, renderer.style(), maxExtent);
 
-    auto gradient = Gradient::create(WTFMove(data));
-    gradient->setSortedColorStops(WTFMove(stops));
-    return gradient;
+    return Gradient::create(WTFMove(data), colorInterpolationMethod().method, GradientSpreadMethod::Pad, WTFMove(stops));
 }
 
 bool CSSRadialGradientValue::equals(const CSSRadialGradientValue& other) const
@@ -1207,33 +1292,33 @@ String CSSConicGradientValue::customCSSText() const
 {
     StringBuilder result;
 
-    if (isRepeating())
-        result.appendLiteral("repeating-conic-gradient(");
-    else
-        result.appendLiteral("conic-gradient(");
+    result.append(isRepeating() ? "repeating-conic-gradient(" : "conic-gradient(");
 
     bool wroteSomething = false;
 
-    if (m_angle) {
+    if (m_angle && m_angle->computeDegrees()) {
         result.append("from ", m_angle->cssText());
         wroteSomething = true;
     }
 
-    if (firstX() && firstY()) {
+    if ((firstX() && !firstX()->isCenterPosition()) || (firstY() && !firstY()->isCenterPosition())) {
         if (wroteSomething)
             result.append(' ');
-        result.appendLiteral("at ");
+        result.append("at ");
         appendSpaceSeparatedOptionalCSSPtrText(result, firstX(), firstY());
         wroteSomething = true;
     }
 
+    if (appendColorInterpolationMethod(result, colorInterpolationMethod(), wroteSomething))
+        wroteSomething = true;
+
     if (wroteSomething)
-        result.appendLiteral(", ");
+        result.append(", ");
 
     bool wroteFirstStop = false;
     for (auto& stop : stops()) {
         if (wroteFirstStop)
-            result.appendLiteral(", ");
+            result.append(", ");
         wroteFirstStop = true;
         writeColorStop(result, stop);
     }
@@ -1246,7 +1331,11 @@ Ref<Gradient> CSSConicGradientValue::createGradient(RenderElement& renderer, con
 {
     ASSERT(!size.isEmpty());
 
-    CSSToLengthConversionData conversionData(&renderer.style(), renderer.document().documentElement()->renderStyle(), renderer.parentStyle(), &renderer.view());
+    const RenderStyle* rootStyle = nullptr;
+    if (auto* documentElement = renderer.document().documentElement())
+        rootStyle = documentElement->renderStyle();
+
+    CSSToLengthConversionData conversionData(&renderer.style(), rootStyle, renderer.parentStyle(), &renderer.view());
 
     FloatPoint centerPoint = computeEndPoint(firstX(), firstY(), conversionData, size);
     if (!firstX())
@@ -1262,9 +1351,7 @@ Ref<Gradient> CSSConicGradientValue::createGradient(RenderElement& renderer, con
     ConicGradientAdapter adapter;
     auto stops = computeStops(adapter, conversionData, renderer.style(), 1);
 
-    auto gradient = Gradient::create(WTFMove(data));
-    gradient->setSortedColorStops(WTFMove(stops));
-    return gradient;
+    return Gradient::create(WTFMove(data), colorInterpolationMethod().method, GradientSpreadMethod::Pad, WTFMove(stops));
 }
 
 bool CSSConicGradientValue::equals(const CSSConicGradientValue& other) const
