@@ -56,6 +56,7 @@ CSSSelector::CSSSelector(const QualifiedName& tagQName, bool tagIsForNamespaceRu
     , m_match(Tag)
     , m_pseudoType(0)
     , m_isLastInSelectorList(false)
+    , m_isFirstInTagHistory(true)
     , m_isLastInTagHistory(true)
     , m_hasRareData(false)
     , m_hasNameWithCase(false)
@@ -90,14 +91,13 @@ void CSSSelector::createRareData()
     m_hasRareData = true;
 }
 
-static unsigned simpleSelectorSpecificityInternal(const CSSSelector& simpleSelector, bool isComputingMaximumSpecificity);
+static unsigned simpleSelectorSpecificityInternal(const CSSSelector& simpleSelector);
 
-static unsigned selectorSpecificity(const CSSSelector& firstSimpleSelector, bool isComputingMaximumSpecificity)
+static unsigned selectorSpecificity(const CSSSelector& firstSimpleSelector)
 {
-    unsigned total = simpleSelectorSpecificityInternal(firstSimpleSelector, isComputingMaximumSpecificity);
-
-    for (const CSSSelector* selector = firstSimpleSelector.tagHistory(); selector; selector = selector->tagHistory())
-        total = CSSSelector::addSpecificities(total, simpleSelectorSpecificityInternal(*selector, isComputingMaximumSpecificity));
+    unsigned total = 0;
+    for (const CSSSelector* selector = &firstSimpleSelector; selector; selector = selector->tagHistory())
+        total = CSSSelector::addSpecificities(total, simpleSelectorSpecificityInternal(*selector));
     return total;
 }
 
@@ -105,11 +105,11 @@ static unsigned maxSpecificity(const CSSSelectorList& selectorList)
 {
     unsigned maxSpecificity = 0;
     for (const CSSSelector* subSelector = selectorList.first(); subSelector; subSelector = CSSSelectorList::next(subSelector))
-        maxSpecificity = std::max(maxSpecificity, selectorSpecificity(*subSelector, true));
+        maxSpecificity = std::max(maxSpecificity, selectorSpecificity(*subSelector));
     return maxSpecificity;
 }
 
-static unsigned simpleSelectorSpecificityInternal(const CSSSelector& simpleSelector, bool isComputingMaximumSpecificity)
+static unsigned simpleSelectorSpecificityInternal(const CSSSelector& simpleSelector)
 {
     ASSERT_WITH_MESSAGE(!simpleSelector.isForPage(), "At the time of this writing, page selectors are not treated as real selectors that are matched. The value computed here only account for real selectors.");
 
@@ -120,18 +120,22 @@ static unsigned simpleSelectorSpecificityInternal(const CSSSelector& simpleSelec
     case CSSSelector::PagePseudoClass:
         break;
     case CSSSelector::PseudoClass:
-        if (simpleSelector.pseudoClassType() == CSSSelector::PseudoClassMatches) {
-            ASSERT_WITH_MESSAGE(simpleSelector.selectorList() && simpleSelector.selectorList()->first(), "The parser should never generate a valid selector for an empty :matches().");
-            if (!isComputingMaximumSpecificity)
-                return 0;
+        switch (simpleSelector.pseudoClassType()) {
+        case CSSSelector::PseudoClassIs:
+        case CSSSelector::PseudoClassMatches:
+        case CSSSelector::PseudoClassNot:
+        case CSSSelector::PseudoClassHas:
             return maxSpecificity(*simpleSelector.selectorList());
+        case CSSSelector::PseudoClassWhere:
+            return 0;
+        case CSSSelector::PseudoClassNthChild:
+        case CSSSelector::PseudoClassNthLastChild:
+        case CSSSelector::PseudoClassHost:
+            return CSSSelector::addSpecificities(static_cast<unsigned>(SelectorSpecificityIncrement::ClassB), simpleSelector.selectorList() ? maxSpecificity(*simpleSelector.selectorList()) : 0);
+        default:
+            break;
         }
-
-        if (simpleSelector.pseudoClassType() == CSSSelector::PseudoClassNot) {
-            ASSERT_WITH_MESSAGE(simpleSelector.selectorList() && simpleSelector.selectorList()->first(), "The parser should never generate a valid selector for an empty :not().");
-            return maxSpecificity(*simpleSelector.selectorList());
-        }
-        FALLTHROUGH;
+        return static_cast<unsigned>(SelectorSpecificityIncrement::ClassB);
     case CSSSelector::Exact:
     case CSSSelector::Class:
     case CSSSelector::Set:
@@ -144,6 +148,10 @@ static unsigned simpleSelectorSpecificityInternal(const CSSSelector& simpleSelec
     case CSSSelector::Tag:
         return (simpleSelector.tagQName().localName() != starAtom()) ? static_cast<unsigned>(SelectorSpecificityIncrement::ClassC) : 0;
     case CSSSelector::PseudoElement:
+        // Slotted only competes with other slotted selectors for specificity,
+        // so whether we add the ClassC specificity shouldn't be observable.
+        if (simpleSelector.pseudoElementType() == CSSSelector::PseudoElementSlotted)
+            return maxSpecificity(*simpleSelector.selectorList());
         return static_cast<unsigned>(SelectorSpecificityIncrement::ClassC);
     case CSSSelector::Unknown:
         return 0;
@@ -154,63 +162,12 @@ static unsigned simpleSelectorSpecificityInternal(const CSSSelector& simpleSelec
 
 unsigned CSSSelector::simpleSelectorSpecificity() const
 {
-    return simpleSelectorSpecificityInternal(*this, false);
+    return simpleSelectorSpecificityInternal(*this);
 }
 
-static unsigned staticSpecificityInternal(const CSSSelector& firstSimpleSelector, bool& ok);
-
-static unsigned simpleSelectorFunctionalPseudoClassStaticSpecificity(const CSSSelector& simpleSelector, bool& ok)
+unsigned CSSSelector::computeSpecificity() const
 {
-    if (simpleSelector.match() == CSSSelector::PseudoClass) {
-        CSSSelector::PseudoClassType pseudoClassType = simpleSelector.pseudoClassType();
-        if (pseudoClassType == CSSSelector::PseudoClassMatches || pseudoClassType == CSSSelector::PseudoClassNthChild || pseudoClassType == CSSSelector::PseudoClassNthLastChild) {
-            const CSSSelectorList* selectorList = simpleSelector.selectorList();
-            if (!selectorList) {
-                ASSERT_WITH_MESSAGE(pseudoClassType != CSSSelector::PseudoClassMatches, ":matches() should never be created without a valid selector list.");
-                return 0;
-            }
-
-            const CSSSelector& firstSubselector = *selectorList->first();
-
-            unsigned initialSpecificity = staticSpecificityInternal(firstSubselector, ok);
-            if (!ok)
-                return 0;
-
-            const CSSSelector* subselector = &firstSubselector;
-            while ((subselector = CSSSelectorList::next(subselector))) {
-                unsigned subSelectorSpecificity = staticSpecificityInternal(*subselector, ok);
-                if (initialSpecificity != subSelectorSpecificity)
-                    ok = false;
-                if (!ok)
-                    return 0;
-            }
-            return initialSpecificity;
-        }
-    }
-    return 0;
-}
-
-static unsigned functionalPseudoClassStaticSpecificity(const CSSSelector& firstSimpleSelector, bool& ok)
-{
-    unsigned total = 0;
-    for (const CSSSelector* selector = &firstSimpleSelector; selector; selector = selector->tagHistory()) {
-        total = CSSSelector::addSpecificities(total, simpleSelectorFunctionalPseudoClassStaticSpecificity(*selector, ok));
-        if (!ok)
-            return 0;
-    }
-    return total;
-}
-
-static unsigned staticSpecificityInternal(const CSSSelector& firstSimpleSelector, bool& ok)
-{
-    unsigned staticSpecificity = selectorSpecificity(firstSimpleSelector, false);
-    return CSSSelector::addSpecificities(staticSpecificity, functionalPseudoClassStaticSpecificity(firstSimpleSelector, ok));
-}
-
-unsigned CSSSelector::staticSpecificity(bool &ok) const
-{
-    ok = true;
-    return staticSpecificityInternal(*this, ok);
+    return selectorSpecificity(*this);
 }
 
 unsigned CSSSelector::addSpecificities(unsigned a, unsigned b)
@@ -281,6 +238,8 @@ PseudoId CSSSelector::pseudoId(PseudoElementType type)
         return PseudoId::Highlight;
     case PseudoElementMarker:
         return PseudoId::Marker;
+    case PseudoElementBackdrop:
+        return PseudoId::Backdrop;
     case PseudoElementBefore:
         return PseudoId::Before;
     case PseudoElementAfter:
@@ -299,7 +258,7 @@ PseudoId CSSSelector::pseudoId(PseudoElementType type)
         return PseudoId::ScrollbarTrackPiece;
     case PseudoElementResizer:
         return PseudoId::Resizer;
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
     case PseudoElementCue:
 #endif
     case PseudoElementSlotted:
@@ -328,12 +287,21 @@ CSSSelector::PseudoElementType CSSSelector::parsePseudoElementType(StringView na
     if (type == PseudoElementHighlight && !RuntimeEnabledFeatures::sharedFeatures().highlightAPIEnabled())
         return PseudoElementUnknown;
 
-    if (type == PseudoElementPart && !RuntimeEnabledFeatures::sharedFeatures().cssShadowPartsEnabled())
-        return PseudoElementUnknown;
-
     return type;
 }
 
+const CSSSelector* CSSSelector::firstInCompound() const
+{
+    auto* selector = this;
+    while (!selector->isFirstInTagHistory()) {
+        auto* previousSelector = selector - 1;
+        if (previousSelector->relation() != Subselector)
+            break;
+        selector = previousSelector;
+    }
+    return selector;
+}
+
 bool CSSSelector::operator==(const CSSSelector& other) const
 {
     const CSSSelector* sel1 = this;
@@ -387,14 +355,8 @@ static void appendPseudoClassFunctionTail(StringBuilder& builder, const CSSSelec
 
 static void appendLangArgumentList(StringBuilder& builder, const Vector<AtomString>& argumentList)
 {
-    unsigned argumentListSize = argumentList.size();
-    for (unsigned i = 0; i < argumentListSize; ++i) {
-        builder.append('"');
-        builder.append(argumentList[i]);
-        builder.append('"');
-        if (i != argumentListSize - 1)
-            builder.appendLiteral(", ");
-    }
+    for (unsigned i = 0, size = argumentList.size(); i < size; ++i)
+        builder.append('"', argumentList[i], '"', i != size - 1 ? ", " : "");
 }
 
 // http://dev.w3.org/csswg/css-syntax/#serializing-anb
@@ -440,11 +402,8 @@ String CSSSelector::selectorText(const String& rightSide) const
     if (match() == CSSSelector::Tag && !m_tagIsForNamespaceRule) {
         if (tagQName().prefix().isNull())
             builder.append(tagQName().localName());
-        else {
-            builder.append(tagQName().prefix().string());
-            builder.append('|');
-            builder.append(tagQName().localName());
-        }
+        else
+            builder.append(tagQName().prefix().string(), '|', tagQName().localName());
     }
 
     const CSSSelector* cs = this;
@@ -459,256 +418,308 @@ String CSSSelector::selectorText(const String& rightSide) const
             switch (cs->pseudoClassType()) {
 #if ENABLE(FULLSCREEN_API)
             case CSSSelector::PseudoClassAnimatingFullScreenTransition:
-                builder.appendLiteral(":-webkit-animating-full-screen-transition");
+                builder.append(":-webkit-animating-full-screen-transition");
                 break;
 #endif
             case CSSSelector::PseudoClassAny: {
-                builder.appendLiteral(":-webkit-any(");
+                builder.append(":-webkit-any(");
                 cs->selectorList()->buildSelectorsText(builder);
                 builder.append(')');
                 break;
             }
             case CSSSelector::PseudoClassAnyLink:
-                builder.appendLiteral(":any-link");
+                builder.append(":any-link");
                 break;
             case CSSSelector::PseudoClassAnyLinkDeprecated:
-                builder.appendLiteral(":-webkit-any-link");
+                builder.append(":-webkit-any-link");
                 break;
             case CSSSelector::PseudoClassAutofill:
-                builder.appendLiteral(":-webkit-autofill");
+                builder.append(":autofill");
+                break;
+            case CSSSelector::PseudoClassAutofillAndObscured:
+                builder.append(":-webkit-autofill-and-obscured");
                 break;
             case CSSSelector::PseudoClassAutofillStrongPassword:
-                builder.appendLiteral(":-webkit-autofill-strong-password");
+                builder.append(":-webkit-autofill-strong-password");
                 break;
             case CSSSelector::PseudoClassAutofillStrongPasswordViewable:
-                builder.appendLiteral(":-webkit-autofill-strong-password-viewable");
-                break;
-            case CSSSelector::PseudoClassDirectFocus:
-                builder.appendLiteral(":-webkit-direct-focus");
+                builder.append(":-webkit-autofill-strong-password-viewable");
                 break;
             case CSSSelector::PseudoClassDrag:
-                builder.appendLiteral(":-webkit-drag");
+                builder.append(":-webkit-drag");
                 break;
             case CSSSelector::PseudoClassFullPageMedia:
-                builder.appendLiteral(":-webkit-full-page-media");
+                builder.append(":-webkit-full-page-media");
                 break;
 #if ENABLE(FULLSCREEN_API)
             case CSSSelector::PseudoClassFullScreen:
-                builder.appendLiteral(":-webkit-full-screen");
+                builder.append(":-webkit-full-screen");
                 break;
             case CSSSelector::PseudoClassFullScreenAncestor:
-                builder.appendLiteral(":-webkit-full-screen-ancestor");
+                builder.append(":-webkit-full-screen-ancestor");
                 break;
             case CSSSelector::PseudoClassFullScreenDocument:
-                builder.appendLiteral(":-webkit-full-screen-document");
+                builder.append(":-webkit-full-screen-document");
                 break;
             case CSSSelector::PseudoClassFullScreenControlsHidden:
-                builder.appendLiteral(":-webkit-full-screen-controls-hidden");
+                builder.append(":-webkit-full-screen-controls-hidden");
                 break;
 #endif
 #if ENABLE(PICTURE_IN_PICTURE_API)
             case CSSSelector::PseudoClassPictureInPicture:
-                builder.appendLiteral(":picture-in-picture");
+                builder.append(":picture-in-picture");
                 break;
 #endif
             case CSSSelector::PseudoClassActive:
-                builder.appendLiteral(":active");
+                builder.append(":active");
                 break;
             case CSSSelector::PseudoClassChecked:
-                builder.appendLiteral(":checked");
+                builder.append(":checked");
                 break;
             case CSSSelector::PseudoClassCornerPresent:
-                builder.appendLiteral(":corner-present");
+                builder.append(":corner-present");
                 break;
             case CSSSelector::PseudoClassDecrement:
-                builder.appendLiteral(":decrement");
+                builder.append(":decrement");
                 break;
             case CSSSelector::PseudoClassDefault:
-                builder.appendLiteral(":default");
+                builder.append(":default");
                 break;
 #if ENABLE(CSS_SELECTORS_LEVEL4)
             case CSSSelector::PseudoClassDir:
-                builder.appendLiteral(":dir(");
+                builder.append(":dir(");
                 appendPseudoClassFunctionTail(builder, cs);
                 break;
 #endif
             case CSSSelector::PseudoClassDisabled:
-                builder.appendLiteral(":disabled");
+                builder.append(":disabled");
                 break;
             case CSSSelector::PseudoClassDoubleButton:
-                builder.appendLiteral(":double-button");
+                builder.append(":double-button");
                 break;
             case CSSSelector::PseudoClassEmpty:
-                builder.appendLiteral(":empty");
+                builder.append(":empty");
                 break;
             case CSSSelector::PseudoClassEnabled:
-                builder.appendLiteral(":enabled");
+                builder.append(":enabled");
                 break;
             case CSSSelector::PseudoClassEnd:
-                builder.appendLiteral(":end");
+                builder.append(":end");
                 break;
             case CSSSelector::PseudoClassFirstChild:
-                builder.appendLiteral(":first-child");
+                builder.append(":first-child");
                 break;
             case CSSSelector::PseudoClassFirstOfType:
-                builder.appendLiteral(":first-of-type");
+                builder.append(":first-of-type");
                 break;
             case CSSSelector::PseudoClassFocus:
-                builder.appendLiteral(":focus");
+                builder.append(":focus");
+                break;
+            case CSSSelector::PseudoClassFocusVisible:
+                builder.append(":focus-visible");
                 break;
             case CSSSelector::PseudoClassFocusWithin:
-                builder.appendLiteral(":focus-within");
+                builder.append(":focus-within");
                 break;
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
             case CSSSelector::PseudoClassFuture:
-                builder.appendLiteral(":future");
+                builder.append(":future");
+                break;
+            case CSSSelector::PseudoClassPlaying:
+                builder.append(":playing");
+                break;
+            case CSSSelector::PseudoClassPaused:
+                builder.append(":paused");
+                break;
+            case CSSSelector::PseudoClassSeeking:
+                builder.append(":seeking");
+                break;
+            case CSSSelector::PseudoClassBuffering:
+                builder.append(":buffering");
+                break;
+            case CSSSelector::PseudoClassStalled:
+                builder.append(":stalled");
+                break;
+            case CSSSelector::PseudoClassMuted:
+                builder.append(":muted");
+                break;
+            case CSSSelector::PseudoClassVolumeLocked:
+                builder.append(":volume-locked");
                 break;
 #endif
+            case CSSSelector::PseudoClassHas:
+                builder.append(":has(");
+                cs->selectorList()->buildSelectorsText(builder);
+                builder.append(')');
+                break;
 #if ENABLE(ATTACHMENT_ELEMENT)
             case CSSSelector::PseudoClassHasAttachment:
-                builder.appendLiteral(":has-attachment");
+                builder.append(":has-attachment");
                 break;
 #endif
             case CSSSelector::PseudoClassHorizontal:
-                builder.appendLiteral(":horizontal");
+                builder.append(":horizontal");
                 break;
             case CSSSelector::PseudoClassHover:
-                builder.appendLiteral(":hover");
+                builder.append(":hover");
                 break;
             case CSSSelector::PseudoClassInRange:
-                builder.appendLiteral(":in-range");
+                builder.append(":in-range");
                 break;
             case CSSSelector::PseudoClassIncrement:
-                builder.appendLiteral(":increment");
+                builder.append(":increment");
                 break;
             case CSSSelector::PseudoClassIndeterminate:
-                builder.appendLiteral(":indeterminate");
+                builder.append(":indeterminate");
                 break;
             case CSSSelector::PseudoClassInvalid:
-                builder.appendLiteral(":invalid");
+                builder.append(":invalid");
                 break;
             case CSSSelector::PseudoClassLang:
-                builder.appendLiteral(":lang(");
+                builder.append(":lang(");
                 ASSERT_WITH_MESSAGE(cs->argumentList() && !cs->argumentList()->isEmpty(), "An empty :lang() is invalid and should never be generated by the parser.");
                 appendLangArgumentList(builder, *cs->argumentList());
                 builder.append(')');
                 break;
             case CSSSelector::PseudoClassLastChild:
-                builder.appendLiteral(":last-child");
+                builder.append(":last-child");
                 break;
             case CSSSelector::PseudoClassLastOfType:
-                builder.appendLiteral(":last-of-type");
+                builder.append(":last-of-type");
                 break;
             case CSSSelector::PseudoClassLink:
-                builder.appendLiteral(":link");
+                builder.append(":link");
+                break;
+            case CSSSelector::PseudoClassModalDialog:
+                builder.append(":-internal-modal-dialog");
                 break;
             case CSSSelector::PseudoClassNoButton:
-                builder.appendLiteral(":no-button");
+                builder.append(":no-button");
                 break;
             case CSSSelector::PseudoClassNot:
-                builder.appendLiteral(":not(");
+                builder.append(":not(");
                 cs->selectorList()->buildSelectorsText(builder);
                 builder.append(')');
                 break;
             case CSSSelector::PseudoClassNthChild:
-                builder.appendLiteral(":nth-child(");
+                builder.append(":nth-child(");
                 outputNthChildAnPlusB(*cs, builder);
                 if (const CSSSelectorList* selectorList = cs->selectorList()) {
-                    builder.appendLiteral(" of ");
+                    builder.append(" of ");
                     selectorList->buildSelectorsText(builder);
                 }
                 builder.append(')');
                 break;
             case CSSSelector::PseudoClassNthLastChild:
-                builder.appendLiteral(":nth-last-child(");
+                builder.append(":nth-last-child(");
                 outputNthChildAnPlusB(*cs, builder);
                 if (const CSSSelectorList* selectorList = cs->selectorList()) {
-                    builder.appendLiteral(" of ");
+                    builder.append(" of ");
                     selectorList->buildSelectorsText(builder);
                 }
                 builder.append(')');
                 break;
             case CSSSelector::PseudoClassNthLastOfType:
-                builder.appendLiteral(":nth-last-of-type(");
+                builder.append(":nth-last-of-type(");
                 appendPseudoClassFunctionTail(builder, cs);
                 break;
             case CSSSelector::PseudoClassNthOfType:
-                builder.appendLiteral(":nth-of-type(");
+                builder.append(":nth-of-type(");
                 appendPseudoClassFunctionTail(builder, cs);
                 break;
             case CSSSelector::PseudoClassOnlyChild:
-                builder.appendLiteral(":only-child");
+                builder.append(":only-child");
                 break;
             case CSSSelector::PseudoClassOnlyOfType:
-                builder.appendLiteral(":only-of-type");
+                builder.append(":only-of-type");
                 break;
             case CSSSelector::PseudoClassOptional:
-                builder.appendLiteral(":optional");
+                builder.append(":optional");
                 break;
+            case CSSSelector::PseudoClassIs: {
+                builder.append(":is(");
+                cs->selectorList()->buildSelectorsText(builder);
+                builder.append(')');
+                break;
+            }
             case CSSSelector::PseudoClassMatches: {
-                builder.appendLiteral(":matches(");
+                builder.append(":matches(");
+                cs->selectorList()->buildSelectorsText(builder);
+                builder.append(')');
+                break;
+            }
+            case CSSSelector::PseudoClassWhere: {
+                builder.append(":where(");
                 cs->selectorList()->buildSelectorsText(builder);
                 builder.append(')');
                 break;
             }
             case CSSSelector::PseudoClassPlaceholderShown:
-                builder.appendLiteral(":placeholder-shown");
+                builder.append(":placeholder-shown");
                 break;
             case CSSSelector::PseudoClassOutOfRange:
-                builder.appendLiteral(":out-of-range");
+                builder.append(":out-of-range");
                 break;
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
             case CSSSelector::PseudoClassPast:
-                builder.appendLiteral(":past");
+                builder.append(":past");
                 break;
 #endif
             case CSSSelector::PseudoClassReadOnly:
-                builder.appendLiteral(":read-only");
+                builder.append(":read-only");
                 break;
             case CSSSelector::PseudoClassReadWrite:
-                builder.appendLiteral(":read-write");
+                builder.append(":read-write");
                 break;
             case CSSSelector::PseudoClassRequired:
-                builder.appendLiteral(":required");
+                builder.append(":required");
                 break;
 #if ENABLE(CSS_SELECTORS_LEVEL4)
             case CSSSelector::PseudoClassRole:
-                builder.appendLiteral(":role(");
+                builder.append(":role(");
                 appendPseudoClassFunctionTail(builder, cs);
                 break;
 #endif
             case CSSSelector::PseudoClassRoot:
-                builder.appendLiteral(":root");
+                builder.append(":root");
                 break;
             case CSSSelector::PseudoClassScope:
-                builder.appendLiteral(":scope");
+                builder.append(":scope");
                 break;
+            case CSSSelector::PseudoClassRelativeScope:
+                // Just remove the space from the start to generate a relative selector string like in ":has(> foo)".
+                return rightSide.substring(1);
             case CSSSelector::PseudoClassSingleButton:
-                builder.appendLiteral(":single-button");
+                builder.append(":single-button");
                 break;
             case CSSSelector::PseudoClassStart:
-                builder.appendLiteral(":start");
+                builder.append(":start");
                 break;
             case CSSSelector::PseudoClassTarget:
-                builder.appendLiteral(":target");
+                builder.append(":target");
                 break;
             case CSSSelector::PseudoClassValid:
-                builder.appendLiteral(":valid");
+                builder.append(":valid");
                 break;
             case CSSSelector::PseudoClassVertical:
-                builder.appendLiteral(":vertical");
+                builder.append(":vertical");
                 break;
             case CSSSelector::PseudoClassVisited:
-                builder.appendLiteral(":visited");
+                builder.append(":visited");
                 break;
             case CSSSelector::PseudoClassWindowInactive:
-                builder.appendLiteral(":window-inactive");
+                builder.append(":window-inactive");
                 break;
             case CSSSelector::PseudoClassHost:
-                builder.appendLiteral(":host");
+                builder.append(":host");
+                if (auto* selectorList = cs->selectorList()) {
+                    builder.append('(');
+                    selectorList->buildSelectorsText(builder);
+                    builder.append(')');
+                }
                 break;
             case CSSSelector::PseudoClassDefined:
-                builder.appendLiteral(":defined");
+                builder.append(":defined");
                 break;
             case CSSSelector::PseudoClassUnknown:
                 ASSERT_NOT_REACHED();
@@ -716,48 +727,46 @@ String CSSSelector::selectorText(const String& rightSide) const
         } else if (cs->match() == CSSSelector::PseudoElement) {
             switch (cs->pseudoElementType()) {
             case CSSSelector::PseudoElementSlotted:
-                builder.appendLiteral("::slotted(");
+                builder.append("::slotted(");
                 cs->selectorList()->buildSelectorsText(builder);
                 builder.append(')');
                 break;
             case CSSSelector::PseudoElementPart: {
-                builder.appendLiteral("::part(");
+                builder.append("::part(");
                 bool isFirst = true;
                 for (auto& partName : *cs->argumentList()) {
                     if (!isFirst)
                         builder.append(' ');
                     isFirst = false;
-                    builder.append(partName);
+                    serializeIdentifier(partName, builder);
                 }
                 builder.append(')');
                 break;
             }
             case CSSSelector::PseudoElementWebKitCustomLegacyPrefixed:
                 if (cs->value() == "placeholder")
-                    builder.appendLiteral("::-webkit-input-placeholder");
+                    builder.append("::-webkit-input-placeholder");
+                if (cs->value() == "file-selector-button")
+                    builder.append("::-webkit-file-upload-button");
                 break;
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
             case CSSSelector::PseudoElementCue: {
                 if (auto* selectorList = cs->selectorList()) {
-                    builder.appendLiteral("::cue(");
+                    builder.append("::cue(");
                     selectorList->buildSelectorsText(builder);
                     builder.append(')');
                 } else
-                    builder.appendLiteral("::cue");
+                    builder.append("::cue");
                 break;
             }
 #endif
             default:
-                builder.appendLiteral("::");
-                builder.append(cs->serializingValue());
+                builder.append("::", cs->serializingValue());
             }
         } else if (cs->isAttributeSelector()) {
             builder.append('[');
-            const AtomString& prefix = cs->attribute().prefix();
-            if (!prefix.isEmpty()) {
-                builder.append(prefix);
-                builder.append('|');
-            }
+            if (auto& prefix = cs->attribute().prefix(); !prefix.isEmpty())
+                builder.append(prefix, '|');
             builder.append(cs->attribute().localName());
             switch (cs->match()) {
                 case CSSSelector::Exact:
@@ -768,19 +777,19 @@ String CSSSelector::selectorText(const String& rightSide) const
                     builder.append(']');
                     break;
                 case CSSSelector::List:
-                    builder.appendLiteral("~=");
+                    builder.append("~=");
                     break;
                 case CSSSelector::Hyphen:
-                    builder.appendLiteral("|=");
+                    builder.append("|=");
                     break;
                 case CSSSelector::Begin:
-                    builder.appendLiteral("^=");
+                    builder.append("^=");
                     break;
                 case CSSSelector::End:
-                    builder.appendLiteral("$=");
+                    builder.append("$=");
                     break;
                 case CSSSelector::Contain:
-                    builder.appendLiteral("*=");
+                    builder.append("*=");
                     break;
                 default:
                     break;
@@ -788,20 +797,20 @@ String CSSSelector::selectorText(const String& rightSide) const
             if (cs->match() != CSSSelector::Set) {
                 serializeString(cs->serializingValue(), builder);
                 if (cs->attributeValueMatchingIsCaseInsensitive())
-                    builder.appendLiteral(" i]");
+                    builder.append(" i]");
                 else
                     builder.append(']');
             }
         } else if (cs->match() == CSSSelector::PagePseudoClass) {
             switch (cs->pagePseudoClassType()) {
             case PagePseudoClassFirst:
-                builder.appendLiteral(":first");
+                builder.append(":first");
                 break;
             case PagePseudoClassLeft:
-                builder.appendLiteral(":left");
+                builder.append(":left");
                 break;
             case PagePseudoClassRight:
-                builder.appendLiteral(":right");
+                builder.append(":right");
                 break;
             }
         }
@@ -827,10 +836,14 @@ String CSSSelector::selectorText(const String& rightSide) const
             FALLTHROUGH;
 #endif
         case CSSSelector::ShadowDescendant:
-            return tagHistory->selectorText(builder.toString() + rightSide);
+        case CSSSelector::ShadowPartDescendant:
+        case CSSSelector::ShadowSlotted:
+            builder.append(rightSide);
+            return tagHistory->selectorText(builder.toString());
         }
     }
-    return builder.toString() + rightSide;
+    builder.append(rightSide);
+    return builder.toString();
 }
 
 void CSSSelector::setAttribute(const QualifiedName& value, bool convertToLowercase, AttributeMatchType matchType)
