@@ -25,6 +25,7 @@
 
 #include "AXObjectCache.h"
 #include "AllDescendantsCollection.h"
+#include "ChildChangeInvalidation.h"
 #include "ChildListMutationScope.h"
 #include "ClassCollection.h"
 #include "CommonVM.h"
@@ -38,10 +39,11 @@
 #include "HTMLOptionsCollection.h"
 #include "HTMLSlotElement.h"
 #include "HTMLTableRowsCollection.h"
-#include "InlineTextBox.h"
 #include "InspectorInstrumentation.h"
 #include "JSNode.h"
 #include "LabelsNodeList.h"
+#include "LegacyInlineTextBox.h"
+#include "LegacyRootInlineBox.h"
 #include "MutationEvent.h"
 #include "NameNodeList.h"
 #include "NodeRareData.h"
@@ -51,9 +53,8 @@
 #include "RenderTheme.h"
 #include "RenderTreeUpdater.h"
 #include "RenderWidget.h"
-#include "RootInlineBox.h"
 #include "SVGDocumentExtensions.h"
-#include "SVGElement.h"
+#include "SVGElementTypeHelpers.h"
 #include "SVGNames.h"
 #include "SVGUseElement.h"
 #include "ScriptDisallowedScope.h"
@@ -61,8 +62,8 @@
 #include "SlotAssignment.h"
 #include "TemplateContentDocumentFragment.h"
 #include <algorithm>
+#include <variant>
 #include <wtf/IsoMallocInlines.h>
-#include <wtf/Variant.h>
 
 namespace WebCore {
 
@@ -78,11 +79,21 @@ unsigned ScriptDisallowedScope::s_count = 0;
 ScriptDisallowedScope::EventAllowedScope* ScriptDisallowedScope::EventAllowedScope::s_currentScope = nullptr;
 #endif
 
-ALWAYS_INLINE NodeVector ContainerNode::removeAllChildrenWithScriptAssertion(ChildChangeSource source, DeferChildrenChanged deferChildrenChanged)
+ALWAYS_INLINE NodeVector ContainerNode::removeAllChildrenWithScriptAssertion(ChildChange::Source source, DeferChildrenChanged deferChildrenChanged)
 {
     auto children = collectChildNodes(*this);
 
-    if (source == ContainerNode::ChildChangeSource::API) {
+    if (UNLIKELY(isDocumentFragmentForInnerOuterHTML())) {
+        ScriptDisallowedScope::InMainThread scriptDisallowedScope;
+        RELEASE_ASSERT(!connectedSubframeCount() && !hasRareData() && !wrapper());
+        ASSERT(!weakPtrFactory().isInitialized());
+        while (RefPtr<Node> child = m_firstChild)
+            removeBetween(nullptr, child->nextSibling(), *child);
+        document().incDOMTreeVersion();
+        return children;
+    }
+
+    if (source == ChildChange::Source::API) {
         ChildListMutationScope mutation(*this);
         for (auto& child : children) {
             mutation.willRemoveChild(child.get());
@@ -90,9 +101,9 @@ ALWAYS_INLINE NodeVector ContainerNode::removeAllChildrenWithScriptAssertion(Chi
             dispatchChildRemovalEvents(child);
         }
     } else {
-        ASSERT(source == ContainerNode::ChildChangeSource::Parser);
+        ASSERT(source == ChildChange::Source::Parser);
         ScriptDisallowedScope::InMainThread scriptDisallowedScope;
-        if (UNLIKELY(document().hasMutationObserversOfType(MutationObserver::ChildList))) {
+        if (UNLIKELY(document().hasMutationObserversOfType(MutationObserverOptionType::ChildList))) {
             ChildListMutationScope mutation(*this);
             for (auto& child : children)
                 mutation.willRemoveChild(child.get());
@@ -101,26 +112,54 @@ ALWAYS_INLINE NodeVector ContainerNode::removeAllChildrenWithScriptAssertion(Chi
 
     disconnectSubframesIfNeeded(*this, DescendantsOnly);
 
+    ContainerNode::ChildChange childChange { ChildChange::Type::AllChildrenRemoved, nullptr, nullptr, nullptr, source };
+
     WidgetHierarchyUpdatesSuspensionScope suspendWidgetHierarchyUpdates;
     ScriptDisallowedScope::InMainThread scriptDisallowedScope;
+    {
+        Style::ChildChangeInvalidation styleInvalidation(*this, childChange);
 
-    if (UNLIKELY(isShadowRoot() || isInShadowTree()))
-        containingShadowRoot()->willRemoveAllChildren(*this);
+        if (UNLIKELY(isShadowRoot() || isInShadowTree()))
+            containingShadowRoot()->willRemoveAllChildren(*this);
 
-    document().nodeChildrenWillBeRemoved(*this);
+        document().nodeChildrenWillBeRemoved(*this);
 
-    while (RefPtr<Node> child = m_firstChild) {
-        removeBetween(nullptr, child->nextSibling(), *child);
-        notifyChildNodeRemoved(*this, *child);
+        while (RefPtr<Node> child = m_firstChild) {
+            removeBetween(nullptr, child->nextSibling(), *child);
+            auto subtreeObservability = notifyChildNodeRemoved(*this, *child);
+            if (source == ChildChange::Source::API && subtreeObservability == RemovedSubtreeObservability::MaybeObservableByRefPtr)
+                willCreatePossiblyOrphanedTreeByRemoval(*child);
+        }
     }
 
+    ASSERT_WITH_SECURITY_IMPLICATION(!document().selection().selection().isOrphan());
+
     if (deferChildrenChanged == DeferChildrenChanged::No)
-        childrenChanged(ContainerNode::ChildChange { ContainerNode::AllChildrenRemoved, nullptr, nullptr, source });
+        childrenChanged(childChange);
 
     return children;
 }
 
-ALWAYS_INLINE bool ContainerNode::removeNodeWithScriptAssertion(Node& childToRemove, ChildChangeSource source)
+static ContainerNode::ChildChange makeChildChangeForRemoval(Node& childToRemove, ContainerNode::ChildChange::Source source)
+{
+    auto changeType = [&] {
+        if (is<Element>(childToRemove))
+            return ContainerNode::ChildChange::Type::ElementRemoved;
+        if (is<Text>(childToRemove))
+            return ContainerNode::ChildChange::Type::TextRemoved;
+        return ContainerNode::ChildChange::Type::NonContentsChildRemoved;
+    }();
+
+    return {
+        changeType,
+        dynamicDowncast<Element>(childToRemove),
+        ElementTraversal::previousSibling(childToRemove),
+        ElementTraversal::nextSibling(childToRemove),
+        source
+    };
+}
+
+ALWAYS_INLINE bool ContainerNode::removeNodeWithScriptAssertion(Node& childToRemove, ChildChange::Source source)
 {
     Ref<Node> protectedChildToRemove(childToRemove);
     ASSERT_WITH_SECURITY_IMPLICATION(childToRemove.parentNode() == this);
@@ -130,28 +169,31 @@ ALWAYS_INLINE bool ContainerNode::removeNodeWithScriptAssertion(Node& childToRem
     }
 
     ASSERT_WITH_SECURITY_IMPLICATION(ScriptDisallowedScope::InMainThread::isEventDispatchAllowedInSubtree(childToRemove));
-    if (source == ContainerNode::ChildChangeSource::API) {
+    if (source == ChildChange::Source::API) {
         childToRemove.notifyMutationObserversNodeWillDetach();
         dispatchChildRemovalEvents(protectedChildToRemove);
         if (childToRemove.parentNode() != this)
             return false;
     }
 
-    if (source == ContainerNode::ChildChangeSource::Parser) {
+    if (source == ChildChange::Source::Parser) {
         // FIXME: Merge these two code paths. It's a bug in the parser not to update connectedSubframeCount in time.
         disconnectSubframesIfNeeded(*this, DescendantsOnly);
     } else {
-        if (is<ContainerNode>(childToRemove))
-            disconnectSubframesIfNeeded(downcast<ContainerNode>(childToRemove), RootAndDescendants);
+        if (auto containerChild = dynamicDowncast<ContainerNode>(childToRemove))
+            disconnectSubframesIfNeeded(*containerChild, RootAndDescendants);
     }
 
     if (childToRemove.parentNode() != this)
         return false;
 
-    ChildChange change;
+    auto childChange = makeChildChangeForRemoval(childToRemove, source);
+
+    RemovedSubtreeObservability subtreeObservability;
     {
         WidgetHierarchyUpdatesSuspensionScope suspendWidgetHierarchyUpdates;
         ScriptDisallowedScope::InMainThread scriptDisallowedScope;
+        Style::ChildChangeInvalidation styleInvalidation(*this, childChange);
 
         if (UNLIKELY(isShadowRoot() || isInShadowTree()))
             containingShadowRoot()->resolveSlotsBeforeNodeInsertionOrRemoval();
@@ -163,30 +205,57 @@ ALWAYS_INLINE bool ContainerNode::removeNodeWithScriptAssertion(Node& childToRem
 
         RefPtr<Node> previousSibling = childToRemove.previousSibling();
         RefPtr<Node> nextSibling = childToRemove.nextSibling();
-        removeBetween(previousSibling.get(), nextSibling.get(), childToRemove);
-        notifyChildNodeRemoved(*this, childToRemove);
 
-        change.type = is<Element>(childToRemove) ? ElementRemoved : (is<Text>(childToRemove) ? TextRemoved : NonContentsChildRemoved);
-        change.previousSiblingElement = (!previousSibling || is<Element>(*previousSibling)) ? downcast<Element>(previousSibling.get()) : ElementTraversal::previousSibling(*previousSibling);
-        change.nextSiblingElement = (!nextSibling || is<Element>(*nextSibling)) ? downcast<Element>(nextSibling.get()) : ElementTraversal::nextSibling(*nextSibling);
-        change.source = source;
+        removeBetween(previousSibling.get(), nextSibling.get(), childToRemove);
+        subtreeObservability = notifyChildNodeRemoved(*this, childToRemove);
     }
 
+    if (source == ChildChange::Source::API && subtreeObservability == RemovedSubtreeObservability::MaybeObservableByRefPtr)
+        willCreatePossiblyOrphanedTreeByRemoval(childToRemove);
+
+    ASSERT_WITH_SECURITY_IMPLICATION(!document().selection().selection().isOrphan());
+
     // FIXME: Move childrenChanged into ScriptDisallowedScope block.
-    childrenChanged(change);
+    childrenChanged(childChange);
 
     return true;
 }
 
 enum class ReplacedAllChildren { No, Yes };
 
-template<typename DOMInsertionWork>
-static ALWAYS_INLINE void executeNodeInsertionWithScriptAssertion(ContainerNode& containerNode, Node& child,
-    ContainerNode::ChildChangeSource source, ReplacedAllChildren replacedAllChildren, DOMInsertionWork doNodeInsertion)
+static ContainerNode::ChildChange makeChildChangeForInsertion(ContainerNode& containerNode, Node& child, Node* beforeChild, ContainerNode::ChildChange::Source source, ReplacedAllChildren replacedAllChildren)
 {
+    if (replacedAllChildren == ReplacedAllChildren::Yes)
+        return { ContainerNode::ChildChange::Type::AllChildrenReplaced, nullptr, nullptr, nullptr, source };
+
+    auto changeType = [&] {
+        if (is<Element>(child))
+            return ContainerNode::ChildChange::Type::ElementInserted;
+        if (is<Text>(child))
+            return ContainerNode::ChildChange::Type::TextInserted;
+        return ContainerNode::ChildChange::Type::NonContentsChildInserted;
+    }();
+
+    return {
+        changeType,
+        dynamicDowncast<Element>(child),
+        beforeChild ? ElementTraversal::previousSibling(*beforeChild) : ElementTraversal::lastChild(containerNode),
+        !beforeChild || is<Element>(*beforeChild) ? downcast<Element>(beforeChild) : ElementTraversal::nextSibling(*beforeChild),
+        source
+    };
+}
+
+template<typename DOMInsertionWork>
+static ALWAYS_INLINE void executeNodeInsertionWithScriptAssertion(ContainerNode& containerNode, Node& child, Node* beforeChild,
+    ContainerNode::ChildChange::Source source, ReplacedAllChildren replacedAllChildren, DOMInsertionWork doNodeInsertion)
+{
+    auto childChange = makeChildChangeForInsertion(containerNode, child, beforeChild, source, replacedAllChildren);
+
     NodeVector postInsertionNotificationTargets;
     {
+        WidgetHierarchyUpdatesSuspensionScope suspendWidgetHierarchyUpdates;
         ScriptDisallowedScope::InMainThread scriptDisallowedScope;
+        Style::ChildChangeInvalidation styleInvalidation(containerNode, childChange);
 
         if (UNLIKELY(containerNode.isShadowRoot() || containerNode.isInShadowTree()))
             containerNode.containingShadowRoot()->resolveSlotsBeforeNodeInsertionOrRemoval();
@@ -197,38 +266,36 @@ static ALWAYS_INLINE void executeNodeInsertionWithScriptAssertion(ContainerNode&
     }
 
     // FIXME: Move childrenChanged into ScriptDisallowedScope block.
-    if (replacedAllChildren == ReplacedAllChildren::Yes)
-        containerNode.childrenChanged(ContainerNode::ChildChange { ContainerNode::AllChildrenReplaced, nullptr, nullptr, source });
-    else {
-        containerNode.childrenChanged(ContainerNode::ChildChange {
-            child.isElementNode() ? ContainerNode::ElementInserted : (child.isTextNode() ? ContainerNode::TextInserted : ContainerNode::NonContentsChildInserted),
-            ElementTraversal::previousSibling(child),
-            ElementTraversal::nextSibling(child),
-            source
-        });
-    }
+    containerNode.childrenChanged(childChange);
 
     ASSERT(ScriptDisallowedScope::InMainThread::isEventDispatchAllowedInSubtree(child));
     for (auto& target : postInsertionNotificationTargets)
         target->didFinishInsertingNode();
 
-    if (source == ContainerNode::ChildChangeSource::API)
+    if (source == ContainerNode::ChildChange::Source::API)
         dispatchChildInsertionEvents(child);
 }
 
-static ExceptionOr<void> collectChildrenAndRemoveFromOldParent(Node& node, NodeVector& nodes)
+ExceptionOr<void> ContainerNode::removeSelfOrChildNodesForInsertion(Node& child, NodeVector& nodesForInsertion)
 {
-    if (!is<DocumentFragment>(node)) {
-        nodes.append(node);
-        auto* oldParent = node.parentNode();
-        if (!oldParent)
+    if (auto fragment = dynamicDowncast<DocumentFragment>(child)) {
+        if (!fragment->hasChildNodes())
             return { };
-        return oldParent->removeChild(node);
+
+        auto removedChildNodes = fragment->removeAllChildrenWithScriptAssertion(ContainerNode::ChildChange::Source::API);
+        nodesForInsertion.swap(removedChildNodes);
+
+        fragment->rebuildSVGExtensionsElementsIfNecessary();
+        fragment->dispatchSubtreeModifiedEvent();
+
+        return { };
     }
 
-    nodes = collectChildNodes(node);
-    downcast<DocumentFragment>(node).removeChildren();
-    return { };
+    nodesForInsertion.append(child);
+    RefPtr oldParent = child.parentNode();
+    if (!oldParent)
+        return { };
+    return oldParent->removeChild(child);
 }
 
 // FIXME: This function must get a new name.
@@ -247,21 +314,21 @@ void ContainerNode::removeDetachedChildren()
 
 static inline void destroyRenderTreeIfNeeded(Node& child)
 {
-    bool isElement = is<Element>(child);
-    auto hasDisplayContents = isElement && downcast<Element>(child).hasDisplayContents();
+    auto childAsElement = dynamicDowncast<Element>(child);
+    auto hasDisplayContents = childAsElement && childAsElement->hasDisplayContents();
     if (!child.renderer() && !hasDisplayContents)
         return;
-    if (isElement)
-        RenderTreeUpdater::tearDownRenderers(downcast<Element>(child));
-    else if (is<Text>(child))
-        RenderTreeUpdater::tearDownRenderer(downcast<Text>(child));
+    if (childAsElement)
+        RenderTreeUpdater::tearDownRenderers(*childAsElement);
+    else if (auto text = dynamicDowncast<Text>(child))
+        RenderTreeUpdater::tearDownRenderer(*text);
 }
 
 void ContainerNode::takeAllChildrenFrom(ContainerNode* oldParent)
 {
     ASSERT(oldParent);
 
-    auto children = oldParent->removeAllChildrenWithScriptAssertion(ChildChangeSource::Parser);
+    auto children = oldParent->removeAllChildrenWithScriptAssertion(ChildChange::Source::Parser);
 
     // FIXME: assert that we don't dispatch events here since this container node is still disconnected.
     for (auto& child : children) {
@@ -299,8 +366,8 @@ static bool containsIncludingHostElements(const Node& possibleAncestor, const No
             return true;
         const ContainerNode* parent = currentNode->parentNode();
         if (!parent) {
-            if (is<ShadowRoot>(currentNode))
-                parent = downcast<ShadowRoot>(currentNode)->host();
+            if (auto shadowRoot = dynamicDowncast<ShadowRoot>(currentNode))
+                parent = shadowRoot->host();
             else if (is<DocumentFragment>(*currentNode) && downcast<DocumentFragment>(*currentNode).isTemplateContent())
                 parent = static_cast<const TemplateContentDocumentFragment*>(currentNode)->host();
         }
@@ -333,8 +400,8 @@ static inline ExceptionOr<void> checkAcceptChild(ContainerNode& newParent, Node&
     if (shouldValidateChildParent == ShouldValidateChildParent::Yes && refChild && refChild->parentNode() != &newParent)
         return Exception { NotFoundError };
 
-    if (is<Document>(newParent)) {
-        if (!downcast<Document>(newParent).canAcceptChild(newChild, refChild, operation))
+    if (auto newParentDocument = dynamicDowncast<Document>(newParent)) {
+        if (!newParentDocument->canAcceptChild(newChild, refChild, operation))
             return Exception { HierarchyRequestError };
     } else if (!isChildTypeAllowed(newParent, newChild))
         return Exception { HierarchyRequestError };
@@ -385,13 +452,13 @@ ExceptionOr<void> ContainerNode::insertBefore(Node& newChild, Node* refChild)
     Ref<Node> next(*refChild);
 
     NodeVector targets;
-    auto removeResult = collectChildrenAndRemoveFromOldParent(newChild, targets);
+    auto removeResult = removeSelfOrChildNodesForInsertion(newChild, targets);
     if (removeResult.hasException())
         return removeResult.releaseException();
     if (targets.isEmpty())
         return { };
 
-    // We need this extra check because collectChildrenAndRemoveFromOldParent() can fire mutation events.
+    // We need this extra check because removeSelfOrChildNodesForInsertion() can fire mutation events.
     for (auto& child : targets) {
         auto checkAcceptResult = checkAcceptChildGuaranteedNodeTypes(*this, child);
         if (checkAcceptResult.hasException())
@@ -411,7 +478,7 @@ ExceptionOr<void> ContainerNode::insertBefore(Node& newChild, Node* refChild)
         if (child->parentNode())
             break;
 
-        executeNodeInsertionWithScriptAssertion(*this, child.get(), ChildChangeSource::API, ReplacedAllChildren::No, [&] {
+        executeNodeInsertionWithScriptAssertion(*this, child.get(), next.ptr(), ChildChange::Source::API, ReplacedAllChildren::No, [&] {
             child->setTreeScopeRecursively(treeScope());
             insertBeforeCommon(next, child);
         });
@@ -470,7 +537,7 @@ void ContainerNode::parserInsertBefore(Node& newChild, Node& nextChild)
     if (nextChild.previousSibling() == &newChild || &nextChild == &newChild) // nothing to do
         return;
 
-    executeNodeInsertionWithScriptAssertion(*this, newChild, ChildChangeSource::Parser, ReplacedAllChildren::No, [&] {
+    executeNodeInsertionWithScriptAssertion(*this, newChild, &nextChild, ChildChange::Source::Parser, ReplacedAllChildren::No, [&] {
         if (&document() != &newChild.document())
             document().adoptNode(newChild);
 
@@ -500,12 +567,12 @@ ExceptionOr<void> ContainerNode::replaceChild(Node& newChild, Node& oldChild)
     NodeVector targets;
     {
         ChildListMutationScope mutation(*this);
-        auto collectResult = collectChildrenAndRemoveFromOldParent(newChild, targets);
+        auto collectResult = removeSelfOrChildNodesForInsertion(newChild, targets);
         if (collectResult.hasException())
             return collectResult.releaseException();
     }
 
-    // Do this one more time because collectChildrenAndRemoveFromOldParent() fires a MutationEvent.
+    // Do this one more time because removeSelfOrChildNodesForInsertion() fires a MutationEvent.
     for (auto& child : targets) {
         validityResult = checkPreReplacementValidity(*this, child, oldChild, ShouldValidateChildParent::No);
         if (validityResult.hasException())
@@ -544,7 +611,7 @@ ExceptionOr<void> ContainerNode::replaceChild(Node& newChild, Node& oldChild)
         if (child->parentNode())
             break;
 
-        executeNodeInsertionWithScriptAssertion(*this, child.get(), ChildChangeSource::API, ReplacedAllChildren::No, [&] {
+        executeNodeInsertionWithScriptAssertion(*this, child.get(), refChild.get(), ChildChange::Source::API, ReplacedAllChildren::No, [&] {
             child->setTreeScopeRecursively(treeScope());
             if (refChild)
                 insertBeforeCommon(*refChild, child.get());
@@ -574,7 +641,7 @@ ExceptionOr<void> ContainerNode::removeChild(Node& oldChild)
     if (oldChild.parentNode() != this)
         return Exception { NotFoundError };
 
-    if (!removeNodeWithScriptAssertion(oldChild, ChildChangeSource::API))
+    if (!removeNodeWithScriptAssertion(oldChild, ChildChange::Source::API))
         return Exception { NotFoundError };
 
     rebuildSVGExtensionsElementsIfNecessary();
@@ -593,6 +660,9 @@ void ContainerNode::removeBetween(Node* previousChild, Node* nextChild, Node& ol
 
     destroyRenderTreeIfNeeded(oldChild);
 
+    if (UNLIKELY(hasShadowRootContainingSlots()))
+        shadowRoot()->willRemoveAssignedNode(oldChild);
+
     if (nextChild) {
         nextChild->setPreviousSibling(previousChild);
         oldChild.setNextSibling(nullptr);
@@ -619,45 +689,45 @@ void ContainerNode::removeBetween(Node* previousChild, Node* nextChild, Node& ol
 
 void ContainerNode::parserRemoveChild(Node& oldChild)
 {
-    removeNodeWithScriptAssertion(oldChild, ChildChangeSource::Parser);
+    removeNodeWithScriptAssertion(oldChild, ChildChange::Source::Parser);
 }
 
 // https://dom.spec.whatwg.org/#concept-node-replace-all
-void ContainerNode::replaceAllChildren(std::nullptr_t)
+void ContainerNode::replaceAll(Node* node)
 {
-    ChildListMutationScope mutation(*this);
-    removeChildren();
-}
+    if (!node) {
+        ChildListMutationScope mutation(*this);
+        removeChildren();
+        return;
+    }
 
-// https://dom.spec.whatwg.org/#concept-node-replace-all
-void ContainerNode::replaceAllChildren(Ref<Node>&& node)
-{
-    // This function assumes the input node is not a DocumentFragment and is parentless to decrease complexity.
-    ASSERT(!is<DocumentFragment>(node));
-    ASSERT(!node->parentNode());
+    // FIXME: The code below is roughly correct for a new text node with no parent, but needs enhancement to work properly for more complex cases.
 
     if (!hasChildNodes()) {
-        // appendChildWithoutPreInsertionValidityCheck() can only throw when node has a parent and we already asserted it doesn't.
-        auto result = appendChildWithoutPreInsertionValidityCheck(node);
-        ASSERT_UNUSED(result, !result.hasException());
+        appendChildWithoutPreInsertionValidityCheck(*node);
         return;
     }
 
     Ref<ContainerNode> protectedThis(*this);
     ChildListMutationScope mutation(*this);
-    removeAllChildrenWithScriptAssertion(ChildChangeSource::API, DeferChildrenChanged::Yes);
+    removeAllChildrenWithScriptAssertion(ChildChange::Source::API, DeferChildrenChanged::Yes);
 
-    executeNodeInsertionWithScriptAssertion(*this, node.get(), ChildChangeSource::API, ReplacedAllChildren::Yes, [&] {
-        ASSERT(!ensurePreInsertionValidity(node, nullptr).hasException());
+    executeNodeInsertionWithScriptAssertion(*this, *node, nullptr, ChildChange::Source::API, ReplacedAllChildren::Yes, [&] {
         InspectorInstrumentation::willInsertDOMNode(document(), *this);
         node->setTreeScopeRecursively(treeScope());
-        appendChildCommon(node);
+        appendChildCommon(*node);
     });
 
     rebuildSVGExtensionsElementsIfNecessary();
     dispatchSubtreeModifiedEvent();
 }
 
+// https://dom.spec.whatwg.org/#string-replace-all
+void ContainerNode::stringReplaceAll(const String& string)
+{
+    replaceAll(string.isEmpty() ? nullptr : document().createTextNode(string).ptr());
+}
+
 inline void ContainerNode::rebuildSVGExtensionsElementsIfNecessary()
 {
     if (document().svgExtensions() && !is<SVGUseElement>(shadowHost()))
@@ -672,7 +742,7 @@ void ContainerNode::removeChildren()
         return;
 
     Ref<ContainerNode> protectedThis(*this);
-    removeAllChildrenWithScriptAssertion(ChildChangeSource::API);
+    removeAllChildrenWithScriptAssertion(ChildChange::Source::API);
 
     rebuildSVGExtensionsElementsIfNecessary();
     dispatchSubtreeModifiedEvent();
@@ -697,14 +767,14 @@ ExceptionOr<void> ContainerNode::appendChildWithoutPreInsertionValidityCheck(Nod
     Ref<ContainerNode> protectedThis(*this);
 
     NodeVector targets;
-    auto removeResult = collectChildrenAndRemoveFromOldParent(newChild, targets);
+    auto removeResult = removeSelfOrChildNodesForInsertion(newChild, targets);
     if (removeResult.hasException())
         return removeResult.releaseException();
 
     if (targets.isEmpty())
         return { };
 
-    // We need this extra check because collectChildrenAndRemoveFromOldParent() can fire mutation events.
+    // We need this extra check because removeSelfOrChildNodesForInsertion() can fire mutation events.
     for (auto& child : targets) {
         auto nodeTypeResult = checkAcceptChildGuaranteedNodeTypes(*this, child);
         if (nodeTypeResult.hasException())
@@ -723,7 +793,7 @@ ExceptionOr<void> ContainerNode::appendChildWithoutPreInsertionValidityCheck(Nod
             break;
 
         // Append child to the end of the list
-        executeNodeInsertionWithScriptAssertion(*this, child.get(), ChildChangeSource::API, ReplacedAllChildren::No, [&] {
+        executeNodeInsertionWithScriptAssertion(*this, child.get(), nullptr, ChildChange::Source::API, ReplacedAllChildren::No, [&] {
             child->setTreeScopeRecursively(treeScope());
             appendChildCommon(child);
         });
@@ -739,7 +809,7 @@ void ContainerNode::parserAppendChild(Node& newChild)
     ASSERT(!newChild.isDocumentFragment());
     ASSERT(!hasTagName(HTMLNames::templateTag));
 
-    executeNodeInsertionWithScriptAssertion(*this, newChild, ChildChangeSource::Parser, ReplacedAllChildren::No, [&] {
+    executeNodeInsertionWithScriptAssertion(*this, newChild, nullptr, ChildChange::Source::Parser, ReplacedAllChildren::No, [&] {
         if (&document() != &newChild.document())
             document().adoptNode(newChild);
 
@@ -749,36 +819,26 @@ void ContainerNode::parserAppendChild(Node& newChild)
     });
 }
 
-static bool affectsElements(const ContainerNode::ChildChange& change)
+ExceptionOr<void> ContainerNode::appendChild(ChildChange::Source source, Node& newChild)
 {
-    switch (change.type) {
-    case ContainerNode::ElementInserted:
-    case ContainerNode::ElementRemoved:
-    case ContainerNode::AllChildrenRemoved:
-    case ContainerNode::AllChildrenReplaced:
-        return true;
-    case ContainerNode::TextInserted:
-    case ContainerNode::TextRemoved:
-    case ContainerNode::TextChanged:
-    case ContainerNode::NonContentsChildInserted:
-    case ContainerNode::NonContentsChildRemoved:
-        return false;
+    if (source == ChildChange::Source::Parser) {
+        parserAppendChild(newChild);
+        return { };
     }
-    ASSERT_NOT_REACHED();
-    return false;
+    return appendChild(newChild);
 }
 
 void ContainerNode::childrenChanged(const ChildChange& change)
 {
     document().incDOMTreeVersion();
 
-    if (affectsElements(change))
+    if (change.affectsElements())
         document().invalidateAccessKeyCache();
 
     // FIXME: Unclear why it's always safe to skip this when parser is adding children.
     // FIXME: Seems like it's equally safe to skip for TextInserted and TextRemoved as for TextChanged.
     // FIXME: Should use switch for change type so we remember to update when adding new types.
-    if (change.source == ChildChangeSource::API && change.type != TextChanged)
+    if (change.source == ChildChange::Source::API && change.type != ChildChange::Type::TextChanged)
         document().updateRangesAfterChildrenChanged(*this);
 
     invalidateNodeListAndCollectionCachesInAncestors();
@@ -838,10 +898,6 @@ static void dispatchChildRemovalEvents(Ref<Node>& child)
     if (child->isInShadowTree())
         return;
 
-    // FIXME: This doesn't belong in dispatchChildRemovalEvents.
-    // FIXME: Nodes removed from a shadow tree should also be kept alive.
-    willCreatePossiblyOrphanedTreeByRemoval(child.ptr());
-
     Ref<Document> document = child->document();
 
     // dispatch pre-removal mutation events
@@ -923,10 +979,10 @@ Element* ContainerNode::lastElementChild() const
 unsigned ContainerNode::childElementCount() const
 {
     auto children = childrenOfType<Element>(*this);
-    return std::distance(children.begin(), children.end());
+    return std::distance(children.begin(), { });
 }
 
-ExceptionOr<void> ContainerNode::append(Vector<NodeOrString>&& vector)
+ExceptionOr<void> ContainerNode::append(FixedVector<NodeOrString>&& vector)
 {
     auto result = convertNodesOrStringsIntoNode(WTFMove(vector));
     if (result.hasException())
@@ -939,7 +995,7 @@ ExceptionOr<void> ContainerNode::append(Vector<NodeOrString>&& vector)
     return appendChild(*node);
 }
 
-ExceptionOr<void> ContainerNode::prepend(Vector<NodeOrString>&& vector)
+ExceptionOr<void> ContainerNode::prepend(FixedVector<NodeOrString>&& vector)
 {
     auto result = convertNodesOrStringsIntoNode(WTFMove(vector));
     if (result.hasException())
@@ -952,6 +1008,37 @@ ExceptionOr<void> ContainerNode::prepend(Vector<NodeOrString>&& vector)
     return insertBefore(*node, firstChild());
 }
 
+// https://dom.spec.whatwg.org/#dom-parentnode-replacechildren
+ExceptionOr<void> ContainerNode::replaceChildren(FixedVector<NodeOrString>&& vector)
+{
+    // step 1
+    auto result = convertNodesOrStringsIntoNode(WTFMove(vector));
+    if (result.hasException())
+        return result.releaseException();
+    auto node = result.releaseReturnValue();
+
+    // step 2
+    if (node) {
+        if (auto checkResult = ensurePreInsertionValidity(*node, nullptr); checkResult.hasException())
+            return checkResult;
+    }
+
+    // step 3
+    Ref protectedThis { *this };
+    ChildListMutationScope mutation(*this);
+    removeAllChildrenWithScriptAssertion(ChildChange::Source::API, DeferChildrenChanged::No);
+
+    if (node) {
+        if (auto appendResult = appendChildWithoutPreInsertionValidityCheck(*node); appendResult.hasException())
+            return appendResult;
+    }
+
+    rebuildSVGExtensionsElementsIfNecessary();
+    dispatchSubtreeModifiedEvent();
+
+    return { };
+}
+
 HTMLCollection* ContainerNode::cachedHTMLCollection(CollectionType type)
 {
     return hasRareData() && rareData()->nodeLists() ? rareData()->nodeLists()->cachedCollection<HTMLCollection>(type) : nullptr;
