@@ -26,6 +26,7 @@
 #include "config.h"
 #include "InsertListCommand.h"
 
+#include "Document.h"
 #include "Editing.h"
 #include "ElementTraversal.h"
 #include "HTMLBRElement.h"
@@ -56,6 +57,10 @@ RefPtr<HTMLElement> InsertListCommand::insertList(Document& document, Type type)
 
 HTMLElement* InsertListCommand::fixOrphanedListChild(Node& node)
 {
+    RefPtr parentNode { node.parentNode() };
+    if (parentNode && !parentNode->hasRichlyEditableStyle())
+        return nullptr;
+
     auto listElement = HTMLUListElement::create(document());
     insertNodeBefore(listElement.copyRef(), node);
     if (!listElement->hasEditableStyle())
@@ -70,11 +75,11 @@ HTMLElement* InsertListCommand::fixOrphanedListChild(Node& node)
 Ref<HTMLElement> InsertListCommand::mergeWithNeighboringLists(HTMLElement& list)
 {
     Ref<HTMLElement> protectedList = list;
-    Element* previousList = list.previousElementSibling();
-    if (canMergeLists(previousList, &list))
+    RefPtr previousList = list.previousElementSibling();
+    if (canMergeLists(previousList.get(), &list))
         mergeIdenticalElements(*previousList, list);
 
-    Element* sibling = ElementTraversal::nextSibling(list);
+    RefPtr sibling = ElementTraversal::nextSibling(list);
     if (!is<HTMLElement>(sibling))
         return protectedList;
 
@@ -112,12 +117,13 @@ InsertListCommand::InsertListCommand(Document& document, Type type)
 
 void InsertListCommand::doApply()
 {
-    if (endingSelection().isNoneOrOrphaned() || !endingSelection().isContentRichlyEditable())
-        return;
-
     VisiblePosition visibleEnd = endingSelection().visibleEnd();
     VisiblePosition visibleStart = endingSelection().visibleStart();
-    // When a selection ends at the start of a paragraph, we rarely paint 
+
+    if (visibleEnd.isNull() || visibleStart.isNull() || !endingSelection().isContentRichlyEditable())
+        return;
+
+    // When a selection ends at the start of a paragraph, we rarely paint
     // the selection gap before that paragraph, because there often is no gap.  
     // In a case like this, it's not obvious to the user that the selection 
     // ends "inside" that paragraph, so it would be confusing if InsertUn{Ordered}List 
@@ -134,63 +140,67 @@ void InsertListCommand::doApply()
     auto& listTag = (m_type == Type::OrderedList) ? olTag : ulTag;
     if (endingSelection().isRange()) {
         VisibleSelection selection = selectionForParagraphIteration(endingSelection());
-        ASSERT(selection.isRange());
-        VisiblePosition startOfSelection = selection.visibleStart();
-        VisiblePosition endOfSelection = selection.visibleEnd();
-        VisiblePosition startOfLastParagraph = startOfParagraph(endOfSelection, CanSkipOverEditingBoundary);
+        if (selection.isRange()) {
+            VisiblePosition startOfSelection = selection.visibleStart();
+            VisiblePosition endOfSelection = selection.visibleEnd();
+            VisiblePosition startOfLastParagraph = startOfParagraph(endOfSelection, CanSkipOverEditingBoundary);
 
-        if (startOfParagraph(startOfSelection, CanSkipOverEditingBoundary) != startOfLastParagraph) {
-            bool forceCreateList = !selectionHasListOfType(selection, listTag);
+            if (startOfLastParagraph.isNotNull() && startOfParagraph(startOfSelection, CanSkipOverEditingBoundary) != startOfLastParagraph) {
+                bool forceCreateList = !selectionHasListOfType(selection, listTag);
 
-            RefPtr<Range> currentSelection = endingSelection().firstRange();
-            VisiblePosition startOfCurrentParagraph = startOfSelection;
-            while (!inSameParagraph(startOfCurrentParagraph, startOfLastParagraph, CanCrossEditingBoundary)) {
-                // doApply() may operate on and remove the last paragraph of the selection from the document 
-                // if it's in the same list item as startOfCurrentParagraph.  Return early to avoid an 
-                // infinite loop and because there is no more work to be done.
-                // FIXME(<rdar://problem/5983974>): The endingSelection() may be incorrect here.  Compute 
-                // the new location of endOfSelection and use it as the end of the new selection.
-                if (!startOfLastParagraph.deepEquivalent().anchorNode()->isConnected())
-                    return;
-                setEndingSelection(startOfCurrentParagraph);
-
-                // Save and restore endOfSelection and startOfLastParagraph when necessary
-                // since moveParagraph and movePragraphWithClones can remove nodes.
-                // FIXME: This is an inefficient way to keep selection alive because indexForVisiblePosition walks from
-                // the beginning of the document to the endOfSelection everytime this code is executed.
-                // But not using index is hard because there are so many ways we can lose selection inside doApplyForSingleParagraph.
-                RefPtr<ContainerNode> scope;
-                int indexForEndOfSelection = indexForVisiblePosition(endOfSelection, scope);
-                doApplyForSingleParagraph(forceCreateList, listTag, currentSelection.get());
-                if (endOfSelection.isNull() || endOfSelection.isOrphan() || startOfLastParagraph.isNull() || startOfLastParagraph.isOrphan()) {
-                    endOfSelection = visiblePositionForIndex(indexForEndOfSelection, scope.get());
-                    // If endOfSelection is null, then some contents have been deleted from the document.
-                    // This should never happen and if it did, exit early immediately because we've lost the loop invariant.
-                    ASSERT(endOfSelection.isNotNull());
-                    if (endOfSelection.isNull())
+                auto currentSelection = *endingSelection().firstRange();
+                VisiblePosition startOfCurrentParagraph = startOfSelection;
+                while (startOfCurrentParagraph.isNotNull() && !inSameParagraph(startOfCurrentParagraph, startOfLastParagraph, CanCrossEditingBoundary)) {
+                    // doApply() may operate on and remove the last paragraph of the selection from the document
+                    // if it's in the same list item as startOfCurrentParagraph. Return early to avoid an
+                    // infinite loop and because there is no more work to be done.
+                    // FIXME(<rdar://problem/5983974>): The endingSelection() may be incorrect here. Compute
+                    // the new location of endOfSelection and use it as the end of the new selection.
+                    if (startOfLastParagraph.isOrphan())
                         return;
-                    startOfLastParagraph = startOfParagraph(endOfSelection, CanSkipOverEditingBoundary);
+                    setEndingSelection(startOfCurrentParagraph);
+
+                    // Save and restore endOfSelection and startOfLastParagraph when necessary
+                    // since moveParagraph and movePragraphWithClones can remove nodes.
+                    // FIXME: This is an inefficient way to keep selection alive because indexForVisiblePosition walks from
+                    // the beginning of the document to the endOfSelection everytime this code is executed.
+                    // But not using index is hard because there are so many ways we can lose selection inside doApplyForSingleParagraph.
+                    RefPtr<ContainerNode> scope;
+                    if (endOfSelection.isOrphan())
+                        return;
+                    int indexForEndOfSelection = indexForVisiblePosition(endOfSelection, scope);
+                    doApplyForSingleParagraph(forceCreateList, listTag, currentSelection);
+                    if (endOfSelection.isNull() || endOfSelection.isOrphan() || startOfLastParagraph.isNull() || startOfLastParagraph.isOrphan()) {
+                        endOfSelection = visiblePositionForIndex(indexForEndOfSelection, scope.get());
+                        // If endOfSelection is null, then some contents have been deleted from the document.
+                        // This should never happen and if it did, exit early immediately because we've lost the loop invariant.
+                        ASSERT(endOfSelection.isNotNull());
+                        if (endOfSelection.isNull())
+                            return;
+                        startOfLastParagraph = startOfParagraph(endOfSelection, CanSkipOverEditingBoundary);
+                    }
+
+                    // Fetch the start of the selection after moving the first paragraph,
+                    // because moving the paragraph will invalidate the original start.
+                    // We'll use the new start to restore the original selection after
+                    // we modified all selected paragraphs.
+                    if (startOfCurrentParagraph == startOfSelection)
+                        startOfSelection = endingSelection().visibleStart();
+
+                    startOfCurrentParagraph = startOfNextParagraph(endingSelection().visibleStart());
                 }
-
-                // Fetch the start of the selection after moving the first paragraph,
-                // because moving the paragraph will invalidate the original start.  
-                // We'll use the new start to restore the original selection after 
-                // we modified all selected paragraphs.
-                if (startOfCurrentParagraph == startOfSelection)
-                    startOfSelection = endingSelection().visibleStart();
-
-                startOfCurrentParagraph = startOfNextParagraph(endingSelection().visibleStart());
+                setEndingSelection(endOfSelection);
+                doApplyForSingleParagraph(forceCreateList, listTag, currentSelection);
+                // Fetch the end of the selection, for the reason mentioned above.
+                endOfSelection = endingSelection().visibleEnd();
+                setEndingSelection(VisibleSelection(startOfSelection, endOfSelection, endingSelection().isDirectional()));
+                return;
             }
-            setEndingSelection(endOfSelection);
-            doApplyForSingleParagraph(forceCreateList, listTag, currentSelection.get());
-            // Fetch the end of the selection, for the reason mentioned above.
-            endOfSelection = endingSelection().visibleEnd();
-            setEndingSelection(VisibleSelection(startOfSelection, endOfSelection, endingSelection().isDirectional()));
-            return;
         }
     }
 
-    doApplyForSingleParagraph(false, listTag, endingSelection().firstRange().get());
+    auto range = endingSelection().firstRange();
+    doApplyForSingleParagraph(false, listTag, *range);
 }
 
 EditAction InsertListCommand::editingAction() const
@@ -198,7 +208,7 @@ EditAction InsertListCommand::editingAction() const
     return m_type == Type::OrderedList ? EditAction::InsertOrderedList : EditAction::InsertUnorderedList;
 }
 
-void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HTMLQualifiedName& listTag, Range* currentSelection)
+void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HTMLQualifiedName& listTag, SimpleRange& currentSelection)
 {
     // FIXME: This will produce unexpected results for a selection that starts just before a
     // table and ends inside the first cell, selectionForParagraphIteration should probably
@@ -207,11 +217,11 @@ void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HT
     Node* listChildNode = enclosingListChild(selectionNode);
     bool switchListType = false;
     if (listChildNode) {
-        // Remove the list chlild.
+        // Remove the list child.
         RefPtr<HTMLElement> listNode = enclosingList(listChildNode);
         if (!listNode) {
             RefPtr<HTMLElement> listElement = fixOrphanedListChild(*listChildNode);
-            if (!listElement)
+            if (!listElement || !listElement->isConnected())
                 return;
 
             listNode = mergeWithNeighboringLists(*listElement);
@@ -227,9 +237,9 @@ void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HT
             return;
 
         // If the entire list is selected, then convert the whole list.
-        if (switchListType && isNodeVisiblyContainedWithin(*listNode, *currentSelection)) {
-            bool rangeStartIsInList = visiblePositionBeforeNode(*listNode) == currentSelection->startPosition();
-            bool rangeEndIsInList = visiblePositionAfterNode(*listNode) == currentSelection->endPosition();
+        if (switchListType && isNodeVisiblyContainedWithin(*listNode, currentSelection)) {
+            bool rangeStartIsInList = visiblePositionBeforeNode(*listNode) == makeDeprecatedLegacyPosition(currentSelection.start);
+            bool rangeEndIsInList = visiblePositionAfterNode(*listNode) == makeDeprecatedLegacyPosition(currentSelection.end);
 
             RefPtr<HTMLElement> newList = createHTMLElement(document(), listTag);
             insertNodeBefore(*newList, *listNode);
@@ -252,9 +262,9 @@ void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HT
             // Restore the start and the end of current selection if they started inside listNode
             // because moveParagraphWithClones could have removed them.
             if (rangeStartIsInList && newList)
-                currentSelection->setStart(*newList, 0);
+                currentSelection.start = makeBoundaryPointBeforeNodeContents(*newList);
             if (rangeEndIsInList && newList)
-                currentSelection->setEnd(*newList, lastOffsetInNode(newList.get()));
+                currentSelection.end = makeBoundaryPointAfterNodeContents(*newList);
 
             setEndingSelection(VisiblePosition(firstPositionInNode(newList.get())));
 
@@ -270,10 +280,14 @@ void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HT
 
 void InsertListCommand::unlistifyParagraph(const VisiblePosition& originalStart, HTMLElement* listNode, Node* listChildNode)
 {
-    Node* nextListChild;
-    Node* previousListChild;
+    RefPtr<Node> nextListChild;
+    RefPtr<Node> previousListChild;
     VisiblePosition start;
     VisiblePosition end;
+
+    if (!listNode->parentNode()->hasEditableStyle())
+        return;
+
     if (listChildNode->hasTagName(liTag)) {
         start = firstPositionInNode(listChildNode);
         end = lastPositionInNode(listChildNode);
@@ -288,6 +302,10 @@ void InsertListCommand::unlistifyParagraph(const VisiblePosition& originalStart,
         previousListChild = enclosingListChild(start.previous().deepEquivalent().deprecatedNode(), listNode);
         ASSERT(previousListChild != listChildNode);
     }
+
+    if (start.isNull() || end.isNull())
+        return;
+
     // When removing a list, we must always create a placeholder to act as a point of insertion
     // for the list content being removed.
     auto placeholder = HTMLBRElement::create(document());
@@ -314,7 +332,7 @@ void InsertListCommand::unlistifyParagraph(const VisiblePosition& originalStart,
         // in listNode that comes before listChildNode, as listChildNode could have ancestors
         // between it and listNode. So, we split up to listNode before inserting the placeholder
         // where we're about to move listChildNode to.
-        if (listChildNode->parentNode() != listNode)
+        if (RefPtr listChildNodeParentNode { listChildNode->parentNode() }; listChildNodeParentNode && listChildNodeParentNode != listNode)
             splitElement(*listNode, *splitTreeToNode(*listChildNode, *listNode).get());
         insertNodeBefore(nodeToInsert.releaseNonNull(), *listNode);
     } else
@@ -324,21 +342,21 @@ void InsertListCommand::unlistifyParagraph(const VisiblePosition& originalStart,
     moveParagraphs(start, end, insertionPoint, true);
 }
 
-static Element* adjacentEnclosingList(const VisiblePosition& pos, const VisiblePosition& adjacentPos, const QualifiedName& listTag)
+static RefPtr<Element> adjacentEnclosingList(const VisiblePosition& pos, const VisiblePosition& adjacentPos, const QualifiedName& listTag)
 {
-    Element* listNode = outermostEnclosingList(adjacentPos.deepEquivalent().deprecatedNode());
+    RefPtr<Element> listNode = outermostEnclosingList(adjacentPos.deepEquivalent().deprecatedNode());
 
     if (!listNode)
-        return 0;
+        return nullptr;
 
-    Node* previousCell = enclosingTableCell(pos.deepEquivalent());
-    Node* currentCell = enclosingTableCell(adjacentPos.deepEquivalent());
+    RefPtr previousCell = enclosingTableCell(pos.deepEquivalent());
+    RefPtr currentCell = enclosingTableCell(adjacentPos.deepEquivalent());
 
     if (!listNode->hasTagName(listTag)
         || listNode->contains(pos.deepEquivalent().deprecatedNode())
         || previousCell != currentCell
-        || enclosingList(listNode) != enclosingList(pos.deepEquivalent().deprecatedNode()))
-        return 0;
+        || enclosingList(listNode.get()) != enclosingList(pos.deepEquivalent().deprecatedNode()))
+        return nullptr;
 
     return listNode;
 }
@@ -349,7 +367,7 @@ RefPtr<HTMLElement> InsertListCommand::listifyParagraph(const VisiblePosition& o
     VisiblePosition end = endOfParagraph(start, CanSkipOverEditingBoundary);
     
     if (start.isNull() || end.isNull() || !start.deepEquivalent().containerNode()->hasEditableStyle() || !end.deepEquivalent().containerNode()->hasEditableStyle())
-        return 0;
+        return nullptr;
 
     // Check for adjoining lists.
     auto listItemElement = HTMLLIElement::create(document());
@@ -357,13 +375,13 @@ RefPtr<HTMLElement> InsertListCommand::listifyParagraph(const VisiblePosition& o
     appendNode(placeholder.copyRef(), listItemElement.copyRef());
 
     // Place list item into adjoining lists.
-    Element* previousList = adjacentEnclosingList(start.deepEquivalent(), start.previous(CannotCrossEditingBoundary), listTag);
-    Element* nextList = adjacentEnclosingList(start.deepEquivalent(), end.next(CannotCrossEditingBoundary), listTag);
+    auto previousList = adjacentEnclosingList(start.deepEquivalent(), start.previous(CannotCrossEditingBoundary), listTag);
+    auto nextList = adjacentEnclosingList(start.deepEquivalent(), end.next(CannotCrossEditingBoundary), listTag);
     RefPtr<HTMLElement> listElement;
     if (previousList)
         appendNode(WTFMove(listItemElement), *previousList);
     else if (nextList)
-        insertNodeAt(WTFMove(listItemElement), positionBeforeNode(nextList));
+        insertNodeAt(WTFMove(listItemElement), positionBeforeNode(nextList.get()));
     else {
         // Create the list.
         listElement = createHTMLElement(document(), listTag);
@@ -385,9 +403,12 @@ RefPtr<HTMLElement> InsertListCommand::listifyParagraph(const VisiblePosition& o
         // clean markup when inline elements are pushed down as far as possible.
         Position insertionPos(start.deepEquivalent().upstream());
         // Also avoid the containing list item.
-        Node* listChild = enclosingListChild(insertionPos.deprecatedNode());
+        RefPtr listChild = enclosingListChild(insertionPos.deprecatedNode());
         if (listChild && listChild->hasTagName(liTag))
-            insertionPos = positionInParentBeforeNode(listChild);
+            insertionPos = positionInParentBeforeNode(listChild.get());
+
+        if (!isEditablePosition(insertionPos))
+            return nullptr;
 
         insertNodeAt(*listElement, insertionPos);
 
@@ -407,7 +428,7 @@ RefPtr<HTMLElement> InsertListCommand::listifyParagraph(const VisiblePosition& o
     if (listElement)
         return mergeWithNeighboringLists(*listElement);
 
-    if (canMergeLists(previousList, nextList))
+    if (canMergeLists(previousList.get(), nextList.get()))
         mergeIdenticalElements(*previousList, *nextList);
 
     return listElement;
