@@ -29,9 +29,9 @@
 
 #include "ActiveDOMObject.h"
 #include "CanvasBase.h"
+#include "Document.h"
 #include "FloatRect.h"
 #include "HTMLElement.h"
-#include "ImageBitmapRenderingContextSettings.h"
 #include <memory>
 #include <wtf/Forward.h>
 
@@ -50,8 +50,11 @@ class ImageBuffer;
 class ImageData;
 class MediaSample;
 class MediaStream;
+class OffscreenCanvas;
 class WebGLRenderingContextBase;
 class GPUCanvasContext;
+struct CanvasRenderingContext2DSettings;
+struct ImageBitmapRenderingContextSettings;
 struct UncachedString;
 
 namespace DisplayList {
@@ -71,63 +74,60 @@ public:
     void setSize(const IntSize& newSize) override;
 
     CanvasRenderingContext* renderingContext() const final { return m_context.get(); }
-    ExceptionOr<Optional<RenderingContext>> getContext(JSC::JSGlobalObject&, const String& contextId, Vector<JSC::Strong<JSC::Unknown>>&& arguments);
+    ExceptionOr<std::optional<RenderingContext>> getContext(JSC::JSGlobalObject&, const String& contextId, FixedVector<JSC::Strong<JSC::Unknown>>&& arguments);
 
     CanvasRenderingContext* getContext(const String&);
 
     static bool is2dType(const String&);
-    CanvasRenderingContext2D* createContext2d(const String& type);
-    CanvasRenderingContext2D* getContext2d(const String&);
+    CanvasRenderingContext2D* createContext2d(const String&, CanvasRenderingContext2DSettings&&);
+    CanvasRenderingContext2D* getContext2d(const String&, CanvasRenderingContext2DSettings&&);
 
 #if ENABLE(WEBGL)
+    using WebGLVersion = GraphicsContextGLWebGLVersion;
     static bool isWebGLType(const String&);
-    WebGLRenderingContextBase* createContextWebGL(const String&, WebGLContextAttributes&& = { });
-    WebGLRenderingContextBase* getContextWebGL(const String&, WebGLContextAttributes&& = { });
-#endif
-#if ENABLE(WEBGPU)
-    static bool isWebGPUType(const String&);
-    GPUCanvasContext* createContextWebGPU(const String&);
-    GPUCanvasContext* getContextWebGPU(const String&);
+    static WebGLVersion toWebGLVersion(const String&);
+    WebGLRenderingContextBase* createContextWebGL(WebGLVersion type, WebGLContextAttributes&& = { });
+    WebGLRenderingContextBase* getContextWebGL(WebGLVersion type, WebGLContextAttributes&& = { });
 #endif
 
     static bool isBitmapRendererType(const String&);
-    ImageBitmapRenderingContext* createContextBitmapRenderer(const String&, ImageBitmapRenderingContextSettings&& = { });
-    ImageBitmapRenderingContext* getContextBitmapRenderer(const String&, ImageBitmapRenderingContextSettings&& = { });
+    ImageBitmapRenderingContext* createContextBitmapRenderer(const String&, ImageBitmapRenderingContextSettings&&);
+    ImageBitmapRenderingContext* getContextBitmapRenderer(const String&, ImageBitmapRenderingContextSettings&&);
 
     WEBCORE_EXPORT ExceptionOr<UncachedString> toDataURL(const String& mimeType, JSC::JSValue quality);
     WEBCORE_EXPORT ExceptionOr<UncachedString> toDataURL(const String& mimeType);
-    ExceptionOr<void> toBlob(ScriptExecutionContext&, Ref<BlobCallback>&&, const String& mimeType, JSC::JSValue quality);
+    ExceptionOr<void> toBlob(Ref<BlobCallback>&&, const String& mimeType, JSC::JSValue quality);
+#if ENABLE(OFFSCREEN_CANVAS)
+    ExceptionOr<Ref<OffscreenCanvas>> transferControlToOffscreen();
+#endif
 
     // Used for rendering
-    void didDraw(const FloatRect&) final;
+    void didDraw(const std::optional<FloatRect>&) final;
 
     void paint(GraphicsContext&, const LayoutRect&);
 
 #if ENABLE(MEDIA_STREAM)
     RefPtr<MediaSample> toMediaSample();
-    ExceptionOr<Ref<MediaStream>> captureStream(Document&, Optional<double>&& frameRequestRate);
+    ExceptionOr<Ref<MediaStream>> captureStream(std::optional<double>&& frameRequestRate);
 #endif
 
     Image* copiedImage() const final;
-    void clearCopiedImage();
+    void clearCopiedImage() const final;
     RefPtr<ImageData> getImageData();
-    void makePresentationCopy();
-    void clearPresentationCopy();
 
     SecurityOrigin* securityOrigin() const final;
 
     bool shouldAccelerate(const IntSize&) const;
+    bool shouldAccelerate(unsigned area) const;
 
     WEBCORE_EXPORT void setUsesDisplayListDrawing(bool);
-    WEBCORE_EXPORT void setTracksDisplayListReplay(bool);
-    WEBCORE_EXPORT String displayListAsText(DisplayList::AsTextFlags) const;
-    WEBCORE_EXPORT String replayDisplayListAsText(DisplayList::AsTextFlags) const;
 
     // FIXME: Only some canvas rendering contexts need an ImageBuffer.
     // It would be better to have the contexts own the buffers.
-    void setImageBufferAndMarkDirty(std::unique_ptr<ImageBuffer>&&);
+    void setImageBufferAndMarkDirty(RefPtr<ImageBuffer>&&);
 
-    WEBCORE_EXPORT static void setMaxPixelMemoryForTesting(size_t);
+    WEBCORE_EXPORT static void setMaxPixelMemoryForTesting(std::optional<size_t>);
+    WEBCORE_EXPORT static void setMaxCanvasAreaForTesting(std::optional<size_t>);
 
     bool needsPreparationForDisplay();
     void prepareForDisplay();
@@ -135,6 +135,14 @@ public:
     void setIsSnapshotting(bool isSnapshotting) { m_isSnapshotting = isSnapshotting; }
     bool isSnapshotting() const { return m_isSnapshotting; }
 
+    bool isControlledByOffscreen() const;
+
+    WEBCORE_EXPORT static size_t maxActivePixelMemory();
+
+#if PLATFORM(COCOA)
+    GraphicsContext* drawingContext() const final;
+#endif
+
 private:
     HTMLCanvasElement(const QualifiedName&, Document&);
 
@@ -175,26 +183,22 @@ private:
     Node::InsertedIntoAncestorResult insertedIntoAncestor(InsertionType, ContainerNode&) final;
     void removedFromAncestor(RemovalType, ContainerNode& oldParentOfRemovedTree) final;
 
-    FloatRect m_dirtyRect;
+    std::unique_ptr<CanvasRenderingContext> m_context;
+    mutable RefPtr<Image> m_copiedImage; // FIXME: This is temporary for platforms that have to copy the image buffer to render (and for CSSCanvasValue).
+
+    std::optional<bool> m_usesDisplayListDrawing;
 
     bool m_ignoreReset { false };
-
-    Optional<bool> m_usesDisplayListDrawing;
-    bool m_tracksDisplayListReplay { false };
-
-    std::unique_ptr<CanvasRenderingContext> m_context;
-
     // m_hasCreatedImageBuffer means we tried to malloc the buffer. We didn't necessarily get it.
     mutable bool m_hasCreatedImageBuffer { false };
     mutable bool m_didClearImageBuffer { false };
 #if ENABLE(WEBGL)
     bool m_hasRelevantWebGLEventListener { false };
 #endif
-
     bool m_isSnapshotting { false };
-
-    mutable RefPtr<Image> m_presentedImage;
-    mutable RefPtr<Image> m_copiedImage; // FIXME: This is temporary for platforms that have to copy the image buffer to render (and for CSSCanvasValue).
+#if PLATFORM(COCOA)
+    mutable bool m_mustGuardAgainstUseByPendingLayerTransaction { false };
+#endif
 };
 
 } // namespace WebCore
@@ -210,4 +214,3 @@ private:
     static bool checkTagName(const WebCore::EventTarget& target) { return is<WebCore::Node>(target) && checkTagName(downcast<WebCore::Node>(target)); }
 };
 }
-
