@@ -34,11 +34,11 @@ namespace WebCore {
 
 WTF_MAKE_ISO_ALLOCATED_IMPL(TrackEvent);
 
-static inline Optional<TrackEvent::TrackEventTrack> convertToTrackEventTrack(Ref<TrackBase>&& track)
+static inline std::optional<TrackEvent::TrackEventTrack> convertToTrackEventTrack(Ref<TrackBase>&& track)
 {
     switch (track->type()) {
     case TrackBase::BaseTrack:
-        return WTF::nullopt;
+        return std::nullopt;
     case TrackBase::TextTrack:
         return TrackEvent::TrackEventTrack { RefPtr<TextTrack>(&downcast<TextTrack>(track.get())) };
     case TrackBase::AudioTrack:
@@ -48,7 +48,7 @@ static inline Optional<TrackEvent::TrackEventTrack> convertToTrackEventTrack(Ref
     }
     
     ASSERT_NOT_REACHED();
-    return WTF::nullopt;
+    return std::nullopt;
 }
 
 TrackEvent::TrackEvent(const AtomString& type, CanBubble canBubble, IsCancelable cancelable, Ref<TrackBase>&& track)
