@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,6 +28,8 @@
 
 #include "DisplayListItemBuffer.h"
 #include "DisplayListItems.h"
+#include "DisplayListIterator.h"
+#include "Filter.h"
 #include "Logging.h"
 #include <wtf/FastMalloc.h>
 #include <wtf/StdLibExtras.h>
@@ -37,10 +39,10 @@ namespace WebCore {
 namespace DisplayList {
 
 #if !defined(NDEBUG) || !LOG_DISABLED
-WTF::CString DisplayList::description() const
+CString DisplayList::description() const
 {
     TextStream ts;
-    ts << this;
+    ts << *this;
     return ts.release().utf8();
 }
 
@@ -51,8 +53,7 @@ void DisplayList::dump() const
 #endif
 
 DisplayList::DisplayList(DisplayList&& other)
-    : m_imageBuffers(std::exchange(other.m_imageBuffers, { }))
-    , m_nativeImages(std::exchange(other.m_nativeImages, { }))
+    : m_resourceHeap(std::exchange(other.m_resourceHeap, { }))
     , m_items(std::exchange(other.m_items, nullptr))
     , m_drawingItemExtents(std::exchange(other.m_drawingItemExtents, { }))
 {
@@ -68,8 +69,7 @@ DisplayList::~DisplayList() = default;
 
 DisplayList& DisplayList::operator=(DisplayList&& other)
 {
-    m_imageBuffers = std::exchange(other.m_imageBuffers, { });
-    m_nativeImages = std::exchange(other.m_nativeImages, { });
+    m_resourceHeap = std::exchange(other.m_resourceHeap, { });
     m_items = std::exchange(other.m_items, nullptr);
     m_drawingItemExtents = std::exchange(other.m_drawingItemExtents, { });
     return *this;
@@ -80,8 +80,7 @@ void DisplayList::clear()
     if (m_items)
         m_items->clear();
     m_drawingItemExtents.clear();
-    m_imageBuffers.clear();
-    m_nativeImages.clear();
+    m_resourceHeap.clear();
 }
 
 bool DisplayList::shouldDumpForFlags(AsTextFlags flags, ItemHandle item)
@@ -114,13 +113,14 @@ bool DisplayList::shouldDumpForFlags(AsTextFlags flags, ItemHandle item)
 String DisplayList::asText(AsTextFlags flags) const
 {
     TextStream stream(TextStream::LineMode::MultipleLine, TextStream::Formatting::SVGStyleRect);
-    for (auto [item, extent, itemSizeInBuffer] : *this) {
-        if (!shouldDumpForFlags(flags, *item))
+    for (auto displayListItem : *this) {
+        auto [item, extent, itemSizeInBuffer] = displayListItem.value();
+        if (!shouldDumpForFlags(flags, item))
             continue;
 
         TextStream::GroupScope group(stream);
         stream << item;
-        if (item->isDrawingItem())
+        if (item.isDrawingItem())
             stream << " extent " << extent;
     }
     return stream.release();
@@ -131,10 +131,11 @@ void DisplayList::dump(TextStream& ts) const
     TextStream::GroupScope group(ts);
     ts << "display list";
 
-    for (auto [item, extent, itemSizeInBuffer] : *this) {
+    for (auto displayListItem : *this) {
+        auto [item, extent, itemSizeInBuffer] = displayListItem.value();
         TextStream::GroupScope group(ts);
         ts << item;
-        if (item->isDrawingItem())
+        if (item.isDrawingItem())
             ts << " extent " << extent;
     }
     ts.startGroup();
@@ -159,12 +160,20 @@ ItemBuffer& DisplayList::itemBuffer()
     return *m_items;
 }
 
-void DisplayList::setItemBufferClient(ItemBufferReadingClient* client)
+void DisplayList::shrinkToFit()
+{
+    if (auto* itemBuffer = itemBufferIfExists())
+        itemBuffer->shrinkToFit();
+
+    m_drawingItemExtents.shrinkToFit();
+}
+
+void DisplayList::setItemBufferReadingClient(ItemBufferReadingClient* client)
 {
     itemBuffer().setClient(client);
 }
 
-void DisplayList::setItemBufferClient(ItemBufferWritingClient* client)
+void DisplayList::setItemBufferWritingClient(ItemBufferWritingClient* client)
 {
     itemBuffer().setClient(client);
 }
@@ -182,7 +191,7 @@ void DisplayList::forEachItemBuffer(Function<void(const ItemBufferHandle&)>&& ma
 
 void DisplayList::setTracksDrawingItemExtents(bool value)
 {
-    RELEASE_ASSERT(!m_items || m_items->isEmpty());
+    RELEASE_ASSERT(isEmpty());
     m_tracksDrawingItemExtents = value;
 }
 
@@ -203,8 +212,6 @@ void DisplayList::append(ItemHandle item)
         return append<ConcatenateCTM>(item.get<ConcatenateCTM>());
     case ItemType::SetCTM:
         return append<SetCTM>(item.get<SetCTM>());
-    case ItemType::SetInlineFillGradient:
-        return append<SetInlineFillGradient>(item.get<SetInlineFillGradient>());
     case ItemType::SetInlineFillColor:
         return append<SetInlineFillColor>(item.get<SetInlineFillColor>());
     case ItemType::SetInlineStrokeColor:
@@ -233,10 +240,8 @@ void DisplayList::append(ItemHandle item)
         return append<ClipOutToPath>(item.get<ClipOutToPath>());
     case ItemType::ClipPath:
         return append<ClipPath>(item.get<ClipPath>());
-    case ItemType::BeginClipToDrawingCommands:
-        return append<BeginClipToDrawingCommands>(item.get<BeginClipToDrawingCommands>());
-    case ItemType::EndClipToDrawingCommands:
-        return append<EndClipToDrawingCommands>(item.get<EndClipToDrawingCommands>());
+    case ItemType::DrawFilteredImageBuffer:
+        return append<DrawFilteredImageBuffer>(item.get<DrawFilteredImageBuffer>());
     case ItemType::DrawGlyphs:
         return append<DrawGlyphs>(item.get<DrawGlyphs>());
     case ItemType::DrawImageBuffer:
@@ -274,8 +279,14 @@ void DisplayList::append(ItemHandle item)
     case ItemType::FillRectWithRoundedHole:
         return append<FillRectWithRoundedHole>(item.get<FillRectWithRoundedHole>());
 #if ENABLE(INLINE_PATH_DATA)
-    case ItemType::FillInlinePath:
-        return append<FillInlinePath>(item.get<FillInlinePath>());
+    case ItemType::FillLine:
+        return append<FillLine>(item.get<FillLine>());
+    case ItemType::FillArc:
+        return append<FillArc>(item.get<FillArc>());
+    case ItemType::FillQuadCurve:
+        return append<FillQuadCurve>(item.get<FillQuadCurve>());
+    case ItemType::FillBezierCurve:
+        return append<FillBezierCurve>(item.get<FillBezierCurve>());
 #endif
     case ItemType::FillPath:
         return append<FillPath>(item.get<FillPath>());
@@ -283,21 +294,21 @@ void DisplayList::append(ItemHandle item)
         return append<FillEllipse>(item.get<FillEllipse>());
     case ItemType::FlushContext:
         return append<FlushContext>(item.get<FlushContext>());
-    case ItemType::MetaCommandChangeDestinationImageBuffer:
-        return append<MetaCommandChangeDestinationImageBuffer>(item.get<MetaCommandChangeDestinationImageBuffer>());
-    case ItemType::MetaCommandChangeItemBuffer:
-        return append<MetaCommandChangeItemBuffer>(item.get<MetaCommandChangeItemBuffer>());
-    case ItemType::PutImageData:
-        return append<PutImageData>(item.get<PutImageData>());
+#if ENABLE(VIDEO)
     case ItemType::PaintFrameForMedia:
         return append<PaintFrameForMedia>(item.get<PaintFrameForMedia>());
+#endif
     case ItemType::StrokeRect:
         return append<StrokeRect>(item.get<StrokeRect>());
     case ItemType::StrokeLine:
         return append<StrokeLine>(item.get<StrokeLine>());
 #if ENABLE(INLINE_PATH_DATA)
-    case ItemType::StrokeInlinePath:
-        return append<StrokeInlinePath>(item.get<StrokeInlinePath>());
+    case ItemType::StrokeArc:
+        return append<StrokeArc>(item.get<StrokeArc>());
+    case ItemType::StrokeQuadCurve:
+        return append<StrokeQuadCurve>(item.get<StrokeQuadCurve>());
+    case ItemType::StrokeBezierCurve:
+        return append<StrokeBezierCurve>(item.get<StrokeBezierCurve>());
 #endif
     case ItemType::StrokePath:
         return append<StrokePath>(item.get<StrokePath>());
@@ -320,113 +331,22 @@ void DisplayList::append(ItemHandle item)
     }
 }
 
-bool DisplayList::iterator::atEnd() const
+auto DisplayList::begin() const -> Iterator
 {
-    if (m_displayList.isEmpty() || !m_isValid)
-        return true;
-
-    auto& items = *m_displayList.m_items;
-    auto endCursor = items.m_writableBuffer.data + items.m_writtenNumberOfBytes;
-    return m_cursor == endCursor;
+    return { *this };
 }
 
-void DisplayList::iterator::updateCurrentItem()
+auto DisplayList::end() const -> Iterator
 {
-    clearCurrentItem();
-
-    if (atEnd())
-        return;
-
-    auto& items = *m_displayList.m_items;
-    auto itemType = static_cast<ItemType>(m_cursor[0]);
-    if (isDrawingItem(itemType) && !m_displayList.m_drawingItemExtents.isEmpty()) {
-        m_currentExtent = m_displayList.m_drawingItemExtents[m_drawingItemIndex];
-        m_drawingItemIndex++;
-    } else
-        m_currentExtent = WTF::nullopt;
-
-    auto* client = items.m_readingClient;
-    auto paddedSizeOfTypeAndItem = paddedSizeOfTypeAndItemInBytes(itemType);
-    m_currentBufferForItem = paddedSizeOfTypeAndItem <= sizeOfFixedBufferForCurrentItem ? m_fixedBufferForCurrentItem : reinterpret_cast<uint8_t*>(fastMalloc(paddedSizeOfTypeAndItem));
-    if (!isInlineItem(itemType) && client) {
-        auto dataLength = reinterpret_cast<uint64_t*>(m_cursor)[1];
-        auto* startOfData = m_cursor + 2 * sizeof(uint64_t);
-        auto decodedItemHandle = client->decodeItem(startOfData, dataLength, itemType, m_currentBufferForItem);
-        if (UNLIKELY(!decodedItemHandle))
-            m_isValid = false;
-
-        m_currentBufferForItem[0] = static_cast<uint8_t>(itemType);
-        m_currentItemSizeInBuffer = 2 * sizeof(uint64_t) + roundUpToMultipleOf(alignof(uint64_t), dataLength);
-    } else {
-        if (UNLIKELY(!ItemHandle { m_cursor }.safeCopy({ m_currentBufferForItem })))
-            m_isValid = false;
-
-        m_currentItemSizeInBuffer = paddedSizeOfTypeAndItem;
-    }
+    return { *this, Iterator::ImmediatelyMoveToEnd::Yes };
 }
 
-void DisplayList::iterator::advance()
-{
-    if (atEnd())
-        return;
-
-    m_cursor += m_currentItemSizeInBuffer;
-
-    if (m_cursor == m_currentEndOfBuffer && m_readOnlyBufferIndex < m_displayList.m_items->m_readOnlyBuffers.size()) {
-        m_readOnlyBufferIndex++;
-        moveCursorToStartOfCurrentBuffer();
-    }
-
-    updateCurrentItem();
-}
-
-void DisplayList::iterator::clearCurrentItem()
-{
-    if (m_currentBufferForItem) {
-        if (LIKELY(m_isValid))
-            ItemHandle { m_currentBufferForItem }.destroy();
-
-        if (UNLIKELY(m_currentBufferForItem != m_fixedBufferForCurrentItem))
-            fastFree(m_currentBufferForItem);
-    }
-
-    m_currentItemSizeInBuffer = 0;
-    m_currentBufferForItem = nullptr;
-}
-
-void DisplayList::iterator::moveToEnd()
-{
-    if (auto& items = m_displayList.m_items) {
-        m_cursor = items->m_writableBuffer.data + items->m_writtenNumberOfBytes;
-        m_currentEndOfBuffer = m_cursor;
-        m_readOnlyBufferIndex = items->m_readOnlyBuffers.size();
-    }
-}
-
-void DisplayList::iterator::moveCursorToStartOfCurrentBuffer()
-{
-    auto& items = m_displayList.m_items;
-    if (!items)
-        return;
-
-    auto numberOfReadOnlyBuffers = items->m_readOnlyBuffers.size();
-    if (m_readOnlyBufferIndex < numberOfReadOnlyBuffers) {
-        auto& nextBufferHandle = items->m_readOnlyBuffers[m_readOnlyBufferIndex];
-        m_cursor = nextBufferHandle.data;
-        m_currentEndOfBuffer = m_cursor + nextBufferHandle.capacity;
-    } else if (m_readOnlyBufferIndex == numberOfReadOnlyBuffers) {
-        m_cursor = items->m_writableBuffer.data;
-        m_currentEndOfBuffer = m_cursor + items->m_writtenNumberOfBytes;
-    }
-}
-
-
-} // namespace DisplayList
-
-TextStream& operator<<(TextStream& ts, const DisplayList::DisplayList& displayList)
+TextStream& operator<<(TextStream& ts, const DisplayList& displayList)
 {
     displayList.dump(ts);
     return ts;
 }
 
+} // namespace DisplayList
+
 } // namespace WebCore
