@@ -23,14 +23,20 @@
 
 #if USE(GSTREAMER)
 
+#include "ApplicationGLib.h"
 #include "GLVideoSinkGStreamer.h"
+#include "GStreamerAudioMixer.h"
+#include "GUniquePtrGStreamer.h"
 #include "GstAllocatorFastMalloc.h"
 #include "IntSize.h"
+#include "RuntimeApplicationChecks.h"
 #include "SharedBuffer.h"
+#include "WebKitAudioSinkGStreamer.h"
 #include <gst/audio/audio-info.h>
 #include <gst/gst.h>
 #include <mutex>
-#include <wtf/glib/GLibUtilities.h>
+#include <wtf/FileSystem.h>
+#include <wtf/Scope.h>
 #include <wtf/glib/GUniquePtr.h>
 #include <wtf/glib/RunLoopSourcePriority.h>
 
@@ -50,19 +56,21 @@
 
 #if ENABLE(MEDIA_STREAM)
 #include "GStreamerMediaStreamSource.h"
+#include "GStreamerVideoEncoder.h"
 #endif
 
-#if ENABLE(ENCRYPTED_MEDIA)
-#include "WebKitClearKeyDecryptorGStreamer.h"
-#if ENABLE(THUNDER)
+#if ENABLE(ENCRYPTED_MEDIA) && ENABLE(THUNDER)
+#include "CDMThunder.h"
 #include "WebKitThunderDecryptorGStreamer.h"
 #endif
-#endif
 
 #if ENABLE(VIDEO)
 #include "WebKitWebSourceGStreamer.h"
 #endif
 
+GST_DEBUG_CATEGORY(webkit_gst_common_debug);
+#define GST_CAT_DEFAULT webkit_gst_common_debug
+
 namespace WebCore {
 
 GstPad* webkitGstGhostPadFromStaticTemplate(GstStaticPadTemplate* staticPadTemplate, const gchar* name, GstPad* target)
@@ -81,7 +89,7 @@ GstPad* webkitGstGhostPadFromStaticTemplate(GstStaticPadTemplate* staticPadTempl
 }
 
 #if ENABLE(VIDEO)
-bool getVideoSizeAndFormatFromCaps(GstCaps* caps, WebCore::IntSize& size, GstVideoFormat& format, int& pixelAspectRatioNumerator, int& pixelAspectRatioDenominator, int& stride)
+bool getVideoSizeAndFormatFromCaps(const GstCaps* caps, WebCore::IntSize& size, GstVideoFormat& format, int& pixelAspectRatioNumerator, int& pixelAspectRatioDenominator, int& stride)
 {
     if (!doCapsHaveType(caps, GST_VIDEO_CAPS_TYPE_PREFIX)) {
         GST_WARNING("Failed to get the video size and format, these are not a video caps");
@@ -119,11 +127,11 @@ bool getVideoSizeAndFormatFromCaps(GstCaps* caps, WebCore::IntSize& size, GstVid
     return true;
 }
 
-Optional<FloatSize> getVideoResolutionFromCaps(const GstCaps* caps)
+std::optional<FloatSize> getVideoResolutionFromCaps(const GstCaps* caps)
 {
     if (!doCapsHaveType(caps, GST_VIDEO_CAPS_TYPE_PREFIX)) {
         GST_WARNING("Failed to get the video resolution, these are not a video caps");
-        return WTF::nullopt;
+        return std::nullopt;
     }
 
     int width = 0, height = 0;
@@ -138,7 +146,7 @@ Optional<FloatSize> getVideoResolutionFromCaps(const GstCaps* caps)
         GstVideoInfo info;
         gst_video_info_init(&info);
         if (!gst_video_info_from_caps(&info, caps))
-            return WTF::nullopt;
+            return std::nullopt;
 
         width = GST_VIDEO_INFO_WIDTH(&info);
         height = GST_VIDEO_INFO_HEIGHT(&info);
@@ -146,7 +154,7 @@ Optional<FloatSize> getVideoResolutionFromCaps(const GstCaps* caps)
         pixelAspectRatioDenominator = GST_VIDEO_INFO_PAR_D(&info);
     }
 
-    return makeOptional(FloatSize(width, height * (static_cast<float>(pixelAspectRatioDenominator) / static_cast<float>(pixelAspectRatioNumerator))));
+    return std::make_optional(FloatSize(width, height * (static_cast<float>(pixelAspectRatioDenominator) / static_cast<float>(pixelAspectRatioNumerator))));
 }
 
 bool getSampleVideoInfo(GstSample* sample, GstVideoInfo& videoInfo)
@@ -176,7 +184,7 @@ const char* capsMediaType(const GstCaps* caps)
         return nullptr;
     }
 #if ENABLE(ENCRYPTED_MEDIA)
-    if (gst_structure_has_name(structure, "application/x-cenc") || gst_structure_has_name(structure, "application/x-webm-enc"))
+    if (gst_structure_has_name(structure, "application/x-cenc") || gst_structure_has_name(structure, "application/x-cbcs") || gst_structure_has_name(structure, "application/x-webm-enc"))
         return gst_structure_get_string(structure, "original-media-type");
 #endif
     return gst_structure_get_name(structure);
@@ -208,6 +216,12 @@ bool areEncryptedCaps(const GstCaps* caps)
 #endif
 }
 
+static std::optional<Vector<String>> s_UIProcessCommandLineOptions;
+void setGStreamerOptionsFromUIProcess(Vector<String>&& options)
+{
+    s_UIProcessCommandLineOptions = WTFMove(options);
+}
+
 Vector<String> extractGStreamerOptionsFromCommandLine()
 {
     GUniqueOutPtr<char> contents;
@@ -224,11 +238,12 @@ Vector<String> extractGStreamerOptionsFromCommandLine()
     return options;
 }
 
-bool initializeGStreamer(Optional<Vector<String>>&& options)
+bool ensureGStreamerInitialized()
 {
+    RELEASE_ASSERT(isInWebProcess());
     static std::once_flag onceFlag;
     static bool isGStreamerInitialized;
-    std::call_once(onceFlag, [options = WTFMove(options)] {
+    std::call_once(onceFlag, [] {
         isGStreamerInitialized = false;
 
         // USE_PLAYBIN3 is dangerous for us because its potential sneaky effect
@@ -239,10 +254,11 @@ bool initializeGStreamer(Optional<Vector<String>>&& options)
             WTFLogAlways("The USE_PLAYBIN3 variable was detected in the environment. Expect playback issues or please unset it.");
 
 #if ENABLE(VIDEO) || ENABLE(WEB_AUDIO)
-        Vector<String> parameters = options.valueOr(extractGStreamerOptionsFromCommandLine());
+        Vector<String> parameters = s_UIProcessCommandLineOptions.value_or(extractGStreamerOptionsFromCommandLine());
+        s_UIProcessCommandLineOptions.reset();
         char** argv = g_new0(char*, parameters.size() + 2);
         int argc = parameters.size() + 1;
-        argv[0] = g_strdup(getCurrentExecutableName().data());
+        argv[0] = g_strdup(FileSystem::currentExecutableName().data());
         for (unsigned i = 0; i < parameters.size(); i++)
             argv[i + 1] = g_strdup(parameters[i].utf8().data());
 
@@ -250,6 +266,7 @@ bool initializeGStreamer(Optional<Vector<String>>&& options)
         isGStreamerInitialized = gst_init_check(&argc, &argv, &error.outPtr());
         ASSERT_WITH_MESSAGE(isGStreamerInitialized, "GStreamer initialization failed: %s", error ? error->message : "unknown error occurred");
         g_strfreev(argv);
+        GST_DEBUG_CATEGORY_INIT(webkit_gst_common_debug, "webkitcommon", 0, "WebKit Common utilities");
 
         if (isFastMallocEnabled()) {
             const char* disableFastMalloc = getenv("WEBKIT_GST_DISABLE_FAST_MALLOC");
@@ -285,27 +302,31 @@ bool isThunderRanked()
 }
 #endif
 
-bool initializeGStreamerAndRegisterWebKitElements()
+void registerWebKitGStreamerElements()
 {
-    if (!initializeGStreamer())
-        return false;
-
     static std::once_flag onceFlag;
     std::call_once(onceFlag, [] {
 #if USE(GSTREAMER_FULL)
         gst_init_static_plugins();
 #endif
 
-#if ENABLE(ENCRYPTED_MEDIA)
-        gst_element_register(nullptr, "webkitclearkey", GST_RANK_PRIMARY + 200, WEBKIT_TYPE_MEDIA_CK_DECRYPT);
-#if ENABLE(THUNDER)
-        unsigned thunderRank = isThunderRanked() ? 300 : 100;
-        gst_element_register(nullptr, "webkitthunder", GST_RANK_PRIMARY + thunderRank, WEBKIT_TYPE_MEDIA_THUNDER_DECRYPT);
+#if ENABLE(ENCRYPTED_MEDIA) && ENABLE(THUNDER)
+        if (!CDMFactoryThunder::singleton().supportedKeySystems().isEmpty()) {
+            unsigned thunderRank = isThunderRanked() ? 300 : 100;
+            gst_element_register(nullptr, "webkitthunder", GST_RANK_PRIMARY + thunderRank, WEBKIT_TYPE_MEDIA_THUNDER_DECRYPT);
+        }
+#ifndef NDEBUG
+        else if (isThunderRanked()) {
+            GST_WARNING("Thunder is up-ranked as preferred decryptor but Thunder is not supporting any encryption system. Is "
+                "Thunder running? Are the plugins built?");
+        }
 #endif
+
 #endif
 
 #if ENABLE(MEDIA_STREAM)
         gst_element_register(nullptr, "mediastreamsrc", GST_RANK_PRIMARY, WEBKIT_TYPE_MEDIA_STREAM_SRC);
+        gst_element_register(nullptr, "webrtcvideoencoder", GST_RANK_PRIMARY + 100, WEBKIT_TYPE_WEBRTC_VIDEO_ENCODER);
 #endif
 
 #if ENABLE(MEDIA_SOURCE)
@@ -315,9 +336,11 @@ bool initializeGStreamerAndRegisterWebKitElements()
 #if ENABLE(VIDEO)
         gst_element_register(0, "webkitwebsrc", GST_RANK_PRIMARY + 100, WEBKIT_TYPE_WEB_SRC);
 #if USE(GSTREAMER_GL)
-        gst_element_register(0, "webkitglvideosink", GST_RANK_PRIMARY, WEBKIT_TYPE_GL_VIDEO_SINK);
+        gst_element_register(0, "webkitglvideosink", GST_RANK_NONE, WEBKIT_TYPE_GL_VIDEO_SINK);
 #endif
 #endif
+        // We don't want autoaudiosink to autoplug our sink.
+        gst_element_register(0, "webkitaudiosink", GST_RANK_NONE, WEBKIT_TYPE_AUDIO_SINK);
 
         // If the FDK-AAC decoder is available, promote it and downrank the
         // libav AAC decoders, due to their broken LC support, as reported in:
@@ -334,7 +357,6 @@ bool initializeGStreamerAndRegisterWebKitElements()
             }
         }
     });
-    return true;
 }
 
 unsigned getGstPlayFlag(const char* nick)
@@ -360,57 +382,72 @@ uint64_t toGstUnsigned64Time(const MediaTime& mediaTime)
     return time.timeValue();
 }
 
-static void simpleBusMessageCallback(GstBus*, GstMessage* message, GstBin* pipeline)
-{
-    switch (GST_MESSAGE_TYPE(message)) {
-    case GST_MESSAGE_ERROR:
-        GST_ERROR_OBJECT(pipeline, "Got message: %" GST_PTR_FORMAT, message);
-        {
-            WTF::String dotFileName = makeString(GST_OBJECT_NAME(pipeline), "_error");
-            GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS(pipeline, GST_DEBUG_GRAPH_SHOW_ALL, dotFileName.utf8().data());
-        }
-        break;
-    case GST_MESSAGE_STATE_CHANGED:
-        if (GST_MESSAGE_SRC(message) == GST_OBJECT(pipeline)) {
-            GstState oldState, newState, pending;
-            gst_message_parse_state_changed(message, &oldState, &newState, &pending);
-
-            GST_INFO_OBJECT(pipeline, "State changed (old: %s, new: %s, pending: %s)",
-                gst_element_state_get_name(oldState),
-                gst_element_state_get_name(newState),
-                gst_element_state_get_name(pending));
-
-            WTF::String dotFileName = makeString(
-                GST_OBJECT_NAME(pipeline), '_',
-                gst_element_state_get_name(oldState), '_',
-                gst_element_state_get_name(newState));
-
-            GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS(GST_BIN(pipeline), GST_DEBUG_GRAPH_SHOW_ALL, dotFileName.utf8().data());
-        }
-        break;
-    default:
-        break;
-    }
-}
-
 void disconnectSimpleBusMessageCallback(GstElement* pipeline)
 {
-    GRefPtr<GstBus> bus = adoptGRef(gst_pipeline_get_bus(GST_PIPELINE(pipeline)));
-    g_signal_handlers_disconnect_by_func(bus.get(), reinterpret_cast<gpointer>(simpleBusMessageCallback), pipeline);
+    auto bus = adoptGRef(gst_pipeline_get_bus(GST_PIPELINE(pipeline)));
+    g_signal_handlers_disconnect_by_data(bus.get(), pipeline);
+    gst_bus_remove_signal_watch(bus.get());
 }
 
-void connectSimpleBusMessageCallback(GstElement* pipeline)
+struct CustomMessageHandlerHolder {
+    explicit CustomMessageHandlerHolder(Function<void(GstMessage*)>&& handler)
+    {
+        this->handler = WTFMove(handler);
+    }
+    Function<void(GstMessage*)> handler;
+};
+
+void connectSimpleBusMessageCallback(GstElement* pipeline, Function<void(GstMessage*)>&& customHandler)
 {
-    GRefPtr<GstBus> bus = adoptGRef(gst_pipeline_get_bus(GST_PIPELINE(pipeline)));
+    auto bus = adoptGRef(gst_pipeline_get_bus(GST_PIPELINE(pipeline)));
     gst_bus_add_signal_watch_full(bus.get(), RunLoopSourcePriority::RunLoopDispatcher);
-    g_signal_connect(bus.get(), "message", G_CALLBACK(simpleBusMessageCallback), pipeline);
+
+    auto* holder = new CustomMessageHandlerHolder(WTFMove(customHandler));
+    GQuark quark = g_quark_from_static_string("pipeline-custom-message-handler");
+    g_object_set_qdata_full(G_OBJECT(pipeline), quark, holder, [](gpointer data) {
+        delete reinterpret_cast<CustomMessageHandlerHolder*>(data);
+    });
+
+    g_signal_connect(bus.get(), "message", G_CALLBACK(+[](GstBus*, GstMessage* message, GstElement* pipeline) {
+        switch (GST_MESSAGE_TYPE(message)) {
+        case GST_MESSAGE_ERROR: {
+            GST_ERROR_OBJECT(pipeline, "Got message: %" GST_PTR_FORMAT, message);
+            auto dotFileName = makeString(GST_OBJECT_NAME(pipeline), "_error");
+            GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS(GST_BIN_CAST(pipeline), GST_DEBUG_GRAPH_SHOW_ALL, dotFileName.utf8().data());
+            break;
+        }
+        case GST_MESSAGE_STATE_CHANGED: {
+            if (GST_MESSAGE_SRC(message) != GST_OBJECT_CAST(pipeline))
+                break;
+
+            GstState oldState;
+            GstState newState;
+            GstState pending;
+            gst_message_parse_state_changed(message, &oldState, &newState, &pending);
+
+            GST_INFO_OBJECT(pipeline, "State changed (old: %s, new: %s, pending: %s)", gst_element_state_get_name(oldState),
+                gst_element_state_get_name(newState), gst_element_state_get_name(pending));
+
+            auto dotFileName = makeString(GST_OBJECT_NAME(pipeline), '_', gst_element_state_get_name(oldState), '_', gst_element_state_get_name(newState));
+            GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS(GST_BIN_CAST(pipeline), GST_DEBUG_GRAPH_SHOW_ALL, dotFileName.utf8().data());
+            break;
+        }
+        default:
+            break;
+        }
+
+        GQuark quark = g_quark_from_static_string("pipeline-custom-message-handler");
+        auto* holder = reinterpret_cast<CustomMessageHandlerHolder*>(g_object_get_qdata(G_OBJECT(pipeline), quark));
+        if (!holder)
+            return;
+
+        holder->handler(message);
+    }), pipeline);
 }
 
 Vector<uint8_t> GstMappedBuffer::createVector() const
 {
-    Vector<uint8_t> vector;
-    vector.append(data(), size());
-    return vector;
+    return { data(), size() };
 }
 
 Ref<SharedBuffer> GstMappedOwnedBuffer::createSharedBuffer()
@@ -426,6 +463,180 @@ bool isGStreamerPluginAvailable(const char* name)
     return plugin;
 }
 
+bool gstElementFactoryEquals(GstElement* element, const char* name)
+{
+    return equal(GST_OBJECT_NAME(gst_element_get_factory(element)), name);
+}
+
+GstElement* createAutoAudioSink(const String& role)
+{
+    auto* audioSink = makeGStreamerElement("autoaudiosink", nullptr);
+    g_signal_connect_data(audioSink, "child-added", G_CALLBACK(+[](GstChildProxy*, GObject* object, gchar*, gpointer userData) {
+        auto* role = reinterpret_cast<StringImpl*>(userData);
+        auto* objectClass = G_OBJECT_GET_CLASS(object);
+        if (role && g_object_class_find_property(objectClass, "stream-properties")) {
+            GUniquePtr<GstStructure> properties(gst_structure_new("stream-properties", "media.role", G_TYPE_STRING, role->utf8().data(), nullptr));
+            g_object_set(object, "stream-properties", properties.get(), nullptr);
+            GST_DEBUG("Set media.role as %s on %" GST_PTR_FORMAT, role->utf8().data(), GST_ELEMENT_CAST(object));
+        }
+        if (g_object_class_find_property(objectClass, "client-name"))
+            g_object_set(object, "client-name", getApplicationName(), nullptr);
+    }), role.isolatedCopy().releaseImpl().leakRef(), static_cast<GClosureNotify>([](gpointer userData, GClosure*) {
+        reinterpret_cast<StringImpl*>(userData)->deref();
+    }), static_cast<GConnectFlags>(0));
+    return audioSink;
+}
+
+GstElement* createPlatformAudioSink(const String& role)
+{
+    GstElement* audioSink = webkitAudioSinkNew();
+    if (!audioSink) {
+        // This means the WebKit audio sink configuration failed. It can happen for the following reasons:
+        // - audio mixing was not requested using the WEBKIT_GST_ENABLE_AUDIO_MIXER
+        // - audio mixing was requested using the WEBKIT_GST_ENABLE_AUDIO_MIXER but the audio mixer
+        //   runtime requirements are not fullfilled.
+        // - the sink was created for the WPE port, audio mixing was not requested and no
+        //   WPEBackend-FDO audio receiver has been registered at runtime.
+        audioSink = createAutoAudioSink(role);
+    }
+
+    return audioSink;
+}
+
+bool webkitGstSetElementStateSynchronously(GstElement* pipeline, GstState targetState, Function<bool(GstMessage*)>&& messageHandler)
+{
+    GST_DEBUG_OBJECT(pipeline, "Setting state to %s", gst_element_state_get_name(targetState));
+
+    GstState currentState;
+    auto result = gst_element_get_state(pipeline, &currentState, nullptr, 10);
+    if (result == GST_STATE_CHANGE_SUCCESS && currentState == targetState) {
+        GST_DEBUG_OBJECT(pipeline, "Target state already reached");
+        return true;
+    }
+
+    auto bus = adoptGRef(gst_pipeline_get_bus(GST_PIPELINE(pipeline)));
+    gst_bus_enable_sync_message_emission(bus.get());
+
+    auto cleanup = makeScopeExit([bus = GRefPtr<GstBus>(bus), pipeline, targetState] {
+        gst_bus_disable_sync_message_emission(bus.get());
+        GstState currentState;
+        auto result = gst_element_get_state(pipeline, &currentState, nullptr, 0);
+        GST_DEBUG_OBJECT(pipeline, "Task finished, result: %s, target state reached: %s", gst_element_state_change_return_get_name(result), boolForPrinting(currentState == targetState));
+    });
+
+    result = gst_element_set_state(pipeline, targetState);
+    if (result == GST_STATE_CHANGE_FAILURE)
+        return false;
+
+    if (result == GST_STATE_CHANGE_ASYNC) {
+        while (auto message = adoptGRef(gst_bus_timed_pop_filtered(bus.get(), GST_CLOCK_TIME_NONE, GST_MESSAGE_STATE_CHANGED))) {
+            if (!messageHandler(message.get()))
+                return false;
+
+            result = gst_element_get_state(pipeline, &currentState, nullptr, 10);
+            if (result == GST_STATE_CHANGE_FAILURE)
+                return false;
+
+            if (currentState == targetState)
+                return true;
+        }
+    }
+    return true;
+}
+
+GstBuffer* gstBufferNewWrappedFast(void* data, size_t length)
+{
+    return gst_buffer_new_wrapped_full(static_cast<GstMemoryFlags>(0), data, length, 0, length, data, fastFree);
+}
+
+GstElement* makeGStreamerElement(const char* factoryName, const char* name)
+{
+    auto* element = gst_element_factory_make(factoryName, name);
+    if (!element)
+        WTFLogAlways("GStreamer element %s not found. Please install it", factoryName);
+    return element;
+}
+
+GstElement* makeGStreamerBin(const char* description, bool ghostUnlinkedPads)
+{
+    GUniqueOutPtr<GError> error;
+    auto* bin = gst_parse_bin_from_description(description, ghostUnlinkedPads, &error.outPtr());
+    if (!bin)
+        WTFLogAlways("Unable to create bin for description: \"%s\". Error: %s", description, error->message);
+    return bin;
+}
+
+static RefPtr<JSON::Value> gstStructureToJSON(const GstStructure*);
+
+static std::optional<RefPtr<JSON::Value>> gstStructureValueToJSON(const GValue* value)
+{
+    if (GST_VALUE_HOLDS_STRUCTURE(value))
+        return gstStructureToJSON(gst_value_get_structure(value));
+
+    if (GST_VALUE_HOLDS_ARRAY(value)) {
+        unsigned size = gst_value_array_get_size(value);
+        auto array = JSON::Array::create();
+        for (unsigned i = 0; i < size; i++) {
+            if (auto innerJson = gstStructureValueToJSON(gst_value_array_get_value(value, i)))
+                array->pushValue(innerJson->releaseNonNull());
+        }
+        return array->asArray()->asValue();
+    }
+    auto valueType = G_VALUE_TYPE(value);
+    if (valueType == G_TYPE_BOOLEAN)
+        return JSON::Value::create(g_value_get_boolean(value))->asValue();
+
+    if (valueType == G_TYPE_INT)
+        return JSON::Value::create(g_value_get_int(value))->asValue();
+
+    if (valueType == G_TYPE_UINT)
+        return JSON::Value::create(static_cast<int>(g_value_get_uint(value)))->asValue();
+
+    if (valueType == G_TYPE_DOUBLE)
+        return JSON::Value::create(g_value_get_double(value))->asValue();
+
+    if (valueType == G_TYPE_FLOAT)
+        return JSON::Value::create(static_cast<double>(g_value_get_float(value)))->asValue();
+
+    // FIXME: bigint support missing in JSON.
+    if (valueType == G_TYPE_UINT64)
+        return JSON::Value::create(static_cast<int>(g_value_get_uint64(value)))->asValue();
+
+    if (valueType == G_TYPE_STRING)
+        return JSON::Value::create(makeString(g_value_get_string(value)))->asValue();
+
+    GST_WARNING("Unhandled GValue type: %s", G_VALUE_TYPE_NAME(value));
+    return { };
+}
+
+static gboolean parseGstStructureValue(GQuark fieldId, const GValue* value, gpointer userData)
+{
+    if (auto jsonValue = gstStructureValueToJSON(value)) {
+        auto* object = reinterpret_cast<JSON::Object*>(userData);
+        object->setValue(g_quark_to_string(fieldId), jsonValue->releaseNonNull());
+    }
+    return TRUE;
+}
+
+static RefPtr<JSON::Value> gstStructureToJSON(const GstStructure* structure)
+{
+    auto jsonObject = JSON::Object::create();
+    auto resultValue = jsonObject->asObject();
+    if (!resultValue)
+        return nullptr;
+
+    gst_structure_foreach(structure, parseGstStructureValue, resultValue.get());
+    return resultValue;
+}
+
+String gstStructureToJSONString(const GstStructure* structure)
+{
+    auto value = gstStructureToJSON(structure);
+    if (!value)
+        return { };
+    return value->toJSONString();
+}
+
 }
 
 #endif // USE(GSTREAMER)
