@@ -37,10 +37,14 @@
 #include <type_traits>
 #include <wtf/text/TextStream.h>
 
+#if PLATFORM(WPE) || PLATFORM(GTK)
+#include "ThemeAdwaita.h"
+#endif
+
 namespace Nicosia {
 using namespace WebCore;
 
-PlatformContextCairo& contextForReplay(PaintingOperationReplay& operationReplay)
+GraphicsContextCairo& contextForReplay(PaintingOperationReplay& operationReplay)
 {
     return static_cast<PaintingOperationReplayCairo&>(operationReplay).platformContext;
 }
@@ -72,14 +76,13 @@ auto createCommand() -> std::enable_if_t<std::is_base_of<OperationData<>, T>::va
     return makeUnique<T>();
 }
 
-CairoOperationRecorder::CairoOperationRecorder(GraphicsContext& context, PaintingOperations& commandList)
-    : GraphicsContextImpl(context, FloatRect { }, AffineTransform { })
-    , m_commandList(commandList)
+CairoOperationRecorder::CairoOperationRecorder(PaintingOperations& commandList)
+    : m_commandList(commandList)
 {
     m_stateStack.append({ { }, { }, FloatRect::infiniteRect() });
 }
 
-void CairoOperationRecorder::updateState(const GraphicsContextState& state, GraphicsContextState::StateChangeFlags flags)
+void CairoOperationRecorder::didUpdateState(const GraphicsContextState& state, GraphicsContextState::StateChangeFlags flags)
 {
     if (flags & GraphicsContextState::StrokeThicknessChange) {
         struct StrokeThicknessChange final : PaintingOperation, OperationData<float> {
@@ -154,10 +157,6 @@ void CairoOperationRecorder::updateState(const GraphicsContextState& state, Grap
     }
 }
 
-void CairoOperationRecorder::clearShadow()
-{
-}
-
 void CairoOperationRecorder::setLineCap(LineCap lineCap)
 {
     struct SetLineCap final : PaintingOperation, OperationData<LineCap> {
@@ -250,7 +249,7 @@ void CairoOperationRecorder::fillRect(const FloatRect& rect)
         }
     };
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<FillRect>(rect, Cairo::FillSource(state), Cairo::ShadowState(state)));
 }
 
@@ -270,7 +269,7 @@ void CairoOperationRecorder::fillRect(const FloatRect& rect, const Color& color)
         }
     };
 
-    append(createCommand<FillRect>(rect, color, Cairo::ShadowState(graphicsContext().state())));
+    append(createCommand<FillRect>(rect, color, Cairo::ShadowState(state())));
 }
 
 void CairoOperationRecorder::fillRect(const FloatRect& rect, Gradient& gradient)
@@ -281,9 +280,9 @@ void CairoOperationRecorder::fillRect(const FloatRect& rect, Gradient& gradient)
         void execute(PaintingOperationReplay& replayer) override
         {
             auto& platformContext = contextForReplay(replayer);
-            Cairo::save(platformContext);
+            platformContext.save();
             Cairo::fillRect(platformContext, arg<0>(), arg<1>().get());
-            Cairo::restore(platformContext);
+            platformContext.restore();
         }
 
         void dump(TextStream& ts) override
@@ -292,7 +291,8 @@ void CairoOperationRecorder::fillRect(const FloatRect& rect, Gradient& gradient)
         }
     };
 
-    append(createCommand<FillRect>(rect, gradient.createPattern(1.0)));
+    auto& state = this->state();
+    append(createCommand<FillRect>(rect, gradient.createPattern(1.0, state.fillGradientSpaceTransform)));
 }
 
 void CairoOperationRecorder::fillRect(const FloatRect& rect, const Color& color, CompositeOperator compositeOperator, BlendMode blendMode)
@@ -315,7 +315,7 @@ void CairoOperationRecorder::fillRect(const FloatRect& rect, const Color& color,
         }
     };
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<FillRect>(rect, color, compositeOperator, blendMode, Cairo::ShadowState(state), state.compositeOperator));
 }
 
@@ -345,7 +345,7 @@ void CairoOperationRecorder::fillRoundedRect(const FloatRoundedRect& roundedRect
         }
     };
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<FillRoundedRect>(roundedRect, color, state.compositeOperator, blendMode, Cairo::ShadowState(state)));
 }
 
@@ -366,7 +366,7 @@ void CairoOperationRecorder::fillRectWithRoundedHole(const FloatRect& rect, cons
     };
 
     UNUSED_PARAM(color);
-    append(createCommand<FillRectWithRoundedHole>(rect, roundedHoleRect, Cairo::ShadowState(graphicsContext().state())));
+    append(createCommand<FillRectWithRoundedHole>(rect, roundedHoleRect, Cairo::ShadowState(state())));
 }
 
 void CairoOperationRecorder::fillPath(const Path& path)
@@ -385,7 +385,10 @@ void CairoOperationRecorder::fillPath(const Path& path)
         }
     };
 
-    auto& state = graphicsContext().state();
+    if (path.isEmpty())
+        return;
+
+    auto& state = this->state();
     append(createCommand<FillPath>(path, Cairo::FillSource(state), Cairo::ShadowState(state)));
 }
 
@@ -407,7 +410,7 @@ void CairoOperationRecorder::fillEllipse(const FloatRect& rect)
         }
     };
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<FillEllipse>(rect, Cairo::FillSource(state), Cairo::ShadowState(state)));
 }
 
@@ -427,7 +430,7 @@ void CairoOperationRecorder::strokeRect(const FloatRect& rect, float lineWidth)
         }
     };
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<StrokeRect>(rect, lineWidth, Cairo::StrokeSource(state), Cairo::ShadowState(state)));
 }
 
@@ -447,7 +450,10 @@ void CairoOperationRecorder::strokePath(const Path& path)
         }
     };
 
-    auto& state = graphicsContext().state();
+    if (path.isEmpty())
+        return;
+
+    auto& state = this->state();
     append(createCommand<StrokePath>(path, Cairo::StrokeSource(state), Cairo::ShadowState(state)));
 }
 
@@ -469,7 +475,7 @@ void CairoOperationRecorder::strokeEllipse(const FloatRect& rect)
         }
     };
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<StrokeEllipse>(rect, Cairo::StrokeSource(state), Cairo::ShadowState(state)));
 }
 
@@ -492,7 +498,7 @@ void CairoOperationRecorder::clearRect(const FloatRect& rect)
     append(createCommand<ClearRect>(rect));
 }
 
-void CairoOperationRecorder::drawGlyphs(const Font& font, const GlyphBuffer& glyphBuffer, unsigned from, unsigned numGlyphs, const FloatPoint& point, FontSmoothingMode fontSmoothing)
+void CairoOperationRecorder::drawGlyphs(const Font& font, const GlyphBufferGlyph* glyphs, const GlyphBufferAdvance* advances, unsigned numGlyphs, const FloatPoint& point, FontSmoothingMode fontSmoothing)
 {
     struct DrawGlyphs final : PaintingOperation, OperationData<Cairo::FillSource, Cairo::StrokeSource, Cairo::ShadowState, FloatPoint, RefPtr<cairo_scaled_font_t>, float, Vector<cairo_glyph_t>, float, TextDrawingModeFlags, float, FloatSize, Color, FontSmoothingMode> {
         virtual ~DrawGlyphs() = default;
@@ -513,50 +519,36 @@ void CairoOperationRecorder::drawGlyphs(const Font& font, const GlyphBuffer& gly
         return;
 
     auto xOffset = point.x();
-    Vector<cairo_glyph_t> glyphs(numGlyphs);
+    Vector<cairo_glyph_t> cairoGlyphs(numGlyphs);
     {
-        ASSERT(from + numGlyphs <= glyphBuffer.size());
-        auto* glyphsData = glyphBuffer.glyphs(from);
-        auto* advances = glyphBuffer.advances(from);
-
         auto yOffset = point.y();
         for (size_t i = 0; i < numGlyphs; ++i) {
-            glyphs[i] = { glyphsData[i], xOffset, yOffset };
+            cairoGlyphs[i] = { glyphs[i], xOffset, yOffset };
             xOffset += advances[i].width();
         }
     }
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<DrawGlyphs>(Cairo::FillSource(state), Cairo::StrokeSource(state),
         Cairo::ShadowState(state), point,
         RefPtr<cairo_scaled_font_t>(font.platformData().scaledFont()),
-        font.syntheticBoldOffset(), WTFMove(glyphs), xOffset, state.textDrawingMode,
+        font.syntheticBoldOffset(), WTFMove(cairoGlyphs), xOffset, state.textDrawingMode,
         state.strokeThickness, state.shadowOffset, state.shadowColor, fontSmoothing));
 }
 
-ImageDrawResult CairoOperationRecorder::drawImage(Image& image, const FloatRect& destination, const FloatRect& source, const ImagePaintingOptions& imagePaintingOptions)
+void CairoOperationRecorder::drawImageBuffer(ImageBuffer&, const FloatRect&, const FloatRect&, const ImagePaintingOptions&)
 {
-    return GraphicsContextImpl::drawImageImpl(graphicsContext(), image, destination, source, imagePaintingOptions);
+    // FIXME: Not implemented.
 }
 
-ImageDrawResult CairoOperationRecorder::drawTiledImage(Image& image, const FloatRect& destination, const FloatPoint& source, const FloatSize& tileSize, const FloatSize& spacing, const ImagePaintingOptions& imagePaintingOptions)
-{
-    return GraphicsContextImpl::drawTiledImageImpl(graphicsContext(), image, destination, source, tileSize, spacing, imagePaintingOptions);
-}
-
-ImageDrawResult CairoOperationRecorder::drawTiledImage(Image& image, const FloatRect& destination, const FloatRect& source, const FloatSize& tileScaleFactor, Image::TileRule hRule, Image::TileRule vRule, const ImagePaintingOptions& imagePaintingOptions)
-{
-    return GraphicsContextImpl::drawTiledImageImpl(graphicsContext(), image, destination, source, tileScaleFactor, hRule, vRule, imagePaintingOptions);
-}
-
-void CairoOperationRecorder::drawNativeImage(const NativeImagePtr& image, const FloatSize& imageSize, const FloatRect& destRect, const FloatRect& srcRect, const ImagePaintingOptions& options)
+void CairoOperationRecorder::drawNativeImage(NativeImage& nativeImage, const FloatSize& imageSize, const FloatRect& destRect, const FloatRect& srcRect, const ImagePaintingOptions& options)
 {
     struct DrawNativeImage final : PaintingOperation, OperationData<RefPtr<cairo_surface_t>, FloatRect, FloatRect, ImagePaintingOptions, float, Cairo::ShadowState> {
         virtual ~DrawNativeImage() = default;
 
         void execute(PaintingOperationReplay& replayer) override
         {
-            Cairo::drawNativeImage(contextForReplay(replayer), arg<0>().get(), arg<1>(), arg<2>(), arg<3>(), arg<4>(), arg<5>());
+            Cairo::drawPlatformImage(contextForReplay(replayer), arg<0>().get(), arg<1>(), arg<2>(), arg<3>(), arg<4>(), arg<5>());
         }
 
         void dump(TextStream& ts) override
@@ -566,11 +558,11 @@ void CairoOperationRecorder::drawNativeImage(const NativeImagePtr& image, const
     };
 
     UNUSED_PARAM(imageSize);
-    auto& state = graphicsContext().state();
-    append(createCommand<DrawNativeImage>(RefPtr<cairo_surface_t>(image.get()), destRect, srcRect, ImagePaintingOptions(options, state.imageInterpolationQuality), state.alpha, Cairo::ShadowState(state)));
+    auto& state = this->state();
+    append(createCommand<DrawNativeImage>(nativeImage.platformImage(), destRect, srcRect, ImagePaintingOptions(options, state.imageInterpolationQuality), state.alpha, Cairo::ShadowState(state)));
 }
 
-void CairoOperationRecorder::drawPattern(Image& image, const FloatRect& destRect, const FloatRect& tileRect, const AffineTransform& patternTransform, const FloatPoint& phase, const FloatSize& spacing, const ImagePaintingOptions& options)
+void CairoOperationRecorder::drawPattern(NativeImage& nativeImage, const FloatRect& destRect, const FloatRect& tileRect, const AffineTransform& patternTransform, const FloatPoint& phase, const FloatSize& spacing, const ImagePaintingOptions& options)
 {
     struct DrawPattern final : PaintingOperation, OperationData<RefPtr<cairo_surface_t>, IntSize, FloatRect, FloatRect, AffineTransform, FloatPoint, ImagePaintingOptions> {
         virtual ~DrawPattern() = default;
@@ -587,8 +579,7 @@ void CairoOperationRecorder::drawPattern(Image& image, const FloatRect& destRect
     };
 
     UNUSED_PARAM(spacing);
-    if (auto surface = image.nativeImageForCurrentFrame())
-        append(createCommand<DrawPattern>(WTFMove(surface), IntSize(image.size()), destRect, tileRect, patternTransform, phase, options));
+    append(createCommand<DrawPattern>(nativeImage.platformImage(), nativeImage.size(), destRect, tileRect, patternTransform, phase, options));
 }
 
 void CairoOperationRecorder::drawRect(const FloatRect& rect, float borderThickness)
@@ -607,7 +598,7 @@ void CairoOperationRecorder::drawRect(const FloatRect& rect, float borderThickne
         }
     };
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<DrawRect>(rect, borderThickness, state.fillColor, state.strokeStyle, state.strokeColor));
 }
 
@@ -627,11 +618,14 @@ void CairoOperationRecorder::drawLine(const FloatPoint& point1, const FloatPoint
         }
     };
 
-    auto& state = graphicsContext().state();
+    if (strokeStyle() == NoStroke)
+        return;
+
+    auto& state = this->state();
     append(createCommand<DrawLine>(point1, point2, state.strokeStyle, state.strokeColor, state.strokeThickness, state.shouldAntialias));
 }
 
-void CairoOperationRecorder::drawLinesForText(const FloatPoint& point, float thickness, const DashArray& widths, bool printing, bool doubleUnderlines)
+void CairoOperationRecorder::drawLinesForText(const FloatPoint& point, float thickness, const DashArray& widths, bool printing, bool doubleUnderlines, StrokeStyle)
 {
     struct DrawLinesForText final : PaintingOperation, OperationData<FloatPoint, float, DashArray, bool, bool, Color> {
         virtual ~DrawLinesForText() = default;
@@ -647,7 +641,10 @@ void CairoOperationRecorder::drawLinesForText(const FloatPoint& point, float thi
         }
     };
 
-    auto& state = graphicsContext().state();
+    if (widths.isEmpty())
+        return;
+
+    auto& state = this->state();
     append(createCommand<DrawLinesForText>(point, thickness, widths, printing, doubleUnderlines, state.strokeColor));
 }
 
@@ -686,16 +683,16 @@ void CairoOperationRecorder::drawEllipse(const FloatRect& rect)
         }
     };
 
-    auto& state = graphicsContext().state();
+    auto& state = this->state();
     append(createCommand<DrawEllipse>(rect, state.fillColor, state.strokeStyle, state.strokeColor, state.strokeThickness));
 }
 
-void CairoOperationRecorder::drawPath(const Path&)
-{
-}
-
 void CairoOperationRecorder::drawFocusRing(const Path& path, float width, float offset, const Color& color)
 {
+#if PLATFORM(WPE) || PLATFORM(GTK)
+    ThemeAdwaita::paintFocus(*this, path, color);
+    UNUSED_PARAM(width);
+#else
     struct DrawFocusRing final : PaintingOperation, OperationData<Path, float, Color> {
         virtual ~DrawFocusRing() = default;
 
@@ -710,12 +707,17 @@ void CairoOperationRecorder::drawFocusRing(const Path& path, float width, float
         }
     };
 
-    UNUSED_PARAM(offset);
     append(createCommand<DrawFocusRing>(path, width, color));
+#endif
+    UNUSED_PARAM(offset);
 }
 
 void CairoOperationRecorder::drawFocusRing(const Vector<FloatRect>& rects, float width, float offset, const Color& color)
 {
+#if PLATFORM(WPE) || PLATFORM(GTK)
+    ThemeAdwaita::paintFocus(*this, rects, color);
+    UNUSED_PARAM(width);
+#else
     struct DrawFocusRing final : PaintingOperation, OperationData<Vector<FloatRect>, float, Color> {
         virtual ~DrawFocusRing() = default;
 
@@ -730,8 +732,9 @@ void CairoOperationRecorder::drawFocusRing(const Vector<FloatRect>& rects, float
         }
     };
 
-    UNUSED_PARAM(offset);
     append(createCommand<DrawFocusRing>(rects, width, color));
+#endif
+    UNUSED_PARAM(offset);
 }
 
 void CairoOperationRecorder::save()
@@ -741,7 +744,7 @@ void CairoOperationRecorder::save()
 
         void execute(PaintingOperationReplay& replayer) override
         {
-            Cairo::save(contextForReplay(replayer));
+            contextForReplay(replayer).save();
         }
 
         void dump(TextStream& ts) override
@@ -750,6 +753,8 @@ void CairoOperationRecorder::save()
         }
     };
 
+    GraphicsContext::save();
+
     append(createCommand<Save>());
 
     m_stateStack.append(m_stateStack.last());
@@ -762,7 +767,7 @@ void CairoOperationRecorder::restore()
 
         void execute(PaintingOperationReplay& replayer) override
         {
-            Cairo::restore(contextForReplay(replayer));
+            contextForReplay(replayer).restore();
         }
 
         void dump(TextStream& ts) override
@@ -771,9 +776,16 @@ void CairoOperationRecorder::restore()
         }
     };
 
+    if (!stackSize())
+        return;
+
+    GraphicsContext::restore();
+
+    if (m_stateStack.isEmpty())
+        return;
+
     append(createCommand<Restore>());
 
-    ASSERT(!m_stateStack.isEmpty());
     m_stateStack.removeLast();
     if (m_stateStack.isEmpty())
         m_stateStack.clear();
@@ -917,7 +929,7 @@ void CairoOperationRecorder::setCTM(const AffineTransform& transform)
     state.ctmInverse = inverse.value();
 }
 
-AffineTransform CairoOperationRecorder::getCTM(GraphicsContext::IncludeDeviceScale)
+AffineTransform CairoOperationRecorder::getCTM(GraphicsContext::IncludeDeviceScale) const
 {
     return m_stateStack.last().ctm;
 }
@@ -938,6 +950,8 @@ void CairoOperationRecorder::beginTransparencyLayer(float opacity)
         }
     };
 
+    GraphicsContext::beginTransparencyLayer(opacity);
+
     append(createCommand<BeginTransparencyLayer>(opacity));
 }
 
@@ -957,6 +971,8 @@ void CairoOperationRecorder::endTransparencyLayer()
         }
     };
 
+    GraphicsContext::endTransparencyLayer();
+
     append(createCommand<EndTransparencyLayer>());
 }
 
@@ -1046,7 +1062,7 @@ void CairoOperationRecorder::clipPath(const Path& path, WindRule clipRule)
     }
 }
 
-IntRect CairoOperationRecorder::clipBounds()
+IntRect CairoOperationRecorder::clipBounds() const
 {
     auto& state = m_stateStack.last();
     return enclosingIntRect(state.ctmInverse.mapRect(state.clipBounds));
@@ -1072,8 +1088,8 @@ void CairoOperationRecorder::clipToImageBuffer(ImageBuffer& buffer, const FloatR
     if (!image)
         return;
 
-    if (auto surface = image->nativeImageForCurrentFrame())
-        append(createCommand<ClipToImageBuffer>(RefPtr<cairo_surface_t>(surface.get()), destRect));
+    if (auto nativeImage = image->nativeImageForCurrentFrame())
+        append(createCommand<ClipToImageBuffer>(nativeImage->platformImage(), destRect));
 }
 
 void CairoOperationRecorder::applyDeviceScaleFactor(float)
@@ -1090,4 +1106,12 @@ void CairoOperationRecorder::append(std::unique_ptr<PaintingOperation>&& command
     m_commandList.append(WTFMove(command));
 }
 
+#if ENABLE(VIDEO)
+void CairoOperationRecorder::paintFrameForMedia(MediaPlayer& player, const FloatRect& destination)
+{
+    // FIXME: Not implemented.
+    GraphicsContext::paintFrameForMedia(player, destination);
+}
+#endif
+
 } // namespace Nicosia
