@@ -39,7 +39,10 @@
 #include "HTMLSlotElement.h"
 #include "HTMLStyleElement.h"
 #include "InspectorInstrumentation.h"
+#include "Logging.h"
 #include "ProcessingInstruction.h"
+#include "RenderView.h"
+#include "SVGElementTypeHelpers.h"
 #include "SVGStyleElement.h"
 #include "Settings.h"
 #include "ShadowRoot.h"
@@ -76,61 +79,82 @@ Scope::~Scope()
     ASSERT(!hasPendingSheets());
 }
 
-bool Scope::shouldUseSharedUserAgentShadowTreeStyleResolver() const
-{
-    if (!m_shadowRoot)
-        return false;
-    if (m_shadowRoot->mode() != ShadowRootMode::UserAgent)
-        return false;
-    // If we have stylesheets in the user agent shadow tree use per-scope resolver.
-    if (!m_styleSheetCandidateNodes.isEmpty())
-        return false;
-    return true;
-}
-
 Resolver& Scope::resolver()
 {
-    if (shouldUseSharedUserAgentShadowTreeStyleResolver())
-        return m_document.userAgentShadowTreeStyleResolver();
-
     if (!m_resolver) {
-        SetForScope<bool> isUpdatingStyleResolver { m_isUpdatingStyleResolver, true };
-
-        m_resolver = makeUnique<Resolver>(m_document);
-
-        if (!m_shadowRoot) {
-            m_document.fontSelector().buildStarted();
-            m_resolver->ruleSets().initializeUserStyle();
-        } else {
-            m_resolver->ruleSets().setIsForShadowScope();
-            m_resolver->ruleSets().setUsesSharedUserStyle(m_shadowRoot->mode() != ShadowRootMode::UserAgent);
-        }
-
-        m_resolver->addCurrentSVGFontFaceRules();
-        m_resolver->appendAuthorStyleSheets(m_activeStyleSheets);
-
-        if (!m_shadowRoot)
-            m_document.fontSelector().buildCompleted();
+        if (m_shadowRoot)
+            createOrFindSharedShadowTreeResolver();
+        else
+            createDocumentResolver();
+        
+        if (m_resolver->ruleSets().features().usesHasPseudoClass())
+            m_usesHasPseudoClass = true;
     }
-    ASSERT(!m_shadowRoot || &m_document == &m_shadowRoot->document());
-    ASSERT(&m_resolver->document() == &m_document);
     return *m_resolver;
 }
 
-Resolver* Scope::resolverIfExists()
+void Scope::createDocumentResolver()
 {
-    if (shouldUseSharedUserAgentShadowTreeStyleResolver())
-        return &m_document.userAgentShadowTreeStyleResolver();
+    ASSERT(!m_resolver);
+    ASSERT(!m_shadowRoot);
 
-    return m_resolver.get();
+    SetForScope<bool> isUpdatingStyleResolver { m_isUpdatingStyleResolver, true };
+
+    m_resolver = Resolver::create(m_document);
+
+    m_document.fontSelector().buildStarted();
+
+    m_resolver->ruleSets().initializeUserStyle();
+    m_resolver->addCurrentSVGFontFaceRules();
+    m_resolver->appendAuthorStyleSheets(m_activeStyleSheets);
+
+    m_document.fontSelector().buildCompleted();
+}
+
+void Scope::createOrFindSharedShadowTreeResolver()
+{
+    ASSERT(!m_resolver);
+    ASSERT(m_shadowRoot);
+
+    auto key = makeResolverSharingKey();
+
+    auto result = documentScope().m_sharedShadowTreeResolvers.ensure(WTFMove(key), [&] {
+        SetForScope<bool> isUpdatingStyleResolver { m_isUpdatingStyleResolver, true };
+
+        m_resolver = Resolver::create(m_document);
+
+        m_resolver->ruleSets().setUsesSharedUserStyle(!isForUserAgentShadowTree());
+        m_resolver->appendAuthorStyleSheets(m_activeStyleSheets);
+
+        return Ref { *m_resolver };
+    });
+
+    if (!result.isNewEntry) {
+        m_resolver = result.iterator->value.ptr();
+        m_resolver->setSharedBetweenShadowTrees();
+    }
+}
+
+void Scope::unshareShadowTreeResolverBeforeMutation()
+{
+    ASSERT(m_shadowRoot);
+
+    documentScope().m_sharedShadowTreeResolvers.remove(makeResolverSharingKey());
+}
+
+auto Scope::makeResolverSharingKey() -> ResolverSharingKey
+{
+    constexpr bool isNonEmptyHashTableValue = true;
+    return {
+        m_activeStyleSheets.map([&](auto& sheet) { return RefPtr { &sheet->contents() }; }),
+        isForUserAgentShadowTree(),
+        isNonEmptyHashTableValue
+    };
 }
 
 void Scope::clearResolver()
 {
     m_resolver = nullptr;
-
-    if (!m_shadowRoot)
-        m_document.didClearStyleResolver();
 }
 
 void Scope::releaseMemory()
@@ -150,6 +174,8 @@ void Scope::releaseMemory()
     }
 #endif
     clearResolver();
+
+    m_sharedShadowTreeResolvers.clear();
 }
 
 Scope& Scope::forNode(Node& node)
@@ -163,31 +189,18 @@ Scope& Scope::forNode(Node& node)
 
 Scope* Scope::forOrdinal(Element& element, ScopeOrdinal ordinal)
 {
-    switch (ordinal) {
-    case ScopeOrdinal::Element:
+    if (ordinal == ScopeOrdinal::Element)
         return &forNode(element);
-    case ScopeOrdinal::ContainingHost: {
-        auto* containingShadowRoot = element.containingShadowRoot();
-        if (!containingShadowRoot)
-            return nullptr;
-        return &forNode(*containingShadowRoot->host());
-    }
-    case ScopeOrdinal::Shadow: {
+    if (ordinal == ScopeOrdinal::Shadow) {
         auto* shadowRoot = element.shadowRoot();
-        if (!shadowRoot)
-            return nullptr;
-        return &shadowRoot->styleScope();
-    }
-    default: {
-        ASSERT(ordinal >= ScopeOrdinal::FirstSlot);
-        auto slotIndex = ScopeOrdinal::FirstSlot;
-        for (auto* slot = element.assignedSlot(); slot; slot = slot->assignedSlot(), ++slotIndex) {
-            if (slotIndex == ordinal)
-                return &forNode(*slot);
-        }
-        return nullptr;
+        return shadowRoot ? &shadowRoot->styleScope() : nullptr;
     }
+    if (ordinal <= ScopeOrdinal::ContainingHost) {
+        auto* host = hostForScopeOrdinal(element, ordinal);
+        return host ? &forNode(*host) : nullptr;
     }
+    auto* slot = assignedSlotForScopeOrdinal(element, ordinal);
+    return slot ? &forNode(*slot) : nullptr;
 }
 
 void Scope::setPreferredStylesheetSetName(const String& name)
@@ -203,6 +216,9 @@ void Scope::addPendingSheet(const Element& element)
     ASSERT(!hasPendingSheet(element));
 
     bool isInHead = ancestorsOfType<HTMLHeadElement>(element).first();
+
+    LOG_WITH_STREAM(StyleSheets, stream << "Scope " << this << " addPendingSheet() " << element << " isInHead " << isInHead);
+
     if (isInHead)
         m_elementsInHeadWithPendingSheets.add(&element);
     else
@@ -291,7 +307,9 @@ void Scope::addStyleSheetCandidateNode(Node& node, bool createdByParser)
         }
         followingNode = n;
     } while (it != begin);
-    
+
+    LOG_WITH_STREAM(StyleSheets, stream << "Scope " << this << " addStyleSheetCandidateNode() " << node);
+
     m_styleSheetCandidateNodes.insertBefore(followingNode, &node);
 }
 
@@ -319,6 +337,8 @@ void Scope::collectActiveStyleSheets(Vector<RefPtr<StyleSheet>>& sheets)
     if (!m_document.settings().authorAndUserStylesEnabled())
         return;
 
+    LOG_WITH_STREAM(StyleSheets, stream << "Scope " << this << " collectActiveStyleSheets()");
+
     for (auto& node : m_styleSheetCandidateNodes) {
         RefPtr<StyleSheet> sheet;
         if (is<ProcessingInstruction>(*node)) {
@@ -326,6 +346,7 @@ void Scope::collectActiveStyleSheets(Vector<RefPtr<StyleSheet>>& sheets)
                 continue;
             // We don't support linking to embedded CSS stylesheets, see <https://bugs.webkit.org/show_bug.cgi?id=49281> for discussion.
             sheet = downcast<ProcessingInstruction>(*node).sheet();
+            LOG_WITH_STREAM(StyleSheets, stream << " adding sheet " << sheet << " from ProcessingInstruction node " << *node);
         } else if (is<HTMLLinkElement>(*node) || is<HTMLStyleElement>(*node) || is<SVGStyleElement>(*node)) {
             Element& element = downcast<Element>(*node);
             AtomString title = element.isInShadowTree() ? nullAtom() : element.attributeWithoutSynchronization(titleAttr);
@@ -355,6 +376,7 @@ void Scope::collectActiveStyleSheets(Vector<RefPtr<StyleSheet>>& sheets)
                 sheet = downcast<HTMLLinkElement>(element).sheet();
             else
                 sheet = downcast<HTMLStyleElement>(element).sheet();
+
             // Check to see if this sheet belongs to a styleset
             // (thus making it PREFERRED or ALTERNATE rather than
             // PERSISTENT).
@@ -375,6 +397,9 @@ void Scope::collectActiveStyleSheets(Vector<RefPtr<StyleSheet>>& sheets)
 
             if (rel.contains("alternate") && title.isEmpty())
                 sheet = nullptr;
+
+            if (sheet)
+                LOG_WITH_STREAM(StyleSheets, stream << " adding sheet " << sheet << " from " << *node);
         }
         if (sheet)
             sheets.append(WTFMove(sheet));
@@ -385,7 +410,11 @@ Scope::StyleSheetChange Scope::analyzeStyleSheetChange(const Vector<RefPtr<CSSSt
 {
     unsigned newStylesheetCount = newStylesheets.size();
 
-    if (!resolverIfExists())
+    auto* resolver = resolverIfExists();
+    if (!resolver)
+        return { ResolverUpdateType::Reconstruct };
+
+    if (resolver->isSharedBetweenShadowTrees())
         return { ResolverUpdateType::Reconstruct };
 
     // Find out which stylesheets are new.
@@ -454,10 +483,16 @@ void Scope::updateActiveStyleSheets(UpdateType updateType)
     collectActiveStyleSheets(activeStyleSheets);
 
     Vector<RefPtr<CSSStyleSheet>> activeCSSStyleSheets;
-    activeCSSStyleSheets.appendVector(m_document.extensionStyleSheets().injectedAuthorStyleSheets());
-    activeCSSStyleSheets.appendVector(m_document.extensionStyleSheets().authorStyleSheetsForTesting());
+
+    if (!isForUserAgentShadowTree()) {
+        activeCSSStyleSheets.appendVector(m_document.extensionStyleSheets().injectedAuthorStyleSheets());
+        activeCSSStyleSheets.appendVector(m_document.extensionStyleSheets().authorStyleSheetsForTesting());
+    }
+
     filterEnabledNonemptyCSSStyleSheets(activeCSSStyleSheets, activeStyleSheets);
 
+    LOG_WITH_STREAM(StyleSheets, stream << "Scope::updateActiveStyleSheets for document " << m_document << " sheets " << activeCSSStyleSheets);
+
     auto styleSheetChange = StyleSheetChange { ResolverUpdateType::Reconstruct };
     if (updateType == UpdateType::ActiveSet)
         styleSheetChange = analyzeStyleSheetChange(activeCSSStyleSheets);
@@ -475,11 +510,17 @@ void Scope::updateActiveStyleSheets(UpdateType updateType)
             m_usesStyleBasedEditability = true;
     }
 
+    if (m_resolver && m_resolver->ruleSets().features().usesHasPseudoClass())
+        m_usesHasPseudoClass = true;
+
     invalidateStyleAfterStyleSheetChange(styleSheetChange);
 }
 
 void Scope::invalidateStyleAfterStyleSheetChange(const StyleSheetChange& styleSheetChange)
 {
+    if (m_shadowRoot && !m_shadowRoot->isConnected())
+        return;
+
     // If we are already parsing the body and so may have significant amount of elements, put some effort into trying to avoid style recalcs.
     bool invalidateAll = !m_document.bodyOrFrameset() || m_document.hasNodesWithNonFinalStyle() || m_document.hasNodesWithMissingStyle();
 
@@ -498,19 +539,25 @@ void Scope::updateResolver(Vector<RefPtr<CSSStyleSheet>>& activeStyleSheets, Res
         clearResolver();
         return;
     }
-    auto& styleResolver = resolver();
+
+    if (m_shadowRoot)
+        unshareShadowTreeResolverBeforeMutation();
 
     SetForScope<bool> isUpdatingStyleResolver { m_isUpdatingStyleResolver, true };
+
     if (updateType == ResolverUpdateType::Reset) {
-        styleResolver.ruleSets().resetAuthorStyle();
-        styleResolver.appendAuthorStyleSheets(activeStyleSheets);
-    } else {
-        ASSERT(updateType == ResolverUpdateType::Additive);
-        unsigned firstNewIndex = m_activeStyleSheets.size();
-        Vector<RefPtr<CSSStyleSheet>> newStyleSheets;
-        newStyleSheets.appendRange(activeStyleSheets.begin() + firstNewIndex, activeStyleSheets.end());
-        styleResolver.appendAuthorStyleSheets(newStyleSheets);
+        m_resolver->ruleSets().resetAuthorStyle();
+        m_resolver->appendAuthorStyleSheets(activeStyleSheets);
+        return;
     }
+
+    ASSERT(updateType == ResolverUpdateType::Additive);
+    ASSERT(activeStyleSheets.size() >= m_activeStyleSheets.size());
+
+    unsigned firstNewIndex = m_activeStyleSheets.size();
+    Vector<RefPtr<CSSStyleSheet>> newStyleSheets;
+    newStyleSheets.appendRange(activeStyleSheets.begin() + firstNewIndex, activeStyleSheets.end());
+    m_resolver->appendAuthorStyleSheets(newStyleSheets);
 }
 
 const Vector<RefPtr<CSSStyleSheet>> Scope::activeStyleSheetsForInspector()
@@ -555,6 +602,7 @@ void Scope::flushPendingSelfUpdate()
     auto updateType = *m_pendingUpdate;
 
     clearPendingUpdate();
+    
     updateActiveStyleSheets(updateType);
 }
 
@@ -577,8 +625,10 @@ void Scope::scheduleUpdate(UpdateType update)
 {
     if (update == UpdateType::ContentsOrInterpretation) {
         // :host and ::slotted rules might go away.
-        if (m_shadowRoot)
+        if (m_shadowRoot) {
             Invalidator::invalidateHostAndSlottedStyleIfNeeded(*m_shadowRoot);
+            unshareShadowTreeResolverBeforeMutation();
+        }
         // FIXME: Animation code may trigger resource load in middle of style recalc and that can add a rule to a content extension stylesheet.
         //        Fix and remove isResolvingTreeStyle() test below, see https://bugs.webkit.org/show_bug.cgi?id=194335
         // FIXME: The m_isUpdatingStyleResolver test is here because extension stylesheets can get us here from Resolver::appendAuthorStyleSheets.
@@ -589,7 +639,7 @@ void Scope::scheduleUpdate(UpdateType update)
     if (!m_pendingUpdate || *m_pendingUpdate < update) {
         m_pendingUpdate = update;
         if (m_shadowRoot)
-            m_document.styleScope().m_hasDescendantWithPendingUpdate = true;
+            documentScope().m_hasDescendantWithPendingUpdate = true;
     }
 
     if (m_pendingUpdateTimer.isActive())
@@ -599,6 +649,13 @@ void Scope::scheduleUpdate(UpdateType update)
 
 void Scope::evaluateMediaQueriesForViewportChange()
 {
+    auto viewportState = mediaQueryViewportStateForDocument(m_document);
+
+    if (m_viewportStateOnPreviousMediaQueryEvaluation && *m_viewportStateOnPreviousMediaQueryEvaluation == viewportState)
+        return;
+    // This doesn't need to be invalidated as any changes to the rules will compute their media queries to correct values.
+    m_viewportStateOnPreviousMediaQueryEvaluation = viewportState;
+
     evaluateMediaQueries([] (Resolver& resolver) {
         return resolver.evaluateDynamicMediaQueries();
     });
@@ -618,33 +675,55 @@ void Scope::evaluateMediaQueriesForAppearanceChange()
     });
 }
 
+auto Scope::collectResolverScopes() -> ResolverScopes
+{
+    ASSERT(!m_shadowRoot);
+
+    if (!resolverIfExists())
+        return { };
+
+    ResolverScopes resolverScopes;
+
+    resolverScopes.add(*resolverIfExists(), Vector<CheckedPtr<Scope>> { this });
+
+    for (auto* shadowRoot : m_document.inDocumentShadowRoots()) {
+        auto& scope = shadowRoot->styleScope();
+        auto* resolver = scope.resolverIfExists();
+        if (!resolver)
+            continue;
+        resolverScopes.add(*resolver, Vector<CheckedPtr<Scope>> { }).iterator->value.append(&scope);
+    }
+    return resolverScopes;
+}
+
 template <typename TestFunction>
 void Scope::evaluateMediaQueries(TestFunction&& testFunction)
 {
-    auto* resolver = resolverIfExists();
-    if (!resolver)
-        return;
+    bool hadChanges = false;
 
-    auto evaluationChanges = testFunction(*resolver);
-    if (evaluationChanges) {
-        switch (evaluationChanges->type) {
-        case DynamicMediaQueryEvaluationChanges::Type::InvalidateStyle: {
-            Invalidator invalidator(evaluationChanges->invalidationRuleSets);
-            invalidator.invalidateStyle(*this);
-            break;
-        }
-        case DynamicMediaQueryEvaluationChanges::Type::ResetStyle:
-            scheduleUpdate(UpdateType::ContentsOrInterpretation);
-            break;
-        }
+    auto resolverScopes = collectResolverScopes();
+    for (auto& [resolver, scopes] : resolverScopes) {
+        auto evaluationChanges = testFunction(resolver.get());
+        if (!evaluationChanges)
+            continue;
+        hadChanges = true;
 
+        for (auto& scope : scopes) {
+            switch (evaluationChanges->type) {
+            case DynamicMediaQueryEvaluationChanges::Type::InvalidateStyle: {
+                Invalidator invalidator(evaluationChanges->invalidationRuleSets);
+                invalidator.invalidateStyle(*scope);
+                break;
+            }
+            case DynamicMediaQueryEvaluationChanges::Type::ResetStyle:
+                scope->scheduleUpdate(UpdateType::ContentsOrInterpretation);
+                break;
+            }
+        }
+    }
+
+    if (hadChanges)
         InspectorInstrumentation::mediaQueryResultChanged(m_document);
-    }
-
-    if (!m_shadowRoot) {
-        for (auto* descendantShadowRoot : m_document.inDocumentShadowRoots())
-            descendantShadowRoot->styleScope().evaluateMediaQueries(testFunction);
-    }
 }
 
 void Scope::didChangeActiveStyleSheetCandidates()
@@ -660,6 +739,8 @@ void Scope::didChangeStyleSheetContents()
 void Scope::didChangeStyleSheetEnvironment()
 {
     if (!m_shadowRoot) {
+        m_sharedShadowTreeResolvers.clear();
+
         for (auto* descendantShadowRoot : m_document.inDocumentShadowRoots()) {
             // Stylesheets is author shadow roots are potentially affected.
             if (descendantShadowRoot->mode() != ShadowRootMode::UserAgent)
@@ -680,7 +761,6 @@ void Scope::invalidateMatchedDeclarationsCache()
         resolver->invalidateMatchedDeclarationsCache();
 }
 
-
 void Scope::pendingUpdateTimerFired()
 {
     flushPendingUpdate();
@@ -693,5 +773,75 @@ const Vector<RefPtr<StyleSheet>>& Scope::styleSheetsForStyleSheetList()
     return m_styleSheetsForStyleSheetList;
 }
 
+Scope& Scope::documentScope()
+{
+    return m_document.styleScope();
+}
+
+bool Scope::isForUserAgentShadowTree() const
+{
+    return m_shadowRoot && m_shadowRoot->mode() == ShadowRootMode::UserAgent;
+}
+
+bool Scope::updateQueryContainerState()
+{
+    ASSERT(!m_shadowRoot);
+    ASSERT(m_document.renderView());
+
+    auto previousStates = WTFMove(m_queryContainerStates);
+    m_queryContainerStates.clear();
+
+    Vector<Element*> changedContainers;
+
+    for (auto& containerRenderer : m_document.renderView()->containerQueryBoxes()) {
+        auto* containerElement = containerRenderer.element();
+        if (!containerElement)
+            continue;
+        
+        auto size = containerRenderer.logicalSize();
+
+        auto sizeChanged = [&](LayoutSize oldSize) {
+            switch (containerRenderer.style().containerType()) {
+            case ContainerType::InlineSize:
+                return size.width() != oldSize.width();
+            case ContainerType::Size:
+                return size != oldSize;
+            case ContainerType::None:
+                RELEASE_ASSERT_NOT_REACHED();
+            }
+            RELEASE_ASSERT_NOT_REACHED();
+        };
+
+        auto it = previousStates.find(*containerElement);
+        bool changed = it == previousStates.end() || sizeChanged(it->value);
+        if (changed)
+            changedContainers.append(containerElement);
+        m_queryContainerStates.add(*containerElement, size);
+    }
+
+    for (auto* toInvalidate : changedContainers)
+        toInvalidate->invalidateForQueryContainerChange();
+
+    return !changedContainers.isEmpty();
+}
+
+HTMLSlotElement* assignedSlotForScopeOrdinal(const Element& element, ScopeOrdinal scopeOrdinal)
+{
+    ASSERT(scopeOrdinal >= ScopeOrdinal::FirstSlot);
+    auto* slot = element.assignedSlot();
+    for (auto scopeDepth = ScopeOrdinal::FirstSlot; slot && scopeDepth != scopeOrdinal; ++scopeDepth)
+        slot = slot->assignedSlot();
+    return slot;
+}
+
+Element* hostForScopeOrdinal(const Element& element, ScopeOrdinal scopeOrdinal)
+{
+    ASSERT(scopeOrdinal <= ScopeOrdinal::ContainingHost);
+    auto* host = element.shadowHost();
+    for (auto scopeDepth = ScopeOrdinal::ContainingHost; host && scopeDepth != scopeOrdinal; --scopeDepth)
+        host = host->shadowHost();
+    return host;
+}
+
 }
 }
