@@ -34,9 +34,12 @@
 #include "WebPageGroupData.h"
 #include "WebPageProxyIdentifier.h"
 #include "WebPreferencesStore.h"
+#include "WebURLSchemeHandlerIdentifier.h"
 #include <WebCore/ActivityState.h>
 #include <WebCore/Color.h>
+#include <WebCore/DestinationColorSpace.h>
 #include <WebCore/FloatSize.h>
+#include <WebCore/HighlightVisibility.h>
 #include <WebCore/IntSize.h>
 #include <WebCore/LayoutMilestone.h>
 #include <WebCore/MediaProducer.h>
@@ -49,10 +52,6 @@
 #include <wtf/HashMap.h>
 #include <wtf/text/WTFString.h>
 
-#if PLATFORM(MAC)
-#include "ColorSpaceData.h"
-#endif
-
 #if ENABLE(APPLICATION_MANIFEST)
 #include <WebCore/ApplicationManifest.h>
 #endif
@@ -66,7 +65,7 @@ namespace WebKit {
 
 struct WebPageCreationParameters {
     void encode(IPC::Encoder&) const;
-    static Optional<WebPageCreationParameters> decode(IPC::Decoder&);
+    static std::optional<WebPageCreationParameters> decode(IPC::Decoder&);
 
     WebCore::IntSize viewSize;
 
@@ -85,7 +84,7 @@ struct WebPageCreationParameters {
     bool useFixedLayout;
     WebCore::IntSize fixedLayoutSize;
 
-    Optional<WebCore::FloatRect> viewExposedRect;
+    std::optional<WebCore::FloatRect> viewExposedRect;
 
     bool alwaysShowsHorizontalScroller;
     bool alwaysShowsVerticalScroller;
@@ -116,20 +115,21 @@ struct WebPageCreationParameters {
     float topContentInset;
     
     float mediaVolume;
-    WebCore::MediaProducer::MutedStateFlags muted;
+    WebCore::MediaProducerMutedStateFlags muted;
+    bool openedByDOM { false };
     bool mayStartMediaWhenInWindow;
     bool mediaPlaybackIsSuspended { false };
 
     WebCore::IntSize minimumSizeForAutoLayout;
     WebCore::IntSize sizeToContentAutoSizeMaximumSize;
     bool autoSizingShouldExpandToViewHeight;
-    Optional<WebCore::IntSize> viewportSizeForCSSViewportUnits;
+    std::optional<WebCore::FloatSize> viewportSizeForCSSViewportUnits;
     
     WebCore::ScrollPinningBehavior scrollPinningBehavior;
 
-    // FIXME: This should be Optional<WebCore::ScrollbarOverlayStyle>, but we would need to
+    // FIXME: This should be std::optional<WebCore::ScrollbarOverlayStyle>, but we would need to
     // correctly handle enums inside Optionals when encoding and decoding. 
-    Optional<uint32_t> scrollbarOverlayStyle;
+    std::optional<uint32_t> scrollbarOverlayStyle;
 
     bool backgroundExtendsBeyondPage;
 
@@ -146,7 +146,7 @@ struct WebPageCreationParameters {
     bool useElevatedUserInterfaceLevel { false };
 
 #if PLATFORM(MAC)
-    ColorSpaceData colorSpace;
+    std::optional<WebCore::DestinationColorSpace> colorSpace;
     bool useSystemAppearance;
 #endif
 #if ENABLE(META_VIEWPORT)
@@ -155,15 +155,14 @@ struct WebPageCreationParameters {
     double viewportConfigurationLayoutSizeScaleFactor;
     double viewportConfigurationMinimumEffectiveDeviceWidth;
     WebCore::FloatSize viewportConfigurationViewSize;
-    Optional<WebCore::ViewportArguments> overrideViewportArguments;
-    Optional<SandboxExtension::Handle> frontboardExtensionHandle;
-    Optional<SandboxExtension::Handle> iconServicesExtensionHandle;
+    std::optional<WebCore::ViewportArguments> overrideViewportArguments;
 #endif
 #if PLATFORM(IOS_FAMILY)
     WebCore::FloatSize screenSize;
     WebCore::FloatSize availableScreenSize;
     WebCore::FloatSize overrideScreenSize;
     float textAutosizingWidth;
+    WebCore::FloatSize minimumUnobscuredSize;
     WebCore::FloatSize maximumUnobscuredSize;
     int32_t deviceOrientation { 0 };
     bool keyboardIsAttached { false };
@@ -173,6 +172,11 @@ struct WebPageCreationParameters {
 #if PLATFORM(COCOA)
     bool smartInsertDeleteEnabled;
     Vector<String> additionalSupportedImageTypes;
+    Vector<SandboxExtension::Handle> gpuIOKitExtensionHandles;
+    Vector<SandboxExtension::Handle> gpuMachExtensionHandles;
+#endif
+#if HAVE(STATIC_FONT_REGISTRY)
+    std::optional<SandboxExtension::Handle> fontMachExtensionHandle;
 #endif
 #if HAVE(APP_ACCENT_COLORS)
     WebCore::Color accentColor;
@@ -183,20 +187,22 @@ struct WebPageCreationParameters {
 #if PLATFORM(WIN)
     uint64_t nativeWindowHandle;
 #endif
-    bool appleMailPaginationQuirkEnabled;
-    bool appleMailLinesClampEnabled;
+#if USE(GRAPHICS_LAYER_WC)
+    bool usesOffscreenRendering { false };
+#endif
     bool shouldScaleViewToFitDocument;
 
     WebCore::UserInterfaceLayoutDirection userInterfaceLayoutDirection;
     OptionSet<WebCore::LayoutMilestone> observedLayoutMilestones;
 
     String overrideContentSecurityPolicy;
-    Optional<double> cpuLimit;
+    std::optional<double> cpuLimit;
 
-    HashMap<String, uint64_t> urlSchemeHandlers;
+    HashMap<String, WebURLSchemeHandlerIdentifier> urlSchemeHandlers;
+    Vector<String> urlSchemesWithLegacyCustomProtocolHandlers;
 
 #if ENABLE(APPLICATION_MANIFEST)
-    Optional<WebCore::ApplicationManifest> applicationManifest;
+    std::optional<WebCore::ApplicationManifest> applicationManifest;
 #endif
 
     bool needsFontAttributes { false };
@@ -207,15 +213,15 @@ struct WebPageCreationParameters {
 
     UserContentControllerParameters userContentControllerParameters;
 
-    Optional<WebCore::Color> backgroundColor;
+    std::optional<WebCore::Color> backgroundColor;
 
-    Optional<WebCore::PageIdentifier> oldPageID;
+    std::optional<WebCore::PageIdentifier> oldPageID;
 
     String overriddenMediaType;
     Vector<String> corsDisablingPatterns;
     bool userScriptsShouldWaitUntilNotification { true };
     bool loadsSubresources { true };
-    bool loadsFromNetwork { true };
+    std::optional<HashSet<String>> allowedNetworkHosts;
 
     bool crossOriginAccessControlCheckEnabled { true };
     String processDisplayName;
@@ -225,17 +231,36 @@ struct WebPageCreationParameters {
     bool shouldCaptureVideoInUIProcess { false };
     bool shouldCaptureVideoInGPUProcess { false };
     bool shouldCaptureDisplayInUIProcess { false };
+    bool shouldCaptureDisplayInGPUProcess { false };
     bool shouldRenderCanvasInGPUProcess { false };
+    bool shouldRenderDOMInGPUProcess { false };
+    bool shouldPlayMediaInGPUProcess { false };
+#if ENABLE(WEBGL)
+    bool shouldRenderWebGLInGPUProcess { false };
+#endif
+    bool shouldEnableVP8Decoder { false };
     bool shouldEnableVP9Decoder { false };
     bool shouldEnableVP9SWDecoder { false };
-    bool needsInAppBrowserPrivacyQuirks { false };
+#if ENABLE(APP_BOUND_DOMAINS)
     bool limitsNavigationsToAppBoundDomains { false };
+#endif
+    bool lastNavigationWasAppInitiated { true };
     bool canUseCredentialStorage { true };
 
     WebCore::ShouldRelaxThirdPartyCookieBlocking shouldRelaxThirdPartyCookieBlocking { WebCore::ShouldRelaxThirdPartyCookieBlocking::No };
+    
+    bool httpsUpgradeEnabled { true };
 
-#if PLATFORM(GTK)
-    String themeName;
+#if PLATFORM(IOS)
+    bool allowsDeprecatedSynchronousXMLHttpRequestDuringUnload { false };
+#endif
+    
+#if ENABLE(APP_HIGHLIGHTS)
+    WebCore::HighlightVisibility appHighlightsVisible { WebCore::HighlightVisibility::Hidden };
+#endif
+
+#if HAVE(TOUCH_BAR)
+    bool requiresUserActionForEditingControlsManager { false };
 #endif
 };
 
