@@ -27,6 +27,8 @@
 #include <gio/gio.h>
 #include <wpe/WebKitCookieManager.h>
 #include <wpe/WebKitDefines.h>
+#include <wpe/WebKitMemoryPressureSettings.h>
+#include <wpe/WebKitNetworkProxySettings.h>
 #include <wpe/WebKitWebsiteData.h>
 
 G_BEGIN_DECLS
@@ -38,6 +40,21 @@ G_BEGIN_DECLS
 #define WEBKIT_IS_WEBSITE_DATA_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),  WEBKIT_TYPE_WEBSITE_DATA_MANAGER))
 #define WEBKIT_WEBSITE_DATA_MANAGER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj),  WEBKIT_TYPE_WEBSITE_DATA_MANAGER, WebKitWebsiteDataManagerClass))
 
+/**
+ * WebKitTLSErrorsPolicy:
+ * @WEBKIT_TLS_ERRORS_POLICY_IGNORE: Ignore TLS errors.
+ * @WEBKIT_TLS_ERRORS_POLICY_FAIL: TLS errors will emit
+ *   #WebKitWebView::load-failed-with-tls-errors and, if the signal is handled,
+ *   finish the load. In case the signal is not handled,
+ *   #WebKitWebView::load-failed is emitted before the load finishes.
+ *
+ * Enum values used to denote the TLS errors policy.
+ */
+typedef enum {
+    WEBKIT_TLS_ERRORS_POLICY_IGNORE,
+    WEBKIT_TLS_ERRORS_POLICY_FAIL
+} WebKitTLSErrorsPolicy;
+
 typedef struct _WebKitWebsiteDataManager        WebKitWebsiteDataManager;
 typedef struct _WebKitWebsiteDataManagerClass   WebKitWebsiteDataManagerClass;
 typedef struct _WebKitWebsiteDataManagerPrivate WebKitWebsiteDataManagerPrivate;
@@ -119,6 +136,18 @@ webkit_website_data_manager_set_persistent_credential_storage_enabled (WebKitWeb
 WEBKIT_API gboolean
 webkit_website_data_manager_get_persistent_credential_storage_enabled (WebKitWebsiteDataManager *manager);
 
+WEBKIT_API void
+webkit_website_data_manager_set_tls_errors_policy                     (WebKitWebsiteDataManager *manager,
+                                                                       WebKitTLSErrorsPolicy     policy);
+
+WEBKIT_API WebKitTLSErrorsPolicy
+webkit_website_data_manager_get_tls_errors_policy                     (WebKitWebsiteDataManager *manager);
+
+WEBKIT_API void
+webkit_website_data_manager_set_network_proxy_settings                (WebKitWebsiteDataManager *manager,
+                                                                       WebKitNetworkProxyMode    proxy_mode,
+                                                                       WebKitNetworkProxySettings *proxy_settings);
+
 WEBKIT_API void
 webkit_website_data_manager_fetch                                     (WebKitWebsiteDataManager *manager,
                                                                        WebKitWebsiteDataTypes    types,
@@ -155,6 +184,9 @@ webkit_website_data_manager_clear_finish                               (WebKitWe
                                                                         GAsyncResult             *result,
                                                                         GError                  **error);
 
+WEBKIT_API void
+webkit_website_data_manager_set_memory_pressure_settings               (WebKitMemoryPressureSettings *settings);
+
 
 #define WEBKIT_TYPE_ITP_FIRST_PARTY   (webkit_itp_first_party_get_type())
 
