@@ -31,10 +31,14 @@
 #include "APIUIClient.h"
 #include "APIWebAuthenticationPanel.h"
 #include "APIWebAuthenticationPanelClient.h"
+#include "AuthenticatorPresenterCoordinator.h"
 #include "LocalService.h"
 #include "NfcService.h"
 #include "WebPageProxy.h"
 #include "WebPreferencesKeys.h"
+#include "WebProcessProxy.h"
+#include <WebCore/AuthenticatorAssertionResponse.h>
+#include <WebCore/AuthenticatorAttachment.h>
 #include <WebCore/AuthenticatorTransport.h>
 #include <WebCore/PublicKeyCredentialCreationOptions.h>
 #include <WebCore/WebAuthenticationConstants.h>
@@ -49,7 +53,7 @@ namespace {
 const unsigned maxTimeOutValue = 120000;
 
 // FIXME(188625): Support BLE authenticators.
-static AuthenticatorManager::TransportSet collectTransports(const Optional<PublicKeyCredentialCreationOptions::AuthenticatorSelectionCriteria>& authenticatorSelection)
+static AuthenticatorManager::TransportSet collectTransports(const std::optional<PublicKeyCredentialCreationOptions::AuthenticatorSelectionCriteria>& authenticatorSelection)
 {
     AuthenticatorManager::TransportSet result;
     if (!authenticatorSelection || !authenticatorSelection->authenticatorAttachment) {
@@ -62,12 +66,12 @@ static AuthenticatorManager::TransportSet collectTransports(const Optional<Publi
         return result;
     }
 
-    if (authenticatorSelection->authenticatorAttachment == PublicKeyCredentialCreationOptions::AuthenticatorAttachment::Platform) {
+    if (authenticatorSelection->authenticatorAttachment == AuthenticatorAttachment::Platform) {
         auto addResult = result.add(AuthenticatorTransport::Internal);
         ASSERT_UNUSED(addResult, addResult.isNewEntry);
         return result;
     }
-    if (authenticatorSelection->authenticatorAttachment == PublicKeyCredentialCreationOptions::AuthenticatorAttachment::CrossPlatform) {
+    if (authenticatorSelection->authenticatorAttachment == AuthenticatorAttachment::CrossPlatform) {
         auto addResult = result.add(AuthenticatorTransport::Usb);
         ASSERT_UNUSED(addResult, addResult.isNewEntry);
         addResult = result.add(AuthenticatorTransport::Nfc);
@@ -84,7 +88,7 @@ static AuthenticatorManager::TransportSet collectTransports(const Optional<Publi
 // If it is not specified or any of its credentials doesn't specify its own. We should discover all.
 // This is a variant of Step. 18.*.4 from https://www.w3.org/TR/webauthn/#discover-from-external-source
 // as of 7 August 2018.
-static AuthenticatorManager::TransportSet collectTransports(const Vector<PublicKeyCredentialDescriptor>& allowCredentials)
+static AuthenticatorManager::TransportSet collectTransports(const Vector<PublicKeyCredentialDescriptor>& allowCredentials, const std::optional<AuthenticatorAttachment>& authenticatorAttachment)
 {
     AuthenticatorManager::TransportSet result;
     if (allowCredentials.isEmpty()) {
@@ -94,7 +98,6 @@ static AuthenticatorManager::TransportSet collectTransports(const Vector<PublicK
         ASSERT_UNUSED(addResult, addResult.isNewEntry);
         addResult = result.add(AuthenticatorTransport::Nfc);
         ASSERT_UNUSED(addResult, addResult.isNewEntry);
-        return result;
     }
 
     for (auto& allowCredential : allowCredentials) {
@@ -102,51 +105,60 @@ static AuthenticatorManager::TransportSet collectTransports(const Vector<PublicK
             result.add(AuthenticatorTransport::Internal);
             result.add(AuthenticatorTransport::Usb);
             result.add(AuthenticatorTransport::Nfc);
-            return result;
+
+            break;
         }
 
         for (const auto& transport : allowCredential.transports) {
             if (transport == AuthenticatorTransport::Ble)
                 continue;
+
             result.add(transport);
+
             if (result.size() >= AuthenticatorManager::maxTransportNumber)
-                return result;
+                break;
         }
     }
 
-    ASSERT(result.size() < AuthenticatorManager::maxTransportNumber);
+    if (authenticatorAttachment) {
+        if (authenticatorAttachment == AuthenticatorAttachment::Platform) {
+            result.remove(AuthenticatorTransport::Usb);
+            result.remove(AuthenticatorTransport::Nfc);
+        }
+
+        if (authenticatorAttachment == AuthenticatorAttachment::CrossPlatform)
+            result.remove(AuthenticatorTransport::Internal);
+    }
+
+    ASSERT(result.size() <= AuthenticatorManager::maxTransportNumber);
     return result;
 }
 
 // Only roaming authenticators are supported for Google legacy AppID support.
-static void processGoogleLegacyAppIdSupportExtension(const Optional<AuthenticationExtensionsClientInputs>& extensions, AuthenticatorManager::TransportSet& transports)
+static void processGoogleLegacyAppIdSupportExtension(const std::optional<AuthenticationExtensionsClientInputs>& extensions, AuthenticatorManager::TransportSet& transports)
 {
-    // AuthenticatorCoordinator::create should always set it.
-    ASSERT(!!extensions);
+    if (!extensions) {
+        // AuthenticatorCoordinator::create should always set it.
+        ASSERT_NOT_REACHED();
+        return;
+    }
     if (!extensions->googleLegacyAppidSupport)
         return;
     transports.remove(AuthenticatorTransport::Internal);
 }
 
-static bool isFeatureEnabled(WebPageProxy* page, const String& featureKey)
+static String getRpId(const std::variant<PublicKeyCredentialCreationOptions, PublicKeyCredentialRequestOptions>& options)
 {
-    if (!page)
-        return false;
-    return page->preferences().store().getBoolValueForKey(featureKey);
+    if (std::holds_alternative<PublicKeyCredentialCreationOptions>(options))
+        return std::get<PublicKeyCredentialCreationOptions>(options).rp.id;
+    return std::get<PublicKeyCredentialRequestOptions>(options).rpId;
 }
 
-static String getRpId(const Variant<PublicKeyCredentialCreationOptions, PublicKeyCredentialRequestOptions>& options)
+static String getUserName(const std::variant<PublicKeyCredentialCreationOptions, PublicKeyCredentialRequestOptions>& options)
 {
-    if (WTF::holds_alternative<PublicKeyCredentialCreationOptions>(options))
-        return WTF::get<PublicKeyCredentialCreationOptions>(options).rp.id;
-    return WTF::get<PublicKeyCredentialRequestOptions>(options).rpId;
-}
-
-static ClientDataType getClientDataType(const Variant<PublicKeyCredentialCreationOptions, PublicKeyCredentialRequestOptions>& options)
-{
-    if (WTF::holds_alternative<PublicKeyCredentialCreationOptions>(options))
-        return ClientDataType::Create;
-    return ClientDataType::Get;
+    if (std::holds_alternative<PublicKeyCredentialCreationOptions>(options))
+        return std::get<PublicKeyCredentialCreationOptions>(options).user.name;
+    return emptyString();
 }
 
 } // namespace
@@ -172,10 +184,17 @@ void AuthenticatorManager::handleRequest(WebAuthenticationRequestData&& data, Ca
 
     // 2. Ask clients to show appropriate UI if any and then start the request.
     initTimeOutTimer();
-    runPanel();
+
+    // FIXME<rdar://problem/70822834>: The m_mode is used to determine whether or not we are in the UIProcess.
+    // If so, continue to the old route. Otherwise, use the modern WebAuthn process way.
+    if (m_mode == Mode::Compatible) {
+        runPanel();
+        return;
+    }
+    runPresenter();
 }
 
-void AuthenticatorManager::cancelRequest(const PageIdentifier& pageID, const Optional<FrameIdentifier>& frameID)
+void AuthenticatorManager::cancelRequest(const PageIdentifier& pageID, const std::optional<FrameIdentifier>& frameID)
 {
     if (!m_pendingCompletionHandler)
         return;
@@ -185,9 +204,7 @@ void AuthenticatorManager::cancelRequest(const PageIdentifier& pageID, const Opt
         if (frameID && frameID != pendingFrameID->frameID)
             return;
     }
-    invokePendingCompletionHandler(ExceptionData { NotAllowedError, "Operation timed out."_s });
-    clearState();
-    m_requestTimeOutTimer.stop();
+    cancelRequest();
 }
 
 // The following implements part of Step 20. of https://www.w3.org/TR/webauthn/#createCredential
@@ -198,14 +215,30 @@ void AuthenticatorManager::cancelRequest(const API::WebAuthenticationPanel& pane
     RELEASE_ASSERT(RunLoop::isMain());
     if (!m_pendingCompletionHandler || m_pendingRequestData.panel.get() != &panel)
         return;
-    invokePendingCompletionHandler(ExceptionData { NotAllowedError, "This request has been cancelled by the user."_s });
-    clearState();
-    m_requestTimeOutTimer.stop();
+    cancelRequest();
+}
+
+void AuthenticatorManager::cancel()
+{
+    RELEASE_ASSERT(RunLoop::isMain());
+    if (!m_pendingCompletionHandler)
+        return;
+    cancelRequest();
+}
+
+void AuthenticatorManager::enableModernWebAuthentication()
+{
+    m_mode = Mode::Modern;
+}
+
+void AuthenticatorManager::enableNativeSupport()
+{
+    m_mode = Mode::Native;
 }
 
 void AuthenticatorManager::clearStateAsync()
 {
-    RunLoop::main().dispatch([weakThis = makeWeakPtr(*this)] {
+    RunLoop::main().dispatch([weakThis = WeakPtr { *this }] {
         if (!weakThis)
             return;
         weakThis->clearState();
@@ -219,6 +252,7 @@ void AuthenticatorManager::clearState()
     m_authenticators.clear();
     m_services.clear();
     m_pendingRequestData = { };
+    m_presenter = nullptr;
 }
 
 void AuthenticatorManager::authenticatorAdded(Ref<Authenticator>&& authenticator)
@@ -226,14 +260,22 @@ void AuthenticatorManager::authenticatorAdded(Ref<Authenticator>&& authenticator
     ASSERT(RunLoop::isMain());
     authenticator->setObserver(*this);
     authenticator->handleRequest(m_pendingRequestData);
+    authenticator->setWebAuthenticationModernEnabled(m_mode != Mode::Compatible);
     auto addResult = m_authenticators.add(WTFMove(authenticator));
     ASSERT_UNUSED(addResult, addResult.isNewEntry);
 }
 
 void AuthenticatorManager::serviceStatusUpdated(WebAuthenticationStatus status)
 {
-    if (auto *panel = m_pendingRequestData.panel.get())
-        panel->client().updatePanel(status);
+    // This is for the new UI.
+    if (m_presenter) {
+        m_presenter->updatePresenter(status);
+        return;
+    }
+
+    dispatchPanelClientCall([status] (const API::WebAuthenticationPanel& panel) {
+        panel.client().updatePanel(status);
+    });
 }
 
 void AuthenticatorManager::respondReceived(Respond&& respond)
@@ -243,9 +285,9 @@ void AuthenticatorManager::respondReceived(Respond&& respond)
         return;
     ASSERT(m_pendingCompletionHandler);
 
-    auto shouldComplete = WTF::holds_alternative<Ref<AuthenticatorResponse>>(respond);
+    auto shouldComplete = std::holds_alternative<Ref<AuthenticatorResponse>>(respond);
     if (!shouldComplete)
-        shouldComplete = WTF::get<ExceptionData>(respond).code == InvalidStateError;
+        shouldComplete = std::get<ExceptionData>(respond).code == InvalidStateError;
     if (shouldComplete) {
         invokePendingCompletionHandler(WTFMove(respond));
         clearStateAsync();
@@ -258,7 +300,7 @@ void AuthenticatorManager::respondReceived(Respond&& respond)
 
 void AuthenticatorManager::downgrade(Authenticator* id, Ref<Authenticator>&& downgradedAuthenticator)
 {
-    RunLoop::main().dispatch([weakThis = makeWeakPtr(*this), id] {
+    RunLoop::main().dispatch([weakThis = WeakPtr { *this }, id] {
         if (!weakThis)
             return;
         auto removed = weakThis->m_authenticators.remove(id);
@@ -269,20 +311,88 @@ void AuthenticatorManager::downgrade(Authenticator* id, Ref<Authenticator>&& dow
 
 void AuthenticatorManager::authenticatorStatusUpdated(WebAuthenticationStatus status)
 {
-    if (auto* panel = m_pendingRequestData.panel.get())
-        panel->client().updatePanel(status);
+    // Immediately invalidate the cache if the PIN is incorrect. A status update often means
+    // an error. We don't really care what kind of error it really is.
+    m_pendingRequestData.cachedPin = String();
+
+    // This is for the new UI.
+    if (m_presenter) {
+        m_presenter->updatePresenter(status);
+        return;
+    }
+
+    dispatchPanelClientCall([status] (const API::WebAuthenticationPanel& panel) {
+        panel.client().updatePanel(status);
+    });
 }
 
 void AuthenticatorManager::requestPin(uint64_t retries, CompletionHandler<void(const WTF::String&)>&& completionHandler)
 {
-    if (auto* panel = m_pendingRequestData.panel.get())
-        panel->client().requestPin(retries, WTFMove(completionHandler));
+    // Cache the PIN to improve NFC user experience so that a momentary movement of the NFC key away from the scanner doesn't
+    // force the PIN entry to be re-entered.
+    // We don't distinguish USB and NFC here becuase there is no harms to have this optimization for USB even though it is useless.
+    if (!m_pendingRequestData.cachedPin.isNull()) {
+        completionHandler(m_pendingRequestData.cachedPin);
+        m_pendingRequestData.cachedPin = String();
+        return;
+    }
+
+    auto callback = [weakThis = WeakPtr { *this }, this, completionHandler = WTFMove(completionHandler)] (const WTF::String& pin) mutable {
+        if (!weakThis)
+            return;
+
+        m_pendingRequestData.cachedPin = pin;
+        completionHandler(pin);
+    };
+
+    // This is for the new UI.
+    if (m_presenter) {
+        m_presenter->requestPin(retries, WTFMove(callback));
+        return;
+    }
+
+    dispatchPanelClientCall([retries, callback = WTFMove(callback)] (const API::WebAuthenticationPanel& panel) mutable {
+        panel.client().requestPin(retries, WTFMove(callback));
+    });
 }
 
-void AuthenticatorManager::selectAssertionResponse(const HashSet<Ref<WebCore::AuthenticatorAssertionResponse>>& responses, CompletionHandler<void(const WebCore::AuthenticatorAssertionResponse&)>&& completionHandler)
+void AuthenticatorManager::selectAssertionResponse(Vector<Ref<WebCore::AuthenticatorAssertionResponse>>&& responses, WebAuthenticationSource source, CompletionHandler<void(AuthenticatorAssertionResponse*)>&& completionHandler)
 {
-    if (auto* panel = m_pendingRequestData.panel.get())
-        panel->client().selectAssertionResponse(responses, WTFMove(completionHandler));
+    // This is for the new UI.
+    if (m_presenter) {
+        m_presenter->selectAssertionResponse(WTFMove(responses), source, WTFMove(completionHandler));
+        return;
+    }
+
+    dispatchPanelClientCall([responses = WTFMove(responses), source, completionHandler = WTFMove(completionHandler)] (const API::WebAuthenticationPanel& panel) mutable {
+        panel.client().selectAssertionResponse(WTFMove(responses), source, WTFMove(completionHandler));
+    });
+}
+
+void AuthenticatorManager::decidePolicyForLocalAuthenticator(CompletionHandler<void(LocalAuthenticatorPolicy)>&& completionHandler)
+{
+    dispatchPanelClientCall([completionHandler = WTFMove(completionHandler)] (const API::WebAuthenticationPanel& panel) mutable {
+        panel.client().decidePolicyForLocalAuthenticator(WTFMove(completionHandler));
+    });
+}
+
+void AuthenticatorManager::requestLAContextForUserVerification(CompletionHandler<void(LAContext *)>&& completionHandler)
+{
+    if (m_presenter) {
+        m_presenter->requestLAContextForUserVerification(WTFMove(completionHandler));
+        return;
+    }
+
+    dispatchPanelClientCall([completionHandler = WTFMove(completionHandler)] (const API::WebAuthenticationPanel& panel) mutable {
+        panel.client().requestLAContextForUserVerification(WTFMove(completionHandler));
+    });
+}
+
+void AuthenticatorManager::cancelRequest()
+{
+    invokePendingCompletionHandler(ExceptionData { NotAllowedError, "This request has been cancelled by the user."_s });
+    clearState();
+    m_requestTimeOutTimer.stop();
 }
 
 UniqueRef<AuthenticatorTransportService> AuthenticatorManager::createService(AuthenticatorTransport transport, AuthenticatorTransportService::Observer& observer) const
@@ -296,6 +406,10 @@ void AuthenticatorManager::filterTransports(TransportSet& transports) const
         transports.remove(AuthenticatorTransport::Nfc);
     if (!LocalService::isAvailable())
         transports.remove(AuthenticatorTransport::Internal);
+
+    // For the modern UI, we should only consider invoking it when the operation is triggered by users.
+    if (!m_pendingRequestData.processingUserGesture)
+        transports.clear();
 }
 
 void AuthenticatorManager::startDiscovery(const TransportSet& transports)
@@ -303,9 +417,6 @@ void AuthenticatorManager::startDiscovery(const TransportSet& transports)
     ASSERT(RunLoop::isMain());
     ASSERT(m_services.isEmpty() && transports.size() <= maxTransportNumber);
     for (auto& transport : transports) {
-        // Only allow USB authenticators when clients don't have dedicated UI.
-        if (transport != AuthenticatorTransport::Usb && (m_pendingRequestData.panelResult == WebAuthenticationPanelResult::Unavailable))
-            continue;
         auto service = createService(transport, *this);
         service->startDiscovery();
         m_services.append(WTFMove(service));
@@ -314,20 +425,19 @@ void AuthenticatorManager::startDiscovery(const TransportSet& transports)
 
 void AuthenticatorManager::initTimeOutTimer()
 {
-    Optional<unsigned> timeOutInMs;
+    std::optional<unsigned> timeOutInMs;
     WTF::switchOn(m_pendingRequestData.options, [&](const PublicKeyCredentialCreationOptions& options) {
         timeOutInMs = options.timeout;
     }, [&](const PublicKeyCredentialRequestOptions& options) {
         timeOutInMs = options.timeout;
     });
 
-    unsigned timeOutInMsValue = std::min(maxTimeOutValue, timeOutInMs.valueOr(maxTimeOutValue));
+    unsigned timeOutInMsValue = std::min(maxTimeOutValue, timeOutInMs.value_or(maxTimeOutValue));
     m_requestTimeOutTimer.startOneShot(Seconds::fromMilliseconds(timeOutInMsValue));
 }
 
 void AuthenticatorManager::timeOutTimerFired()
 {
-    ASSERT(m_requestTimeOutTimer.isActive());
     invokePendingCompletionHandler((ExceptionData { NotAllowedError, "Operation timed out."_s }));
     clearState();
 }
@@ -345,28 +455,60 @@ void AuthenticatorManager::runPanel()
     // Get available transports and start discovering authenticators on them.
     auto& options = m_pendingRequestData.options;
     auto transports = getTransports();
-    m_pendingRequestData.panel = API::WebAuthenticationPanel::create(*this, getRpId(options), transports, getClientDataType(options));
+    if (transports.isEmpty()) {
+        cancel();
+        return;
+    }
+
+    m_pendingRequestData.panel = API::WebAuthenticationPanel::create(*this, getRpId(options), transports, getClientDataType(options), getUserName(options));
     auto& panel = *m_pendingRequestData.panel;
-    page->uiClient().runWebAuthenticationPanel(*page, panel, *frame, WebCore::SecurityOriginData { m_pendingRequestData.origin }, [transports = WTFMove(transports), weakPanel = makeWeakPtr(panel), weakThis = makeWeakPtr(*this), this] (WebAuthenticationPanelResult result) {
+    page->uiClient().runWebAuthenticationPanel(*page, panel, *frame, FrameInfoData { m_pendingRequestData.frameInfo }, [transports = WTFMove(transports), weakPanel = WeakPtr { panel }, weakThis = WeakPtr { *this }, this] (WebAuthenticationPanelResult result) {
         // The panel address is used to determine if the current pending request is still the same.
         if (!weakThis || !weakPanel
             || (result == WebAuthenticationPanelResult::DidNotPresent)
             || (weakPanel.get() != m_pendingRequestData.panel.get()))
             return;
-        m_pendingRequestData.panelResult = result;
         startDiscovery(transports);
     });
 }
 
+void AuthenticatorManager::runPresenter()
+{
+    // Get available transports and start discovering authenticators on them.
+    auto transports = getTransports();
+    if (transports.isEmpty()) {
+        cancel();
+        return;
+    }
+
+    startDiscovery(transports);
+
+    // For native API support, we skip the UI part. The native API will handle that.
+    if (m_mode == Mode::Native)
+        return;
+
+    runPresenterInternal(transports);
+}
+
+void AuthenticatorManager::runPresenterInternal(const TransportSet& transports)
+{
+    auto& options = m_pendingRequestData.options;
+    m_presenter = makeUnique<AuthenticatorPresenterCoordinator>(*this, getRpId(options), transports, getClientDataType(options), getUserName(options));
+}
+
 void AuthenticatorManager::invokePendingCompletionHandler(Respond&& respond)
 {
-    if (auto *panel = m_pendingRequestData.panel.get()) {
-        WTF::switchOn(respond, [&](const Ref<AuthenticatorResponse>&) {
-            panel->client().dismissPanel(WebAuthenticationResult::Succeeded);
-        }, [&](const ExceptionData&) {
-            panel->client().dismissPanel(WebAuthenticationResult::Failed);
+    auto result = std::holds_alternative<Ref<AuthenticatorResponse>>(respond) ? WebAuthenticationResult::Succeeded : WebAuthenticationResult::Failed;
+
+    // This is for the new UI.
+    if (m_presenter)
+        m_presenter->dimissPresenter(result);
+    else {
+        dispatchPanelClientCall([result] (const API::WebAuthenticationPanel& panel) {
+            panel.client().dismissPanel(result);
         });
     }
+
     m_pendingCompletionHandler(WTFMove(respond));
 }
 
@@ -383,14 +525,29 @@ auto AuthenticatorManager::getTransports() const -> TransportSet
         transports = collectTransports(options.authenticatorSelection);
         processGoogleLegacyAppIdSupportExtension(options.extensions, transports);
     }, [&](const PublicKeyCredentialRequestOptions& options) {
-        transports = collectTransports(options.allowCredentials);
+        transports = collectTransports(options.allowCredentials, options.authenticatorAttachment);
     });
-    if (!isFeatureEnabled(m_pendingRequestData.page.get(), WebPreferencesKey::webAuthenticationLocalAuthenticatorEnabledKey()))
-        transports.remove(AuthenticatorTransport::Internal);
     filterTransports(transports);
     return transports;
 }
 
+void AuthenticatorManager::dispatchPanelClientCall(Function<void(const API::WebAuthenticationPanel&)>&& call) const
+{
+    auto weakPanel = m_pendingRequestData.weakPanel;
+    if (!weakPanel)
+        weakPanel = m_pendingRequestData.panel;
+    if (!weakPanel)
+        return;
+
+    // Call delegates in the next run loop to prevent clients' reentrance that would potentially modify the state
+    // of the current run loop in unexpected ways.
+    RunLoop::main().dispatch([weakPanel = WTFMove(weakPanel), call = WTFMove(call)] () {
+        if (!weakPanel)
+            return;
+        call(*weakPanel);
+    });
+}
+
 } // namespace WebKit
 
 #endif // ENABLE(WEB_AUTHN)
