@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,14 +27,19 @@
 
 #include "Connection.h"
 #include "DebuggableInfoData.h"
-#include "WebInspectorFrontendAPIDispatcher.h"
 #include "WebPageProxyIdentifier.h"
+#include <WebCore/FrameIdentifier.h>
 #include <WebCore/InspectorDebuggableType.h>
+#include <WebCore/InspectorFrontendAPIDispatcher.h>
 #include <WebCore/InspectorFrontendClient.h>
-#include <WebCore/InspectorFrontendHost.h>
+
+#if ENABLE(INSPECTOR_EXTENSIONS)
+#include "InspectorExtensionTypes.h"
+#endif
 
 namespace WebCore {
 class InspectorController;
+class InspectorFrontendHost;
 class CertificateInfo;
 class FloatRect;
 }
@@ -42,12 +47,19 @@ class FloatRect;
 namespace WebKit {
 
 class WebPage;
+#if ENABLE(INSPECTOR_EXTENSIONS)
+class WebInspectorUIExtensionController;
+#endif
 
-class WebInspectorUI : public RefCounted<WebInspectorUI>, private IPC::Connection::Client, public WebCore::InspectorFrontendClient {
+class WebInspectorUI final
+    : public RefCounted<WebInspectorUI>
+    , private IPC::Connection::Client
+    , public WebCore::InspectorFrontendClient {
 public:
     static Ref<WebInspectorUI> create(WebPage&);
+    virtual ~WebInspectorUI();
 
-    static void enableFrontendFeatures();
+    static void enableFrontendFeatures(WebPage&);
 
     // Implemented in generated WebInspectorUIMessageReceiver.cpp
     void didReceiveMessage(IPC::Connection&, IPC::Decoder&) override;
@@ -86,10 +98,11 @@ public:
     void didSave(const String& url);
     void didAppend(const String& url);
 
-    void sendMessageToFrontend(const String&);
+    void sendMessageToFrontend(const String& message);
+    void evaluateInFrontendForTesting(const String& expression);
 
 #if ENABLE(INSPECTOR_TELEMETRY)
-    void setDiagnosticLoggingAvailable(bool avaliable);
+    void setDiagnosticLoggingAvailable(bool);
 #endif
 
     // WebCore::InspectorFrontendClient
@@ -125,7 +138,7 @@ public:
 
     void changeSheetRect(const WebCore::FloatRect&) override;
 
-    void openInNewTab(const String& url) override;
+    void openURLExternally(const String& url) override;
 
     bool canSave() override;
     void save(const WTF::String& url, const WTF::String& content, bool base64Encoded, bool forceSaveAs) override;
@@ -139,9 +152,19 @@ public:
     bool diagnosticLoggingAvailable() override { return m_diagnosticLoggingAvailable; }
     void logDiagnosticEvent(const WTF::String& eventName, const WebCore::DiagnosticLoggingClient::ValueDictionary&) override;
 #endif
+        
+#if ENABLE(INSPECTOR_EXTENSIONS)
+    bool supportsWebExtensions() override;
+    void didShowExtensionTab(const Inspector::ExtensionID&, const Inspector::ExtensionTabID&, WebCore::FrameIdentifier) override;
+    void didHideExtensionTab(const Inspector::ExtensionID&, const Inspector::ExtensionTabID&) override;
+    void didNavigateExtensionTab(const Inspector::ExtensionID&, const Inspector::ExtensionTabID&, const URL&) override;
+    void inspectedPageDidNavigate(const URL&) override;
+#endif
 
     void sendMessageToBackend(const String&) override;
-
+    WebCore::InspectorFrontendAPIDispatcher& frontendAPIDispatcher() final { return m_frontendAPIDispatcher; }
+    WebCore::Page* frontendPage() final;
+        
     void pagePaused() override;
     void pageUnpaused() override;
 
@@ -151,14 +174,19 @@ private:
     explicit WebInspectorUI(WebPage&);
 
     WebPage& m_page;
-    WebInspectorFrontendAPIDispatcher m_frontendAPIDispatcher;
+    Ref<WebCore::InspectorFrontendAPIDispatcher> m_frontendAPIDispatcher;
     RefPtr<WebCore::InspectorFrontendHost> m_frontendHost;
-    RefPtr<IPC::Connection> m_backendConnection;
 
     // Keep a pointer to the frontend's inspector controller rather than going through
     // corePage(), since we may need it after the frontend's page has started destruction.
     WebCore::InspectorController* m_frontendController { nullptr };
 
+#if ENABLE(INSPECTOR_EXTENSIONS)
+    std::unique_ptr<WebInspectorUIExtensionController> m_extensionController;
+#endif
+
+    RefPtr<IPC::Connection> m_backendConnection;
+
     WebPageProxyIdentifier m_inspectedPageIdentifier;
     bool m_underTest { false };
     DebuggableInfoData m_debuggableInfo;
