@@ -41,7 +41,7 @@ void WebViewTest::resizeView(int width, int height)
     // FIXME: implement.
 }
 
-void WebViewTest::showInWindow()
+void WebViewTest::showInWindow(int, int)
 {
     auto* backend = webkit_web_view_backend_get_wpe_backend(webkit_web_view_get_backend(m_webView));
     wpe_view_backend_add_activity_state(backend, wpe_view_activity_state_visible | wpe_view_activity_state_in_window | wpe_view_activity_state_focused);
